﻿// Accord Vision Library
// The Accord.NET Framework (LGPL)
// http://accord.googlecode.com
//
// Copyright © César Souza, 2009-2013
// cesarsouza at gmail.com
//
// Copyright © Modesto Castrillon-Santana, 2008
// University of Las Palmas de Gran Canaria (ULPGC)
// Distributed under the LGPL with permission.
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

namespace Accord.Vision.Detection.Cascades
{
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.CodeDom.Compiler;

    /// <summary>
    ///   Automatic transcription of haar cascade definitions 
    ///   for facial features by Modesto Castrillon-Santana.
    /// </summary>
    /// 
    /// <remarks>
    /// <para>
    ///   This code has been automatically generated by the Accord.NET Framework
    ///   based on original research by Modesto Castrillon-Santana. The original
    ///   code has been shared under a BSD license in the OpenCV library and has
    ///   been incorporated in the Accord.NET Framework under permission of the
    ///   original author.</para>
    ///   
    /// <code>
    ///   // Copyright (c) 2008, Modesto Castrillon-Santana (IUSIANI, University of
    ///   // Las Palmas de Gran Canaria, Spain).
    ///   //  All rights reserved.
    ///   //  
    ///   // Redistribution and use in source and binary forms, with or without
    ///   // modification, are permitted provided that the following conditions are
    ///   // met:
    ///   //       
    ///   //    * Redistributions of source code must retain the above copyright
    ///   //      notice, this list of conditions and the following disclaimer.
    ///   //    * Redistributions in binary form must reproduce the above
    ///   //      copyright notice, this list of conditions and the following
    ///   //      disclaimer in the documentation and/or other materials provided
    ///   //      with the distribution.  
    ///   //    * The name of Contributor may not used to endorse or promote products 
    ///   //      derived from this software without specific prior written permission.
    ///   //
    ///   //  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ///   //  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ///   //  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ///   //  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    ///   //  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    ///   //  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    ///   //  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    ///   //  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    ///   //  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    ///   //  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    ///   //  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    /// </code>
    /// 
    /// </remarks>
    /// 
    [GeneratedCode("Accord.NET HaarCascadeWriter", "2.7")]
    public class NoseHaarCascade : Accord.Vision.Detection.HaarCascade
    {

        /// <summary>
        ///   Creates a new cascade for nose detection.
        /// </summary>
        /// 
        public NoseHaarCascade()
            : base(18, 15)
        {

            List<HaarCascadeStage> stages = new List<HaarCascadeStage>();
            List<HaarFeatureNode[]> nodes;
            HaarCascadeStage stage;

            HasTiltedFeatures = true;

            #region Stage 0
            stage = new HaarCascadeStage(-1.8310650587081909, -1, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0363217890262604, -0.6772649884223938, 0.668734610080719, true, new int[] { 8, 4, 4, 4, -1 }, new int[] { 8, 4, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0544859282672405, -0.440317690372467, 0.489185094833374, new int[] { 3, 0, 12, 7, -1 }, new int[] { 6, 0, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15089729428291321, 0.63702392578125, -0.28146758675575262, new int[] { 3, 5, 12, 9, -1 }, new int[] { 3, 8, 12, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0794939175248146, 0.63470429182052612, -0.16119189560413361, new int[] { 6, 0, 6, 8, -1 }, new int[] { 6, 0, 3, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.067041739821434, 0.59565997123718262, -0.16454219818115229, new int[] { 3, 8, 12, 4, -1 }, new int[] { 3, 10, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.16542470455169681, -0.0291650108993053, 0.27849620580673218, true, new int[] { 10, 1, 3, 8, -1 }, new int[] { 10, 1, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14491100609302521, -0.15930549800395971, 0.56260192394256592, true, new int[] { 8, 1, 8, 3, -1 }, new int[] { 8, 1, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0126969404518604, -0.6924440860748291, 0.1042767018079758, new int[] { 3, 0, 12, 3, -1 }, new int[] { 3, 1, 12, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052858339622616768, 0.0736001133918762, -0.81359738111495972, new int[] { 8, 0, 2, 3, -1 }, new int[] { 8, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13196030259132391, -0.0852369293570518, 0.64642858505249023, new int[] { 5, 6, 11, 9, -1 }, new int[] { 5, 9, 11, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6259789592586458E-05, -0.252252608537674, 0.27700841426849371, new int[] { 7, 1, 2, 1, -1 }, new int[] { 8, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.9456392743159086E-05, -0.15982529520988459, 0.1796030998229981, new int[] { 9, 1, 2, 1, -1 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0181720405817032, 0.46623438596725458, -0.1598974019289017, new int[] { 4, 0, 9, 7, -1 }, new int[] { 7, 0, 3, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.11940079927444459, 0.58289611339569092, -0.1248269975185394, new int[] { 3, 7, 12, 8, -1 }, new int[] { 3, 9, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.49619960784912109, 0.75930988788604736, -0.0939436629414558, new int[] { 2, 0, 14, 14, -1 }, new int[] { 9, 0, 7, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.18309399485588071, 0.58175492286682129, -0.0883935913443565, new int[] { 3, 4, 12, 9, -1 }, new int[] { 3, 7, 12, 3, 3 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 1
            stage = new HaarCascadeStage(-1.7070330381393433, 0, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0485280007123947, 0.00015333959890995175, -2673.697998046875, new int[] { 5, 3, 6, 1, -1 }, new int[] { 8, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1116186007857323, -0.1391783952713013, 0.47061970829963679, true, new int[] { 9, 2, 6, 4, -1 }, new int[] { 9, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.140942394733429, -0.459025502204895, 0.68740749359130859, true, new int[] { 9, 2, 5, 6, -1 }, new int[] { 9, 2, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15287929773330691, 0.2594836950302124, -0.0452645681798458, true, new int[] { 10, 0, 4, 9, -1 }, new int[] { 7, 3, 4, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0578792616724968, -0.3745568990707398, 0.46996209025383, true, new int[] { 8, 0, 8, 4, -1 }, new int[] { 10, 2, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059482799842953682, -0.33294659852981567, 0.27539891004562378, new int[] { 3, 0, 12, 1, -1 }, new int[] { 7, 0, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1846064031124115, 0.48681849241256708, -0.1640070974826813, new int[] { 2, 6, 14, 9, -1 }, new int[] { 2, 9, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066531449556350708, -0.65238291025161743, 0.11169300228357321, new int[] { 9, 0, 5, 3, -1 }, new int[] { 9, 1, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0080141983926296234, 0.11979129910469059, -0.71780908107757568, new int[] { 4, 0, 10, 2, -1 }, new int[] { 4, 0, 5, 1, 2 }, new int[] { 9, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13707329332828519, -0.1418797969818115, 0.3295237123966217, new int[] { 6, 0, 12, 14, -1 }, new int[] { 9, 0, 6, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0090329283848404884, 0.1041319966316223, -0.73359811305999756, new int[] { 5, 0, 6, 3, -1 }, new int[] { 5, 1, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.18033640086650851, -0.54879492521286011, 0.0710614770650864, new int[] { 14, 0, 4, 14, -1 }, new int[] { 14, 7, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0078154532238841057, -0.689528226852417, 0.1063653975725174, new int[] { 4, 1, 10, 2, -1 }, new int[] { 4, 1, 5, 1, 2 }, new int[] { 9, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1088579967617989, 0.70592081546783447, -0.1002665981650353, new int[] { 2, 9, 14, 4, -1 }, new int[] { 2, 11, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.17265169322490689, 0.48955410718917852, -0.13769739866256711, new int[] { 2, 4, 14, 9, -1 }, new int[] { 2, 7, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0574669800698757, 0.0478747487068176, -0.336111307144165, new int[] { 14, 0, 4, 12, -1 }, new int[] { 14, 6, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1294801980257034, -0.67898839712142944, 0.1097540035843849, new int[] { 0, 0, 4, 12, -1 }, new int[] { 0, 6, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058118398301303387, -0.50810492038726807, 0.0530205518007278, new int[] { 11, 1, 3, 3, -1 }, new int[] { 11, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052181649953126907, -0.7440345287322998, 0.0739578828215599, new int[] { 6, 3, 4, 2, -1 }, new int[] { 6, 3, 2, 1, 2 }, new int[] { 8, 4, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0141012202948332, -0.51200342178344727, 0.0294169094413519, new int[] { 8, 0, 6, 4, -1 }, new int[] { 8, 1, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.20708249509334559, -0.21835790574550629, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0066746207885444164, 0.0782192721962929, -0.58582967519760132, new int[] { 7, 1, 4, 3, -1 }, new int[] { 7, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00659123994410038, -0.65275472402572632, 0.0550902597606182, new int[] { 4, 1, 3, 3, -1 }, new int[] { 4, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.26057091355323792, 0.0209255293011665, -0.64536881446838379, new int[] { 10, 1, 8, 14, -1 }, new int[] { 10, 8, 8, 7, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 2
            stage = new HaarCascadeStage(-1.5818140506744385, 1, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0890733674168587, 0.54986131191253662, -0.50310492515563965, new int[] { 5, 5, 8, 6, -1 }, new int[] { 5, 8, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0470851697027683, 0.385565996170044, -0.16194720566272741, new int[] { 9, 0, 8, 12, -1 }, new int[] { 11, 0, 4, 12, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1344425976276398, -0.31617870926857, 0.56394141912460327, new int[] { 4, 0, 8, 10, -1 }, new int[] { 8, 0, 4, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.004263279028236866, -0.22349369525909421, 0.0977761000394821, true, new int[] { 9, 2, 2, 8, -1 }, new int[] { 9, 2, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1214829981327057, -0.13394290208816531, 0.5355374813079834, true, new int[] { 9, 3, 8, 2, -1 }, new int[] { 9, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0033225349616259336, -0.682870090007782, 0.083227276802063, new int[] { 10, 0, 1, 4, -1 }, new int[] { 10, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0077031590044498444, -0.68243968486785889, 0.10678689926862719, new int[] { 4, 2, 10, 2, -1 }, new int[] { 4, 2, 5, 1, 2 }, new int[] { 9, 3, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0353097803890705, -0.6521000862121582, 0.0987162664532661, new int[] { 0, 0, 18, 4, -1 }, new int[] { 9, 0, 9, 2, 2 }, new int[] { 0, 2, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0304474700242281, 0.2479538023471832, -0.25818860530853271, new int[] { 1, 0, 8, 14, -1 }, new int[] { 3, 0, 4, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048874127678573132, 0.0805528536438942, -0.63403171300888062, new int[] { 7, 0, 4, 2, -1 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14157949388027191, 0.63742321729660034, -0.0921661630272865, new int[] { 3, 4, 12, 8, -1 }, new int[] { 3, 6, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1456591933965683, -0.1032999008893967, 0.58382421731948853, new int[] { 4, 4, 10, 9, -1 }, new int[] { 4, 7, 10, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0116241797804832, -0.688891589641571, 0.0828648507595062, new int[] { 1, 0, 8, 3, -1 }, new int[] { 1, 1, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.02174754999578, -0.6213839054107666, 0.0476981997489929, new int[] { 8, 0, 6, 4, -1 }, new int[] { 8, 2, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.018483079969883, -0.2010547071695328, 0.26797088980674738, new int[] { 3, 0, 12, 2, -1 }, new int[] { 6, 0, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0369827300310135, -0.16930599510669711, 0.22727000713348389, new int[] { 5, 0, 12, 4, -1 }, new int[] { 8, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0168901197612286, 0.0774174928665161, -0.76188772916793823, true, new int[] { 7, 0, 3, 2, -1 }, new int[] { 7, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.2389906048774719, 0.43991729617118841, -0.1319973021745682, new int[] { 2, 6, 14, 9, -1 }, new int[] { 2, 9, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.18494910001754761, 0.73120379447937012, -0.072184756398201, new int[] { 4, 0, 10, 7, -1 }, new int[] { 9, 0, 5, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.009174540638923645, 0.0494462810456753, -0.57036292552948, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0072624902240931988, 0.0598880685865879, -0.70289182662963867, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0525570586323738, -0.0988772809505463, 0.17423820495605469, new int[] { 5, 0, 12, 4, -1 }, new int[] { 8, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0300392601639032, 0.49870780110359192, -0.0794838070869446, new int[] { 5, 0, 8, 7, -1 }, new int[] { 7, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109278596937656, -0.453724592924118, 0.0490351393818855, true, new int[] { 9, 0, 3, 2, -1 }, new int[] { 10, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.008502008393406868, -0.73869508504867554, 0.0514139384031296, true, new int[] { 9, 0, 1, 3, -1 }, new int[] { 8, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0552169494330883, -0.1239347010850906, 0.32208061218261719, new int[] { 1, 9, 16, 6, -1 }, new int[] { 1, 11, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0883669406175613, 0.48289158940315252, -0.0840416923165321, new int[] { 1, 9, 14, 4, -1 }, new int[] { 1, 11, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0171657595783472, -0.1314162015914917, 0.26804590225219732, new int[] { 5, 11, 8, 4, -1 }, new int[] { 5, 13, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.090517058968544, -0.0930236876010895, 0.40674149990081793, true, new int[] { 8, 0, 8, 2, -1 }, new int[] { 8, 0, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.015297899954021, -0.11356069892644879, 0.0976252779364586, new int[] { 5, 0, 12, 4, -1 }, new int[] { 8, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0306295193731785, 0.42534521222114557, -0.0865394771099091, new int[] { 0, 1, 16, 2, -1 }, new int[] { 4, 1, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0798880606889725, 0.0924375280737877, -0.39891800284385681, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 3
            stage = new HaarCascadeStage(-1.5400149822235107, 2, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0614461190998554, -0.45049899816513062, 0.485420286655426, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1895785927772522, -0.0670469328761101, 0.419770210981369, true, new int[] { 10, 0, 6, 7, -1 }, new int[] { 10, 0, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.173656702041626, -0.28913810849189758, 0.52919161319732666, true, new int[] { 8, 0, 7, 6, -1 }, new int[] { 8, 0, 7, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0164134204387665, 0.28622248768806458, -0.17473380267620089, new int[] { 12, 9, 6, 4, -1 }, new int[] { 12, 9, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0107280304655433, 0.31400930881500239, -0.28309330344200129, new int[] { 0, 9, 6, 4, -1 }, new int[] { 3, 9, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0077994461171329021, -0.28578609228134161, 0.22502979636192319, new int[] { 3, 1, 12, 1, -1 }, new int[] { 7, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011308079585433, 0.1045889034867287, -0.74274301528930664, new int[] { 4, 1, 8, 3, -1 }, new int[] { 4, 2, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1032197996973991, -0.1167842000722885, 0.4927442073822022, new int[] { 3, 4, 12, 8, -1 }, new int[] { 3, 6, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0056132972240448, 0.0890597030520439, -0.53440308570861816, new int[] { 6, 0, 6, 3, -1 }, new int[] { 6, 1, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0606942698359489, 0.55840307474136353, -0.0227699298411608, true, new int[] { 11, 5, 4, 4, -1 }, new int[] { 12, 6, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00624879403039813, 0.0758677795529366, -0.58721762895584106, true, new int[] { 9, 1, 1, 3, -1 }, new int[] { 8, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0400232896208763, 0.14124380052089691, -0.0172170307487249, true, new int[] { 11, 5, 4, 4, -1 }, new int[] { 12, 6, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0412207692861557, 0.51341092586517334, -0.085405632853508, true, new int[] { 7, 5, 4, 4, -1 }, new int[] { 6, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035766770597547293, -0.60522651672363281, 0.0409328490495682, new int[] { 10, 0, 2, 2, -1 }, new int[] { 10, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0099679548293352127, -0.60633981227874756, 0.0673605129122734, true, new int[] { 8, 0, 2, 3, -1 }, new int[] { 7, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0067802299745380878, 0.27804800868034357, -0.1798703074455261, new int[] { 6, 0, 6, 1, -1 }, new int[] { 6, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0207993201911449, 0.48167890310287481, -0.1240388005971909, new int[] { 6, 0, 6, 3, -1 }, new int[] { 9, 0, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.139158695936203, -0.044727511703968, 0.58631712198257446, new int[] { 5, 6, 10, 9, -1 }, new int[] { 5, 9, 10, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053711780346930027, 0.203908696770668, -0.233932301402092, new int[] { 6, 8, 6, 2, -1 }, new int[] { 6, 9, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0164771005511284, 0.0404451601207256, -0.625005304813385, new int[] { 15, 10, 3, 5, -1 }, new int[] { 16, 10, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011078979820013, 0.0576713494956493, -0.54169511795043945, new int[] { 0, 10, 3, 5, -1 }, new int[] { 1, 10, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.016222869977355, -0.1663480997085571, 0.20724619925022131, new int[] { 1, 11, 16, 4, -1 }, new int[] { 1, 13, 16, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00716758705675602, -0.47880691289901728, 0.075772762298584, new int[] { 0, 10, 3, 3, -1 }, new int[] { 0, 11, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0589063800871372, -0.0867818593978882, 0.39148110151290888, new int[] { 3, 7, 12, 8, -1 }, new int[] { 3, 9, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0931876674294472, 0.0619301609694958, -0.5739055871963501, new int[] { 0, 1, 4, 14, -1 }, new int[] { 0, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020346969831734896, -0.1360708028078079, 0.0450085289776325, true, new int[] { 15, 11, 3, 1, -1 }, new int[] { 16, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.005236657802015543, -0.18271179497241971, 0.16897720098495481, new int[] { 3, 0, 8, 1, -1 }, new int[] { 7, 0, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0105886701494455, -0.55421608686447144, 0.0492046102881432, true, new int[] { 12, 0, 3, 2, -1 }, new int[] { 13, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0100352102890611, 0.0409362092614174, -0.68710482120513916, true, new int[] { 4, 0, 1, 4, -1 }, new int[] { 3, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0344069004058838, 0.35165968537330627, -0.0428969487547874, new int[] { 4, 1, 12, 4, -1 }, new int[] { 7, 1, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074508260004222393, 0.0498083718121052, -0.61689341068267822, true, new int[] { 5, 0, 1, 3, -1 }, new int[] { 4, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.082342803478241, 0.083641491830349, -0.0810145065188408, true, new int[] { 12, 0, 2, 12, -1 }, new int[] { 12, 0, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0617706216871738, 0.32327979803085333, -0.0792278200387955, new int[] { 1, 0, 12, 5, -1 }, new int[] { 4, 0, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0364590808749199, -0.15961149334907529, 0.1232450976967812, new int[] { 5, 0, 12, 7, -1 }, new int[] { 8, 0, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0474974289536476, -0.1659339964389801, 0.29666280746459961, new int[] { 1, 0, 12, 7, -1 }, new int[] { 4, 0, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0096670873463153839, -0.58818382024765015, 0.0336683988571167, true, new int[] { 9, 1, 1, 3, -1 }, new int[] { 8, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069817090407013893, 0.0585361085832119, -0.47672748565673828, true, new int[] { 9, 1, 3, 1, -1 }, new int[] { 10, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1032517030835152, 0.22064709663391111, -0.1236488968133926, new int[] { 1, 6, 16, 8, -1 }, new int[] { 1, 8, 16, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0696480572223663, -0.10253950208425521, 0.37149900197982788, new int[] { 3, 5, 12, 8, -1 }, new int[] { 3, 7, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0588895305991173, 0.324886292219162, -0.0962660014629364, new int[] { 3, 4, 12, 4, -1 }, new int[] { 3, 6, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0299398303031921, 0.17989009618759161, -0.1531133055686951, new int[] { 1, 0, 8, 11, -1 }, new int[] { 3, 0, 4, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00850120559334755, 0.0426186993718147, -0.51194471120834351, true, new int[] { 11, 1, 3, 1, -1 }, new int[] { 12, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058030229993164539, -0.49628180265426641, 0.059898991137743, true, new int[] { 7, 1, 1, 3, -1 }, new int[] { 6, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0227242801338434, -0.0956752821803093, 0.2338289022445679, new int[] { 5, 12, 12, 2, -1 }, new int[] { 5, 12, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0372309498488903, 0.3216434121131897, -0.0921498537063599, new int[] { 2, 0, 12, 5, -1 }, new int[] { 6, 0, 4, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0166754201054573, 0.0617647506296635, -0.47197958827018738, new int[] { 1, 2, 17, 2, -1 }, new int[] { 1, 3, 17, 1, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 4
            stage = new HaarCascadeStage(-1.5587489604949951, 3, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0564467795193195, -0.4791874885559082, 0.49137350916862488, true, new int[] { 8, 4, 4, 4, -1 }, new int[] { 8, 4, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0106428097933531, -0.144835501909256, 0.31846639513969421, new int[] { 10, 1, 2, 11, -1 }, new int[] { 10, 1, 1, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0598327815532684, -0.36746969819068909, 0.27132889628410339, new int[] { 3, 1, 12, 9, -1 }, new int[] { 3, 4, 12, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0121322497725487, 0.1230909004807472, -0.0897226184606552, true, new int[] { 9, 0, 4, 2, -1 }, new int[] { 9, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031117030885070562, -0.35122260451316828, 0.22136250138282779, true, new int[] { 8, 0, 2, 4, -1 }, new int[] { 8, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0397736988961697, 0.20415990054607391, -0.043302271515131, new int[] { 10, 8, 8, 7, -1 }, new int[] { 10, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0183949507772923, 0.19368380308151251, -0.2287393063306809, new int[] { 0, 9, 8, 5, -1 }, new int[] { 4, 9, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092628989368677139, -0.22149570286273959, 0.2067804038524628, new int[] { 3, 0, 12, 1, -1 }, new int[] { 7, 0, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0098584238439798355, 0.0557319596409798, -0.6437491774559021, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00792868621647358, -0.628904402256012, 0.0527597591280937, new int[] { 8, 0, 8, 2, -1 }, new int[] { 8, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0654434263706207, -0.1031555980443955, 0.44659650325775152, new int[] { 2, 6, 14, 6, -1 }, new int[] { 2, 8, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0322746597230434, -0.1719404011964798, 0.36625158786773682, new int[] { 3, 9, 12, 4, -1 }, new int[] { 3, 11, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0480254292488098, 0.0847395211458206, -0.51354151964187622, new int[] { 0, 1, 3, 14, -1 }, new int[] { 0, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.011461510322988, -0.650554895401001, 0.0551190003752708, new int[] { 2, 0, 14, 2, -1 }, new int[] { 9, 0, 7, 1, 2 }, new int[] { 2, 1, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034770029596984386, -0.16373869776725769, 0.26408019661903381, new int[] { 8, 0, 2, 4, -1 }, new int[] { 9, 0, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0417843498289585, -0.74961292743682861, 0.037305548787117, new int[] { 8, 0, 3, 6, -1 }, new int[] { 9, 2, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.31991851329803472, 0.40143400430679321, -0.1033769026398659, new int[] { 2, 1, 14, 14, -1 }, new int[] { 9, 1, 7, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1278306990861893, 0.27113029360771179, -0.009534287266433239, true, new int[] { 9, 1, 2, 9, -1 }, new int[] { 6, 4, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.063939742743969, -0.13559409976005549, 0.3188548088073731, true, new int[] { 9, 1, 9, 2, -1 }, new int[] { 12, 4, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14868929982185361, -0.0747430101037025, 0.50650840997695923, new int[] { 1, 6, 16, 9, -1 }, new int[] { 1, 9, 16, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0108674801886082, 0.0678603425621986, -0.56486707925796509, true, new int[] { 9, 1, 4, 1, -1 }, new int[] { 10, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.11102750152349471, 0.36937940120697021, -0.1024053022265434, new int[] { 5, 3, 8, 6, -1 }, new int[] { 5, 6, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0554906614124775, -0.13388429582118991, 0.32509210705757141, new int[] { 2, 5, 14, 6, -1 }, new int[] { 2, 7, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1232120022177696, -0.44768521189689642, 0.073690727353096, new int[] { 0, 0, 18, 10, -1 }, new int[] { 9, 0, 9, 5, 2 }, new int[] { 0, 5, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0203750394284725, -0.662591278553009, 0.0422433987259865, new int[] { 0, 4, 18, 2, -1 }, new int[] { 0, 4, 9, 1, 2 }, new int[] { 9, 5, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030578291043639183, 0.18292440474033361, -0.121791198849678, new int[] { 16, 0, 2, 10, -1 }, new int[] { 16, 0, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0161957796663046, -0.63178831338882446, 0.0402268916368485, true, new int[] { 6, 0, 2, 4, -1 }, new int[] { 5, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0509672202169895, -0.0774049535393715, 0.24355340003967291, new int[] { 6, 0, 12, 3, -1 }, new int[] { 9, 0, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0580940917134285, -0.1238128989934921, 0.25356009602546692, new int[] { 3, 0, 9, 9, -1 }, new int[] { 6, 0, 3, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.004231311846524477, -0.53830701112747192, 0.0235711093991995, new int[] { 9, 4, 3, 1, -1 }, new int[] { 10, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0187011696398258, 0.37818440794944758, -0.0800608471035957, new int[] { 5, 0, 8, 8, -1 }, new int[] { 7, 0, 4, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025685389991849661, -0.16534459590911871, 0.16206049919128421, new int[] { 3, 12, 12, 2, -1 }, new int[] { 3, 13, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0019677819218486547, -0.17564539611339569, 0.153071403503418, new int[] { 7, 0, 2, 5, -1 }, new int[] { 8, 0, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.35487169027328491, -0.0136137595400214, 0.36016708612442022, true, new int[] { 12, 0, 3, 12, -1 }, new int[] { 12, 0, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.26808801293373108, -0.0809430927038193, 0.369129091501236, true, new int[] { 6, 0, 12, 3, -1 }, new int[] { 6, 0, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0628807172179222, -0.0913113132119179, 0.32952618598937988, new int[] { 2, 9, 15, 6, -1 }, new int[] { 2, 11, 15, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.024154469370842, -0.0686313733458519, 0.45747300982475281, true, new int[] { 6, 7, 4, 2, -1 }, new int[] { 6, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0091738719493150711, 0.0545422695577145, -0.51373308897018433, new int[] { 3, 1, 12, 2, -1 }, new int[] { 9, 1, 6, 1, 2 }, new int[] { 3, 2, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0130733698606491, -0.59702301025390625, 0.0365914106369019, new int[] { 6, 1, 3, 3, -1 }, new int[] { 7, 2, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068077309988439083, -0.0354327894747257, 0.25199410319328308, new int[] { 11, 8, 3, 2, -1 }, new int[] { 11, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0451491102576256, 0.0638899281620979, -0.38367250561714172, new int[] { 0, 7, 18, 4, -1 }, new int[] { 0, 7, 9, 2, 2 }, new int[] { 9, 9, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0099950553849339485, 0.0132095599547029, -0.45377358794212341, true, new int[] { 9, 0, 3, 1, -1 }, new int[] { 10, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079643689095973969, 0.0337183102965355, -0.653340220451355, true, new int[] { 9, 0, 1, 3, -1 }, new int[] { 8, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.35672768950462341, 0.0322214402258396, -0.58003139495849609, new int[] { 0, 1, 18, 14, -1 }, new int[] { 9, 1, 9, 7, 2 }, new int[] { 0, 8, 9, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0362690612673759, 0.24694380164146421, -0.104957602918148, new int[] { 0, 0, 12, 3, -1 }, new int[] { 3, 0, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0427862294018269, -0.0707177072763443, 0.36938878893852228, new int[] { 5, 0, 8, 3, -1 }, new int[] { 5, 0, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011904439888894558, -0.38284510374069208, 0.061551358550787, new int[] { 8, 1, 1, 2, -1 }, new int[] { 8, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1074014976620674, -0.0219720508903265, 0.1813759058713913, true, new int[] { 17, 2, 1, 12, -1 }, new int[] { 17, 2, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0774416774511337, -0.20107130706310269, 0.11222709715366359, new int[] { 0, 0, 18, 8, -1 }, new int[] { 6, 0, 6, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0711435526609421, -0.0310098994523287, 0.0730640217661858, true, new int[] { 17, 2, 1, 12, -1 }, new int[] { 17, 2, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.057338759303093, 0.4086444079875946, -0.0614440515637398, new int[] { 2, 1, 14, 8, -1 }, new int[] { 2, 3, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.072106160223484, 0.339823991060257, -0.0868131667375565, new int[] { 2, 5, 14, 6, -1 }, new int[] { 2, 7, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0585803911089897, -0.49610468745231628, 0.0615561902523041, true, new int[] { 1, 2, 12, 1, -1 }, new int[] { 1, 2, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044991881586611271, 0.0394841395318508, -0.46022048592567438, new int[] { 7, 5, 4, 2, -1 }, new int[] { 9, 5, 2, 1, 2 }, new int[] { 7, 6, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0579723715782166, -0.11365810036659239, 0.18178419768810269, new int[] { 1, 4, 16, 6, -1 }, new int[] { 1, 6, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.41217011213302612, 0.0172915197908878, -0.80449968576431274, new int[] { 5, 0, 13, 12, -1 }, new int[] { 5, 3, 13, 6, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 5
            stage = new HaarCascadeStage(-1.5197360515594482, 4, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0492322407662869, 0.40377280116081238, -0.42361000180244451, new int[] { 5, 6, 8, 4, -1 }, new int[] { 5, 8, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0273310504853725, -0.13277700543403631, 0.207337498664856, new int[] { 9, 0, 4, 10, -1 }, new int[] { 9, 0, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0451007597148418, 0.3161504864692688, -0.42044240236282349, new int[] { 1, 0, 9, 12, -1 }, new int[] { 4, 0, 3, 12, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.25283211469650269, -0.57497388124465942, 0.0644379332661629, new int[] { 4, 4, 14, 10, -1 }, new int[] { 11, 4, 7, 5, 2 }, new int[] { 4, 9, 7, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0427955314517021, 0.125260204076767, -0.36320659518241882, new int[] { 0, 4, 12, 10, -1 }, new int[] { 0, 4, 6, 5, 2 }, new int[] { 6, 9, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10599119961261751, -0.59337782859802246, 0.1167925000190735, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071173040196299553, -0.202963799238205, 0.2159796953201294, new int[] { 1, 11, 15, 2, -1 }, new int[] { 1, 12, 15, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0115433102473617, -0.56954711675643921, 0.069512702524662, new int[] { 3, 0, 14, 2, -1 }, new int[] { 3, 1, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0259417798370123, 0.0406758897006512, -0.596626877784729, new int[] { 3, 0, 7, 4, -1 }, new int[] { 3, 2, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1111780032515526, 0.39230749011039728, -0.0852632820606232, new int[] { 3, 4, 12, 8, -1 }, new int[] { 3, 6, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.139802098274231, -0.20322300493717191, 0.25884169340133673, new int[] { 2, 1, 13, 12, -1 }, new int[] { 2, 5, 13, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0223447605967522, -0.2217562943696976, 0.1535113006830216, new int[] { 3, 0, 12, 4, -1 }, new int[] { 6, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0356404818594456, -0.11393369734287261, 0.29229050874710077, new int[] { 7, 0, 4, 7, -1 }, new int[] { 9, 0, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0080998390913009644, 0.0395722091197968, -0.66712599992752075, new int[] { 7, 0, 4, 4, -1 }, new int[] { 7, 1, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0534741394221783, -0.0767945721745491, 0.4321976900100708, new int[] { 2, 7, 14, 6, -1 }, new int[] { 2, 9, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0138621004298329, 0.0846036896109581, -0.1605919003486633, new int[] { 11, 1, 7, 10, -1 }, new int[] { 11, 6, 7, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0770997405052185, 0.54772442579269409, -0.066370002925396, new int[] { 4, 0, 10, 3, -1 }, new int[] { 9, 0, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0128013696521521, -0.55477362871170044, 0.0567846409976482, new int[] { 0, 1, 18, 2, -1 }, new int[] { 9, 1, 9, 1, 2 }, new int[] { 0, 2, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00010235139779979363, 0.1450944989919663, -0.19509540498256681, new int[] { 0, 2, 2, 2, -1 }, new int[] { 1, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0070487200282514095, 0.0400543101131916, -0.44429570436477661, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0045558041892945766, -0.43548169732093811, 0.0606299117207527, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0193000100553036, -0.0711913108825684, 0.0810695365071297, true, new int[] { 14, 4, 2, 7, -1 }, new int[] { 14, 4, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054058600217103958, -0.1416722983121872, 0.196803405880928, new int[] { 1, 10, 16, 2, -1 }, new int[] { 1, 11, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0086945146322250366, -0.1313387006521225, 0.0205014292150736, true, new int[] { 13, 6, 2, 6, -1 }, new int[] { 13, 6, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00971742533147335, -0.18720309436321261, 0.1876177042722702, new int[] { 4, 9, 8, 4, -1 }, new int[] { 8, 9, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.11155830323696141, 0.40864959359169012, -0.069993183016777, new int[] { 2, 8, 14, 4, -1 }, new int[] { 2, 10, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.097640797495842, -0.1244983971118927, 0.21617740392684939, new int[] { 3, 0, 12, 9, -1 }, new int[] { 3, 3, 12, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15061390399932861, -0.38674619793891912, 0.0543168187141418, new int[] { 14, 1, 4, 12, -1 }, new int[] { 14, 7, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049472171813249588, 0.0436532311141491, -0.51559001207351685, new int[] { 6, 0, 6, 2, -1 }, new int[] { 6, 0, 3, 1, 2 }, new int[] { 9, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0204955395311117, -0.54416948556900024, 0.0076605947688221931, true, new int[] { 10, 2, 2, 2, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0272786691784859, 0.426749587059021, -0.0565182790160179, new int[] { 5, 0, 8, 6, -1 }, new int[] { 7, 0, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0135246496647596, -0.0507161505520344, 0.1838100999593735, new int[] { 11, 8, 4, 6, -1 }, new int[] { 11, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0949866473674774, -0.4232459962368012, 0.0522982999682426, new int[] { 0, 0, 18, 8, -1 }, new int[] { 0, 0, 9, 4, 2 }, new int[] { 9, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1105156019330025, 0.0035527960862964392, -0.41661360859870911, new int[] { 14, 1, 4, 10, -1 }, new int[] { 14, 6, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1319251954555512, -0.62827968597412109, 0.0391492694616318, new int[] { 0, 1, 4, 10, -1 }, new int[] { 0, 6, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.01942471973598, 0.00065935368184000254, -0.57528150081634521, true, new int[] { 10, 2, 2, 2, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0147077599540353, 0.0390244014561176, -0.56517869234085083, true, new int[] { 8, 2, 2, 2, -1 }, new int[] { 8, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00029291698592714965, -0.1292673051357269, 0.12589070200920111, new int[] { 10, 2, 2, 1, -1 }, new int[] { 10, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021614220459014177, -0.13799719512462619, 0.16510820388793951, new int[] { 7, 0, 2, 3, -1 }, new int[] { 8, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.48753958940505981, 0.43802809715271, -0.0606237016618252, new int[] { 3, 0, 12, 12, -1 }, new int[] { 3, 6, 12, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0505968406796455, -0.0435010008513927, 0.51223617792129517, new int[] { 3, 5, 12, 4, -1 }, new int[] { 3, 7, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1982239037752152, 0.0168439298868179, -0.45089399814605707, new int[] { 12, 1, 6, 14, -1 }, new int[] { 12, 8, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0525614693760872, 0.61911600828170776, -0.0332456789910793, new int[] { 2, 13, 14, 2, -1 }, new int[] { 2, 14, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0394346490502357, -0.13324570655822751, 0.1555656045675278, new int[] { 0, 4, 18, 6, -1 }, new int[] { 0, 6, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00828025583177805, -0.46491861343383789, 0.0463778004050255, new int[] { 0, 8, 2, 4, -1 }, new int[] { 0, 9, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.18781690299510961, -0.0738439187407494, 0.20355209708213809, new int[] { 6, 0, 12, 10, -1 }, new int[] { 10, 0, 4, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0592883005738258, -0.10040319710969931, 0.29306849837303162, new int[] { 0, 0, 12, 10, -1 }, new int[] { 4, 0, 4, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00483306311070919, -0.1236037984490395, 0.18227760493755341, new int[] { 3, 13, 12, 2, -1 }, new int[] { 3, 14, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0134623004123569, -0.08650141954422, 0.2545304000377655, new int[] { 3, 8, 4, 6, -1 }, new int[] { 3, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0112787801772356, 0.035953551530838, -0.36370408535003662, new int[] { 13, 11, 4, 4, -1 }, new int[] { 14, 11, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.11120849847793581, 0.0411560982465744, -0.49355891346931458, new int[] { 0, 1, 5, 14, -1 }, new int[] { 0, 8, 5, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00589548796415329, 0.00860547088086605, -0.57748162746429443, new int[] { 10, 5, 2, 1, -1 }, new int[] { 10, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.0609137765131891E-05, -0.19438520073890689, 0.10896600037813189, new int[] { 6, 5, 2, 1, -1 }, new int[] { 7, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0111626898869872, -0.1052400022745132, 0.17699919641017911, new int[] { 1, 11, 17, 4, -1 }, new int[] { 1, 12, 17, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0147585002705455, 0.0338271111249924, -0.57838040590286255, new int[] { 7, 0, 3, 3, -1 }, new int[] { 8, 1, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.006510044913738966, 0.012222490273416, -0.68323171138763428, new int[] { 9, 1, 2, 2, -1 }, new int[] { 9, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0132402600720525, 0.0317283198237419, -0.496233195066452, new int[] { 1, 11, 4, 4, -1 }, new int[] { 2, 11, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.21011430025100711, -0.49222511053085333, 0.0054596872068941593, new int[] { 8, 3, 10, 8, -1 }, new int[] { 8, 3, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24140250682830811, 0.0314619205892086, -0.56909531354904175, new int[] { 0, 3, 10, 8, -1 }, new int[] { 5, 3, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038006789982318878, -0.0650670900940895, 0.0376422517001629, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12624409794807431, 0.039377398788929, -0.4590097963809967, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 9, 9, 3, 2 }, new int[] { 9, 12, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0130107998847961, -0.0579108111560345, 0.29622611403465271, new int[] { 10, 7, 6, 4, -1 }, new int[] { 13, 7, 3, 2, 2 }, new int[] { 10, 9, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071800998412072659, 0.0342495106160641, -0.56361818313598633, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0242467503994703, -0.1086483970284462, 0.1013154983520508, new int[] { 4, 0, 12, 2, -1 }, new int[] { 8, 0, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1696685999631882, -0.3411920964717865, 0.0499880090355873, new int[] { 1, 5, 16, 10, -1 }, new int[] { 1, 5, 8, 5, 2 }, new int[] { 9, 10, 8, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0204610601067543, -0.2079558074474335, 0.0034589329734444618, true, new int[] { 12, 6, 2, 4, -1 }, new int[] { 12, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0213081296533346, 0.50270938873291016, -0.0400764681398869, true, new int[] { 6, 6, 4, 2, -1 }, new int[] { 6, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109308399260044, 0.15635550022125241, -0.075159102678299, new int[] { 10, 8, 6, 4, -1 }, new int[] { 13, 8, 3, 2, 2 }, new int[] { 10, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0099652167409658432, 0.036286398768425, -0.505298912525177, true, new int[] { 9, 0, 4, 1, -1 }, new int[] { 10, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0033498809207230806, -0.27242329716682429, 0.0273806899785995, new int[] { 17, 10, 1, 3, -1 }, new int[] { 17, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.059739351272583, 0.0268720109015703, -0.63886368274688721, true, new int[] { 3, 0, 11, 2, -1 }, new int[] { 3, 0, 11, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.127812996506691, 0.0014498339733108878, -0.38336989283561712, new int[] { 8, 6, 10, 8, -1 }, new int[] { 13, 6, 5, 4, 2 }, new int[] { 8, 10, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00293133407831192, -0.13099479675292969, 0.12987799942493439, new int[] { 2, 12, 14, 2, -1 }, new int[] { 2, 13, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0061392742209136486, 0.0108347898349166, -0.31701850891113281, new int[] { 17, 10, 1, 3, -1 }, new int[] { 17, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0811345130205154, -0.35706749558448792, 0.0494775287806988, true, new int[] { 7, 4, 8, 3, -1 }, new int[] { 9, 6, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.060443039983511, 0.4088949859142304, -0.0221638102084398, new int[] { 13, 9, 3, 6, -1 }, new int[] { 13, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059390361420810223, -0.1046036034822464, 0.1944513022899628, new int[] { 2, 9, 3, 6, -1 }, new int[] { 2, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.8998396929819137E-05, -0.047956719994545, 0.0571181289851666, new int[] { 17, 10, 1, 3, -1 }, new int[] { 17, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0018057189881801605, -0.29241380095481873, 0.0581192187964916, new int[] { 0, 10, 1, 3, -1 }, new int[] { 0, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0067375837825238705, -0.0886564627289772, 0.0441452711820602, new int[] { 11, 1, 6, 6, -1 }, new int[] { 11, 1, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.5221098591573536E-05, -0.1249044984579086, 0.12661279737949371, new int[] { 3, 6, 2, 1, -1 }, new int[] { 4, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0241630896925926, -0.0133935501798987, 0.3467755913734436, true, new int[] { 15, 0, 3, 3, -1 }, new int[] { 14, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0127861900255084, -0.0568488091230392, 0.27275329828262329, new int[] { 1, 7, 8, 4, -1 }, new int[] { 1, 7, 4, 2, 2 }, new int[] { 5, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00135722104460001, 0.0654089972376823, -0.1414448022842407, new int[] { 8, 8, 4, 2, -1 }, new int[] { 8, 9, 4, 1, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 6
            stage = new HaarCascadeStage(-1.508497953414917, 5, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.1201385036110878, -0.36573138833045959, 0.36293190717697138, true, new int[] { 9, 2, 4, 6, -1 }, new int[] { 9, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.146201103925705, 0.3965567946434021, -0.19461369514465329, new int[] { 2, 6, 14, 8, -1 }, new int[] { 2, 8, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0123430602252483, -0.24749830365180969, 0.22562310099601751, new int[] { 5, 0, 8, 3, -1 }, new int[] { 7, 0, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003274885006248951, 0.0721044987440109, -0.38964301347732538, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24311800301074979, 0.00946643017232418, 1062.68798828125, true, new int[] { 8, 3, 3, 6, -1 }, new int[] { 8, 3, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0399235188961029, -0.12903560698032379, 0.19358199834823611, new int[] { 6, 0, 12, 4, -1 }, new int[] { 9, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070425998419523239, 0.1544698029756546, -0.2654632031917572, new int[] { 0, 8, 6, 7, -1 }, new int[] { 3, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065724221058189869, 0.0737086832523346, -0.58167362213134766, new int[] { 4, 1, 10, 2, -1 }, new int[] { 9, 1, 5, 1, 2 }, new int[] { 4, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0233357399702072, -0.42724540829658508, 0.0886551067233086, new int[] { 3, 0, 6, 6, -1 }, new int[] { 3, 3, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0262159798294306, 0.35602480173110962, -0.1014178022742271, new int[] { 3, 9, 12, 2, -1 }, new int[] { 3, 10, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0114004900678992, -0.1101441010832787, 0.36441218852996832, new int[] { 1, 9, 16, 2, -1 }, new int[] { 1, 10, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145206097513437, 0.0214245207607746, -0.4902862012386322, new int[] { 9, 3, 3, 3, -1 }, new int[] { 10, 4, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0085834655910730362, -0.652571976184845, 0.0546631813049316, new int[] { 5, 3, 8, 2, -1 }, new int[] { 5, 3, 4, 1, 2 }, new int[] { 9, 4, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13745459914207461, -0.50492751598358154, 0.0527309887111187, new int[] { 2, 0, 14, 12, -1 }, new int[] { 9, 0, 7, 6, 2 }, new int[] { 2, 6, 7, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0126157002523541, -0.62455308437347412, 0.0316158086061478, true, new int[] { 7, 0, 2, 3, -1 }, new int[] { 6, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3604110538144596E-05, 0.0987414866685867, -0.0946909487247467, new int[] { 15, 2, 2, 1, -1 }, new int[] { 15, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.8249959693057463E-05, 0.1445119976997376, -0.16137890517711639, new int[] { 1, 2, 2, 1, -1 }, new int[] { 2, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0199512392282486, -0.37731361389160162, 0.0244714803993702, true, new int[] { 13, 0, 4, 4, -1 }, new int[] { 14, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0549685694277287, -0.44058069586753851, 0.0534904003143311, true, new int[] { 8, 5, 4, 5, -1 }, new int[] { 8, 5, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0169392302632332, -0.66650348901748657, 0.031559631228447, new int[] { 5, 0, 8, 4, -1 }, new int[] { 5, 1, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0110901398584247, 0.0311973206698895, -0.547548770904541, true, new int[] { 3, 1, 2, 3, -1 }, new int[] { 2, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0289862100034952, -0.12510849535465241, 0.0918823182582855, new int[] { 5, 0, 12, 4, -1 }, new int[] { 8, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1045346036553383, 0.43575459718704218, -0.0606762506067753, new int[] { 5, 0, 8, 7, -1 }, new int[] { 9, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046273069456219673, 0.0973885133862495, -0.0912084132432938, true, new int[] { 11, 3, 1, 8, -1 }, new int[] { 9, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.51698398590087891, -0.0609911382198334, 0.48797199130058289, new int[] { 3, 3, 12, 9, -1 }, new int[] { 7, 6, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0667436569929123, 0.37274169921875, -0.0635046362876892, new int[] { 1, 4, 16, 6, -1 }, new int[] { 1, 6, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0154703501611948, 0.0610504113137722, -0.48717978596687322, true, new int[] { 6, 1, 3, 2, -1 }, new int[] { 6, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035926289856433868, 0.1421190947294235, -0.1508843004703522, new int[] { 7, 9, 4, 2, -1 }, new int[] { 7, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.20565569400787351, -0.47814959287643433, 0.0436189286410809, new int[] { 1, 5, 14, 10, -1 }, new int[] { 1, 5, 7, 5, 2 }, new int[] { 8, 10, 7, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0296549908816814, -0.035474069416523, 0.1896422952413559, new int[] { 10, 9, 3, 6, -1 }, new int[] { 10, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13284200429916379, 0.0555178187787533, -0.39714470505714422, new int[] { 0, 5, 18, 10, -1 }, new int[] { 0, 5, 9, 5, 2 }, new int[] { 9, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003375923028215766, 0.0415674299001694, -0.36205479502677917, new int[] { 8, 0, 3, 2, -1 }, new int[] { 8, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00054163701133802533, -0.1866434067487717, 0.1040982976555824, new int[] { 5, 1, 2, 5, -1 }, new int[] { 6, 1, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0527310110628605, 0.276021808385849, -0.0270596593618393, new int[] { 4, 0, 12, 7, -1 }, new int[] { 8, 0, 4, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0621075518429279, 0.31340470910072332, -0.0696556121110916, new int[] { 1, 0, 12, 4, -1 }, new int[] { 4, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0139620797708631, 0.0415851585566998, -0.1057448983192444, new int[] { 12, 0, 6, 14, -1 }, new int[] { 12, 7, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0591135807335377, -0.1132714971899986, 0.21400369703769681, new int[] { 2, 0, 12, 5, -1 }, new int[] { 5, 0, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.32472780346870422, -0.2102808952331543, 0.0147817200049758, new int[] { 12, 0, 6, 14, -1 }, new int[] { 12, 7, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0095277121290564537, 0.1057813987135887, -0.21662670373916629, new int[] { 0, 0, 6, 14, -1 }, new int[] { 0, 7, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0557695515453815, 0.27192020416259771, -0.0213698092848063, new int[] { 10, 9, 3, 6, -1 }, new int[] { 10, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.013918150216341, -0.0888932272791862, 0.255586713552475, new int[] { 5, 9, 3, 6, -1 }, new int[] { 5, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0023373179137706757, -0.1157324984669685, 0.15424209833145139, new int[] { 7, 13, 6, 2, -1 }, new int[] { 7, 14, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071918689645826817, 0.0410376191139221, -0.50523638725280762, true, new int[] { 8, 2, 1, 3, -1 }, new int[] { 7, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0085471794009208679, 0.014381350018084, -0.2316330969333649, true, new int[] { 15, 3, 3, 1, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0032956521026790142, -0.28280371427536011, 0.0618998408317566, true, new int[] { 3, 11, 1, 3, -1 }, new int[] { 2, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0220706891268492, 0.1489437073469162, -0.0949123501777649, new int[] { 11, 6, 3, 4, -1 }, new int[] { 11, 8, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.16646440327167511, -0.0590463504195213, 0.45291060209274292, new int[] { 2, 5, 14, 9, -1 }, new int[] { 2, 8, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079817809164524078, -0.0702360421419144, 0.1200437024235725, new int[] { 10, 10, 8, 4, -1 }, new int[] { 14, 10, 4, 2, 2 }, new int[] { 10, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0077218217775225639, 0.0476134307682514, -0.41645199060440058, new int[] { 0, 12, 4, 3, -1 }, new int[] { 1, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.8179560104035772E-05, -0.11355110257864, 0.0995815470814705, new int[] { 8, 0, 8, 2, -1 }, new int[] { 8, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0115354498848319, 0.0479713715612888, -0.4701226949691773, new int[] { 0, 0, 18, 2, -1 }, new int[] { 0, 0, 9, 1, 2 }, new int[] { 9, 1, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0417897514998913, 0.18016649782657621, -0.0923613235354424, new int[] { 7, 1, 4, 12, -1 }, new int[] { 7, 1, 2, 12, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075845858082175255, -0.1170279979705811, 0.1517726927995682, new int[] { 0, 11, 18, 4, -1 }, new int[] { 0, 12, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0117145096883178, -0.0399577096104622, 0.0563791207969189, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0809042006731033, -0.0586656406521797, 0.3254713118076325, new int[] { 0, 8, 18, 6, -1 }, new int[] { 0, 10, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0111858202144504, -0.15692700445652011, 0.1074031963944435, new int[] { 11, 0, 6, 6, -1 }, new int[] { 11, 0, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0207462906837463, -0.0727149471640587, 0.29882580041885382, true, new int[] { 6, 7, 4, 2, -1 }, new int[] { 6, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071547999978065491, 0.0502206012606621, -0.38929650187492371, new int[] { 6, 6, 6, 2, -1 }, new int[] { 9, 6, 3, 1, 2 }, new int[] { 6, 7, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00476626493036747, 0.1062309965491295, -0.1640899926424027, new int[] { 6, 7, 5, 2, -1 }, new int[] { 6, 8, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0132446801289916, -0.0340634994208813, 0.31890881061553961, new int[] { 11, 6, 3, 4, -1 }, new int[] { 11, 8, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020384900271892548, 0.0399366803467274, -0.46564960479736328, new int[] { 7, 0, 1, 2, -1 }, new int[] { 7, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0223837792873383, 0.0195741802453995, -0.31799200177192688, new int[] { 9, 13, 6, 2, -1 }, new int[] { 11, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.006019658874720335, -0.40058508515357971, 0.041111808270216, new int[] { 8, 2, 2, 6, -1 }, new int[] { 8, 2, 1, 3, 2 }, new int[] { 9, 5, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0133403996005654, 0.0072229830548167229, -0.35855838656425482, new int[] { 16, 8, 2, 3, -1 }, new int[] { 16, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.16548049449920649, 0.0360200293362141, -0.44204410910606379, true, new int[] { 6, 1, 10, 4, -1 }, new int[] { 6, 1, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0172677896916866, 0.0957728773355484, -0.030379680916667, new int[] { 10, 10, 8, 4, -1 }, new int[] { 14, 10, 4, 2, 2 }, new int[] { 10, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003787358058616519, -0.13409850001335141, 0.1292660981416702, new int[] { 0, 10, 8, 4, -1 }, new int[] { 0, 10, 4, 2, 2 }, new int[] { 4, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055727548897266388, -0.0669078826904297, 0.17382170259952551, true, new int[] { 13, 6, 3, 4, -1 }, new int[] { 14, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0085729602724313736, 0.0307218804955482, -0.58534258604049683, new int[] { 0, 8, 2, 3, -1 }, new int[] { 0, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0263858195394278, 0.17780029773712161, -0.0393683984875679, true, new int[] { 13, 6, 3, 4, -1 }, new int[] { 14, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0118999304249883, -0.0571489408612251, 0.30101099610328669, true, new int[] { 5, 6, 4, 3, -1 }, new int[] { 4, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0683530792593956, 0.0291851498186588, -0.15513670444488531, new int[] { 10, 3, 8, 3, -1 }, new int[] { 10, 3, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0108240302652121, -0.1347029060125351, 0.1385277062654495, new int[] { 0, 3, 8, 3, -1 }, new int[] { 4, 3, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0880321934819222, -0.0365363508462906, 0.2360302060842514, new int[] { 4, 2, 14, 2, -1 }, new int[] { 4, 2, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.025776170194149, 0.1835854053497315, -0.13343839347362521, new int[] { 0, 1, 12, 4, -1 }, new int[] { 3, 1, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0820100232958794, 0.0118177495896816, -0.31878089904785162, new int[] { 13, 1, 4, 10, -1 }, new int[] { 13, 6, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0203707292675972, 0.250352293252945, -0.0702304020524025, new int[] { 0, 2, 14, 2, -1 }, new int[] { 7, 2, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0784170925617218, 0.0254040490835905, -0.21633470058441159, new int[] { 4, 12, 12, 3, -1 }, new int[] { 8, 12, 4, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054000681266188622, 0.0398776307702065, -0.38197609782218928, new int[] { 0, 9, 2, 3, -1 }, new int[] { 0, 10, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0116557897999883, 0.008572491817176342, -0.46817851066589361, new int[] { 10, 6, 2, 2, -1 }, new int[] { 10, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.1775790527462959E-05, -0.17354169487953189, 0.0904209986329079, new int[] { 6, 6, 2, 2, -1 }, new int[] { 7, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0180264692753553, -0.79275929927825928, 0.0092333797365427017, true, new int[] { 15, 2, 3, 1, -1 }, new int[] { 16, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021709210705012083, -0.0846288874745369, 0.1654430031776428, new int[] { 4, 7, 3, 3, -1 }, new int[] { 4, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.082279697060585, 0.21551139652729029, -0.091900646686554, new int[] { 3, 5, 12, 4, -1 }, new int[] { 3, 7, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0102933598682284, 0.0234903004020453, -0.67681080102920532, true, new int[] { 3, 2, 1, 3, -1 }, new int[] { 2, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.218819797039032, 0.50478667020797729, -0.0318927802145481, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 11, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0221189390867949, -0.63159322738647461, 0.025988319888711, new int[] { 0, 4, 18, 2, -1 }, new int[] { 0, 4, 9, 1, 2 }, new int[] { 9, 5, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0229423604905605, -0.040672279894352, 0.35672950744628912, true, new int[] { 15, 3, 3, 3, -1 }, new int[] { 14, 4, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0567631609737873, 0.35523039102554321, -0.038303978741169, new int[] { 2, 2, 14, 6, -1 }, new int[] { 2, 4, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075660292059183121, -0.37110349535942078, 0.0192387793213129, true, new int[] { 9, 2, 1, 3, -1 }, new int[] { 8, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1234833970665932, 0.0215323101729155, -0.63291150331497192, new int[] { 0, 1, 6, 10, -1 }, new int[] { 0, 6, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7259990019956604E-05, -0.12036570161581039, 0.1052009984850884, new int[] { 9, 3, 6, 2, -1 }, new int[] { 9, 4, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0855550765991211, 0.0342116691172123, -0.48727419972419739, new int[] { 0, 0, 18, 6, -1 }, new int[] { 0, 0, 9, 3, 2 }, new int[] { 9, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.149810403585434, 0.42568850517272949, -0.0406881310045719, new int[] { 4, 2, 10, 6, -1 }, new int[] { 4, 5, 10, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0249004401266575, -0.0469012595713139, 0.28062260150909418, new int[] { 0, 5, 9, 3, -1 }, new int[] { 3, 5, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0078607350587844849, 0.0052375709637999535, -0.9763677716255188, new int[] { 9, 3, 2, 1, -1 }, new int[] { 9, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.3002476710826159E-05, -0.16680990159511569, 0.1061896979808807, new int[] { 7, 3, 2, 1, -1 }, new int[] { 8, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1778886020183563, -0.0167296305298805, 0.17790630459785459, true, new int[] { 9, 2, 6, 4, -1 }, new int[] { 9, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0129577601328492, 0.0327777788043022, -0.44296708703041082, new int[] { 7, 2, 3, 3, -1 }, new int[] { 8, 3, 1, 1, 9 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 7
            stage = new HaarCascadeStage(-1.4449690580368042, 6, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0671501830220222, 0.39577248692512512, -0.31510940194129938, true, new int[] { 7, 4, 4, 6, -1 }, new int[] { 5, 6, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0489628501236439, -0.26961261034011841, 0.16869769990444181, new int[] { 4, 2, 14, 8, -1 }, new int[] { 4, 4, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002719441894441843, -0.35195991396903992, 0.2283660024404526, true, new int[] { 9, 0, 1, 4, -1 }, new int[] { 9, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0051611121743917465, 0.24076780676841739, -0.22074960172176361, new int[] { 12, 7, 6, 7, -1 }, new int[] { 12, 7, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.23630170524120331, -0.0165349505841732, -791.90631103515625, new int[] { 2, 9, 8, 4, -1 }, new int[] { 2, 11, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0192054994404316, 0.36792600154876709, -0.0511916503310204, new int[] { 13, 6, 1, 6, -1 }, new int[] { 13, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038221171125769615, -0.145134299993515, 0.32845288515090942, new int[] { 4, 6, 1, 6, -1 }, new int[] { 4, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0114400796592236, -0.35804128646850591, 0.1191418990492821, new int[] { 8, 0, 6, 4, -1 }, new int[] { 8, 2, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0088761039078235626, -0.21450379490852359, 0.17957879602909091, true, new int[] { 9, 0, 6, 1, -1 }, new int[] { 9, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0084572024643421173, -0.069746732711792, 0.16367790102958679, new int[] { 6, 0, 12, 1, -1 }, new int[] { 9, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1268958002328873, 0.24832360446453089, -0.12166699767112731, new int[] { 2, 7, 14, 8, -1 }, new int[] { 2, 9, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046295030042529106, -0.0560571514070034, 0.35743680596351618, new int[] { 11, 8, 3, 2, -1 }, new int[] { 11, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.5959236710332334E-05, 0.1490119993686676, -0.18527039885520941, new int[] { 0, 0, 2, 3, -1 }, new int[] { 1, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13179309666156769, 0.0314710587263107, -0.65023940801620483, new int[] { 10, 0, 6, 8, -1 }, new int[] { 10, 4, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0135068297386169, 0.049855500459671, -0.52044898271560669, true, new int[] { 6, 0, 3, 2, -1 }, new int[] { 6, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13922819495201111, -0.42741641402244568, 0.022189699113369, new int[] { 14, 5, 4, 10, -1 }, new int[] { 14, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0602215304970741, 0.0557326711714268, -0.4318253099918366, new int[] { 0, 5, 4, 10, -1 }, new int[] { 0, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13498260080814359, -0.71942609548568726, 0.000654424715321511, new int[] { 12, 8, 6, 6, -1 }, new int[] { 12, 8, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003972203005105257, 0.1110355034470558, -0.206549197435379, new int[] { 0, 8, 6, 6, -1 }, new int[] { 3, 8, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0218843296170235, -0.2502841055393219, 0.0452274195849895, new int[] { 10, 0, 6, 6, -1 }, new int[] { 10, 3, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0562942214310169, 0.0373776294291019, -0.62178802490234375, new int[] { 2, 0, 6, 6, -1 }, new int[] { 2, 3, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0416125096380711, -0.58709871768951416, 0.0327165089547634, new int[] { 0, 0, 18, 4, -1 }, new int[] { 9, 0, 9, 2, 2 }, new int[] { 0, 2, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073085748590528965, -0.1344400942325592, 0.18418920040130621, new int[] { 2, 10, 14, 2, -1 }, new int[] { 2, 11, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0391575917601585, -0.0723762214183807, 0.0374199710786343, true, new int[] { 11, 5, 3, 6, -1 }, new int[] { 9, 7, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00821463018655777, -0.2051306068897247, 0.1153298020362854, new int[] { 1, 4, 16, 1, -1 }, new int[] { 5, 4, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0034585020039230585, 0.0500501617789268, -0.0578955002129078, new int[] { 10, 8, 4, 4, -1 }, new int[] { 10, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040681189857423306, -0.0944659411907196, 0.292072594165802, new int[] { 4, 8, 4, 4, -1 }, new int[] { 4, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0549114495515823, -0.3530954122543335, 0.0700343772768974, new int[] { 3, 6, 12, 8, -1 }, new int[] { 9, 6, 6, 4, 2 }, new int[] { 3, 10, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0693727433681488, 0.0222254004329443, -0.71920287609100342, new int[] { 3, 12, 9, 3, -1 }, new int[] { 6, 12, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.079585537314415, -0.0380740091204643, 0.303349107503891, true, new int[] { 11, 4, 6, 4, -1 }, new int[] { 13, 6, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0544063299894333, 0.0448827184736729, -0.44952940940856928, new int[] { 0, 0, 3, 12, -1 }, new int[] { 0, 6, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2690613865852356, -0.0360089801251888, 0.53076601028442383, new int[] { 4, 0, 10, 9, -1 }, new int[] { 4, 0, 5, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041156299412250519, -0.1003653034567833, 0.18043400347232821, new int[] { 7, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1438598036766052, -0.62012898921966553, 0.0115139102563262, new int[] { 6, 9, 8, 5, -1 }, new int[] { 6, 9, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0144033199176192, -0.0768772587180138, 0.26086720824241638, true, new int[] { 5, 3, 3, 5, -1 }, new int[] { 6, 4, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039774607867002487, 0.0425334200263023, -0.46169069409370422, new int[] { 8, 0, 2, 4, -1 }, new int[] { 8, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0468562692403793, 0.48750248551368708, -0.0433990210294724, new int[] { 0, 13, 18, 2, -1 }, new int[] { 0, 14, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052139908075332642, 0.1103964000940323, -0.18073910474777219, new int[] { 6, 8, 6, 2, -1 }, new int[] { 6, 9, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0067679318599402905, -0.52303707599639893, 0.0307772196829319, new int[] { 4, 0, 6, 2, -1 }, new int[] { 4, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0071862619370222092, 0.18328289687633509, -0.0569993406534195, true, new int[] { 15, 4, 3, 3, -1 }, new int[] { 14, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00096733449026942253, 0.15355390310287481, -0.10831949859857561, new int[] { 0, 8, 2, 2, -1 }, new int[] { 1, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0292031392455101, -0.037776630371809, 0.10933200269937519, true, new int[] { 11, 4, 6, 4, -1 }, new int[] { 13, 6, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0098407091572880745, -0.10926169902086259, 0.16795679926872251, new int[] { 2, 0, 12, 1, -1 }, new int[] { 5, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44505208730697632, 0.0268258899450302, -0.780637800693512, new int[] { 1, 1, 16, 11, -1 }, new int[] { 5, 1, 8, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0061639058403670788, -0.49384048581123352, 0.0311304796487093, true, new int[] { 4, 0, 1, 3, -1 }, new int[] { 3, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.049183439463377, -0.32318601012229919, 0.0469044297933578, new int[] { 4, 3, 10, 8, -1 }, new int[] { 9, 3, 5, 4, 2 }, new int[] { 4, 7, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6128649551537819E-05, -0.1063510999083519, 0.1544602960348129, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0368313007056713, 0.28206101059913641, -0.0126016000285745, true, new int[] { 12, 8, 3, 3, -1 }, new int[] { 13, 9, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0718847513198853, 0.23140460252761841, -0.0733308866620064, new int[] { 1, 5, 16, 6, -1 }, new int[] { 1, 7, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0574985891580582, -0.0964356362819672, 0.20507499575614929, new int[] { 0, 5, 18, 6, -1 }, new int[] { 0, 7, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039720349013805389, 0.0360010303556919, -0.54572492837905884, new int[] { 0, 7, 1, 3, -1 }, new int[] { 0, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0026467780116945505, -0.0441318899393082, 0.0756502225995064, true, new int[] { 12, 8, 3, 3, -1 }, new int[] { 13, 9, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0088836792856454849, -0.46108219027519232, 0.032768789678812, true, new int[] { 8, 6, 3, 1, -1 }, new int[] { 9, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0128562701866031, 0.0721951574087143, -0.0297321807593107, true, new int[] { 12, 8, 3, 3, -1 }, new int[] { 13, 9, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0120727699249983, -0.0505888797342777, 0.29055860638618469, true, new int[] { 6, 8, 3, 3, -1 }, new int[] { 5, 9, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00018108480435330421, -0.0714614391326904, 0.0798238515853882, new int[] { 11, 13, 1, 2, -1 }, new int[] { 11, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0160763803869486, 0.0476631112396717, -0.32759100198745728, new int[] { 1, 13, 16, 2, -1 }, new int[] { 1, 13, 8, 1, 2 }, new int[] { 9, 14, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0095250606536865234, -0.18988420069217679, 0.00708581879734993, true, new int[] { 16, 12, 2, 1, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0062362798489630222, -0.42836889624595642, 0.0339706018567085, true, new int[] { 2, 12, 1, 2, -1 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.080308698117733, 0.11084640026092531, new int[] { 12, 9, 2, 2, -1 }, new int[] { 13, 9, 1, 1, 2 }, new int[] { 12, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011949270265176892, 0.2256557047367096, -0.0626343935728073, new int[] { 4, 9, 2, 2, -1 }, new int[] { 4, 9, 1, 1, 2 }, new int[] { 5, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.5406976975500584E-05, -0.1237920969724655, 0.0894999876618385, new int[] { 11, 2, 2, 1, -1 }, new int[] { 11, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0155067397281528, 0.31002271175384521, -0.0654744282364845, new int[] { 5, 0, 8, 9, -1 }, new int[] { 7, 0, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051327929832041264, 0.0204462595283985, -0.49159330129623408, new int[] { 11, 2, 2, 1, -1 }, new int[] { 11, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.8783698730403557E-05, -0.172290101647377, 0.1088512986898422, new int[] { 5, 2, 2, 1, -1 }, new int[] { 6, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.006178875919431448, 0.0195190999656916, -0.31397709250450129, true, new int[] { 9, 1, 1, 3, -1 }, new int[] { 8, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.17130610346794131, 0.0172466896474361, -0.77260631322860718, new int[] { 4, 9, 8, 6, -1 }, new int[] { 8, 9, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0429867096245289, 0.1577536016702652, -0.0482686497271061, new int[] { 8, 1, 4, 10, -1 }, new int[] { 8, 1, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052703949622809887, -0.46245059370994568, 0.0392020307481289, true, new int[] { 9, 1, 3, 1, -1 }, new int[] { 10, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2032378017902374, 0.0357716716825962, -0.39400190114974981, new int[] { 0, 5, 18, 10, -1 }, new int[] { 9, 5, 9, 5, 2 }, new int[] { 0, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0182179491966963, -0.0407346189022064, 0.37419110536575317, true, new int[] { 3, 2, 3, 3, -1 }, new int[] { 4, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00010606779687805101, 0.10123260319232941, -0.0911243632435799, new int[] { 17, 4, 1, 2, -1 }, new int[] { 17, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038906659465283155, -0.15201710164546969, 0.0934790223836899, new int[] { 0, 6, 10, 1, -1 }, new int[] { 5, 6, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0125372298061848, -0.0601580515503883, 0.25583261251449591, new int[] { 7, 0, 4, 4, -1 }, new int[] { 7, 0, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0099574513733387, 0.13798029720783231, -0.1249634027481079, new int[] { 3, 5, 6, 5, -1 }, new int[] { 5, 5, 2, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036789269652217627, 0.0427718199789524, -0.30630341172218323, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037803261075168848, 0.0323704518377781, -0.41383808851242071, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.8372930400073528E-05, -0.0645466670393944, 0.0794665068387985, new int[] { 16, 2, 2, 2, -1 }, new int[] { 17, 2, 1, 1, 2 }, new int[] { 16, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3996631070040166E-05, 0.1355656981468201, -0.11014910042285921, new int[] { 0, 2, 2, 2, -1 }, new int[] { 0, 2, 1, 1, 2 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.3484519564080983E-05, 0.128577396273613, -0.0937314331531525, new int[] { 16, 2, 2, 2, -1 }, new int[] { 17, 2, 1, 1, 2 }, new int[] { 16, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0100723998621106, -0.382802814245224, 0.034546609967947, new int[] { 7, 3, 4, 4, -1 }, new int[] { 7, 3, 2, 2, 2 }, new int[] { 9, 5, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103168003261089, 0.12971499562263489, -0.1024452969431877, new int[] { 5, 6, 8, 2, -1 }, new int[] { 5, 7, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0107137700542808, -0.0704529136419296, 0.23588269948959351, true, new int[] { 7, 4, 5, 4, -1 }, new int[] { 6, 5, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0262797605246305, -0.1242780014872551, 0.0811929032206535, new int[] { 5, 0, 12, 4, -1 }, new int[] { 8, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055222269147634506, 0.0614674314856529, -0.26426988840103149, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074345488101243973, -0.0884712487459183, 0.14741429686546331, new int[] { 0, 12, 18, 3, -1 }, new int[] { 0, 13, 18, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00581725500524044, -0.313044011592865, 0.0437002405524254, new int[] { 8, 4, 2, 6, -1 }, new int[] { 8, 4, 1, 3, 2 }, new int[] { 9, 7, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0365137197077274, 0.32511061429977423, -0.0333890803158283, new int[] { 5, 0, 9, 4, -1 }, new int[] { 8, 0, 3, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0463338792324066, 0.50428932905197144, -0.0255471803247929, new int[] { 1, 12, 15, 3, -1 }, new int[] { 1, 13, 15, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00015593919670209289, -0.0568273402750492, 0.07766093313694, new int[] { 17, 9, 1, 3, -1 }, new int[] { 17, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082058515399694443, 0.0321849994361401, -0.4203890860080719, true, new int[] { 1, 11, 3, 1, -1 }, new int[] { 2, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.044285461306572, -0.3896655142307282, 0.011912340298295, true, new int[] { 12, 7, 6, 1, -1 }, new int[] { 12, 7, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0258340202271938, 0.0417318902909756, -0.33182808756828308, true, new int[] { 6, 7, 1, 6, -1 }, new int[] { 6, 7, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0309912301599979, 0.0173530708998442, -0.66546082496643066, new int[] { 7, 7, 4, 6, -1 }, new int[] { 8, 7, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0112233497202396, -0.0643179565668106, 0.21755810081958771, new int[] { 1, 8, 16, 3, -1 }, new int[] { 1, 9, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010795110138133168, 0.0604902096092701, -0.12580770254135129, new int[] { 9, 7, 2, 2, -1 }, new int[] { 9, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15915779769420621, 0.0323631800711155, -0.40798279643058782, true, new int[] { 5, 0, 10, 4, -1 }, new int[] { 5, 0, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.564980968600139E-05, -0.0744273290038109, 0.0895882174372673, new int[] { 16, 2, 2, 2, -1 }, new int[] { 17, 2, 1, 1, 2 }, new int[] { 16, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.093008317053318, 0.13343870639801031, new int[] { 0, 2, 2, 2, -1 }, new int[] { 0, 2, 1, 1, 2 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0146180903539062, 0.0191540997475386, -0.64152318239212036, new int[] { 7, 1, 4, 4, -1 }, new int[] { 9, 1, 2, 2, 2 }, new int[] { 7, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0235322006046772, -0.0603582113981247, 0.21782620251178739, new int[] { 4, 9, 4, 6, -1 }, new int[] { 4, 11, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015804159920662642, -0.10721720010042191, 0.0938933715224266, new int[] { 5, 12, 9, 2, -1 }, new int[] { 5, 13, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1098610013723373, 0.0602713786065578, -0.23471720516681671, true, new int[] { 2, 1, 10, 2, -1 }, new int[] { 2, 1, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0099525712430477142, -0.59630388021469116, 0.0226748306304216, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002722450066357851, -0.34362030029296881, 0.0317178517580032, new int[] { 0, 9, 1, 3, -1 }, new int[] { 0, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0325947701931, 0.20315499603748319, -0.0711073279380798, new int[] { 0, 8, 18, 2, -1 }, new int[] { 0, 9, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031989789567887783, 0.0400660485029221, -0.31384450197219849, new int[] { 0, 5, 1, 4, -1 }, new int[] { 0, 6, 1, 2, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 8
            stage = new HaarCascadeStage(-1.4003620147705078, 7, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0778383314609528, -0.28954571485519409, 0.33590829372406011, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.018956370651722, 0.13711029291152951, -0.11915580183267591, new int[] { 8, 7, 9, 7, -1 }, new int[] { 11, 7, 3, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0290122292935848, 0.2680377066135407, -0.28188169002532959, new int[] { 0, 8, 12, 4, -1 }, new int[] { 3, 8, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0009855274111032486, -0.081531323492527, 0.15281049907207489, true, new int[] { 10, 0, 2, 1, -1 }, new int[] { 10, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00010328469943488017, -0.24661579728126529, 0.17609159648418429, true, new int[] { 8, 0, 1, 2, -1 }, new int[] { 8, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0045671691186726093, -0.48002299666404719, 0.0658785030245781, new int[] { 6, 0, 7, 2, -1 }, new int[] { 6, 1, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0235463008284569, -0.16119800508022311, 0.17704969644546509, new int[] { 1, 10, 15, 4, -1 }, new int[] { 1, 12, 15, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.10163839906454091, 0.0247533395886421, -0.56535172462463379, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.011764950118959, 0.057793740183115, -0.3604769110679627, new int[] { 8, 3, 2, 12, -1 }, new int[] { 8, 9, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029407900292426348, -0.0568644516170025, 0.32670629024505621, new int[] { 12, 8, 2, 2, -1 }, new int[] { 12, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0120360003784299, 0.0500290505588055, -0.43046820163726812, true, new int[] { 9, 0, 3, 3, -1 }, new int[] { 8, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.2945342506282032E-05, 0.1441446989774704, -0.12317640334367751, new int[] { 15, 1, 2, 1, -1 }, new int[] { 15, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10069269686937329, -0.4235703051090241, 0.0498026795685291, new int[] { 0, 1, 4, 10, -1 }, new int[] { 0, 6, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.014581729657948, 0.0301772207021713, -0.66406387090682983, new int[] { 4, 0, 10, 2, -1 }, new int[] { 4, 1, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.543241033796221E-05, 0.12506960332393649, -0.16383630037307739, true, new int[] { 3, 0, 1, 2, -1 }, new int[] { 3, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0099888555705547333, -0.3976281881332398, 0.031741239130497, true, new int[] { 15, 0, 3, 2, -1 }, new int[] { 16, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145155703648925, -0.0675602331757545, 0.32044398784637451, true, new int[] { 4, 2, 3, 4, -1 }, new int[] { 5, 3, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044144429266452789, -0.110104501247406, 0.1062017008662224, true, new int[] { 10, 0, 6, 1, -1 }, new int[] { 10, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0190477203577757, 0.4359183013439179, -0.0567054599523544, true, new int[] { 3, 3, 3, 3, -1 }, new int[] { 4, 4, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0119225401431322, 0.0226012095808983, -0.34638860821723938, true, new int[] { 15, 0, 3, 2, -1 }, new int[] { 16, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0316638201475143, -0.0697475075721741, 0.334603488445282, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 9, 0, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040487637743353844, -0.37775671482086182, 0.0412449985742569, new int[] { 8, 1, 4, 2, -1 }, new int[] { 8, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0085836304351687431, 0.0405867286026478, -0.46596848964691162, true, new int[] { 3, 0, 2, 3, -1 }, new int[] { 2, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2546002864837647, 0.0292705502361059, -0.6189153790473938, new int[] { 0, 5, 18, 10, -1 }, new int[] { 9, 5, 9, 5, 2 }, new int[] { 0, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0027734090108424425, 0.1460099071264267, -0.1248235031962395, new int[] { 0, 0, 2, 6, -1 }, new int[] { 0, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0091764237731695175, 0.2481728941202164, -0.0557485483586788, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074874111451208591, -0.1071233004331589, 0.1664687991142273, new int[] { 6, 0, 6, 1, -1 }, new int[] { 9, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0503873117268085, -0.0504896901547909, 0.12678450345993039, true, new int[] { 10, 0, 3, 8, -1 }, new int[] { 8, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0775756686925888, 0.1210061982274056, -0.17718310654163361, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104536600410938, -0.0304590705782175, 0.2466717064380646, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0119400899857283, 0.14313019812107089, -0.1400607973337174, new int[] { 5, 8, 8, 2, -1 }, new int[] { 5, 9, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021164349745959044, 0.0545042082667351, -0.0924128219485283, new int[] { 11, 8, 3, 2, -1 }, new int[] { 11, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048259901814162731, -0.0795849785208702, 0.42220059037208563, new int[] { 4, 8, 3, 2, -1 }, new int[] { 4, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0080155059695243835, 0.019714679569006, -0.479563295841217, new int[] { 10, 2, 3, 2, -1 }, new int[] { 10, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0072104120627045631, -0.46714499592781072, 0.0325505807995796, new int[] { 2, 0, 8, 2, -1 }, new int[] { 2, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0316700302064419, 0.37553250789642328, -0.0109495399519801, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.006346333771944046, -0.0652034804224968, 0.24626299738883969, true, new int[] { 2, 2, 3, 2, -1 }, new int[] { 3, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036191360559314489, -0.1709388941526413, 0.0311141796410084, new int[] { 17, 8, 1, 4, -1 }, new int[] { 17, 9, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0043581780046224594, 0.0366473011672497, -0.4237492978572846, new int[] { 0, 8, 1, 4, -1 }, new int[] { 0, 9, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071306470781564713, 0.0361863411962986, -0.35813450813293463, new int[] { 3, 0, 12, 2, -1 }, new int[] { 9, 0, 6, 1, 2 }, new int[] { 3, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.20273950695991519, -0.0464575290679932, 0.32370680570602423, true, new int[] { 9, 1, 8, 3, -1 }, new int[] { 9, 1, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0068010999821126461, 0.1703307926654816, -0.0903682932257652, new int[] { 8, 0, 2, 6, -1 }, new int[] { 8, 0, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0198947098106146, 0.0316714681684971, -0.6259496808052063, true, new int[] { 8, 0, 3, 2, -1 }, new int[] { 8, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00072822818765416741, -0.0703171566128731, 0.0968886613845825, new int[] { 8, 8, 6, 2, -1 }, new int[] { 11, 8, 3, 1, 2 }, new int[] { 8, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.36959239840507507, 0.0186286699026823, -0.77441781759262085, new int[] { 0, 3, 18, 12, -1 }, new int[] { 0, 9, 18, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0101259099319577, -0.0668892487883568, 0.15247039496898651, new int[] { 13, 8, 3, 6, -1 }, new int[] { 14, 10, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1245594993233681, 0.289630800485611, -0.0485628917813301, new int[] { 2, 8, 14, 4, -1 }, new int[] { 2, 10, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025091960560530424, -0.0350436493754387, 0.1112501993775368, true, new int[] { 13, 9, 3, 1, -1 }, new int[] { 14, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.28475138545036321, 0.35674199461936951, -0.0428154803812504, new int[] { 4, 0, 10, 15, -1 }, new int[] { 9, 0, 5, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016454169526696205, 0.1969088017940521, -0.0439714081585407, new int[] { 9, 0, 2, 1, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015759950038045645, -0.15584190189838409, 0.1092967018485069, new int[] { 5, 0, 4, 1, -1 }, new int[] { 7, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.70181107125245E-05, -0.093722403049469, 0.0794489830732346, new int[] { 10, 2, 3, 2, -1 }, new int[] { 10, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045426278375089169, 0.0382768400013447, -0.42568549513816828, new int[] { 5, 2, 3, 2, -1 }, new int[] { 5, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00058855221141129732, 0.0603053607046604, -0.14615760743618009, new int[] { 10, 7, 1, 2, -1 }, new int[] { 10, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0134366303682327, -0.23946529626846311, 0.0633801072835922, new int[] { 6, 6, 6, 4, -1 }, new int[] { 6, 6, 3, 2, 2 }, new int[] { 9, 8, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046623498201370239, -0.0411083400249481, 0.0386099815368652, true, new int[] { 10, 7, 4, 3, -1 }, new int[] { 11, 8, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0196607392281294, -0.0376873910427094, 0.39592269062995911, true, new int[] { 8, 7, 3, 4, -1 }, new int[] { 7, 8, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0082754753530025482, 0.1025618016719818, -0.0427510403096676, true, new int[] { 11, 9, 4, 1, -1 }, new int[] { 11, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0317808799445629, 0.36264151334762568, -0.040603369474411, true, new int[] { 7, 7, 3, 3, -1 }, new int[] { 6, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0216846503317356, 0.0229385606944561, -0.35124549269676208, new int[] { 11, 13, 6, 2, -1 }, new int[] { 13, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0154039999470115, 0.29343938827514648, -0.0483902990818024, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081902230158448219, -0.32770949602127081, 0.0413685590028763, new int[] { 2, 1, 14, 2, -1 }, new int[] { 9, 1, 7, 1, 2 }, new int[] { 2, 2, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0079587763175368309, -0.58493942022323608, 0.0197221394628286, true, new int[] { 9, 2, 3, 1, -1 }, new int[] { 10, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0223498903214931, 0.0063248360529541969, -0.0670235827565193, true, new int[] { 9, 3, 2, 8, -1 }, new int[] { 7, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001803660998120904, -0.0722102373838425, 0.20629370212554929, new int[] { 5, 7, 1, 4, -1 }, new int[] { 5, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0204626396298409, -0.34459498524665833, 0.0262401904910803, new int[] { 11, 13, 6, 2, -1 }, new int[] { 13, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(9.1937501565553248E-05, -0.1117258965969086, 0.1140339002013207, true, new int[] { 5, 9, 1, 3, -1 }, new int[] { 4, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020170810166746378, 0.0586952790617943, -0.0434083491563797, new int[] { 11, 13, 6, 2, -1 }, new int[] { 13, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016941629583016038, 0.0660928636789322, -0.2047823965549469, true, new int[] { 4, 11, 1, 2, -1 }, new int[] { 4, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.112091101706028, -0.00039467259193770587, -0.5106043815612793, true, new int[] { 9, 3, 2, 8, -1 }, new int[] { 7, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0729039311408997, -0.0399064607918262, 0.33780521154403692, true, new int[] { 9, 3, 8, 2, -1 }, new int[] { 11, 5, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030249240808188915, 0.1124901026487351, -0.14893929660320279, new int[] { 7, 6, 4, 2, -1 }, new int[] { 7, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0179907791316509, -0.24895049631595609, 0.0522084012627602, new int[] { 6, 7, 6, 3, -1 }, new int[] { 8, 8, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0281639993190765, 0.34624269604682922, -0.0468134209513664, new int[] { 5, 0, 8, 8, -1 }, new int[] { 7, 0, 4, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1455519050359726, -0.137273296713829, 0.0992739796638489, new int[] { 0, 0, 18, 8, -1 }, new int[] { 6, 0, 6, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1902603954076767, 0.0178888794034719, -0.71033167839050293, true, new int[] { 14, 0, 4, 9, -1 }, new int[] { 14, 0, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1708780974149704, 0.0214544609189034, -0.56766891479492188, true, new int[] { 4, 0, 9, 4, -1 }, new int[] { 4, 0, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0493922904133797, 0.4660165011882782, -0.0284054595977068, new int[] { 3, 13, 13, 2, -1 }, new int[] { 3, 14, 13, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039778267964720726, -0.10497090220451361, 0.1207138001918793, new int[] { 1, 13, 16, 2, -1 }, new int[] { 1, 14, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1800612956285477, 0.38309639692306519, -0.0141020696610212, new int[] { 11, 9, 6, 6, -1 }, new int[] { 13, 11, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0033417791128158569, -0.1053301990032196, 0.129559800028801, new int[] { 1, 9, 6, 6, -1 }, new int[] { 3, 11, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0289579704403877, -0.32808870077133179, 0.0085954880341887474, new int[] { 11, 13, 6, 2, -1 }, new int[] { 13, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0129891699180007, 0.0406576991081238, -0.34399700164794922, new int[] { 1, 13, 6, 2, -1 }, new int[] { 3, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.009318917989730835, 0.0200005602091551, -0.30933129787445068, new int[] { 11, 3, 5, 2, -1 }, new int[] { 11, 4, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.2429470088100061E-05, 0.126863107085228, -0.0951527133584023, new int[] { 7, 0, 2, 2, -1 }, new int[] { 7, 0, 1, 1, 2 }, new int[] { 8, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.6926601246232167E-05, -0.0697774663567543, 0.1006100997328758, new int[] { 9, 0, 2, 2, -1 }, new int[] { 10, 0, 1, 1, 2 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066324290819466114, -0.37384641170501709, 0.0329254008829594, true, new int[] { 7, 2, 1, 3, -1 }, new int[] { 6, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0018024910241365433, 0.0833972916007042, -0.0764525309205055, new int[] { 13, 0, 3, 8, -1 }, new int[] { 14, 0, 1, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0651966035366058, 0.031775739043951, -0.36805319786071777, new int[] { 1, 11, 16, 4, -1 }, new int[] { 1, 11, 8, 2, 2 }, new int[] { 9, 13, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0174991004168987, -0.25744670629501343, 0.0206988304853439, new int[] { 13, 0, 3, 8, -1 }, new int[] { 14, 0, 1, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00872408039867878, -0.0517450198531151, 0.226482704281807, true, new int[] { 3, 0, 3, 3, -1 }, new int[] { 4, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0024927619379013777, 0.0974271073937416, -0.0842309221625328, new int[] { 12, 2, 3, 13, -1 }, new int[] { 13, 2, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.044600460678339, -0.76867169141769409, 0.0147034004330635, new int[] { 3, 0, 3, 15, -1 }, new int[] { 4, 0, 1, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0325057990849018, 0.0300058592110872, -0.49162200093269348, new int[] { 17, 1, 1, 14, -1 }, new int[] { 17, 8, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.564980968600139E-05, 0.1131459027528763, -0.0940568000078201, new int[] { 0, 0, 2, 2, -1 }, new int[] { 0, 0, 1, 1, 2 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3604110538144596E-05, 0.0883647277951241, -0.0680588483810425, new int[] { 16, 0, 2, 2, -1 }, new int[] { 17, 0, 1, 1, 2 }, new int[] { 16, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.621649946377147E-05, -0.0913942903280258, 0.1227736994624138, new int[] { 0, 0, 2, 2, -1 }, new int[] { 0, 0, 1, 1, 2 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002901752945035696, -0.151534304022789, 0.0306931808590889, new int[] { 10, 1, 5, 2, -1 }, new int[] { 10, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068409377709031105, 0.0285490602254868, -0.37030708789825439, new int[] { 3, 1, 5, 2, -1 }, new int[] { 3, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12914350628852839, 0.0526567809283733, -0.20276160538196561, new int[] { 0, 5, 18, 10, -1 }, new int[] { 9, 5, 9, 5, 2 }, new int[] { 0, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1138025000691414, 0.222510501742363, -0.0516252294182777, true, new int[] { 8, 3, 5, 6, -1 }, new int[] { 6, 5, 5, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082800639793276787, -0.065930999815464, 0.0602529682219028, new int[] { 11, 4, 3, 6, -1 }, new int[] { 12, 6, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0530367009341717, -0.46652480959892267, 0.0276027899235487, true, new int[] { 8, 6, 1, 8, -1 }, new int[] { 8, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1186264008283615, -0.0335345789790154, 0.37986829876899719, new int[] { 1, 7, 16, 6, -1 }, new int[] { 1, 9, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0030761719681322575, -0.1226020976901054, 0.1153718009591103, new int[] { 4, 4, 3, 6, -1 }, new int[] { 5, 6, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0001753035030560568, 0.08503807336092, -0.0923559591174126, true, new int[] { 16, 8, 2, 2, -1 }, new int[] { 16, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0667972564697266, 0.0270407292991877, -0.45982721447944641, new int[] { 1, 8, 16, 4, -1 }, new int[] { 1, 8, 8, 2, 2 }, new int[] { 9, 10, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.02337945997715, -0.0620422512292862, 0.1758442968130112, new int[] { 1, 11, 17, 4, -1 }, new int[] { 1, 12, 17, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00010949910210911185, -0.1238159984350205, 0.0968135967850685, new int[] { 0, 13, 6, 2, -1 }, new int[] { 0, 13, 3, 1, 2 }, new int[] { 3, 14, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0338632389903069, 0.0139471795409918, -0.18364569544792181, true, new int[] { 12, 6, 1, 4, -1 }, new int[] { 12, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0349671207368374, -0.80809932947158813, 0.014799409545958, new int[] { 2, 0, 3, 8, -1 }, new int[] { 3, 0, 1, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.45521798729896551, 0.0136053897440434, -0.60478818416595459, new int[] { 0, 4, 18, 6, -1 }, new int[] { 6, 4, 6, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0160876307636499, 0.0580550096929073, -0.19826529920101171, true, new int[] { 6, 6, 4, 1, -1 }, new int[] { 6, 6, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.17235469818115229, 0.0074058459140360355, -0.51899272203445435, new int[] { 8, 0, 6, 10, -1 }, new int[] { 10, 0, 2, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015957270516082645, -0.0428939200937748, 0.26449468731880188, new int[] { 6, 9, 2, 2, -1 }, new int[] { 6, 9, 1, 1, 2 }, new int[] { 7, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0076875099912285805, -0.27318629622459412, 0.0131092797964811, new int[] { 17, 9, 1, 4, -1 }, new int[] { 17, 10, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015951599925756455, 0.20967930555343631, -0.0498337894678116, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.010349789634347, 0.0072593181394040585, -0.44166409969329828, new int[] { 17, 9, 1, 4, -1 }, new int[] { 17, 10, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069909151643514633, 0.0249945204705, -0.4013820886611939, true, new int[] { 3, 2, 1, 3, -1 }, new int[] { 2, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0047854268923401833, 0.0235026106238365, -0.099097803235054, true, new int[] { 15, 1, 3, 2, -1 }, new int[] { 16, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043787118047475815, -0.36183780431747442, 0.0264573395252228, new int[] { 0, 9, 1, 4, -1 }, new int[] { 0, 10, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031168339774012566, -0.0457625910639763, 0.1117715016007423, new int[] { 12, 7, 3, 3, -1 }, new int[] { 12, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0118435099720955, 0.27435851097106928, -0.0350703783333302, new int[] { 7, 0, 4, 2, -1 }, new int[] { 9, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00015275570331141353, 0.084554448723793, -0.0753161907196045, new int[] { 9, 0, 2, 2, -1 }, new int[] { 9, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0862143188714981, 0.13820220530033109, -0.0711062476038933, new int[] { 1, 5, 15, 4, -1 }, new int[] { 1, 7, 15, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0363043397665024, -0.0381477884948254, 0.11627230048179631, new int[] { 9, 4, 4, 8, -1 }, new int[] { 9, 6, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014807139523327351, -0.1041129976511002, 0.1122824996709824, new int[] { 7, 0, 2, 2, -1 }, new int[] { 8, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073545570485293865, 0.0333745889365673, -0.3583162128925324, new int[] { 4, 3, 10, 2, -1 }, new int[] { 9, 3, 5, 1, 2 }, new int[] { 4, 4, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0344681590795517, -0.0549360811710358, 0.20390030741691589, new int[] { 4, 0, 6, 11, -1 }, new int[] { 6, 0, 2, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0592398792505264, 0.43228080868721008, -0.0247077196836472, new int[] { 3, 11, 12, 4, -1 }, new int[] { 3, 12, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2427041977643967, 0.0220374502241611, -0.54193401336669922, new int[] { 1, 7, 12, 6, -1 }, new int[] { 5, 9, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0122847901657224, -0.37384429574012762, 0.0092992689460515976, true, new int[] { 14, 0, 4, 1, -1 }, new int[] { 15, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0116195902228355, -0.58757847547531128, 0.0175772104412317, true, new int[] { 4, 0, 1, 4, -1 }, new int[] { 3, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0212285108864307, 0.0056798839941620827, -0.31449121236801147, new int[] { 12, 7, 3, 3, -1 }, new int[] { 12, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015732479514554143, -0.0799057930707932, 0.13976779580116269, new int[] { 3, 7, 3, 3, -1 }, new int[] { 3, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.61120092868804932, 0.0133211901411414, -0.55098742246627808, new int[] { 5, 0, 12, 15, -1 }, new int[] { 5, 5, 12, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00020905339624732733, 0.1030462011694908, -0.0948901474475861, new int[] { 6, 9, 2, 2, -1 }, new int[] { 6, 9, 1, 1, 2 }, new int[] { 7, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.5772361014969647E-05, -0.0856239274144173, 0.0874491631984711, new int[] { 13, 3, 2, 2, -1 }, new int[] { 13, 4, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0481263995170593, 0.2119800001382828, -0.0476449094712734, new int[] { 1, 3, 15, 8, -1 }, new int[] { 1, 5, 15, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0086747817695140839, -0.42384940385818481, 0.021367609500885, new int[] { 9, 6, 2, 3, -1 }, new int[] { 9, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081669818609952927, -0.0525886192917824, 0.20056459307670591, true, new int[] { 6, 5, 4, 3, -1 }, new int[] { 5, 6, 4, 1, 3 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 9
            stage = new HaarCascadeStage(-1.4018720388412476, 8, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0085009383037686348, -0.42771288752555853, 0.28500860929489141, new int[] { 5, 2, 4, 4, -1 }, new int[] { 7, 2, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036675720475614071, 0.18305620551109311, -0.43906581401824951, new int[] { 8, 7, 4, 2, -1 }, new int[] { 8, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0154511099681258, -0.25173941254615778, 0.188665896654129, true, new int[] { 4, 3, 8, 2, -1 }, new int[] { 4, 3, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.30046209692955023, -0.05403883010149, 0.48624160885810852, new int[] { 2, 3, 16, 10, -1 }, new int[] { 2, 3, 8, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.36772501468658447, 0.0251029599457979, -958.71881103515625, new int[] { 2, 2, 12, 8, -1 }, new int[] { 2, 4, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040474338456988335, 0.213357001543045, -0.0978919863700867, new int[] { 14, 2, 4, 2, -1 }, new int[] { 14, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0533141195774078, -0.61614441871643066, 0.0559876188635826, new int[] { 0, 4, 2, 9, -1 }, new int[] { 0, 7, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.27916610240936279, 0.407837986946106, -0.1185386031866074, new int[] { 4, 3, 10, 8, -1 }, new int[] { 4, 7, 10, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036125730257481337, 0.23250600695610049, -0.1566430926322937, new int[] { 0, 2, 4, 3, -1 }, new int[] { 2, 2, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002672628965228796, 0.17571009695529941, -0.15493810176849371, new int[] { 8, 1, 2, 4, -1 }, new int[] { 8, 1, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0118291797116399, -0.66747820377349854, 0.0454935915768147, true, new int[] { 7, 0, 1, 4, -1 }, new int[] { 6, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00441691605374217, -0.2293940931558609, 0.10542789846658709, new int[] { 3, 0, 12, 1, -1 }, new int[] { 6, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10357840359210969, 0.34294271469116211, -0.0699092075228691, new int[] { 2, 8, 14, 4, -1 }, new int[] { 2, 10, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034325949382036924, -0.18468430638313291, 0.16796229779720309, new int[] { 1, 10, 16, 2, -1 }, new int[] { 1, 11, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0220014695078135, -0.4447999894618988, 0.0476888418197632, true, new int[] { 2, 9, 4, 2, -1 }, new int[] { 2, 9, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034049700479954481, -0.0612011514604092, 0.134934201836586, new int[] { 7, 7, 8, 2, -1 }, new int[] { 11, 7, 4, 1, 2 }, new int[] { 7, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.16375419497489929, -0.49726039171218872, 0.0431142188608646, new int[] { 0, 0, 18, 10, -1 }, new int[] { 0, 0, 9, 5, 2 }, new int[] { 9, 5, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0426831394433975, 0.19057090580463409, -0.0452457703649998, true, new int[] { 11, 0, 5, 10, -1 }, new int[] { 11, 0, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058941352181136608, 0.1255677938461304, -0.15506540238857269, new int[] { 6, 6, 6, 2, -1 }, new int[] { 6, 7, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0168734900653362, -0.0661193132400513, 0.34744951128959661, new int[] { 7, 3, 4, 6, -1 }, new int[] { 7, 6, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0430995784699917, 0.0575836002826691, -0.33952900767326361, new int[] { 0, 1, 4, 14, -1 }, new int[] { 0, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.019477229565382, -0.80392777919769287, 0.0024795620702207088, true, new int[] { 12, 11, 2, 1, -1 }, new int[] { 12, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.6851670049363747E-05, 0.1161905005574226, -0.17257040739059451, true, new int[] { 6, 11, 1, 2, -1 }, new int[] { 6, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0618079304695129, 0.4056524932384491, -0.0552820302546024, new int[] { 3, 4, 12, 6, -1 }, new int[] { 3, 6, 12, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0398896597325802, -0.28519150614738459, 0.0710409730672836, new int[] { 2, 0, 4, 8, -1 }, new int[] { 2, 4, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0517902411520481, 0.0102649601176381, -0.33244749903678888, true, new int[] { 15, 0, 2, 10, -1 }, new int[] { 15, 0, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055987639352679253, -0.23741720616817469, 0.0760814696550369, true, new int[] { 3, 0, 10, 2, -1 }, new int[] { 3, 0, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.3729403018951416, -0.0144576001912355, 0.276643306016922, true, new int[] { 11, 1, 4, 10, -1 }, new int[] { 11, 1, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.28402900695800781, -0.0665690526366234, 0.3055528998374939, true, new int[] { 7, 1, 10, 4, -1 }, new int[] { 7, 1, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0336107090115547, 0.37678858637809748, -0.038632158190012, new int[] { 5, 0, 9, 7, -1 }, new int[] { 8, 0, 3, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00414227694272995, -0.11140339821577069, 0.1607939004898071, new int[] { 7, 2, 2, 4, -1 }, new int[] { 8, 2, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0784781575202942, 0.52872437238693237, -0.0308714397251606, new int[] { 3, 2, 12, 8, -1 }, new int[] { 3, 4, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073427408933639526, -0.0886204317212105, 0.1757823973894119, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003665081923827529, -0.14013199508190161, 0.0889945700764656, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0249476097524166, -0.0572457909584045, 0.2909868061542511, true, new int[] { 6, 7, 5, 2, -1 }, new int[] { 6, 7, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0095206424593925476, -0.507489025592804, 0.0299209896475077, new int[] { 11, 13, 4, 2, -1 }, new int[] { 12, 13, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00526978587731719, -0.33674290776252752, 0.0424879901111126, new int[] { 4, 0, 10, 2, -1 }, new int[] { 4, 0, 5, 1, 2 }, new int[] { 9, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052029830403625965, -0.38729768991470342, 0.0390708781778812, new int[] { 5, 0, 8, 2, -1 }, new int[] { 9, 0, 4, 1, 2 }, new int[] { 5, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0155430398881435, -0.0815093889832497, 0.1808387041091919, new int[] { 3, 9, 6, 6, -1 }, new int[] { 3, 9, 3, 3, 2 }, new int[] { 6, 12, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.052419401705265, -0.553170382976532, 0.0184993594884872, new int[] { 6, 13, 9, 2, -1 }, new int[] { 9, 13, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0111103300005198, -0.7034459114074707, 0.0181828700006008, true, new int[] { 8, 0, 1, 3, -1 }, new int[] { 7, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024250999558717012, -0.0457252115011215, 0.0519403293728828, true, new int[] { 14, 10, 3, 2, -1 }, new int[] { 15, 11, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0080726835876703262, -0.22301289439201361, 0.0591846518218517, new int[] { 5, 0, 3, 6, -1 }, new int[] { 5, 3, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.083049550652504, -0.0779340714216232, 0.0390878692269325, new int[] { 4, 0, 12, 8, -1 }, new int[] { 8, 0, 4, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0832247883081436, 0.29764831066131592, -0.0553525611758232, new int[] { 2, 0, 12, 8, -1 }, new int[] { 6, 0, 4, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0287941191345453, 0.1785778999328613, -0.0220392197370529, new int[] { 5, 0, 12, 5, -1 }, new int[] { 8, 0, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0564895309507847, -0.0698909312486649, 0.21076519787311551, new int[] { 1, 0, 12, 5, -1 }, new int[] { 4, 0, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0616075918078423, -0.67098802328109741, 0.0254087205976248, new int[] { 7, 0, 4, 14, -1 }, new int[] { 9, 0, 2, 7, 2 }, new int[] { 7, 7, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0404302515089512, -0.0430069416761398, 0.36125731468200678, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 9, 0, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0816636979579926, 0.037107840180397, -0.40147781372070312, new int[] { 0, 6, 18, 4, -1 }, new int[] { 9, 6, 9, 2, 2 }, new int[] { 0, 8, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0200602691620588, 0.0283941105008125, -0.450969785451889, true, new int[] { 9, 0, 4, 2, -1 }, new int[] { 10, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44809231162071228, -0.0288634896278381, 0.54432421922683716, new int[] { 3, 3, 12, 6, -1 }, new int[] { 7, 5, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0094997808337211609, -0.0631850063800812, 0.20143640041351321, true, new int[] { 4, 1, 3, 4, -1 }, new int[] { 5, 2, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3604110538144596E-05, 0.085501417517662, -0.0625851824879646, new int[] { 16, 2, 2, 2, -1 }, new int[] { 17, 2, 1, 1, 2 }, new int[] { 16, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.9380017016083E-05, 0.12780819833278659, -0.1021258011460304, new int[] { 0, 2, 2, 2, -1 }, new int[] { 0, 2, 1, 1, 2 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00030439419788308442, 0.136238306760788, -0.0963960811495781, new int[] { 17, 2, 1, 2, -1 }, new int[] { 17, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.1386282797902822E-05, 0.120204396545887, -0.1152094006538391, new int[] { 0, 2, 1, 2, -1 }, new int[] { 0, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0074278670363128185, -0.11765129864215849, 0.0256468392908573, true, new int[] { 10, 7, 1, 3, -1 }, new int[] { 9, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(9.1655907453969121E-05, -0.1066583022475243, 0.11622580140829091, new int[] { 1, 2, 1, 4, -1 }, new int[] { 1, 3, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.8285116362385452E-05, 0.1020200997591019, -0.0947737917304039, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1716001033782959, -0.0963247865438461, 0.13936719298362729, true, new int[] { 9, 0, 8, 6, -1 }, new int[] { 9, 0, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0061614410951733589, -0.0783397704362869, 0.19864350557327271, true, new int[] { 12, 6, 3, 4, -1 }, new int[] { 13, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104880100116134, 0.0224729795008898, -0.58889657258987427, true, new int[] { 3, 3, 1, 3, -1 }, new int[] { 2, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0423890985548496, 0.0032426279503852129, -0.381795197725296, true, new int[] { 11, 7, 3, 4, -1 }, new int[] { 12, 8, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.01894217915833, -0.0385925881564617, 0.34485790133476257, true, new int[] { 7, 7, 4, 3, -1 }, new int[] { 6, 8, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0018505830084905028, 0.0621170587837696, -0.142229899764061, new int[] { 8, 8, 3, 2, -1 }, new int[] { 8, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034762551076710224, -0.0630814731121063, 0.20072069764137271, new int[] { 3, 8, 8, 2, -1 }, new int[] { 3, 8, 4, 1, 2 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082640787586569786, -0.0460104309022427, 0.1130814999341965, new int[] { 11, 8, 4, 6, -1 }, new int[] { 11, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.084993377327919, 0.2154290974140167, -0.0659862980246544, new int[] { 1, 11, 14, 4, -1 }, new int[] { 8, 11, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0231807008385658, -0.34274458885192871, 0.0235659405589104, new int[] { 9, 13, 6, 2, -1 }, new int[] { 11, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0172915291041136, 0.0314326398074627, -0.39180231094360352, new int[] { 3, 13, 6, 2, -1 }, new int[] { 5, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031471049878746271, -0.1212544962763786, 0.0950881168246269, new int[] { 1, 11, 16, 2, -1 }, new int[] { 1, 12, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0957942008972168, 0.374728798866272, -0.0426806211471558, new int[] { 1, 10, 16, 4, -1 }, new int[] { 1, 12, 16, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0265573691576719, -0.479229211807251, 0.0261464007198811, new int[] { 12, 13, 6, 2, -1 }, new int[] { 14, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.1971433246508241E-05, 0.1034777984023094, -0.117579996585846, new int[] { 0, 0, 2, 1, -1 }, new int[] { 1, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0064540100283920765, -0.52700281143188477, 0.0349571593105793, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0330873392522335, -0.39793440699577332, 0.0254548005759716, new int[] { 1, 14, 12, 1, -1 }, new int[] { 5, 14, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0701283663511276, -0.0294641107320786, 0.4120103120803833, new int[] { 6, 3, 6, 6, -1 }, new int[] { 6, 6, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00096940301591530442, 0.12894269824028021, -0.0847874134778976, new int[] { 7, 12, 4, 3, -1 }, new int[] { 8, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0186607595533133, -0.0062266499735414982, 0.36698341369628912, true, new int[] { 9, 9, 1, 2, -1 }, new int[] { 9, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0135134300217032, 0.0170807391405106, -0.710842490196228, true, new int[] { 9, 9, 2, 1, -1 }, new int[] { 9, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00071627681609243155, 0.0951879769563675, -0.0463394597172737, new int[] { 12, 10, 2, 2, -1 }, new int[] { 13, 10, 1, 1, 2 }, new int[] { 12, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054968800395727158, 0.0190170500427485, -0.56606787443161011, new int[] { 0, 6, 1, 3, -1 }, new int[] { 0, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.033988431096077, 0.20532059669494629, -0.0537301301956177, new int[] { 1, 6, 16, 4, -1 }, new int[] { 1, 7, 16, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0094949705526232719, -0.4779914915561676, 0.0261098798364401, true, new int[] { 9, 6, 3, 1, -1 }, new int[] { 10, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00088990468066185713, -0.0538782998919487, 0.1529861986637116, new int[] { 12, 10, 2, 2, -1 }, new int[] { 13, 10, 1, 1, 2 }, new int[] { 12, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.15903118159622E-05, -0.1203349977731705, 0.0874421000480652, new int[] { 4, 2, 2, 2, -1 }, new int[] { 5, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0583840794861317, 0.1957484036684036, -0.0669205635786057, new int[] { 7, 0, 4, 11, -1 }, new int[] { 7, 0, 2, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001628690049983561, -0.1063129976391792, 0.1267475038766861, true, new int[] { 8, 5, 3, 3, -1 }, new int[] { 7, 6, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0797880366444588, 0.0121673298999667, -0.51673012971878052, new int[] { 14, 1, 2, 14, -1 }, new int[] { 14, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0053892009891569614, -0.12911440432071691, 0.088783398270607, new int[] { 5, 0, 8, 2, -1 }, new int[] { 7, 0, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25091820955276489, 0.0321798510849476, -0.3768610954284668, new int[] { 1, 3, 16, 7, -1 }, new int[] { 5, 3, 8, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0172097105532885, 0.0123794004321098, -0.78753459453582764, new int[] { 4, 1, 9, 3, -1 }, new int[] { 4, 2, 9, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1891666054725647, -0.0333567596971989, 0.18951129913330081, new int[] { 6, 2, 8, 13, -1 }, new int[] { 6, 2, 4, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.004811515100300312, 0.2050116956233978, -0.0531618110835552, new int[] { 4, 0, 9, 1, -1 }, new int[] { 7, 0, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0202697701752186, -0.0289377495646477, 0.21850499510765081, true, new int[] { 15, 4, 3, 3, -1 }, new int[] { 14, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.8484037658199668E-05, 0.0575751215219498, -0.18328189849853521, new int[] { 8, 7, 1, 2, -1 }, new int[] { 8, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022350680083036423, -0.0324196107685566, 0.0866090729832649, new int[] { 11, 8, 3, 3, -1 }, new int[] { 11, 9, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0169897098094225, 0.2827008068561554, -0.0383652187883854, true, new int[] { 3, 4, 3, 3, -1 }, new int[] { 4, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0064167408272624016, 0.1313406974077225, -0.0436117313802242, true, new int[] { 15, 3, 3, 3, -1 }, new int[] { 14, 4, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074191158637404442, -0.0706334635615349, 0.17600670456886289, true, new int[] { 3, 3, 3, 3, -1 }, new int[] { 4, 4, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063850679434835911, 0.0321756713092327, -0.39056539535522461, new int[] { 7, 0, 8, 2, -1 }, new int[] { 11, 0, 4, 1, 2 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1251693069934845, -0.81828737258911133, 0.0108839897438884, new int[] { 1, 6, 6, 9, -1 }, new int[] { 3, 6, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044671529904007912, -0.503462016582489, 0.0046763787977397442, new int[] { 11, 5, 2, 2, -1 }, new int[] { 12, 5, 1, 1, 2 }, new int[] { 11, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.7330769272521138E-05, 0.112311102449894, -0.096118189394474, new int[] { 5, 5, 2, 2, -1 }, new int[] { 5, 5, 1, 1, 2 }, new int[] { 6, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0487493798136711, 0.0153942899778485, -0.1379497051239014, new int[] { 14, 1, 3, 14, -1 }, new int[] { 14, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0150579595938325, 0.0967942178249359, -0.1040832027792931, new int[] { 2, 5, 6, 8, -1 }, new int[] { 4, 5, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0128671396523714, -0.559431791305542, 0.0080226631835103035, new int[] { 9, 4, 4, 2, -1 }, new int[] { 10, 4, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.4015636146068573, 0.0144503097981215, -0.69868141412734985, new int[] { 4, 1, 8, 14, -1 }, new int[] { 8, 1, 4, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014811520231887698, -0.060255978256464, 0.0617385916411877, new int[] { 9, 4, 4, 2, -1 }, new int[] { 10, 4, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.036016408354044, -0.766661524772644, 0.0140148000791669, new int[] { 4, 7, 3, 7, -1 }, new int[] { 5, 7, 1, 7, 3 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 10
            stage = new HaarCascadeStage(-1.4323190450668335, 9, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0917561426758766, -0.2386678010225296, 0.41412800550460821, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0639683231711388, 0.23543690145015719, -0.22721849381923681, new int[] { 8, 3, 10, 10, -1 }, new int[] { 13, 3, 5, 5, 2 }, new int[] { 8, 8, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0100612798705697, 0.190331295132637, -0.26683130860328669, new int[] { 0, 0, 8, 5, -1 }, new int[] { 2, 0, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0135615598410368, 0.14927579462528229, -0.18083690106868741, new int[] { 12, 8, 6, 6, -1 }, new int[] { 12, 8, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0150768300518394, 0.2060939967632294, -0.18534150719642639, new int[] { 0, 8, 6, 6, -1 }, new int[] { 3, 8, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0061514219269156456, -0.52573877573013306, 0.017555620521307, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00052476930432021618, -0.1458822041749954, 0.15166099369525909, true, new int[] { 5, 0, 11, 4, -1 }, new int[] { 4, 1, 11, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024739510845392942, -0.1880511939525604, 0.0956946983933449, new int[] { 2, 12, 16, 2, -1 }, new int[] { 2, 13, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051760678179562092, 0.0520320907235146, -0.49382910132408142, true, new int[] { 8, 0, 1, 3, -1 }, new int[] { 7, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007170247845351696, -0.09414292126894, 0.1121701002120972, true, new int[] { 11, 0, 4, 2, -1 }, new int[] { 11, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0200577601790428, -0.59458369016647339, 0.0365518406033516, true, new int[] { 8, 0, 3, 4, -1 }, new int[] { 7, 1, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.20991469919681549, 0.26298180222511292, -0.1024070009589195, new int[] { 5, 0, 8, 14, -1 }, new int[] { 5, 0, 4, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.004216671921312809, 0.13226920366287229, -0.15037320554256439, new int[] { 6, 7, 3, 2, -1 }, new int[] { 6, 8, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0149440001696348, 0.0650079399347305, -0.0314821898937225, true, new int[] { 10, 0, 3, 8, -1 }, new int[] { 8, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0916189774870873, 0.114597499370575, -0.2158081978559494, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013998460490256548, -0.1513507068157196, 0.1351508945226669, new int[] { 7, 0, 4, 2, -1 }, new int[] { 8, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.062787801027298, -0.1066391989588738, 0.2077779024839401, true, new int[] { 9, 1, 9, 2, -1 }, new int[] { 12, 4, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.16034470498561859, -0.067444883286953, 0.30661910772323608, new int[] { 3, 6, 12, 9, -1 }, new int[] { 3, 6, 6, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0100808003917336, 0.22366729378700259, -0.0887190401554108, new int[] { 5, 5, 8, 2, -1 }, new int[] { 5, 6, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0218050591647625, -0.0556704215705395, 0.13599489629268649, true, new int[] { 13, 6, 2, 6, -1 }, new int[] { 13, 6, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0624005310237408, -0.44345939159393311, 0.031536590307951, new int[] { 0, 0, 18, 6, -1 }, new int[] { 0, 0, 9, 3, 2 }, new int[] { 9, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0338275581598282, 0.2535226047039032, -0.0142370602115989, true, new int[] { 13, 6, 2, 6, -1 }, new int[] { 13, 6, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0249442607164383, -0.0565281696617603, 0.26071038842201227, true, new int[] { 5, 6, 6, 2, -1 }, new int[] { 5, 6, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0286747291684151, -0.0299342703074217, 0.39638450741767878, true, new int[] { 12, 8, 4, 3, -1 }, new int[] { 13, 9, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0907829701900482, 0.0478614382445812, -0.3908458948135376, new int[] { 0, 5, 18, 8, -1 }, new int[] { 0, 5, 9, 4, 2 }, new int[] { 9, 9, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0098480619490146637, -0.53130447864532471, 0.015104659833014, true, new int[] { 14, 0, 3, 1, -1 }, new int[] { 15, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0077331489883363247, 0.0242120604962111, -0.56011068820953369, true, new int[] { 4, 0, 1, 3, -1 }, new int[] { 3, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0097148418426513672, -0.0773390233516693, 0.20035690069198611, true, new int[] { 13, 0, 5, 3, -1 }, new int[] { 12, 1, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0038716041017323732, 0.0935838297009468, -0.1630876958370209, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045740120112895966, -0.0741003602743149, 0.186732605099678, true, new int[] { 12, 7, 3, 3, -1 }, new int[] { 13, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0035367589443922043, -0.13378569483757019, 0.1311887055635452, new int[] { 0, 10, 18, 2, -1 }, new int[] { 0, 11, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.006738745141774416, 0.0191045496612787, -0.26714089512825012, new int[] { 16, 10, 2, 2, -1 }, new int[] { 16, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092638395726680756, 0.0389440283179283, -0.3811526894569397, true, new int[] { 8, 7, 2, 1, -1 }, new int[] { 8, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0180356502532959, -0.0563138388097286, 0.26199018955230707, new int[] { 6, 4, 6, 4, -1 }, new int[] { 6, 6, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0061390590853989124, 0.0667682513594627, -0.24741740524768829, true, new int[] { 9, 6, 3, 1, -1 }, new int[] { 10, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0207422897219658, 0.1581667959690094, -0.0370551086962223, true, new int[] { 12, 7, 3, 3, -1 }, new int[] { 13, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071745091117918491, -0.0627238526940346, 0.24000909924507141, true, new int[] { 6, 7, 3, 3, -1 }, new int[] { 5, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0139801297336817, -0.25686889886856079, 0.0244082696735859, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0050162561237812042, 0.0346935093402863, -0.3694097101688385, new int[] { 0, 10, 2, 2, -1 }, new int[] { 0, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062731141224503517, -0.0931362733244896, 0.0891287103295326, new int[] { 12, 4, 2, 6, -1 }, new int[] { 12, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00614327983930707, -0.38624298572540278, 0.0327900089323521, true, new int[] { 2, 12, 1, 2, -1 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0054340949282050133, 0.1252959072589874, -0.0733088776469231, true, new int[] { 14, 0, 4, 3, -1 }, new int[] { 13, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0264763794839382, 0.0196925196796656, -0.65207397937774658, new int[] { 7, 5, 3, 7, -1 }, new int[] { 8, 5, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0531985610723495, -0.0389075092971325, 0.34459230303764338, new int[] { 7, 0, 4, 11, -1 }, new int[] { 7, 0, 2, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00068159057991579175, -0.1429661959409714, 0.110514797270298, true, new int[] { 9, 2, 1, 2, -1 }, new int[] { 9, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0273211896419525, -0.0230135805904865, 0.3866828978061676, true, new int[] { 14, 0, 4, 3, -1 }, new int[] { 13, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0164375193417072, -0.0503561496734619, 0.25431129336357122, true, new int[] { 4, 3, 3, 4, -1 }, new int[] { 5, 4, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0113530699163675, -0.385333389043808, 0.0233515705913305, new int[] { 3, 0, 14, 2, -1 }, new int[] { 10, 0, 7, 1, 2 }, new int[] { 3, 1, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0056346738710999489, 0.18512620031833649, -0.0785678625106812, new int[] { 0, 0, 4, 6, -1 }, new int[] { 0, 0, 2, 3, 2 }, new int[] { 2, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069470210000872612, 0.0369826108217239, -0.1762986034154892, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165615193545818, -0.498485803604126, 0.0288834199309349, true, new int[] { 4, 0, 2, 4, -1 }, new int[] { 3, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0768493562936783, -0.31578719615936279, 0.0435194000601768, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0151811297982931, 0.2342346012592316, -0.062591478228569, true, new int[] { 3, 0, 4, 3, -1 }, new int[] { 4, 1, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0194898601621389, 0.0099025378003716469, -0.38761860132217407, new int[] { 12, 5, 6, 2, -1 }, new int[] { 12, 6, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0180505998432636, -0.04393070936203, 0.33341428637504578, true, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0029345480725169182, 0.0809545367956162, -0.0499147698283196, new int[] { 15, 0, 3, 15, -1 }, new int[] { 16, 0, 1, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0263634100556374, 0.0291267596185207, -0.50750941038131714, new int[] { 0, 0, 3, 15, -1 }, new int[] { 1, 0, 1, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024248650297522545, 0.0349614284932613, -0.28733271360397339, new int[] { 9, 0, 2, 2, -1 }, new int[] { 9, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049459808506071568, 0.0231612101197243, -0.50714761018753052, new int[] { 7, 0, 2, 1, -1 }, new int[] { 8, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15279249846935269, -0.32881578803062439, 0.0251827891916037, new int[] { 15, 1, 3, 14, -1 }, new int[] { 15, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00014403219392988831, 0.0755192562937737, -0.18179009854793551, new int[] { 0, 1, 4, 12, -1 }, new int[] { 0, 7, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.28954431414604193, 0.0112048899754882, -0.38397979736328131, new int[] { 8, 2, 10, 7, -1 }, new int[] { 8, 2, 5, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0487764589488506, -0.38399431109428411, 0.0332496799528599, new int[] { 2, 0, 9, 6, -1 }, new int[] { 2, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0326264388859272, 0.31781470775604248, -0.0470084510743618, new int[] { 3, 3, 12, 6, -1 }, new int[] { 3, 5, 12, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075620561838150024, -0.1639129966497421, 0.0883946195244789, new int[] { 0, 5, 10, 2, -1 }, new int[] { 5, 5, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055116498842835426, -0.0453669391572475, 0.1035958006978035, true, new int[] { 13, 9, 3, 3, -1 }, new int[] { 14, 10, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00789603590965271, 0.0258352104574442, -0.4117685854434967, new int[] { 1, 11, 3, 3, -1 }, new int[] { 2, 11, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0255158301442862, 0.0233579408377409, -0.1015767008066177, new int[] { 13, 3, 5, 12, -1 }, new int[] { 13, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0027663391083478928, -0.083083488047123, 0.1461292952299118, new int[] { 0, 8, 4, 4, -1 }, new int[] { 0, 8, 2, 2, 2 }, new int[] { 2, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030674580484628677, 0.0921359285712242, -0.0571467913687229, new int[] { 12, 8, 6, 1, -1 }, new int[] { 14, 8, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092945564538240433, 0.0387363918125629, -0.35326778888702393, new int[] { 0, 5, 1, 6, -1 }, new int[] { 0, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.06742312759161, -0.0752417668700218, 0.17596650123596189, new int[] { 1, 5, 16, 6, -1 }, new int[] { 1, 7, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044064600951969624, 0.0977936610579491, -0.15189309418201449, new int[] { 6, 6, 6, 2, -1 }, new int[] { 6, 7, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0498286001384258, -0.457902193069458, 0.0068976799957454205, new int[] { 10, 3, 3, 6, -1 }, new int[] { 11, 5, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0365433506667614, 0.0514394491910934, -0.26903149485588068, new int[] { 5, 3, 3, 6, -1 }, new int[] { 6, 5, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0641553029417992, -0.0376881808042526, 0.0356850884854794, new int[] { 13, 3, 5, 12, -1 }, new int[] { 13, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016559410141780973, -0.0784540399909019, 0.14457669854164121, new int[] { 1, 8, 4, 2, -1 }, new int[] { 2, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0435861088335514, -0.68510597944259644, 0.0130487699061632, new int[] { 12, 4, 3, 11, -1 }, new int[] { 13, 4, 1, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.22230669856071469, -0.57761538028717041, 0.0171249397099018, new int[] { 0, 3, 5, 12, -1 }, new int[] { 0, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0246731601655483, 0.0118981599807739, -0.40522110462188721, new int[] { 12, 4, 3, 11, -1 }, new int[] { 13, 4, 1, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0119292298331857, 0.33518779277801508, -0.0336703099310398, new int[] { 5, 13, 4, 2, -1 }, new int[] { 5, 14, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0001231971982633695, -0.0857188627123833, 0.0837130919098854, new int[] { 11, 13, 1, 2, -1 }, new int[] { 11, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0093408823013305664, -0.28543150424957281, 0.0407378897070885, new int[] { 0, 4, 7, 2, -1 }, new int[] { 0, 5, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.007462651003152132, 0.1119131967425346, -0.0340123288333416, true, new int[] { 14, 4, 3, 3, -1 }, new int[] { 13, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0137237096205354, 0.24986229836940771, -0.045033760368824, new int[] { 5, 0, 8, 7, -1 }, new int[] { 7, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15219870209693909, -0.0910210907459259, 0.0909610465168953, new int[] { 4, 0, 12, 15, -1 }, new int[] { 8, 0, 4, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.725960313109681E-05, -0.10590860247612, 0.110557496547699, new int[] { 0, 0, 1, 3, -1 }, new int[] { 0, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049416758120059967, 0.0241890698671341, -0.3095433115959168, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.008453715592622757, -0.4988319873809815, 0.0197901595383883, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00015807019372005016, 0.0810882821679115, -0.0969615131616592, new int[] { 16, 10, 2, 1, -1 }, new int[] { 16, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0371250584721565, -0.66581451892852783, 0.0148829696699977, new int[] { 3, 2, 3, 13, -1 }, new int[] { 4, 2, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0268303193151951, -0.0143090495839715, 0.18943400681018829, true, new int[] { 14, 4, 3, 3, -1 }, new int[] { 13, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0502456203103065, 0.29321768879890442, -0.0342677310109138, new int[] { 5, 3, 8, 4, -1 }, new int[] { 5, 5, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0069950302131474018, -0.3633973896503449, 0.0245582703500986, true, new int[] { 12, 0, 3, 1, -1 }, new int[] { 13, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0658775717020035, -0.0696238428354263, 0.16893179714679721, new int[] { 4, 0, 8, 13, -1 }, new int[] { 6, 0, 4, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0134680103510618, -0.57445019483566284, 0.0076498151756823063, true, new int[] { 11, 1, 1, 4, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065795979462563992, 0.0468714609742165, -0.26042988896369929, true, new int[] { 7, 1, 4, 1, -1 }, new int[] { 8, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0837022736668587, -0.0026280758902430534, 0.95396530628204346, true, new int[] { 14, 4, 3, 3, -1 }, new int[] { 13, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.026914669200778, 0.43413200974464422, -0.0251872204244137, true, new int[] { 4, 4, 3, 3, -1 }, new int[] { 5, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0681707710027695, 0.0113553795963526, -0.1976965069770813, true, new int[] { 17, 2, 1, 8, -1 }, new int[] { 17, 2, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0183866992592812, -0.30161228775978088, 0.0400681607425213, true, new int[] { 1, 2, 8, 1, -1 }, new int[] { 1, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0048888311721384525, -0.0474995188415051, 0.0279497597366571, true, new int[] { 11, 6, 3, 4, -1 }, new int[] { 12, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0120319798588753, -0.0417588092386723, 0.2567807137966156, true, new int[] { 7, 6, 4, 3, -1 }, new int[] { 6, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.045282568782568, -0.0120907295495272, 0.59624272584915161, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0164286494255066, 0.0317231491208076, -0.34151419997215271, new int[] { 0, 4, 8, 3, -1 }, new int[] { 0, 5, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0158072896301746, -0.0876926332712173, 0.0733993873000145, new int[] { 10, 3, 6, 6, -1 }, new int[] { 10, 5, 6, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0738655477762222, 0.0175666399300098, -0.58591890335083008, new int[] { 4, 1, 6, 8, -1 }, new int[] { 4, 1, 3, 4, 2 }, new int[] { 7, 5, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0817420035600662, -0.0146944299340248, 0.38172268867492681, true, new int[] { 10, 4, 4, 2, -1 }, new int[] { 10, 4, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0001620129041839391, -0.1015762984752655, 0.1007106006145477, new int[] { 6, 13, 1, 2, -1 }, new int[] { 6, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00965146068483591, -0.0391967110335827, 0.15712510049343109, new int[] { 9, 11, 3, 4, -1 }, new int[] { 9, 12, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1139461994171143, 0.021624019369483, -0.49949270486831671, new int[] { 1, 11, 10, 4, -1 }, new int[] { 1, 13, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0031548771075904369, 0.0503181293606758, -0.0436193607747555, true, new int[] { 14, 1, 4, 2, -1 }, new int[] { 14, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0443513505160809, 0.30843031406402588, -0.0323894284665585, new int[] { 3, 1, 12, 8, -1 }, new int[] { 3, 3, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0593373291194439, 0.008863481692969799, -0.43402770161628718, true, new int[] { 12, 4, 2, 8, -1 }, new int[] { 12, 4, 1, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0084961997345089912, -0.16435340046882629, 0.0720200389623642, true, new int[] { 6, 4, 8, 2, -1 }, new int[] { 6, 4, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0126119097694755, -0.0547339096665382, 0.26740849018096918, new int[] { 7, 0, 6, 1, -1 }, new int[] { 7, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1005614027380943, 0.0964706912636757, -0.1237357035279274, new int[] { 3, 5, 12, 6, -1 }, new int[] { 7, 7, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0654680281877518, 0.0757642164826393, new int[] { 10, 3, 1, 2, -1 }, new int[] { 10, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0173253808170557, 0.0493854694068432, -0.20938959717750549, true, new int[] { 9, 0, 6, 3, -1 }, new int[] { 8, 1, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081096980720758438, -0.23129729926586151, 0.0138064604252577, new int[] { 16, 5, 2, 3, -1 }, new int[] { 16, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0070394109934568405, -0.0485932305455208, 0.21045120060443881, new int[] { 2, 7, 4, 4, -1 }, new int[] { 2, 7, 2, 2, 2 }, new int[] { 4, 9, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010678370017558336, 0.098571203649044, -0.0456795394420624, new int[] { 16, 5, 2, 3, -1 }, new int[] { 16, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079888887703418732, 0.0227227304130793, -0.47305500507354742, new int[] { 0, 5, 2, 3, -1 }, new int[] { 0, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0028562550432980061, -0.12667450308799741, 0.0263468995690346, true, new int[] { 12, 6, 2, 4, -1 }, new int[] { 12, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0282390993088484, -0.48173430562019348, 0.0202802792191505, new int[] { 4, 13, 8, 2, -1 }, new int[] { 6, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035814680159091949, 0.13375559449195859, -0.0751768574118614, new int[] { 8, 0, 2, 3, -1 }, new int[] { 8, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1443670988082886, -0.31298300623893738, 0.0385885089635849, new int[] { 4, 3, 10, 12, -1 }, new int[] { 4, 3, 5, 6, 2 }, new int[] { 9, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1250455975532532, 0.0065982979722321033, -0.81579452753067017, new int[] { 7, 11, 8, 4, -1 }, new int[] { 7, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0130116604268551, 0.12922100722789759, -0.0797087624669075, new int[] { 3, 11, 8, 4, -1 }, new int[] { 7, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017209460493177176, 0.18410180509090421, -0.0381583906710148, new int[] { 13, 6, 2, 2, -1 }, new int[] { 14, 6, 1, 1, 2 }, new int[] { 13, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.2962076703552157E-05, -0.0808445066213608, 0.1240184977650642, new int[] { 3, 6, 2, 2, -1 }, new int[] { 3, 6, 1, 1, 2 }, new int[] { 4, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065386621281504631, 0.025721050798893, -0.34728491306304932, true, new int[] { 12, 0, 3, 1, -1 }, new int[] { 13, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.007602212019264698, -0.13279519975185389, 0.0695039033889771, new int[] { 4, 1, 4, 6, -1 }, new int[] { 4, 4, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00012741329555865377, 0.073461078107357, -0.056750301271677, new int[] { 17, 0, 1, 2, -1 }, new int[] { 17, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00874832272529602, -0.38747811317443848, 0.0252428594976664, new int[] { 5, 0, 7, 3, -1 }, new int[] { 5, 1, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00018606209778226912, -0.0807940736413002, 0.1112494990229607, new int[] { 17, 0, 1, 2, -1 }, new int[] { 17, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00023457060160581023, 0.13575780391693121, -0.0805138573050499, new int[] { 0, 0, 1, 2, -1 }, new int[] { 0, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017333909636363387, -0.0408243499696255, 0.0704857334494591, new int[] { 17, 0, 1, 6, -1 }, new int[] { 17, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025763779412955046, -0.1058242991566658, 0.0882512032985687, new int[] { 3, 11, 12, 2, -1 }, new int[] { 3, 12, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031439519952982664, 0.0228503905236721, -0.2287800014019013, new int[] { 17, 11, 1, 2, -1 }, new int[] { 17, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066810711286962032, -0.55194759368896484, 0.0166440196335316, new int[] { 7, 1, 4, 2, -1 }, new int[] { 7, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0102156195789576, 0.1151650995016098, -0.0309206396341324, true, new int[] { 15, 2, 3, 3, -1 }, new int[] { 14, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068375351838767529, 0.0355978682637215, -0.257957398891449, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00316672888584435, -0.1131158992648125, 0.0593770816922188, new int[] { 9, 2, 8, 2, -1 }, new int[] { 9, 2, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.178461104631424, -0.091090977191925, 0.102155402302742, new int[] { 1, 0, 12, 15, -1 }, new int[] { 7, 0, 6, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0023922319523990154, 0.1054854989051819, -0.0409410186111927, new int[] { 17, 0, 1, 6, -1 }, new int[] { 17, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00082479801494628191, -0.0925479605793953, 0.1070403009653091, new int[] { 0, 0, 1, 6, -1 }, new int[] { 0, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002321355976164341, 0.0474837012588978, -0.0448017083108425, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 14, 0, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0099881906062364578, -0.0531012415885925, 0.18933349847793579, true, new int[] { 5, 0, 3, 5, -1 }, new int[] { 6, 1, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0072582447901368141, 0.0154708195477724, -0.16273790597915649, new int[] { 13, 0, 3, 4, -1 }, new int[] { 14, 0, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.12209150195121771, -0.65882581472396851, 0.0144322402775288, true, new int[] { 9, 3, 4, 8, -1 }, new int[] { 9, 3, 2, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0429302901029587, -0.0089507391676306725, 0.70037537813186646, true, new int[] { 15, 3, 2, 3, -1 }, new int[] { 14, 4, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.014183740131557, 0.28738090395927429, -0.0324238389730453, true, new int[] { 3, 3, 3, 2, -1 }, new int[] { 4, 4, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.5566753619350493E-05, -0.0600121095776558, 0.0723430663347244, new int[] { 12, 3, 2, 2, -1 }, new int[] { 13, 3, 1, 1, 2 }, new int[] { 12, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-9.16737990337424E-05, 0.1241253018379211, -0.0886371731758118, new int[] { 4, 3, 2, 2, -1 }, new int[] { 4, 3, 1, 1, 2 }, new int[] { 5, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104515701532364, 0.0198976993560791, -0.54859578609466553, new int[] { 15, 0, 3, 3, -1 }, new int[] { 16, 0, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071406508795917034, 0.0218714401125908, -0.399595707654953, new int[] { 0, 0, 3, 3, -1 }, new int[] { 1, 0, 1, 3, 3 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 11
            stage = new HaarCascadeStage(-1.3140599727630615, 10, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0790023133158684, 0.32428950071334839, -0.25313940644264221, new int[] { 4, 6, 10, 4, -1 }, new int[] { 4, 8, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0223373007029295, -0.0941315069794655, 0.13784369826316831, new int[] { 9, 2, 4, 8, -1 }, new int[] { 9, 2, 2, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0666114836931229, 0.17535580694675451, -0.26326939463615417, new int[] { 1, 0, 9, 12, -1 }, new int[] { 4, 0, 3, 12, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.018115570768714, 0.1001667976379395, -0.25084051489830023, new int[] { 6, 4, 12, 10, -1 }, new int[] { 12, 4, 6, 5, 2 }, new int[] { 6, 9, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0422082990407944, -0.0464601181447506, 0.50753402709960938, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 9, 0, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0219473801553249, -0.0351926311850548, 0.2941356897354126, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.039068479090929, 0.0343180112540722, -0.59637278318405151, new int[] { 2, 0, 10, 4, -1 }, new int[] { 2, 2, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0171588398516178, 0.22071230411529541, -0.0628029406070709, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.5410808272426948E-05, 0.19250670075416571, -0.0979116931557655, new int[] { 7, 1, 2, 2, -1 }, new int[] { 7, 1, 1, 1, 2 }, new int[] { 8, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0577130392193794, -0.0177523493766785, 0.39690899848937988, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0276702996343374, 0.2730920016765595, -0.0699228271842003, true, new int[] { 6, 6, 4, 3, -1 }, new int[] { 5, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041078277863562107, -0.0490987785160542, 0.2490742951631546, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.82316390844062E-05, -0.1242284029722214, 0.1748877018690109, new int[] { 7, 1, 2, 2, -1 }, new int[] { 7, 1, 1, 1, 2 }, new int[] { 8, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054101468995213509, -0.1163510009646416, 0.1120261996984482, new int[] { 10, 0, 8, 6, -1 }, new int[] { 14, 0, 4, 3, 2 }, new int[] { 10, 3, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1215678006410599, 0.0358167998492718, -0.42390239238739008, new int[] { 0, 0, 18, 10, -1 }, new int[] { 0, 0, 9, 5, 2 }, new int[] { 9, 5, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.045798659324646, -0.39612388610839838, 0.026914669200778, new int[] { 16, 0, 2, 8, -1 }, new int[] { 16, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043434510007500648, 0.1517422944307327, -0.15247189998626709, new int[] { 0, 0, 2, 8, -1 }, new int[] { 0, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00054885639110580087, -0.1039891019463539, 0.1021101996302605, true, new int[] { 14, 10, 4, 1, -1 }, new int[] { 15, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024605579674243927, -0.0920632407069206, 0.20085799694061279, new int[] { 4, 7, 2, 4, -1 }, new int[] { 4, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0204001795500517, 0.3931783139705658, 0.0058226548135280609, true, new int[] { 14, 10, 4, 1, -1 }, new int[] { 15, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00023037819482851774, -0.1504732072353363, 0.106061302125454, true, new int[] { 4, 10, 1, 4, -1 }, new int[] { 3, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0042928410694003105, 0.0726602599024773, -0.0793565437197685, true, new int[] { 13, 0, 1, 4, -1 }, new int[] { 13, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1863780021667481, -0.1124956011772156, 0.15694859623909, true, new int[] { 9, 0, 8, 6, -1 }, new int[] { 9, 0, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0264334604144096, -0.39095601439476008, 0.0494861491024494, new int[] { 1, 0, 16, 4, -1 }, new int[] { 9, 0, 8, 2, 2 }, new int[] { 1, 2, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.24137930572032931, -0.67887061834335327, 0.0180502496659756, new int[] { 1, 3, 16, 11, -1 }, new int[] { 5, 3, 8, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.030466640368104, 0.002720230957493186, -0.63896268606185913, true, new int[] { 9, 1, 4, 1, -1 }, new int[] { 9, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0027874959632754326, -0.083127573132515, 0.17751370370388031, new int[] { 3, 9, 4, 4, -1 }, new int[] { 3, 10, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12827770411968231, -0.0936257764697075, 0.16796620190143591, new int[] { 2, 3, 14, 9, -1 }, new int[] { 2, 6, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0037217219360172749, 0.1679864972829819, -0.10740660130977631, new int[] { 7, 5, 4, 2, -1 }, new int[] { 7, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0251063294708729, 0.0170449391007423, -0.49812939763069147, true, new int[] { 13, 0, 2, 4, -1 }, new int[] { 13, 0, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0085740294307470322, 0.0389305390417576, -0.33503991365432739, new int[] { 0, 11, 4, 4, -1 }, new int[] { 1, 11, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0162992291152477, -0.17728500068187711, 0.0059367809444665909, true, new int[] { 13, 0, 2, 4, -1 }, new int[] { 13, 0, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.013755589723587, 0.0492921508848667, -0.2990570068359375, true, new int[] { 5, 0, 4, 2, -1 }, new int[] { 5, 0, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0101705603301525, 0.0125693203881383, -0.32717370986938482, true, new int[] { 13, 0, 4, 1, -1 }, new int[] { 14, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1183888018131256, -0.30642750859260559, 0.0404061898589134, new int[] { 0, 1, 4, 12, -1 }, new int[] { 0, 7, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.287784606218338, 0.00862664170563221, -0.58403861522674561, new int[] { 2, 3, 16, 12, -1 }, new int[] { 10, 3, 8, 6, 2 }, new int[] { 2, 9, 8, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0107093695551157, -0.45812180638313288, 0.0267107002437115, new int[] { 5, 5, 8, 2, -1 }, new int[] { 5, 5, 4, 1, 2 }, new int[] { 9, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168365407735109, -0.4834601879119873, 0.0014101839624345303, true, new int[] { 13, 0, 4, 1, -1 }, new int[] { 14, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.026871969923377, 0.302361011505127, -0.0401738695800304, true, new int[] { 6, 0, 3, 6, -1 }, new int[] { 7, 1, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020822209771722555, 0.0263978503644466, -0.0711281672120094, true, new int[] { 13, 0, 4, 1, -1 }, new int[] { 14, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.18307130038738251, 0.0315734706819057, -0.4311215877532959, true, new int[] { 7, 0, 8, 7, -1 }, new int[] { 9, 2, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053969710133969784, -0.0999102368950844, 0.0134910000488162, true, new int[] { 13, 0, 4, 1, -1 }, new int[] { 14, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065924688242375851, 0.0344651006162167, -0.4054282009601593, true, new int[] { 5, 0, 1, 4, -1 }, new int[] { 4, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00769148301333189, -0.0393002107739449, 0.1681717932224274, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0134877096861601, 0.31880301237106318, -0.0385033711791039, true, new int[] { 2, 2, 3, 2, -1 }, new int[] { 3, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0132067799568176, 0.1150619015097618, -0.0261230692267418, true, new int[] { 14, 0, 4, 3, -1 }, new int[] { 13, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0095766428858041763, -0.0562361218035221, 0.22048389911651611, true, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0030655260197818279, -0.0801741108298302, 0.1032200008630753, new int[] { 8, 13, 7, 2, -1 }, new int[] { 8, 14, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.6779087723698467E-05, -0.1722442954778671, 0.0690877288579941, true, new int[] { 8, 3, 1, 2, -1 }, new int[] { 8, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0961858332157135, 0.0015162150375545025, -0.55438750982284546, true, new int[] { 9, 0, 4, 8, -1 }, new int[] { 10, 1, 2, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0381203815340996, 0.0515935495495796, -0.26273688673973078, true, new int[] { 9, 0, 8, 4, -1 }, new int[] { 8, 1, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.505683422088623, 0.0104669099673629, -0.51577657461166382, new int[] { 5, 0, 12, 10, -1 }, new int[] { 9, 0, 4, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0121925799176097, 0.30584090948104858, -0.0400131605565548, true, new int[] { 7, 7, 3, 2, -1 }, new int[] { 7, 7, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12820640206336981, 0.0224020406603813, -0.27763271331787109, new int[] { 9, 5, 9, 10, -1 }, new int[] { 9, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1294344961643219, -0.0615348294377327, 0.2134552001953125, new int[] { 1, 0, 12, 10, -1 }, new int[] { 5, 0, 4, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0757145211100578, 0.15290339291095731, -0.11667010188102719, new int[] { 5, 0, 8, 7, -1 }, new int[] { 5, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3732179367216304E-05, 0.12800370156764981, -0.0978259593248367, new int[] { 6, 0, 2, 2, -1 }, new int[] { 6, 0, 1, 1, 2 }, new int[] { 7, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075803599320352077, -0.0979151725769043, 0.12620359659194949, new int[] { 0, 11, 18, 4, -1 }, new int[] { 0, 12, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0686360225081444, 0.0404322184622288, -0.313297301530838, new int[] { 0, 5, 5, 10, -1 }, new int[] { 0, 10, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0114607503637671, 0.0253615006804466, -0.48540189862251282, new int[] { 8, 9, 3, 3, -1 }, new int[] { 9, 9, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6128649551537819E-05, -0.104320302605629, 0.1133332997560501, new int[] { 0, 13, 2, 2, -1 }, new int[] { 0, 13, 1, 1, 2 }, new int[] { 1, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.4630657511297613E-05, -0.1048785969614983, 0.12740099430084231, new int[] { 16, 13, 2, 2, -1 }, new int[] { 17, 13, 1, 1, 2 }, new int[] { 16, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.151140496134758, -0.10252150148153311, new int[] { 0, 13, 2, 2, -1 }, new int[] { 0, 13, 1, 1, 2 }, new int[] { 1, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0116111198440194, 0.0148869697004557, -0.28674951195716858, true, new int[] { 9, 0, 4, 1, -1 }, new int[] { 10, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0124207204207778, -0.0620668604969978, 0.17772339284420011, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0234262607991695, -0.0847592502832413, 0.14415900409221649, true, new int[] { 14, 5, 2, 6, -1 }, new int[] { 14, 5, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14368200302124021, 0.0257685091346502, -0.49598070979118353, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 9, 9, 3, 2 }, new int[] { 9, 12, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036740589421242476, -0.34700030088424683, 0.0128000602126122, new int[] { 9, 1, 1, 4, -1 }, new int[] { 9, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.14955900143832E-05, -0.10679510235786439, 0.0999599397182465, new int[] { 1, 0, 1, 4, -1 }, new int[] { 1, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0099259437993168831, 0.0326209701597691, -0.35369759798049932, new int[] { 1, 0, 16, 2, -1 }, new int[] { 9, 0, 8, 1, 2 }, new int[] { 1, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1487040764186531E-05, 0.12531200051307681, -0.0952782332897186, new int[] { 8, 0, 2, 2, -1 }, new int[] { 8, 0, 1, 1, 2 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.027326600626111, -0.0089491289108991623, 0.0647247210144997, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0223257504403591, 0.0140139004215598, -0.74047172069549561, new int[] { 6, 8, 3, 4, -1 }, new int[] { 7, 8, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0402809605002403, 0.0010004050564020872, -0.1177709996700287, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0218933299183846, -0.0508843213319778, 0.22789579629898071, true, new int[] { 6, 7, 4, 2, -1 }, new int[] { 6, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0071642571128904819, 0.12857060134410861, -0.0535524301230907, new int[] { 12, 8, 3, 2, -1 }, new int[] { 12, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.080841138958931, 0.2065366059541702, -0.0666172280907631, new int[] { 0, 5, 17, 6, -1 }, new int[] { 0, 7, 17, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00051331298891454935, -0.0544428005814552, 0.149631604552269, new int[] { 14, 6, 2, 2, -1 }, new int[] { 15, 6, 1, 1, 2 }, new int[] { 14, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0086274370551109314, 0.0308179594576359, -0.36723139882087708, true, new int[] { 8, 8, 3, 1, -1 }, new int[] { 9, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00087373692076653242, 0.1390278041362763, -0.0632526502013206, new int[] { 14, 6, 2, 2, -1 }, new int[] { 15, 6, 1, 1, 2 }, new int[] { 14, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0117200398817658, -0.47670018672943121, 0.0244123209267855, true, new int[] { 9, 7, 3, 1, -1 }, new int[] { 10, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.048860989511013, 0.0100850900635123, -0.4659259021282196, true, new int[] { 9, 0, 1, 6, -1 }, new int[] { 9, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0186931006610394, -0.071992039680481, 0.1769388020038605, true, new int[] { 9, 0, 6, 1, -1 }, new int[] { 9, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0539086498320103, 0.14675259590148931, -0.0904555171728134, new int[] { 6, 5, 6, 10, -1 }, new int[] { 6, 5, 3, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0083356387913227081, 0.0223987400531769, -0.49412518739700317, true, new int[] { 9, 1, 1, 2, -1 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7100899387733079E-05, -0.0535624101758003, 0.0771028995513916, new int[] { 14, 6, 2, 2, -1 }, new int[] { 15, 6, 1, 1, 2 }, new int[] { 14, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.9839400162454695E-05, -0.0879170671105385, 0.12769749760627749, new int[] { 2, 6, 2, 2, -1 }, new int[] { 2, 6, 1, 1, 2 }, new int[] { 3, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.5873789127217606E-05, 0.0862401127815247, -0.0919469594955444, new int[] { 14, 6, 2, 2, -1 }, new int[] { 15, 6, 1, 1, 2 }, new int[] { 14, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.5616321585839614E-05, 0.1086385995149612, -0.0997067466378212, new int[] { 2, 6, 2, 2, -1 }, new int[] { 2, 6, 1, 1, 2 }, new int[] { 3, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034546090755611658, 0.0336912795901299, -0.25994610786437988, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0304389707744122, 0.369629293680191, -0.0292082708328962, new int[] { 5, 0, 8, 5, -1 }, new int[] { 7, 0, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.43956300616264338, -0.023035079240799, 0.44141438603401179, new int[] { 3, 3, 12, 6, -1 }, new int[] { 7, 5, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.8688350691227242E-05, -0.1096998974680901, 0.0987688973546028, new int[] { 4, 6, 1, 3, -1 }, new int[] { 4, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00840908195823431, -0.0491456389427185, 0.17818750441074371, new int[] { 13, 7, 2, 6, -1 }, new int[] { 13, 9, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0149121098220348, -0.42131778597831732, 0.0264007300138474, new int[] { 1, 4, 16, 2, -1 }, new int[] { 1, 4, 8, 1, 2 }, new int[] { 9, 5, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0209064893424511, -0.29467320442199713, 0.0150551898404956, new int[] { 10, 13, 6, 2, -1 }, new int[] { 12, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.3503939852816984E-05, -0.080975167453289, 0.12568619847297671, new int[] { 3, 7, 2, 2, -1 }, new int[] { 3, 7, 1, 1, 2 }, new int[] { 4, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010656829690560699, 0.0537998713552952, -0.14916649460792539, new int[] { 9, 7, 2, 2, -1 }, new int[] { 9, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0148796895518899, 0.0201143808662891, -0.47147929668426508, new int[] { 7, 4, 4, 4, -1 }, new int[] { 7, 4, 2, 2, 2 }, new int[] { 9, 6, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0184495002031326, 0.0162126608192921, -0.260709285736084, new int[] { 10, 13, 6, 2, -1 }, new int[] { 12, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011283960193395615, -0.0618423111736774, 0.15736180543899539, new int[] { 3, 6, 1, 4, -1 }, new int[] { 3, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0417683906853199, 0.0045171868987381458, -0.52301770448684692, new int[] { 10, 13, 6, 2, -1 }, new int[] { 12, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046589840203523636, -0.2460370063781738, 0.0389899984002113, new int[] { 3, 0, 11, 2, -1 }, new int[] { 3, 1, 11, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0121205700561404, 0.0129689900204539, -0.677115797996521, new int[] { 6, 0, 6, 2, -1 }, new int[] { 6, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051322788931429386, 0.0152305504307151, -0.55883342027664185, new int[] { 0, 9, 1, 3, -1 }, new int[] { 0, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0852644816040993, 0.0017884389963001013, -0.57048821449279785, new int[] { 6, 5, 12, 4, -1 }, new int[] { 12, 5, 6, 2, 2 }, new int[] { 6, 7, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.027729919180274, -0.0375315397977829, 0.31022569537162781, new int[] { 0, 5, 12, 4, -1 }, new int[] { 0, 5, 6, 2, 2 }, new int[] { 6, 7, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041674780659377575, -0.0953240767121315, 0.0961099192500114, new int[] { 10, 3, 4, 2, -1 }, new int[] { 10, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0350565910339355, -0.37690278887748718, 0.0244747009128332, new int[] { 0, 4, 4, 4, -1 }, new int[] { 0, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0171847604215145, -0.0070347599685192108, 0.48588299751281738, true, new int[] { 16, 8, 2, 1, -1 }, new int[] { 16, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037842839956283569, 0.0439080595970154, -0.2523730993270874, true, new int[] { 2, 8, 1, 2, -1 }, new int[] { 2, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.82066917419433594, 0.0151718696579337, -0.53948462009429932, new int[] { 0, 0, 18, 15, -1 }, new int[] { 6, 0, 6, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0100911604240537, -0.0969208627939224, 0.11189579963684081, new int[] { 1, 2, 6, 4, -1 }, new int[] { 4, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0160295106470585, -0.2344131022691727, 0.0234555192291737, new int[] { 13, 2, 2, 13, -1 }, new int[] { 13, 2, 1, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0108496798202395, 0.0441476404666901, -0.26963528990745539, new int[] { 3, 2, 2, 13, -1 }, new int[] { 4, 2, 1, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0130452997982502, 0.002215320011600852, -0.797849178314209, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0112366396933794, -0.0430468209087849, 0.240149199962616, true, new int[] { 3, 0, 3, 3, -1 }, new int[] { 4, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0077543058432638645, -0.3550145030021668, 0.0110251400619745, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0033010800834745169, 0.0303408205509186, -0.37136289477348328, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065340842120349407, -0.0858052521944046, 0.0916388481855392, new int[] { 2, 12, 16, 2, -1 }, new int[] { 2, 13, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0476196818053722, 0.40863269567489618, -0.026420159265399, new int[] { 2, 13, 14, 2, -1 }, new int[] { 2, 14, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00068403937621042132, -0.0323128588497639, 0.0880808010697365, new int[] { 16, 0, 1, 2, -1 }, new int[] { 16, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.6149452070239931E-05, 0.1152559965848923, -0.089074932038784, new int[] { 1, 0, 1, 2, -1 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0609943717718124, 0.0818466916680336, new int[] { 15, 0, 2, 2, -1 }, new int[] { 16, 0, 1, 1, 2 }, new int[] { 15, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.2685357483569533E-05, 0.11239729821681981, -0.087840661406517, new int[] { 1, 0, 2, 2, -1 }, new int[] { 1, 0, 1, 1, 2 }, new int[] { 2, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.118195986142382E-05, 0.1241813972592354, -0.0961579829454422, new int[] { 15, 0, 2, 2, -1 }, new int[] { 16, 0, 1, 1, 2 }, new int[] { 15, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0080426130443811417, -0.4060375988483429, 0.0250931605696678, true, new int[] { 4, 0, 1, 4, -1 }, new int[] { 3, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0734931826591492, 0.0902145579457283, new int[] { 15, 0, 2, 2, -1 }, new int[] { 16, 0, 1, 1, 2 }, new int[] { 15, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0119768275180832E-05, -0.0829944536089897, 0.11394649744033809, new int[] { 1, 0, 2, 2, -1 }, new int[] { 1, 0, 1, 1, 2 }, new int[] { 2, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0009592528804205358, -0.0712060630321503, 0.0428064316511154, new int[] { 8, 3, 8, 2, -1 }, new int[] { 8, 4, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002021104097366333, 0.0255169607698917, -0.35512170195579529, new int[] { 6, 0, 1, 3, -1 }, new int[] { 6, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0122425798326731, 0.0187698900699615, -0.19807919859886169, new int[] { 8, 6, 4, 4, -1 }, new int[] { 10, 6, 2, 2, 2 }, new int[] { 8, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142810503020883, 0.19607509672641751, -0.0502470508217812, new int[] { 5, 1, 7, 6, -1 }, new int[] { 5, 3, 7, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.40956941246986389, 0.0131073901429772, -0.7247236967086792, new int[] { 7, 0, 6, 15, -1 }, new int[] { 7, 5, 6, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6600460842018947E-05, -0.0870764032006264, 0.11106210201978679, new int[] { 6, 6, 4, 4, -1 }, new int[] { 6, 6, 2, 2, 2 }, new int[] { 8, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011234419653192163, 0.0774560794234276, -0.13284559547901151, new int[] { 8, 6, 2, 2, -1 }, new int[] { 8, 7, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00564270606264472, 0.0484460406005383, -0.21871030330657959, true, new int[] { 7, 7, 2, 1, -1 }, new int[] { 7, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0135915102437139, 0.0825356394052505, -0.0227083601057529, true, new int[] { 13, 6, 4, 3, -1 }, new int[] { 12, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0115914195775986, -0.0487906895577908, 0.1949059069156647, true, new int[] { 5, 6, 3, 4, -1 }, new int[] { 6, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.12608569860458371, 0.41815188527107239, -0.00957962591201067, new int[] { 13, 0, 2, 12, -1 }, new int[] { 13, 6, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0263312608003616, 0.0167261492460966, -0.57491612434387207, new int[] { 3, 1, 1, 10, -1 }, new int[] { 3, 6, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0410546697676182, -0.0108851799741387, 0.3410010039806366, true, new int[] { 10, 3, 1, 8, -1 }, new int[] { 8, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0710404366254807, -0.0139168696478009, 0.60548651218414307, true, new int[] { 8, 3, 8, 1, -1 }, new int[] { 10, 5, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168137494474649, -0.41529890894889832, 0.0231689400970936, new int[] { 0, 3, 18, 2, -1 }, new int[] { 9, 3, 9, 1, 2 }, new int[] { 0, 4, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0169783309102058, 0.22032840549945831, -0.0398988015949726, true, new int[] { 4, 1, 3, 4, -1 }, new int[] { 5, 2, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-9.5234332547988743E-05, 0.0811500027775764, -0.134388193488121, new int[] { 15, 9, 2, 6, -1 }, new int[] { 15, 9, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0171206202358007, -0.42468288540840149, 0.0203172601759434, new int[] { 1, 9, 2, 6, -1 }, new int[] { 2, 9, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0212412606924772, 0.0140559002757072, -0.54326081275939941, new int[] { 15, 9, 3, 6, -1 }, new int[] { 16, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0468163415789604, 0.39923959970474238, -0.0228534191846848, new int[] { 1, 12, 14, 3, -1 }, new int[] { 1, 13, 14, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0220952108502388, -0.41975128650665278, 0.011670240201056, new int[] { 15, 9, 3, 6, -1 }, new int[] { 16, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22133700549602511, 0.0133688803762197, -0.58491647243499756, new int[] { 0, 3, 9, 12, -1 }, new int[] { 0, 6, 9, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0057718330062925816, -0.0393010601401329, 0.0762483775615692, true, new int[] { 11, 6, 3, 4, -1 }, new int[] { 12, 7, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092696389183402061, -0.0408090092241764, 0.2058036029338837, true, new int[] { 7, 6, 4, 3, -1 }, new int[] { 6, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016822699690237641, -0.06055973097682, 0.0894235521554947, new int[] { 10, 10, 8, 2, -1 }, new int[] { 14, 10, 4, 1, 2 }, new int[] { 10, 11, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0152791002765298, -0.39893868565559393, 0.0227994602173567, new int[] { 7, 6, 3, 3, -1 }, new int[] { 8, 7, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0031749838963150978, 0.13225950300693509, -0.046028770506382, new int[] { 9, 2, 2, 3, -1 }, new int[] { 9, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028258180245757103, -0.1063044965267181, 0.0968753024935722, new int[] { 7, 1, 4, 4, -1 }, new int[] { 8, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.4384778195526451E-05, 0.0512824915349483, -0.0842741429805756, true, new int[] { 14, 1, 4, 2, -1 }, new int[] { 14, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145618002861738, -0.0433528609573841, 0.19777399301528931, true, new int[] { 4, 2, 3, 4, -1 }, new int[] { 5, 3, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00043724558781832457, -0.0508190095424652, 0.1038798987865448, new int[] { 10, 11, 2, 2, -1 }, new int[] { 11, 11, 1, 1, 2 }, new int[] { 10, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10908489674329761, -0.33270779252052313, 0.0268289800733328, new int[] { 1, 8, 16, 6, -1 }, new int[] { 1, 8, 8, 3, 2 }, new int[] { 9, 11, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00070241180947050452, 0.0761685222387314, -0.0645192116498947, new int[] { 15, 8, 3, 7, -1 }, new int[] { 16, 8, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.015636570751667, -0.44809681177139282, 0.0202762503176928, new int[] { 0, 8, 3, 7, -1 }, new int[] { 1, 8, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0118979997932911, -0.49537119269371033, 0.0044984170235693455, new int[] { 9, 9, 2, 4, -1 }, new int[] { 10, 9, 1, 2, 2 }, new int[] { 9, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015789919998496771, 0.1295803040266037, -0.0726606398820877, new int[] { 7, 9, 2, 4, -1 }, new int[] { 7, 9, 1, 2, 2 }, new int[] { 8, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.49960118532180792, -0.66730189323425293, 0.0079309539869427681, new int[] { 3, 6, 14, 9, -1 }, new int[] { 3, 6, 7, 9, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 12
            stage = new HaarCascadeStage(-1.2872380018234253, 11, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0789403170347214, 0.329888790845871, -0.19701880216598511, true, new int[] { 8, 5, 3, 6, -1 }, new int[] { 6, 7, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0173211302608252, 0.21981470286846161, -0.0811920836567879, new int[] { 9, 0, 8, 3, -1 }, new int[] { 11, 0, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0123552503064275, -0.30988898873329163, 0.14423920214176181, new int[] { 3, 3, 8, 2, -1 }, new int[] { 7, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1042677983641625, 0.1562684029340744, -0.18359909951686859, new int[] { 1, 6, 16, 8, -1 }, new int[] { 1, 8, 16, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0851838812232018, -0.29022741317749018, 0.12742319703102109, new int[] { 2, 7, 14, 8, -1 }, new int[] { 2, 7, 7, 4, 2 }, new int[] { 9, 11, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13357129693031311, -0.30198419094085688, -0.016821650788188, new int[] { 9, 7, 4, 6, -1 }, new int[] { 9, 9, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22293360531330109, 0.0184083096683025, -916.78131103515625, new int[] { 5, 6, 3, 9, -1 }, new int[] { 5, 9, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0277230702340603, 0.0996664837002754, -0.1188244000077248, new int[] { 12, 7, 6, 8, -1 }, new int[] { 12, 7, 3, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.18182699382305151, -0.0572614409029484, 0.46252810955047607, true, new int[] { 9, 2, 9, 4, -1 }, new int[] { 12, 5, 3, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0246847905218601, 0.0688610523939133, -0.19284169375896451, new int[] { 12, 7, 6, 8, -1 }, new int[] { 12, 7, 3, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0138146495446563, -0.0780585184693336, 0.3078015148639679, new int[] { 4, 7, 3, 4, -1 }, new int[] { 4, 9, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0245245005935431, -0.26867350935935969, 0.0682309865951538, new int[] { 8, 0, 8, 6, -1 }, new int[] { 8, 3, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0070112771354615688, -0.18542979657649991, 0.1132294982671738, new int[] { 3, 0, 12, 1, -1 }, new int[] { 6, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1054819002747536, -0.34024599194526672, 0.010903449729085, new int[] { 12, 9, 6, 6, -1 }, new int[] { 12, 9, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0063391570001840591, 0.10419529676437379, -0.2051645964384079, new int[] { 0, 9, 6, 6, -1 }, new int[] { 3, 9, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.078947477042675, 0.0161181092262268, -0.41540539264678961, new int[] { 15, 1, 3, 14, -1 }, new int[] { 15, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048509850166738033, 0.0488411597907543, -0.38384801149368292, new int[] { 5, 1, 8, 2, -1 }, new int[] { 5, 1, 4, 1, 2 }, new int[] { 9, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0458627305924892, -0.15829730033874509, 0.1020084023475647, new int[] { 5, 0, 12, 5, -1 }, new int[] { 8, 0, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0134294098243117, 0.0545731112360954, -0.36586639285087591, new int[] { 5, 0, 4, 4, -1 }, new int[] { 5, 2, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0191512107849121, 0.0119114201515913, -0.43721321225166321, true, new int[] { 12, 0, 2, 3, -1 }, new int[] { 12, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.2203599959611893, 0.3832859992980957, -0.0577213913202286, new int[] { 4, 0, 10, 15, -1 }, new int[] { 9, 0, 5, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0423834510147572, -0.0653426200151443, 0.0784513726830482, new int[] { 5, 0, 12, 3, -1 }, new int[] { 8, 0, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.030524730682373, 0.0496221706271172, -0.34946510195732122, new int[] { 0, 1, 2, 14, -1 }, new int[] { 0, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.01950404047966, -0.0683437287807465, 0.26461350917816162, new int[] { 5, 4, 8, 4, -1 }, new int[] { 5, 6, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.009846939705312252, -0.0779279768466949, 0.2089402973651886, new int[] { 2, 9, 14, 2, -1 }, new int[] { 2, 10, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.032195370644331, 0.26800119876861572, -0.0700547993183136, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0098907537758350372, 0.1219308972358704, -0.139754593372345, new int[] { 5, 6, 8, 2, -1 }, new int[] { 5, 7, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0164340194314718, 0.0296364594250917, -0.23874099552631381, new int[] { 10, 5, 3, 3, -1 }, new int[] { 11, 6, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.7646512838546187E-05, 0.1085129007697105, -0.13716340065002439, new int[] { 0, 2, 2, 1, -1 }, new int[] { 1, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0145368697121739, -0.3846626877784729, 0.0236762408167124, true, new int[] { 12, 0, 4, 2, -1 }, new int[] { 13, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0117109399288893, 0.0416956692934036, -0.31956040859222412, true, new int[] { 6, 0, 3, 2, -1 }, new int[] { 6, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.01164176966995, -0.28680109977722168, 0.0145577499642968, new int[] { 10, 5, 3, 3, -1 }, new int[] { 11, 6, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0212982799857855, 0.0255194008350372, -0.48966890573501592, new int[] { 5, 5, 3, 3, -1 }, new int[] { 6, 6, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0092027969658374786, -0.62252938747406006, 0.0087586138397455215, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0201745200902224, 0.30807420611381531, -0.0395388789474964, new int[] { 4, 13, 10, 2, -1 }, new int[] { 4, 14, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0106579503044486, 0.0104256300255656, -0.37197288870811462, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045577301643788815, -0.1160800009965897, 0.1050620973110199, new int[] { 1, 12, 14, 2, -1 }, new int[] { 1, 13, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.059895858168602, -0.0082911262288689613, 0.0757109001278877, true, new int[] { 10, 5, 4, 6, -1 }, new int[] { 8, 7, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0925180464982986, -0.39722099900245672, 0.0354158990085125, true, new int[] { 8, 5, 6, 4, -1 }, new int[] { 10, 7, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0053780227899551392, -0.0451698005199432, 0.10165379941463471, new int[] { 15, 6, 3, 6, -1 }, new int[] { 15, 9, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021006090100854635, 0.0736289173364639, -0.1836252957582474, new int[] { 7, 8, 3, 2, -1 }, new int[] { 7, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00844130665063858, -0.0506231300532818, 0.27132049202919012, new int[] { 2, 8, 14, 2, -1 }, new int[] { 2, 9, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0289131104946136, -0.2333088964223862, 0.0561418682336807, new int[] { 3, 0, 3, 8, -1 }, new int[] { 3, 4, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0894289314746857, 0.0421395003795624, -0.2966344952583313, new int[] { 0, 1, 18, 8, -1 }, new int[] { 9, 1, 9, 4, 2 }, new int[] { 0, 5, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0222117304801941, 0.32237181067466741, -0.0411601513624191, new int[] { 5, 0, 8, 7, -1 }, new int[] { 7, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037851219531148672, -0.0707370936870575, 0.10991329699754721, true, new int[] { 10, 1, 4, 1, -1 }, new int[] { 10, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0083305174484848976, -0.1936282962560654, 0.0662610232830048, true, new int[] { 2, 0, 10, 2, -1 }, new int[] { 2, 0, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0234631896018982, -0.2286916971206665, 0.0538989901542664, new int[] { 4, 4, 10, 6, -1 }, new int[] { 9, 4, 5, 3, 2 }, new int[] { 4, 7, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0010604270501062274, -0.0725375488400459, 0.1586951017379761, new int[] { 5, 8, 4, 2, -1 }, new int[] { 5, 8, 2, 1, 2 }, new int[] { 7, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0659593567252159, 0.0056216111406683922, -0.39239290356636047, new int[] { 15, 6, 3, 6, -1 }, new int[] { 15, 9, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0548790097236633, 0.28525480628013611, -0.0444187112152576, new int[] { 1, 4, 16, 6, -1 }, new int[] { 1, 6, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034504090435802937, 0.0136751402169466, -0.44305860996246338, new int[] { 9, 0, 1, 4, -1 }, new int[] { 9, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00797334685921669, 0.0208843499422073, -0.50481712818145752, new int[] { 0, 7, 2, 3, -1 }, new int[] { 0, 8, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0184303596615791, -0.0379651300609112, 0.21417169272899631, true, new int[] { 15, 5, 3, 3, -1 }, new int[] { 14, 6, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0060115829110145569, -0.34198600053787231, 0.029979920014739, true, new int[] { 8, 0, 1, 3, -1 }, new int[] { 7, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0407630987465382, 0.24182400107383731, -0.0324762500822544, new int[] { 6, 1, 12, 3, -1 }, new int[] { 9, 1, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0456319898366928, 0.194716602563858, -0.0898651406168938, new int[] { 0, 1, 12, 3, -1 }, new int[] { 3, 1, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0130249597132206, 0.18374669551849371, -0.0397638715803623, true, new int[] { 15, 5, 3, 3, -1 }, new int[] { 14, 6, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.035364780575037, -0.0993380174040794, 0.1346897035837174, new int[] { 0, 0, 8, 7, -1 }, new int[] { 4, 0, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1877132058143616, 0.0116381403058767, -0.342296302318573, true, new int[] { 14, 4, 4, 6, -1 }, new int[] { 14, 4, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075244922190904617, -0.20901820063591, 0.0642698332667351, true, new int[] { 4, 4, 6, 4, -1 }, new int[] { 4, 4, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0345222912728786, 0.352169394493103, -0.0368988513946533, new int[] { 4, 1, 10, 8, -1 }, new int[] { 4, 3, 10, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011451860191300511, 0.0721520334482193, -0.2084126025438309, new int[] { 8, 7, 2, 2, -1 }, new int[] { 8, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0108127798885107, -0.33911031484603882, 0.0102402996271849, true, new int[] { 11, 0, 3, 2, -1 }, new int[] { 12, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044051618315279484, 0.0448350198566914, -0.232111006975174, true, new int[] { 7, 0, 2, 3, -1 }, new int[] { 6, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0061400611884891987, -0.26839160919189448, 0.0390401408076286, new int[] { 3, 0, 12, 2, -1 }, new int[] { 9, 0, 6, 1, 2 }, new int[] { 3, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.5988669221987948E-05, 0.1104065030813217, -0.0973475277423859, new int[] { 0, 2, 2, 2, -1 }, new int[] { 0, 2, 1, 1, 2 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00977076031267643, 0.1318017989397049, -0.042217381298542, true, new int[] { 15, 1, 3, 3, -1 }, new int[] { 14, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0146375196054578, -0.0399371199309826, 0.26679611206054688, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0173694007098675, 0.0430083684623241, -0.26838469505310059, new int[] { 2, 0, 14, 4, -1 }, new int[] { 9, 0, 7, 2, 2 }, new int[] { 2, 2, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0207157004624605, -0.0441390685737133, 0.252885103225708, true, new int[] { 6, 1, 3, 6, -1 }, new int[] { 7, 2, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044260770082473755, -0.0181482806801796, 0.0637400820851326, new int[] { 16, 6, 2, 4, -1 }, new int[] { 16, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0218196604400873, -0.45305460691452032, 0.024142699316144, new int[] { 0, 6, 2, 4, -1 }, new int[] { 0, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048437709920108318, 0.0123435202986002, -0.15617559850215909, new int[] { 9, 2, 3, 2, -1 }, new int[] { 9, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0027822460979223251, -0.30781841278076172, 0.0338872000575066, new int[] { 6, 2, 3, 2, -1 }, new int[] { 6, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0014766600215807557, 0.037661001086235, -0.0371170900762081, new int[] { 13, 0, 4, 4, -1 }, new int[] { 14, 0, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.020395090803504, 0.0135211497545242, -0.72870039939880371, new int[] { 1, 0, 4, 4, -1 }, new int[] { 2, 0, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014377470361068845, -0.055464219301939, 0.0552656501531601, new int[] { 8, 13, 10, 2, -1 }, new int[] { 13, 13, 5, 1, 2 }, new int[] { 8, 14, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.029832549393177, 0.42611280083656311, -0.0218381006270647, true, new int[] { 4, 4, 3, 3, -1 }, new int[] { 5, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.030555859208107, 0.0176318995654583, -0.60954070091247559, new int[] { 7, 7, 4, 6, -1 }, new int[] { 8, 7, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1229958981275559, -0.0266627203673124, 0.369583398103714, true, new int[] { 7, 1, 8, 2, -1 }, new int[] { 9, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0229585207998753, -0.46332129836082458, 0.0184264499694109, new int[] { 4, 1, 11, 4, -1 }, new int[] { 4, 2, 11, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0132682900875807, -0.43808931112289429, 0.0190128590911627, new int[] { 4, 12, 4, 3, -1 }, new int[] { 5, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0461827516555786, -0.7000507116317749, 0.0115271303802729, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 8, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0263124592602253, -0.0715227574110031, 0.127688005566597, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 11, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.8344743340276182E-05, -0.0716612488031387, 0.0649365931749344, new int[] { 12, 13, 1, 2, -1 }, new int[] { 12, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0374639108777046, -0.3165304958820343, 0.0307877492159605, true, new int[] { 8, 5, 1, 8, -1 }, new int[] { 8, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0563586615025997, 0.00842954870313406, -0.60672062635421753, new int[] { 10, 2, 4, 13, -1 }, new int[] { 11, 2, 2, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0073837172240018845, 0.0977723896503448, -0.0991689264774323, new int[] { 4, 2, 4, 13, -1 }, new int[] { 5, 2, 2, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.9623919544974342E-05, -0.0549541302025318, 0.0757452771067619, new int[] { 11, 8, 3, 3, -1 }, new int[] { 12, 9, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1653591990470886, 0.0260911695659161, -0.35252508521080023, true, new int[] { 5, 0, 10, 4, -1 }, new int[] { 5, 0, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0830756202340126, -0.53609657287597656, 0.0153222400695086, new int[] { 0, 7, 18, 4, -1 }, new int[] { 9, 7, 9, 2, 2 }, new int[] { 0, 9, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013314849929884076, -0.0434926301240921, 0.21460059285163879, new int[] { 4, 8, 2, 2, -1 }, new int[] { 4, 8, 1, 1, 2 }, new int[] { 5, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0240376498550177, 0.3358427882194519, -0.0249130893498659, new int[] { 7, 0, 8, 3, -1 }, new int[] { 9, 0, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062097259797155857, 0.0491514205932617, -0.19901290535926819, true, new int[] { 8, 6, 3, 1, -1 }, new int[] { 9, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0736415982246399, -0.087231457233429, 0.109493300318718, new int[] { 2, 3, 14, 8, -1 }, new int[] { 2, 5, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.028918519616127, 0.0510564483702183, -0.2057587951421738, true, new int[] { 8, 6, 1, 8, -1 }, new int[] { 8, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00372535502538085, -0.0367016084492207, 0.1051134988665581, new int[] { 11, 8, 4, 4, -1 }, new int[] { 11, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082107484340667725, 0.0238303001970053, -0.35800701379776, new int[] { 1, 11, 4, 2, -1 }, new int[] { 2, 11, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0018392279744148254, -0.044707790017128, 0.1189830973744392, new int[] { 10, 8, 4, 2, -1 }, new int[] { 12, 8, 2, 1, 2 }, new int[] { 10, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0068104080855846405, -0.16840079426765439, 0.0483481995761395, new int[] { 7, 7, 3, 3, -1 }, new int[] { 8, 8, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0033966489136219025, -0.0308044198900461, 0.13462269306182861, new int[] { 13, 8, 4, 2, -1 }, new int[] { 15, 8, 2, 1, 2 }, new int[] { 13, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0003391581994947046, -0.0775286927819252, 0.11303810030221941, new int[] { 1, 8, 4, 2, -1 }, new int[] { 1, 8, 2, 1, 2 }, new int[] { 3, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1835324019193649, 0.0953205227851868, -0.0324969291687012, new int[] { 5, 3, 10, 12, -1 }, new int[] { 5, 3, 5, 12, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44860368967056269, 0.0139211900532246, -0.72890061140060425, new int[] { 3, 3, 10, 12, -1 }, new int[] { 8, 3, 5, 12, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0888018906116486, -0.0640209093689919, 0.0364004485309124, new int[] { 5, 0, 12, 8, -1 }, new int[] { 9, 0, 4, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1080844029784203, -0.0643229931592941, 0.19376879930496221, new int[] { 1, 0, 12, 8, -1 }, new int[] { 5, 0, 4, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0069059031084179878, -0.31092429161071777, 0.0205565802752972, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025598949287086725, -0.0915503427386284, 0.092027373611927, true, new int[] { 9, 0, 2, 2, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.000493561674375087, -0.0242713205516338, 0.0657608583569527, true, new int[] { 9, 1, 2, 2, -1 }, new int[] { 9, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0153526701033115, 0.0173107199370861, -0.48900419473648071, true, new int[] { 9, 1, 2, 2, -1 }, new int[] { 9, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00970359519124031, 0.0089735705405473709, -0.4127190113067627, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021431730128824711, -0.1955125033855438, 0.0380251109600067, new int[] { 6, 0, 4, 2, -1 }, new int[] { 6, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.30845791217871E-05, 0.0705076232552528, -0.047128938138485, new int[] { 9, 0, 1, 2, -1 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0868036672472954, -0.0163518991321325, 0.47820520401000982, true, new int[] { 9, 5, 2, 4, -1 }, new int[] { 9, 5, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0110789397731423, -0.0255244206637144, 0.1099068000912666, new int[] { 8, 8, 6, 4, -1 }, new int[] { 11, 8, 3, 2, 2 }, new int[] { 8, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0061349938623607159, -0.3572841882705689, 0.0223970897495747, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.004765429999679327, -0.0850082710385323, 0.0223076492547989, true, new int[] { 15, 1, 3, 2, -1 }, new int[] { 16, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0122526502236724, 0.0178576093167067, -0.41976860165596008, true, new int[] { 3, 1, 2, 3, -1 }, new int[] { 2, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011971440166235, -0.0210712291300297, 0.23789730668067929, true, new int[] { 16, 1, 2, 3, -1 }, new int[] { 15, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032991201151162386, -0.0615648999810219, 0.13292570412158969, true, new int[] { 2, 1, 3, 2, -1 }, new int[] { 3, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0184490196406841, 0.14298330247402191, -0.0252068098634481, true, new int[] { 14, 1, 4, 2, -1 }, new int[] { 14, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0074155619367957115, 0.17994129657745361, -0.0498336292803288, true, new int[] { 4, 1, 2, 4, -1 }, new int[] { 4, 1, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0482065714895725, 0.0272459890693426, -0.38131779432296747, new int[] { 13, 2, 5, 6, -1 }, new int[] { 13, 5, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011687170481309295, 0.0469573400914669, -0.1817303001880646, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1361666023731232, 0.40798899531364441, -0.0224768593907356, new int[] { 2, 0, 14, 9, -1 }, new int[] { 2, 3, 14, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.1014733985066414, -0.0845235288143158, new int[] { 2, 1, 1, 2, -1 }, new int[] { 2, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0767729580402374, 0.0064514591358602047, -0.46041280031204218, new int[] { 13, 2, 5, 6, -1 }, new int[] { 13, 5, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0634575635194778, -0.0202501695603132, 0.397266298532486, new int[] { 0, 0, 8, 9, -1 }, new int[] { 2, 0, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003344458993524313, 0.15261699259281161, -0.0526536405086517, new int[] { 8, 5, 2, 2, -1 }, new int[] { 8, 6, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0572412200272083, -0.13445749878883359, 0.0807463303208351, true, new int[] { 9, 0, 8, 5, -1 }, new int[] { 11, 2, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0416314415633678, -0.1082227975130081, 0.0224370695650578, new int[] { 13, 2, 5, 6, -1 }, new int[] { 13, 5, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0149030797183514, 0.0450070798397064, -0.22001840174198151, new int[] { 0, 2, 5, 6, -1 }, new int[] { 0, 5, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22303420305252081, 0.0124958604574203, -0.60045099258422852, new int[] { 3, 4, 12, 10, -1 }, new int[] { 9, 4, 6, 5, 2 }, new int[] { 3, 9, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0169060304760933, 0.0127502698451281, -0.5323861837387085, new int[] { 6, 6, 2, 3, -1 }, new int[] { 7, 6, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24477340281009671, 0.0031138889025896788, -0.57128059864044189, true, new int[] { 11, 1, 6, 6, -1 }, new int[] { 13, 3, 2, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1874004006385803, 0.43744760751724238, -0.0196508895605803, true, new int[] { 7, 1, 6, 6, -1 }, new int[] { 5, 3, 6, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0050131231546401978, -0.0674036368727684, 0.101325199007988, new int[] { 13, 4, 1, 6, -1 }, new int[] { 13, 6, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012101340107619762, 0.0345095582306385, -0.2193517982959747, new int[] { 8, 0, 1, 2, -1 }, new int[] { 8, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109212100505829, -0.15897870063781741, 0.0067669888958334923, new int[] { 10, 13, 6, 2, -1 }, new int[] { 13, 13, 3, 1, 2 }, new int[] { 10, 14, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0010091220028698444, -0.0808166116476059, 0.0902162864804268, new int[] { 2, 13, 6, 2, -1 }, new int[] { 2, 13, 3, 1, 2 }, new int[] { 5, 14, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0791598334908485, -0.4955776035785675, 0.00905772764235735, new int[] { 5, 12, 9, 3, -1 }, new int[] { 8, 12, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0231257900595665, 0.0261550601571798, -0.2640474140644074, new int[] { 1, 14, 12, 1, -1 }, new int[] { 5, 14, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25399661064147949, -0.0417557582259178, 0.0842676386237144, new int[] { 4, 0, 12, 15, -1 }, new int[] { 8, 0, 4, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0413385704159737, -0.0543079786002636, 0.16323280334472659, new int[] { 3, 0, 8, 14, -1 }, new int[] { 5, 0, 4, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00798014272004366, -0.0563799887895584, 0.0850874036550522, new int[] { 10, 10, 8, 4, -1 }, new int[] { 14, 10, 4, 2, 2 }, new int[] { 10, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0221821498125792, 0.15680639445781711, -0.0526730790734291, new int[] { 2, 0, 12, 5, -1 }, new int[] { 6, 0, 4, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.8383043475914747E-05, -0.11258769780397419, 0.0710221901535988, true, new int[] { 12, 0, 2, 1, -1 }, new int[] { 12, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0050613721832633018, -0.37599068880081182, 0.0229838006198406, true, new int[] { 6, 0, 1, 2, -1 }, new int[] { 6, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0636510029435158, 0.00411559920758009, -0.4183712899684906, true, new int[] { 12, 5, 2, 8, -1 }, new int[] { 12, 5, 1, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0198200307786465, -0.0826675072312355, 0.0975382328033447, true, new int[] { 6, 5, 8, 2, -1 }, new int[] { 6, 5, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012445739703252912, -0.033446729183197, 0.1453846991062164, new int[] { 12, 7, 2, 2, -1 }, new int[] { 13, 7, 1, 1, 2 }, new int[] { 12, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.11178659647703169, 0.25024509429931641, -0.035332940518856, new int[] { 2, 4, 14, 4, -1 }, new int[] { 2, 6, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0024203520733863115, 0.17330379784107211, -0.0227931998670101, new int[] { 12, 7, 2, 2, -1 }, new int[] { 13, 7, 1, 1, 2 }, new int[] { 12, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00012127320223953575, -0.0742904022336006, 0.11935789883136751, new int[] { 4, 7, 2, 2, -1 }, new int[] { 4, 7, 1, 1, 2 }, new int[] { 5, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0086516663432121277, 0.0119632603600621, -0.28482851386070251, new int[] { 12, 5, 1, 4, -1 }, new int[] { 12, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.577970917627681E-05, -0.11878810077905661, 0.0836797133088112, new int[] { 5, 5, 1, 4, -1 }, new int[] { 5, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0066892090253531933, -0.0259499493986368, 0.0986363664269447, true, new int[] { 12, 7, 3, 2, -1 }, new int[] { 13, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073373341001570225, -0.0568680502474308, 0.1380600035190582, new int[] { 3, 9, 6, 4, -1 }, new int[] { 3, 9, 3, 2, 2 }, new int[] { 6, 11, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0048734410665929317, 0.0774335265159607, -0.0352366790175438, new int[] { 9, 12, 6, 1, -1 }, new int[] { 9, 12, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.4124629716388881E-05, -0.124569296836853, 0.0716082230210304, new int[] { 7, 3, 4, 1, -1 }, new int[] { 8, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0303157493472099, -0.19579620659351349, 0.0308573506772518, new int[] { 6, 13, 8, 2, -1 }, new int[] { 6, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0350410714745522, 0.17880150675773621, -0.0489667803049088, true, new int[] { 9, 0, 6, 2, -1 }, new int[] { 9, 0, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0419709086418152, -0.0401918590068817, 0.129463404417038, new int[] { 7, 11, 10, 4, -1 }, new int[] { 7, 11, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0408818498253822, 0.13018250465393069, -0.0782763436436653, new int[] { 1, 11, 15, 4, -1 }, new int[] { 6, 11, 5, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052412762306630611, -0.18295650184154511, 0.0371690504252911, new int[] { 7, 0, 6, 4, -1 }, new int[] { 7, 1, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0555911002447829E-05, -0.0837283581495285, 0.0939808636903763, new int[] { 1, 10, 2, 2, -1 }, new int[] { 1, 10, 1, 1, 2 }, new int[] { 2, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0165926907211542, 0.0057793757878243923, -0.81482458114624023, new int[] { 8, 10, 3, 2, -1 }, new int[] { 9, 10, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063152369111776352, 0.0213363692164421, -0.32484549283981318, new int[] { 0, 8, 2, 3, -1 }, new int[] { 0, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0568882115185261, -0.41595301032066351, 0.0036880860570818186, new int[] { 11, 8, 4, 4, -1 }, new int[] { 11, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034150490537285805, -0.0535964109003544, 0.1404040008783341, new int[] { 3, 8, 4, 4, -1 }, new int[] { 3, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1477995961904526, 0.0049799410626292229, -0.6226087212562561, new int[] { 2, 7, 16, 2, -1 }, new int[] { 6, 7, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0695117115974426, -0.43304800987243652, 0.0189262200146914, new int[] { 0, 7, 16, 2, -1 }, new int[] { 4, 7, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016076939646154642, -0.0367941483855248, 0.0683272704482079, new int[] { 12, 10, 4, 2, -1 }, new int[] { 14, 10, 2, 1, 2 }, new int[] { 12, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015456780092790723, -0.0668036863207817, 0.1335151940584183, new int[] { 2, 10, 4, 2, -1 }, new int[] { 2, 10, 2, 1, 2 }, new int[] { 4, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0159673895686865, 0.0069505311548709869, -0.47136560082435608, new int[] { 16, 9, 2, 3, -1 }, new int[] { 16, 10, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.28711509704589838, -0.015348769724369, 0.47458758950233459, new int[] { 6, 4, 6, 9, -1 }, new int[] { 8, 7, 2, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.34093499183654791, 0.0054452791810035706, -0.79175651073455811, new int[] { 8, 0, 4, 15, -1 }, new int[] { 8, 5, 4, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0066727129742503166, 0.0294574107974768, -0.25477468967437739, new int[] { 8, 7, 2, 3, -1 }, new int[] { 8, 8, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026719029992818832, -0.17070059478282931, 0.0357673391699791, new int[] { 6, 1, 7, 2, -1 }, new int[] { 6, 2, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052617820911109447, -0.0336550511419773, 0.21332639455795291, new int[] { 0, 7, 6, 2, -1 }, new int[] { 0, 7, 3, 1, 2 }, new int[] { 3, 8, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00810788944363594, 0.0301098693162203, -0.0460237488150597, true, new int[] { 12, 3, 5, 3, -1 }, new int[] { 11, 4, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0167319998145103, -0.043719969689846, 0.19436429440975189, true, new int[] { 6, 3, 3, 5, -1 }, new int[] { 7, 4, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0191528107970953, 0.0174971204251051, -0.428276002407074, new int[] { 7, 8, 4, 3, -1 }, new int[] { 7, 9, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14171889424324041, -0.38993918895721441, 0.0170895904302597, new int[] { 2, 1, 6, 14, -1 }, new int[] { 2, 1, 3, 7, 2 }, new int[] { 5, 8, 3, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038122260011732578, -0.11586090177297589, 0.0506625697016716, new int[] { 10, 1, 8, 9, -1 }, new int[] { 10, 1, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0170307997614145, -0.5399131178855896, 0.0119414301589131, new int[] { 7, 7, 4, 4, -1 }, new int[] { 8, 7, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0088250916451215744, -0.33240219950675959, 0.0083178747445344925, new int[] { 9, 9, 2, 4, -1 }, new int[] { 10, 9, 1, 2, 2 }, new int[] { 9, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0059308991767466068, 0.221118301153183, -0.0314335711300373, new int[] { 3, 9, 4, 2, -1 }, new int[] { 3, 9, 2, 1, 2 }, new int[] { 5, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0037457819562405348, -0.1030357033014298, 0.0240999702364206, new int[] { 16, 9, 2, 3, -1 }, new int[] { 16, 10, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048495861701667309, 0.0257306694984436, -0.26656630635261541, new int[] { 0, 9, 2, 3, -1 }, new int[] { 0, 10, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.30769100785255432, 0.0261018890887499, -0.18695330619812009, new int[] { 2, 0, 16, 9, -1 }, new int[] { 6, 0, 8, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0117959501221776, -0.1118796989321709, 0.0688933432102203, new int[] { 1, 1, 8, 4, -1 }, new int[] { 5, 1, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1020568981766701, 0.1641097962856293, -0.0039911000058054924, true, new int[] { 9, 3, 2, 6, -1 }, new int[] { 7, 5, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1050693020224571, -0.017098480835557, 0.42889660596847529, true, new int[] { 9, 3, 6, 2, -1 }, new int[] { 11, 5, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.8301670176442713E-05, -0.0416239388287067, 0.0495718717575073, new int[] { 14, 1, 2, 2, -1 }, new int[] { 15, 1, 1, 1, 2 }, new int[] { 14, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022682799026370049, -0.0688075497746468, 0.10216739773750309, true, new int[] { 2, 3, 3, 2, -1 }, new int[] { 3, 4, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0060366461984813213, -0.1738830953836441, 0.0198664106428623, true, new int[] { 15, 0, 2, 2, -1 }, new int[] { 15, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049747680313885212, 0.0331093408167362, -0.23262310028076169, true, new int[] { 3, 0, 2, 2, -1 }, new int[] { 3, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0342620797455311, -0.21563960611820221, 0.0115074804052711, true, new int[] { 10, 0, 1, 8, -1 }, new int[] { 8, 2, 1, 4, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 13
            stage = new HaarCascadeStage(-1.2998509407043457, 12, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0882937535643578, -0.248940497636795, 0.26465269923210138, new int[] { 3, 2, 12, 8, -1 }, new int[] { 3, 4, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165174994617701, 0.1308764964342117, -0.048301700502634, new int[] { 8, 0, 9, 2, -1 }, new int[] { 11, 0, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2429573982954025, 0.0002460803952999413, -1211.89697265625, new int[] { 4, 5, 9, 6, -1 }, new int[] { 4, 8, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.017855629324913, -0.0218822807073593, 0.0629134327173233, new int[] { 8, 0, 9, 2, -1 }, new int[] { 11, 0, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0112768700346351, 0.18169599771499631, -0.2307166010141373, new int[] { 1, 0, 9, 2, -1 }, new int[] { 4, 0, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0232120305299759, 0.1088896989822388, -0.28105589747428888, new int[] { 7, 0, 8, 4, -1 }, new int[] { 7, 2, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0334626212716103, 0.42646810412406921, -0.1128323003649712, true, new int[] { 7, 6, 3, 3, -1 }, new int[] { 6, 7, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0309944301843643, 0.0578055083751678, -0.39169758558273321, new int[] { 2, 0, 14, 6, -1 }, new int[] { 9, 0, 7, 3, 2 }, new int[] { 2, 3, 7, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15080560743808749, -0.44636020064353937, 0.0689948424696922, new int[] { 0, 0, 4, 14, -1 }, new int[] { 0, 7, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.19667649269104, 0.0504155196249485, -0.51629501581192017, new int[] { 0, 5, 18, 10, -1 }, new int[] { 9, 5, 9, 5, 2 }, new int[] { 0, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022066079545766115, -0.0707260966300964, 0.27825760841369629, new int[] { 5, 7, 1, 3, -1 }, new int[] { 5, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1075704991817474, 0.2446808069944382, -0.0725844725966454, new int[] { 3, 5, 12, 4, -1 }, new int[] { 3, 7, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0601789988577366, -0.0937738493084908, 0.209071695804596, new int[] { 2, 5, 14, 6, -1 }, new int[] { 2, 7, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0721643567085266, 0.0246197003871202, -0.377494603395462, new int[] { 11, 2, 6, 6, -1 }, new int[] { 11, 5, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0018397889798507094, -0.365955114364624, 0.0356928594410419, new int[] { 6, 0, 2, 2, -1 }, new int[] { 6, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0033323359675705433, 0.0274193398654461, -0.21830609440803531, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25542399287223821, 0.0424718111753464, -0.40455558896064758, new int[] { 0, 0, 9, 15, -1 }, new int[] { 3, 5, 3, 5, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0083238910883665085, -0.0382980890572071, 0.1997260004281998, new int[] { 10, 8, 5, 3, -1 }, new int[] { 10, 9, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036837169900536537, 0.0516507886350155, -0.314887285232544, new int[] { 6, 0, 3, 4, -1 }, new int[] { 6, 1, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15801090002059939, 0.0079839415848255157, -0.64591610431671143, new int[] { 7, 9, 8, 6, -1 }, new int[] { 7, 9, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1195484027266502, 0.0303646996617317, -0.483592689037323, new int[] { 4, 9, 8, 5, -1 }, new int[] { 8, 9, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.1479396612849087E-05, 0.0919145867228508, -0.1064620986580849, new int[] { 16, 1, 2, 1, -1 }, new int[] { 16, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002526798052713275, 0.0452573001384735, -0.3438262939453125, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.17898750305175781, 0.0144175197929144, -0.502654492855072, true, new int[] { 9, 0, 6, 8, -1 }, new int[] { 11, 2, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0395551882684231, -0.35880690813064581, 0.0342500805854797, true, new int[] { 9, 8, 1, 6, -1 }, new int[] { 9, 8, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0076789730228483677, -0.1114436984062195, 0.1351636946201325, new int[] { 0, 10, 18, 2, -1 }, new int[] { 0, 11, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0105727799236774, -0.0437579788267612, 0.31598579883575439, new int[] { 3, 8, 5, 3, -1 }, new int[] { 3, 9, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0357067584991455, -0.1592438071966171, 0.0833674669265747, new int[] { 1, 1, 16, 4, -1 }, new int[] { 5, 1, 8, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0151766203343868, -0.1096644029021263, 0.14354470372200009, new int[] { 6, 0, 6, 2, -1 }, new int[] { 9, 0, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0519099794328213, 0.13713189959526059, -0.0289334002882242, true, new int[] { 14, 4, 4, 7, -1 }, new int[] { 15, 5, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0249809008091688, 0.1281910985708237, -0.101640097796917, true, new int[] { 4, 4, 7, 4, -1 }, new int[] { 3, 5, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051697930321097374, 0.0397001393139362, -0.16936889290809631, new int[] { 8, 2, 6, 2, -1 }, new int[] { 8, 3, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0047851498238742352, -0.28047218918800348, 0.0424798987805843, new int[] { 4, 2, 10, 2, -1 }, new int[] { 4, 2, 5, 1, 2 }, new int[] { 9, 3, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0114343902096152, -0.30073690414428711, 0.0279115606099367, true, new int[] { 16, 11, 2, 2, -1 }, new int[] { 16, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0310384295880795, -0.0384156294167042, 0.31910240650177, new int[] { 2, 12, 14, 3, -1 }, new int[] { 2, 13, 14, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0019539990462362766, 0.0490082204341888, -0.243400901556015, new int[] { 16, 12, 2, 2, -1 }, new int[] { 16, 12, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015783209819346666, 0.0490619093179703, -0.21728959679603579, new int[] { 0, 12, 2, 2, -1 }, new int[] { 1, 12, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.141022801399231, 0.12385349720716481, -0.0194560904055834, new int[] { 10, 7, 6, 6, -1 }, new int[] { 12, 9, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.025759469717741, -0.0577305890619755, 0.22352460026741031, new int[] { 2, 7, 6, 6, -1 }, new int[] { 4, 9, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13943019509315491, -0.43312790989875788, 0.0051124738529324532, true, new int[] { 11, 5, 1, 9, -1 }, new int[] { 8, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0970044583082199, -0.58657991886138916, 0.0171818397939205, new int[] { 0, 5, 18, 4, -1 }, new int[] { 0, 5, 9, 2, 2 }, new int[] { 9, 7, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0095027927309274673, -0.0287947598844767, 0.29738929867744451, true, new int[] { 17, 3, 1, 3, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0262469295412302, -0.21234120428562159, 0.049407571554184, true, new int[] { 4, 5, 6, 4, -1 }, new int[] { 4, 5, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0285178907215595, -0.41019749641418463, 0.0107241403311491, new int[] { 11, 13, 6, 2, -1 }, new int[] { 13, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0079501066356897354, 0.29748660326004028, -0.0357652083039284, true, new int[] { 1, 3, 3, 1, -1 }, new int[] { 2, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0294742994010448, -0.27445879578590388, 0.0378581508994102, new int[] { 0, 1, 18, 4, -1 }, new int[] { 9, 1, 9, 2, 2 }, new int[] { 0, 3, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0197004098445177, -0.3731251060962677, 0.0246061906218529, new int[] { 1, 13, 6, 2, -1 }, new int[] { 3, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0202972404658794, -0.0114561002701521, 0.1300147026777268, true, new int[] { 11, 5, 1, 9, -1 }, new int[] { 8, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0733654201030731, -0.33396759629249573, 0.0288594998419285, true, new int[] { 7, 5, 9, 1, -1 }, new int[] { 10, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063272351399064064, -0.0767316669225693, 0.1508390009403229, new int[] { 1, 9, 16, 2, -1 }, new int[] { 1, 10, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13661600649356839, 0.16243369877338409, -0.0956437736749649, new int[] { 1, 7, 16, 8, -1 }, new int[] { 1, 9, 16, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0107580302283168, -0.23738150298595431, 0.0315589606761932, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0666851326823235, 0.0154138403013349, -0.62513381242752075, true, new int[] { 3, 1, 11, 2, -1 }, new int[] { 3, 1, 11, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.30325201153755188, -0.0291348807513714, 0.36113429069519037, new int[] { 5, 3, 9, 6, -1 }, new int[] { 8, 5, 3, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0158231593668461, -0.40985879302024841, 0.0231184493750334, true, new int[] { 6, 0, 2, 4, -1 }, new int[] { 5, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0253745596855879, -0.0204721000045538, 0.27052029967308039, true, new int[] { 15, 2, 3, 3, -1 }, new int[] { 14, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0163469407707453, -0.0353308208286762, 0.28036290407180792, true, new int[] { 3, 2, 3, 3, -1 }, new int[] { 4, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034061360638588667, -0.1116679012775421, 0.0920868366956711, new int[] { 10, 3, 4, 2, -1 }, new int[] { 10, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2318589985370636, -0.0533741116523743, 0.226513996720314, new int[] { 3, 1, 12, 6, -1 }, new int[] { 9, 1, 6, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0087358150631189346, 0.0622405707836151, -0.1609788984060288, new int[] { 7, 5, 4, 6, -1 }, new int[] { 9, 5, 2, 3, 2 }, new int[] { 7, 8, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0479816384613514, 0.0325308404862881, -0.27026590704917908, new int[] { 3, 6, 12, 6, -1 }, new int[] { 3, 6, 6, 3, 2 }, new int[] { 9, 9, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0325526595115662, -0.0267996098846197, 0.36133301258087158, new int[] { 7, 4, 4, 6, -1 }, new int[] { 7, 7, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052017602138221264, -0.22696959972381589, 0.0536908693611622, true, new int[] { 8, 6, 2, 2, -1 }, new int[] { 8, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0520097799599171, 0.51674157381057739, -0.020591339096427, new int[] { 2, 12, 14, 2, -1 }, new int[] { 2, 13, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0050841891206800938, 0.0838762521743774, -0.1215421035885811, new int[] { 2, 8, 6, 7, -1 }, new int[] { 4, 8, 2, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0053035072050988674, 0.031446810811758, -0.1233906000852585, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.004594006109982729, -0.0627442970871925, 0.14181789755821231, new int[] { 3, 6, 3, 3, -1 }, new int[] { 4, 7, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069754808209836483, 0.0279876105487347, -0.30492180585861212, new int[] { 3, 0, 12, 2, -1 }, new int[] { 9, 0, 6, 1, 2 }, new int[] { 3, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00339008797891438, -0.21763899922370911, 0.0362194888293743, new int[] { 0, 13, 4, 2, -1 }, new int[] { 1, 13, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0095793427899479866, -0.0433258786797524, 0.16427470743656161, true, new int[] { 13, 6, 3, 5, -1 }, new int[] { 14, 7, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0550329610705376, -0.26936888694763178, 0.0320559591054916, true, new int[] { 7, 4, 6, 3, -1 }, new int[] { 9, 6, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.095517598092556, 0.2161073982715607, -0.0582397803664207, new int[] { 1, 5, 16, 6, -1 }, new int[] { 1, 7, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.000885121407918632, 0.0752959027886391, -0.1217793971300125, new int[] { 8, 9, 2, 2, -1 }, new int[] { 8, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0084586488083004951, -0.0455720499157906, 0.2856633067131043, new int[] { 8, 9, 8, 2, -1 }, new int[] { 12, 9, 4, 1, 2 }, new int[] { 8, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1383175998926163, -0.0303479190915823, 0.28037178516387939, new int[] { 0, 0, 12, 15, -1 }, new int[] { 3, 0, 6, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.008588903583586216, 0.25955429673194891, -0.0248014405369759, new int[] { 10, 3, 4, 2, -1 }, new int[] { 10, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0026830460410565138, -0.13567750155925751, 0.0750199928879738, new int[] { 4, 3, 4, 2, -1 }, new int[] { 6, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0561147592961788, -0.13314700126647949, 0.0675303786993027, new int[] { 6, 0, 9, 9, -1 }, new int[] { 9, 0, 3, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0064768209122121334, -0.0428345091640949, 0.2283774018287659, new int[] { 0, 2, 8, 2, -1 }, new int[] { 2, 2, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075396071188151836, 0.0175717808306217, -0.4712331891059876, true, new int[] { 15, 0, 2, 1, -1 }, new int[] { 15, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.032276589423418, 0.16673429310321811, -0.0572832897305489, new int[] { 8, 1, 2, 8, -1 }, new int[] { 8, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081356316804885864, 0.027268510311842, -0.1111190989613533, true, new int[] { 16, 3, 2, 2, -1 }, new int[] { 16, 3, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.010477039963007, 0.0260039307177067, -0.36761531233787542, true, new int[] { 2, 3, 2, 2, -1 }, new int[] { 2, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0309956707060337, -0.0286454297602177, 0.33150678873062128, true, new int[] { 14, 2, 4, 3, -1 }, new int[] { 13, 3, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070666121318936348, -0.40544331073760992, 0.0251925494521856, new int[] { 5, 3, 8, 2, -1 }, new int[] { 5, 3, 4, 1, 2 }, new int[] { 9, 4, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036987180355936289, 0.063140757381916, -0.0327784791588783, true, new int[] { 13, 1, 5, 3, -1 }, new int[] { 12, 2, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0306662693619728, 0.32546588778495789, -0.027702359482646, true, new int[] { 5, 1, 3, 5, -1 }, new int[] { 6, 2, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0788802430033684, 0.01533810980618, -0.220662996172905, new int[] { 7, 0, 9, 6, -1 }, new int[] { 7, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0326623804867268, -0.26111158728599548, 0.0396143011748791, new int[] { 2, 0, 9, 6, -1 }, new int[] { 2, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.202998697757721, 0.46856230497360229, -0.0211902894079685, new int[] { 4, 0, 10, 8, -1 }, new int[] { 4, 4, 10, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0013156479690223932, 0.0511390715837479, -0.1778022050857544, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2458626925945282, 0.00207719998434186, -0.723025918006897, true, new int[] { 9, 0, 6, 8, -1 }, new int[] { 11, 2, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0096061620861291885, -0.0438566096127033, 0.20256240665912631, true, new int[] { 5, 6, 4, 3, -1 }, new int[] { 4, 7, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0928886383771896, 0.025762340053916, -0.0818297490477562, true, new int[] { 9, 0, 6, 8, -1 }, new int[] { 11, 2, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0018360089743509889, -0.1065806970000267, 0.0778321474790573, new int[] { 1, 11, 6, 4, -1 }, new int[] { 1, 11, 3, 2, 2 }, new int[] { 4, 13, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0101813804358244, -0.0704501271247864, 0.0211151205003262, true, new int[] { 9, 0, 6, 8, -1 }, new int[] { 11, 2, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22913800179958341, 0.0105785802006722, -0.815527617931366, true, new int[] { 9, 0, 8, 6, -1 }, new int[] { 7, 2, 8, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0212600603699684, -0.23754499852657321, 0.0127379801124334, true, new int[] { 14, 0, 4, 4, -1 }, new int[] { 15, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0019725849851965904, 0.057212870568037, -0.13770629465579989, new int[] { 4, 1, 3, 4, -1 }, new int[] { 4, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.641170016839169E-05, 0.0502910390496254, -0.0575029999017715, new int[] { 13, 0, 3, 1, -1 }, new int[] { 14, 0, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.36206799745559692, -0.773370087146759, 0.0101746097207069, new int[] { 0, 7, 11, 8, -1 }, new int[] { 0, 11, 11, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14286839962005621, 0.36285620927810669, -0.0296504106372595, new int[] { 1, 9, 17, 4, -1 }, new int[] { 1, 11, 17, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0601753890514374, 0.10930059850215911, -0.090728648006916, new int[] { 1, 6, 16, 6, -1 }, new int[] { 1, 8, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.76404711662326E-05, -0.0555778108537197, 0.0779178664088249, new int[] { 13, 0, 3, 1, -1 }, new int[] { 14, 0, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.4806099797133356E-05, 0.0850946307182312, -0.0902227982878685, new int[] { 2, 0, 3, 1, -1 }, new int[] { 3, 0, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00825556181371212, 0.16778500378131869, -0.0391292311251163, new int[] { 5, 2, 9, 6, -1 }, new int[] { 5, 4, 9, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0024975580163300037, -0.25427588820457458, 0.0310085993260145, new int[] { 7, 1, 3, 2, -1 }, new int[] { 7, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.16913549602031711, 0.0076711731962859631, -0.47778978943824768, new int[] { 6, 11, 12, 4, -1 }, new int[] { 6, 13, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00806424580514431, 0.032001681625843, -0.2201628983020783, new int[] { 0, 0, 16, 2, -1 }, new int[] { 0, 0, 8, 1, 2 }, new int[] { 8, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.8364861615700647E-05, -0.0927060320973396, 0.0926686972379684, new int[] { 16, 11, 2, 2, -1 }, new int[] { 17, 11, 1, 1, 2 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0242639407515526, 0.30613300204277039, -0.0236746892333031, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1245393976569176, -0.001139872008934617, 0.65001028776168823, true, new int[] { 12, 0, 6, 2, -1 }, new int[] { 14, 2, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.030860610306263, -0.234003096818924, 0.0343167595565319, true, new int[] { 6, 0, 2, 6, -1 }, new int[] { 4, 2, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0127543099224567, -0.0391327291727066, 0.0949018001556396, new int[] { 15, 8, 3, 6, -1 }, new int[] { 16, 10, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0376567393541336, 0.0261963903903961, -0.309109091758728, new int[] { 0, 8, 3, 6, -1 }, new int[] { 1, 10, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0312218796461821, -0.28618350625038153, 0.0050922371447086334, true, new int[] { 14, 4, 3, 3, -1 }, new int[] { 13, 5, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0134689500555396, 0.2125725001096726, -0.0359573401510715, true, new int[] { 4, 4, 3, 3, -1 }, new int[] { 5, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075858170166611671, -0.14510390162467959, 0.0284003801643848, new int[] { 11, 9, 3, 6, -1 }, new int[] { 12, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0325641296803951, 0.2121015936136246, -0.0337405614554882, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 12, 3, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0478576682507992, -0.28939861059188843, 0.0082710552960634232, true, new int[] { 15, 4, 1, 8, -1 }, new int[] { 13, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0408857800066471, 0.0154061401262879, -0.52735280990600586, true, new int[] { 3, 4, 8, 1, -1 }, new int[] { 5, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0111554395407438, 0.204815998673439, -0.0385781601071358, new int[] { 6, 0, 6, 6, -1 }, new int[] { 8, 0, 2, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0436525382101536, -0.56057322025299072, 0.0155440401285887, true, new int[] { 8, 3, 10, 1, -1 }, new int[] { 8, 3, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0237427093088627, -0.78456741571426392, 0.0031750639900565147, new int[] { 8, 8, 3, 5, -1 }, new int[] { 9, 8, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1069891974329948, -0.0261800494045019, 0.27015981078147888, true, new int[] { 7, 2, 8, 2, -1 }, new int[] { 9, 4, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0378550700843334, 0.0065697189420461655, -0.40291649103164667, new int[] { 11, 2, 3, 13, -1 }, new int[] { 12, 2, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0300023406744003, -0.3640936017036438, 0.0191395506262779, new int[] { 4, 2, 3, 13, -1 }, new int[] { 5, 2, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0177240408957005, 0.0121768601238728, -0.36743289232254028, new int[] { 17, 7, 1, 6, -1 }, new int[] { 17, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00792890228331089, -0.234558492898941, 0.0312652811408043, new int[] { 0, 7, 1, 6, -1 }, new int[] { 0, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0411901511251926, 0.1780917942523956, -0.0286607407033443, new int[] { 8, 7, 8, 4, -1 }, new int[] { 12, 7, 4, 2, 2 }, new int[] { 8, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104142995551229, -0.0461356192827225, 0.22065189480781561, new int[] { 2, 7, 8, 4, -1 }, new int[] { 2, 7, 4, 2, 2 }, new int[] { 6, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0623511299490929, -0.60133552551269531, 0.0119700403884053, new int[] { 3, 5, 12, 4, -1 }, new int[] { 9, 5, 6, 2, 2 }, new int[] { 3, 7, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.010768829844892, -0.0378835014998913, 0.19194099307060239, new int[] { 7, 12, 3, 3, -1 }, new int[] { 8, 13, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015350959729403257, 0.134353294968605, -0.0599097199738026, new int[] { 8, 12, 2, 3, -1 }, new int[] { 8, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0049390122294425964, -0.22644749283790591, 0.0331381000578403, new int[] { 5, 0, 6, 3, -1 }, new int[] { 5, 1, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029866439290344715, 0.039536539465189, -0.17985729873180389, new int[] { 7, 0, 4, 2, -1 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.1302180003840476E-05, -0.1217418983578682, 0.0578663200139999, new int[] { 3, 4, 4, 1, -1 }, new int[] { 5, 4, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0141327697783709, -0.0697263032197952, 0.1077838987112045, new int[] { 6, 0, 9, 1, -1 }, new int[] { 9, 0, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00870378315448761, 0.13537360727787021, -0.061749380081892, true, new int[] { 6, 8, 4, 2, -1 }, new int[] { 6, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0396597199141979, 0.28668469190597529, -0.004012012854218483, true, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0165502801537514, -0.0549145303666592, 0.1501951068639755, true, new int[] { 6, 7, 4, 2, -1 }, new int[] { 6, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0182081703096628, -0.0716051831841469, 0.0196856409311295, new int[] { 4, 1, 12, 4, -1 }, new int[] { 7, 1, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0295192506164312, 0.20991930365562439, -0.0432162992656231, new int[] { 2, 1, 12, 4, -1 }, new int[] { 5, 1, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0212850607931614, 0.186916396021843, -0.0237888600677252, new int[] { 5, 1, 12, 3, -1 }, new int[] { 9, 1, 4, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0378306210041046, -0.1275478005409241, 0.0723592489957809, new int[] { 3, 1, 11, 8, -1 }, new int[] { 3, 3, 11, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011643759906292, -0.0464428104460239, 0.1379096060991287, new int[] { 2, 7, 15, 4, -1 }, new int[] { 2, 8, 15, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0059127039276063442, -0.16960899531841281, 0.0449999384582043, true, new int[] { 5, 11, 2, 2, -1 }, new int[] { 5, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0576444491744041, -0.29772061109542852, 0.00851062498986721, new int[] { 6, 10, 8, 5, -1 }, new int[] { 8, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0539292395114899, -0.34829708933830261, 0.0207772795110941, new int[] { 4, 10, 8, 5, -1 }, new int[] { 6, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00077844387851655483, -0.1067842990159988, 0.0631283298134804, new int[] { 1, 11, 17, 2, -1 }, new int[] { 1, 12, 17, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0217015091329813, -0.0430709086358547, 0.2051513940095902, new int[] { 0, 9, 17, 4, -1 }, new int[] { 0, 10, 17, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0142901800572872, 0.0401067808270454, -0.19636610150337219, new int[] { 0, 6, 18, 2, -1 }, new int[] { 9, 6, 9, 1, 2 }, new int[] { 0, 7, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0479065105319023, 0.0266829095780849, -0.2608106136322022, true, new int[] { 7, 1, 3, 6, -1 }, new int[] { 5, 3, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0207046903669834, 0.0082300165668129921, -0.17172940075397489, new int[] { 9, 13, 6, 2, -1 }, new int[] { 11, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0228998996317387, -0.37081000208854681, 0.0185417495667934, new int[] { 3, 13, 6, 2, -1 }, new int[] { 5, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0019879220053553581, 0.1643680930137634, -0.0217982996255159, new int[] { 12, 6, 2, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }, new int[] { 12, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.498683822224848E-05, -0.0649014934897423, 0.1062330007553101, new int[] { 4, 6, 2, 2, -1 }, new int[] { 4, 6, 1, 1, 2 }, new int[] { 5, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001355992048047483, -0.0245978496968746, 0.14361660182476041, new int[] { 12, 6, 2, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }, new int[] { 12, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.6802290449268185E-05, 0.0772759467363358, -0.091653436422348, new int[] { 4, 6, 2, 2, -1 }, new int[] { 4, 6, 1, 1, 2 }, new int[] { 5, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0716202333569527, -0.245522603392601, 0.0295341201126575, new int[] { 13, 0, 5, 8, -1 }, new int[] { 13, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.024330910295248, 0.0413995198905468, -0.15903189778327939, true, new int[] { 8, 6, 6, 3, -1 }, new int[] { 10, 8, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0279465708881617, 0.002258610911667347, -0.67318201065063477, true, new int[] { 9, 11, 1, 3, -1 }, new int[] { 8, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0014360989443957806, 0.1064805015921593, -0.0644265785813332, true, new int[] { 9, 11, 3, 1, -1 }, new int[] { 10, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0047291368246078491, 0.0197015404701233, -0.28576970100402832, new int[] { 7, 1, 6, 2, -1 }, new int[] { 10, 1, 3, 1, 2 }, new int[] { 7, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0992026627063751, -0.3520042896270752, 0.01681605912745, new int[] { 1, 5, 16, 5, -1 }, new int[] { 5, 5, 8, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0089718345552682877, 0.0913507118821144, -0.0237340200692415, new int[] { 12, 6, 6, 1, -1 }, new int[] { 14, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032134570647031069, -0.0494450889527798, 0.142311304807663, new int[] { 0, 6, 6, 1, -1 }, new int[] { 2, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010166129795834422, 0.0645815804600716, -0.0191290695220232, new int[] { 15, 4, 2, 1, -1 }, new int[] { 15, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.125310028437525E-05, 0.0835471376776695, -0.0906196907162666, new int[] { 1, 4, 2, 1, -1 }, new int[] { 2, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011647429782897234, -0.17997290194034579, 0.0400951690971851, new int[] { 8, 0, 2, 2, -1 }, new int[] { 8, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0643320977687836, -0.38692688941955572, 0.0174406096339226, new int[] { 0, 0, 2, 10, -1 }, new int[] { 0, 5, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13757969439029691, 0.22808580100536349, -0.0328599512577057, new int[] { 3, 2, 12, 6, -1 }, new int[] { 3, 5, 12, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00731653394177556, 0.0429877601563931, -0.15990610420703891, true, new int[] { 6, 0, 4, 3, -1 }, new int[] { 5, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.021075239405036, 0.013760750181973, -0.0974362194538116, true, new int[] { 9, 0, 3, 6, -1 }, new int[] { 10, 1, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0470838211476803, -0.0716910064220428, 0.10700540244579319, new int[] { 0, 0, 8, 8, -1 }, new int[] { 4, 0, 4, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059396019205451012, -0.0633967369794846, 0.0387225411832333, new int[] { 6, 0, 9, 1, -1 }, new int[] { 9, 0, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.581971287727356, 0.0216003507375717, -0.37873318791389471, new int[] { 0, 0, 18, 9, -1 }, new int[] { 6, 0, 6, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0160421207547188, -0.046681709587574, 0.14364209771156311, new int[] { 5, 11, 9, 4, -1 }, new int[] { 5, 12, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.038316261023283, -0.62408488988876343, 0.0108488202095032, new int[] { 3, 2, 3, 13, -1 }, new int[] { 4, 2, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12451539933681489, -0.0091985529288649559, 0.111726701259613, true, new int[] { 10, 3, 6, 2, -1 }, new int[] { 10, 3, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1228756979107857, -0.0130921201780438, 0.52221369743347168, true, new int[] { 8, 3, 2, 6, -1 }, new int[] { 8, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0091833565384149551, -0.0758661031723022, 0.0255879797041416, true, new int[] { 13, 6, 3, 3, -1 }, new int[] { 12, 7, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0168187208473682, -0.0309611707925797, 0.23137600719928739, true, new int[] { 5, 6, 3, 3, -1 }, new int[] { 6, 7, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.6163040173705667E-05, -0.0593904405832291, 0.0742034986615181, new int[] { 11, 6, 2, 2, -1 }, new int[] { 12, 6, 1, 1, 2 }, new int[] { 11, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0548779107630253, 0.25981691479682922, -0.0269930195063353, new int[] { 7, 0, 4, 11, -1 }, new int[] { 9, 0, 2, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0076188119128346443, 0.13379520177841189, -0.055999118834734, new int[] { 8, 0, 2, 5, -1 }, new int[] { 8, 0, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.233624204993248, 0.32755351066589361, -0.0214694291353226, new int[] { 2, 2, 5, 12, -1 }, new int[] { 2, 8, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1114932000637054, -0.24463839828968051, 0.0362425111234188, new int[] { 0, 5, 18, 10, -1 }, new int[] { 9, 5, 9, 5, 2 }, new int[] { 0, 10, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0441570281982422, 0.4340217113494873, -0.0166491009294987, new int[] { 0, 10, 8, 4, -1 }, new int[] { 0, 10, 4, 2, 2 }, new int[] { 4, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.7168701459886506E-05, 0.0668948367238045, -0.0507181882858276, new int[] { 9, 0, 1, 3, -1 }, new int[] { 9, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.364686876419E-05, -0.0803783014416695, 0.0818097665905952, new int[] { 2, 11, 2, 2, -1 }, new int[] { 2, 11, 1, 1, 2 }, new int[] { 3, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1059508994221687, 0.0050716297701001167, -0.64737159013748169, new int[] { 14, 1, 2, 14, -1 }, new int[] { 14, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0836684033274651, 0.0086071500554680824, -0.65093028545379639, new int[] { 2, 1, 2, 14, -1 }, new int[] { 2, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0083153052255511284, -0.0472831390798092, 0.1902991980314255, true, new int[] { 14, 7, 3, 4, -1 }, new int[] { 15, 8, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0621465183794498, -0.18513560295104981, 0.0434024408459663, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 9, 9, 3, 2 }, new int[] { 9, 12, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015061040176078677, -0.0425548888742924, 0.0472707785665989, true, new int[] { 11, 7, 3, 5, -1 }, new int[] { 12, 8, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0126304496079683, 0.1005629971623421, -0.0700350031256676, true, new int[] { 7, 7, 5, 3, -1 }, new int[] { 6, 8, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0072226561605930328, -0.1351246982812882, 0.0165191907435656, true, new int[] { 15, 0, 3, 2, -1 }, new int[] { 16, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0398441106081009, 0.0061076539568603039, -1.0002349615097046, new int[] { 7, 8, 4, 5, -1 }, new int[] { 8, 8, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.53863292932510376, 0.00042299588676542044, -0.9881020188331604, new int[] { 8, 1, 10, 12, -1 }, new int[] { 8, 5, 10, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0243477690964937, -0.98886072635650635, 0.0046373298391699791, true, new int[] { 3, 0, 2, 3, -1 }, new int[] { 2, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044827940873801708, -0.0541374906897545, 0.138005793094635, true, new int[] { 17, 5, 1, 3, -1 }, new int[] { 16, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0796409398317337, -0.0579614713788033, 0.1078020036220551, new int[] { 0, 0, 15, 4, -1 }, new int[] { 5, 0, 5, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065154298208653927, -0.0951096937060356, 0.0761779919266701, new int[] { 12, 0, 6, 5, -1 }, new int[] { 12, 0, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0639263466000557, 0.022149670869112, -0.36810970306396479, new int[] { 0, 0, 6, 5, -1 }, new int[] { 3, 0, 3, 5, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 14
            stage = new HaarCascadeStage(-1.2603249549865723, 13, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0227022804319859, 0.3458436131477356, -0.1496108025312424, true, new int[] { 8, 6, 2, 4, -1 }, new int[] { 7, 7, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0113259796053171, 0.0946362167596817, -0.1482031047344208, new int[] { 6, 3, 10, 12, -1 }, new int[] { 11, 3, 5, 6, 2 }, new int[] { 6, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010080899810418487, 0.1488129943609238, -0.2323223948478699, new int[] { 3, 0, 6, 1, -1 }, new int[] { 5, 0, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10500989854335791, -0.2153766006231308, 0.0894507020711899, new int[] { 2, 4, 16, 8, -1 }, new int[] { 10, 4, 8, 4, 2 }, new int[] { 2, 8, 8, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.01267762016505, 0.27584138512611389, -0.1148819997906685, new int[] { 1, 6, 4, 4, -1 }, new int[] { 1, 6, 2, 2, 2 }, new int[] { 3, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049704289995133877, 0.0440389215946198, -0.16276310384273529, true, new int[] { 13, 0, 4, 2, -1 }, new int[] { 14, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0061556040309369564, 0.0742129236459732, -0.32477781176567078, true, new int[] { 5, 0, 2, 4, -1 }, new int[] { 4, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0072180028073489666, 0.42525339126586909, -0.0276413895189762, new int[] { 12, 8, 1, 3, -1 }, new int[] { 12, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029266420751810074, -0.0529128387570381, 0.39208140969276428, new int[] { 5, 8, 1, 3, -1 }, new int[] { 5, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079688094556331635, 0.0333337001502514, -0.4196723997592926, new int[] { 10, 0, 4, 4, -1 }, new int[] { 10, 1, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.004510131198912859, -0.0477215312421322, 0.44400349259376531, new int[] { 5, 7, 1, 3, -1 }, new int[] { 5, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00723468279466033, -0.42018109560012817, 0.0553282685577869, new int[] { 3, 0, 12, 2, -1 }, new int[] { 3, 1, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054523041471838951, 0.0427102707326412, -0.400739312171936, new int[] { 4, 0, 4, 4, -1 }, new int[] { 4, 1, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.135473906993866, 0.0132751995697618, -0.4189395010471344, new int[] { 6, 3, 10, 12, -1 }, new int[] { 11, 3, 5, 6, 2 }, new int[] { 6, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0285219997167587, 0.0712370425462723, -0.23564490675926211, new int[] { 2, 3, 10, 12, -1 }, new int[] { 2, 3, 5, 6, 2 }, new int[] { 7, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0678908079862595, -0.60827171802520752, 2.7981699531665072E-05, true, new int[] { 9, 0, 2, 9, -1 }, new int[] { 9, 0, 1, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.7107769710710272E-05, 0.1002285033464432, -0.13644769787788391, new int[] { 0, 3, 2, 1, -1 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.25962561368942261, -0.13785040378570559, 0.0266530998051167, new int[] { 12, 1, 6, 14, -1 }, new int[] { 12, 8, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1188557967543602, 0.0274891909211874, -0.54295271635055542, new int[] { 0, 1, 6, 14, -1 }, new int[] { 0, 8, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.056852251291275, -0.011255219578743, 0.38339531421661383, true, new int[] { 9, 0, 2, 9, -1 }, new int[] { 9, 0, 1, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0415694713592529, -0.0417712591588497, 0.34204569458961492, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 9, 0, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0441399216651917, -0.0225493591278791, 0.46690988540649409, new int[] { 2, 12, 14, 3, -1 }, new int[] { 2, 13, 14, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1063582971692085, 0.0297107696533203, -0.4509320855140686, new int[] { 0, 0, 18, 8, -1 }, new int[] { 0, 0, 9, 4, 2 }, new int[] { 9, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00828692875802517, -0.1222324967384338, 0.0532477386295795, new int[] { 11, 1, 5, 6, -1 }, new int[] { 11, 4, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0367316715419292, 0.0420367904007435, -0.44834709167480469, new int[] { 2, 1, 5, 6, -1 }, new int[] { 2, 4, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0577655285596848, -0.5459136962890625, 0.0074861990287899971, new int[] { 6, 10, 8, 5, -1 }, new int[] { 8, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1748784929513931, 0.0281722098588943, -0.43244078755378718, new int[] { 4, 9, 10, 6, -1 }, new int[] { 9, 9, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.577970917627681E-05, 0.0849614813923836, -0.0936162620782852, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.4103060645284131E-05, -0.15745340287685389, 0.0785599797964096, new int[] { 0, 11, 6, 4, -1 }, new int[] { 0, 11, 3, 2, 2 }, new int[] { 3, 13, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025306469760835171, -0.1860491931438446, 0.0132554396986961, new int[] { 14, 14, 2, 1, -1 }, new int[] { 14, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.564980968600139E-05, 0.1080086007714272, -0.1149718016386032, new int[] { 2, 14, 2, 1, -1 }, new int[] { 3, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.54274487495422363, -0.65146768093109131, 0.019872210919857, new int[] { 0, 3, 18, 8, -1 }, new int[] { 0, 7, 18, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104538202285767, -0.0576840490102768, 0.21809279918670649, true, new int[] { 3, 2, 3, 3, -1 }, new int[] { 4, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.4684870368218981E-05, 0.0703076869249344, -0.0687716603279114, new int[] { 16, 3, 1, 2, -1 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0386879108846188, -0.23570249974727631, 0.0593729391694069, new int[] { 2, 4, 6, 8, -1 }, new int[] { 5, 4, 3, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0146778095513582, -0.0045802700333297253, 0.66445422172546387, new int[] { 8, 9, 4, 2, -1 }, new int[] { 10, 9, 2, 1, 2 }, new int[] { 8, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0101802004501224, 0.52202922105789185, -0.0238862205296755, new int[] { 6, 9, 4, 2, -1 }, new int[] { 6, 9, 2, 1, 2 }, new int[] { 8, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.577970917627681E-05, -0.0755427628755569, 0.1076302006840706, new int[] { 16, 3, 1, 2, -1 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.1134765967726708, -0.1176417991518974, new int[] { 1, 3, 1, 2, -1 }, new int[] { 1, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0110010495409369, -0.4163585901260376, 0.02915552072227, new int[] { 1, 1, 16, 2, -1 }, new int[] { 9, 1, 8, 1, 2 }, new int[] { 1, 2, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0100403595715761, 0.50152337551116943, -0.024473249912262, new int[] { 6, 10, 4, 2, -1 }, new int[] { 6, 10, 2, 1, 2 }, new int[] { 8, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0110518001019955, 0.0379601791501045, -0.29772630333900452, new int[] { 7, 7, 4, 3, -1 }, new int[] { 8, 7, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0120895402505994, -0.51634800434112549, 0.021521920338273, new int[] { 6, 0, 4, 4, -1 }, new int[] { 6, 0, 2, 2, 2 }, new int[] { 8, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0844105631113052, 0.49133801460266108, -0.0146038103848696, new int[] { 12, 6, 6, 3, -1 }, new int[] { 14, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0227140001952648, -0.0488631390035152, 0.23572869598865509, new int[] { 0, 6, 6, 3, -1 }, new int[] { 2, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.064245767891407, 0.0656965523958206, new int[] { 14, 13, 2, 2, -1 }, new int[] { 15, 13, 1, 1, 2 }, new int[] { 14, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.564980968600139E-05, -0.10076279938220981, 0.100671797990799, new int[] { 2, 13, 2, 2, -1 }, new int[] { 2, 13, 1, 1, 2 }, new int[] { 3, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0106822997331619, 0.0119797298684716, -0.47588628530502319, new int[] { 14, 12, 4, 2, -1 }, new int[] { 15, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14251710474491119, 0.0269787404686213, -0.3589037954807282, true, new int[] { 9, 4, 7, 4, -1 }, new int[] { 9, 4, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6178720872849226E-05, -0.0519438087940216, 0.0596988387405872, new int[] { 17, 9, 1, 2, -1 }, new int[] { 17, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015015379758551717, 0.0426829196512699, -0.24742330610752111, new int[] { 0, 9, 1, 2, -1 }, new int[] { 0, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7750380468205549E-05, -0.0659698769450188, 0.0952353179454803, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.0914406329393387, -0.11401320248842239, new int[] { 0, 0, 2, 1, -1 }, new int[] { 1, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0018318339716643095, -0.0358028709888458, 0.28000190854072571, new int[] { 16, 4, 2, 2, -1 }, new int[] { 17, 4, 1, 1, 2 }, new int[] { 16, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.621649946377147E-05, 0.1192717030644417, -0.0900511220097542, new int[] { 0, 4, 2, 2, -1 }, new int[] { 0, 4, 1, 1, 2 }, new int[] { 1, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0184157993644476, 0.0286770407110453, -0.3459722101688385, new int[] { 7, 3, 4, 6, -1 }, new int[] { 9, 3, 2, 3, 2 }, new int[] { 7, 6, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.564980968600139E-05, 0.10555200278759, -0.0939618200063705, new int[] { 0, 0, 2, 2, -1 }, new int[] { 0, 0, 1, 1, 2 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0442830286920071, -0.39377251267433172, 0.0249951407313347, new int[] { 1, 3, 16, 4, -1 }, new int[] { 9, 3, 8, 2, 2 }, new int[] { 1, 5, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0374921411275864, 0.40750551223754877, -0.0246863309293985, new int[] { 2, 12, 14, 2, -1 }, new int[] { 2, 13, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.4684870368218981E-05, 0.0595886707305908, -0.0425871796905994, new int[] { 12, 0, 2, 2, -1 }, new int[] { 13, 0, 1, 1, 2 }, new int[] { 12, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3879110813140869E-05, 0.1165246963500977, -0.0811222568154335, new int[] { 4, 0, 2, 2, -1 }, new int[] { 4, 0, 1, 1, 2 }, new int[] { 5, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0039012550842016935, -0.25430038571357733, 0.0380770415067673, new int[] { 5, 1, 8, 2, -1 }, new int[] { 5, 2, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026903450489044189, 0.30911579728126531, -0.0310623906552792, new int[] { 4, 7, 2, 2, -1 }, new int[] { 4, 7, 1, 1, 2 }, new int[] { 5, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070722219534218311, -0.2149100005626679, 0.0302512794733047, new int[] { 12, 14, 6, 1, -1 }, new int[] { 14, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002191734965890646, 0.0556822307407856, -0.16676320135593409, new int[] { 7, 0, 4, 2, -1 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.590489930298645E-05, -0.12242270261049271, 0.0827013477683067, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00861231330782175, 0.15256710350513461, -0.070295087993145, new int[] { 1, 1, 16, 6, -1 }, new int[] { 1, 3, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0323125012218952, 0.10563810169696811, -0.0887572914361954, new int[] { 8, 7, 10, 8, -1 }, new int[] { 8, 7, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.24041660130023959, -0.56874710321426392, 0.0155827002599835, new int[] { 0, 7, 11, 8, -1 }, new int[] { 0, 9, 11, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036818000953644514, 0.39008429646492, -0.0244826804846525, new int[] { 11, 8, 2, 2, -1 }, new int[] { 12, 8, 1, 1, 2 }, new int[] { 11, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0375609807670116, -0.59190589189529419, 0.0148836802691221, new int[] { 0, 7, 16, 1, -1 }, new int[] { 4, 7, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.26046231389045721, -0.80789750814437866, 0.0080495169386267662, new int[] { 8, 7, 10, 8, -1 }, new int[] { 8, 7, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22003079950809479, 0.0114593897014856, -0.66569620370864868, new int[] { 0, 7, 10, 8, -1 }, new int[] { 5, 7, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0142070800065994, 0.0114870695397258, -0.43284940719604492, true, new int[] { 12, 0, 3, 2, -1 }, new int[] { 13, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001970876008272171, -0.0313467793166637, 0.28304418921470642, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168589502573013, -0.64982718229293823, 0.0090222535654902458, true, new int[] { 12, 0, 3, 2, -1 }, new int[] { 13, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.118768997490406, 0.0299480501562357, -0.2969210147857666, new int[] { 0, 7, 18, 8, -1 }, new int[] { 0, 7, 9, 4, 2 }, new int[] { 9, 11, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0035489429719746113, 0.0224479902535677, -0.118859700858593, new int[] { 14, 12, 4, 2, -1 }, new int[] { 15, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032591039780527353, 0.0439781881868839, -0.20008519291877749, new int[] { 0, 12, 4, 2, -1 }, new int[] { 1, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0069489958696067333, 0.109799899160862, -0.0513728708028793, true, new int[] { 15, 0, 3, 3, -1 }, new int[] { 14, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011651299893856, -0.039162278175354, 0.2311145961284638, true, new int[] { 3, 0, 3, 3, -1 }, new int[] { 4, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020093740895390511, 0.0655085071921349, -0.0361764915287495, true, new int[] { 14, 2, 3, 3, -1 }, new int[] { 13, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044954619370400906, -0.0742958337068558, 0.1480637043714523, true, new int[] { 4, 2, 3, 3, -1 }, new int[] { 5, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040165609680116177, 0.0192055609077215, -0.13202959299087519, true, new int[] { 15, 1, 3, 1, -1 }, new int[] { 16, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051109711639583111, 0.0305455308407545, -0.32131591439247131, true, new int[] { 3, 1, 1, 3, -1 }, new int[] { 2, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0026829841081053019, 0.0255360994488001, -0.1154488995671272, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0032579500693827868, -0.25272831320762629, 0.039438471198082, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0019859049934893847, 0.2665804922580719, -0.0468473583459854, new int[] { 15, 7, 2, 2, -1 }, new int[] { 16, 7, 1, 1, 2 }, new int[] { 15, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1254094988107681, -0.40570110082626343, 0.0230680201202631, true, new int[] { 6, 4, 6, 5, -1 }, new int[] { 8, 6, 2, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054464139975607395, -0.0338515192270279, 0.10910320281982421, new int[] { 6, 9, 10, 2, -1 }, new int[] { 11, 9, 5, 1, 2 }, new int[] { 6, 10, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0291290692985058, 0.0829424485564232, -0.1039045974612236, new int[] { 4, 7, 5, 8, -1 }, new int[] { 4, 9, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0533427894115448, 0.1423411965370178, -0.0637678280472755, new int[] { 2, 5, 15, 6, -1 }, new int[] { 2, 7, 15, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0698260366916656, -0.29960519075393682, 0.038142379373312, new int[] { 3, 0, 2, 15, -1 }, new int[] { 3, 5, 2, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0010430120164528489, -0.048670019954443, 0.22043199837207789, new int[] { 15, 7, 2, 2, -1 }, new int[] { 16, 7, 1, 1, 2 }, new int[] { 15, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048559759743511677, -0.0910003632307053, 0.0976040363311768, new int[] { 0, 10, 18, 2, -1 }, new int[] { 0, 11, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0056559829972684383, 0.0504679903388023, -0.0828957930207253, new int[] { 9, 8, 2, 4, -1 }, new int[] { 9, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.39691910147666931, -0.59703147411346436, 0.0172442905604839, new int[] { 0, 5, 18, 6, -1 }, new int[] { 0, 8, 18, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0546870790421963, 0.39003100991249079, -0.0251556299626827, new int[] { 3, 11, 12, 4, -1 }, new int[] { 3, 12, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0064253779128193855, -0.25506249070167542, 0.0394066199660301, new int[] { 0, 14, 6, 1, -1 }, new int[] { 2, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00857190415263176, 0.0186648592352867, -0.22203260660171509, new int[] { 12, 14, 6, 1, -1 }, new int[] { 14, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012086849892511964, -0.072148896753788, 0.1184407994151115, new int[] { 8, 0, 2, 1, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0130339497700334, 0.20586760342121119, -0.0158201493322849, new int[] { 5, 0, 12, 1, -1 }, new int[] { 8, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0072425887919962406, -0.0630722567439079, 0.14706359803676611, new int[] { 4, 0, 8, 1, -1 }, new int[] { 6, 0, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0152673702687025, -0.26799020171165472, 0.0069345328956842422, new int[] { 12, 14, 6, 1, -1 }, new int[] { 14, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00598661694675684, 0.0335439704358578, -0.26078468561172491, new int[] { 0, 14, 6, 1, -1 }, new int[] { 2, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0108856903389096, 0.0855251327157021, -0.0212142392992973, new int[] { 6, 9, 10, 2, -1 }, new int[] { 11, 9, 5, 1, 2 }, new int[] { 6, 10, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048979911953210831, -0.0451360605657101, 0.2241200953722, new int[] { 4, 9, 6, 2, -1 }, new int[] { 4, 9, 3, 1, 2 }, new int[] { 7, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1925639063119888, -0.63481587171554565, 0.0042262570932507515, true, new int[] { 16, 3, 2, 9, -1 }, new int[] { 13, 6, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1086068972945213, 0.0170917399227619, -0.54510730504989624, true, new int[] { 2, 3, 9, 2, -1 }, new int[] { 5, 6, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0548367016017437, -0.3548921942710877, 0.0045531531795859337, true, new int[] { 11, 0, 6, 2, -1 }, new int[] { 13, 2, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0058792168274521828, 0.0155201097950339, -0.54079991579055786, new int[] { 7, 0, 3, 2, -1 }, new int[] { 7, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075071100145578384, -0.0158542692661285, 0.0666517317295074, true, new int[] { 11, 0, 2, 3, -1 }, new int[] { 11, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0169021207839251, 0.0222053807228804, -0.37370339035987848, true, new int[] { 7, 0, 3, 2, -1 }, new int[] { 7, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-3.1124811357585713E-05, 0.0337283685803413, -0.0621243193745613, true, new int[] { 9, 2, 2, 1, -1 }, new int[] { 9, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0782682672142982, 0.43044888973236078, -0.0193186104297638, new int[] { 4, 2, 10, 8, -1 }, new int[] { 4, 4, 10, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0221087392419577, 0.0139799099415541, -0.42325040698051453, true, new int[] { 11, 0, 3, 3, -1 }, new int[] { 12, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054141050204634666, 0.0420096218585968, -0.18368819355964661, true, new int[] { 7, 0, 3, 3, -1 }, new int[] { 6, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6600460842018947E-05, -0.0531449504196644, 0.0663439631462097, new int[] { 12, 0, 2, 2, -1 }, new int[] { 13, 0, 1, 1, 2 }, new int[] { 12, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0851690322160721, 0.1034568026661873, new int[] { 4, 0, 2, 2, -1 }, new int[] { 4, 0, 1, 1, 2 }, new int[] { 5, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0096517298370599747, -0.0677581280469894, 0.1238183006644249, new int[] { 0, 12, 18, 3, -1 }, new int[] { 0, 13, 18, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.1085200011730194, 0.082693062722683, new int[] { 4, 0, 2, 1, -1 }, new int[] { 5, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025218860246241093, -0.1045825034379959, 0.0663281828165054, new int[] { 11, 1, 4, 2, -1 }, new int[] { 11, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.052996139973402, 0.23921950161457059, -0.041141789406538, new int[] { 0, 0, 15, 2, -1 }, new int[] { 5, 0, 5, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029717630241066217, 0.0353552810847759, -0.15361009538173681, true, new int[] { 12, 0, 3, 1, -1 }, new int[] { 13, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0050528207793831825, -0.28384080529212952, 0.0291973706334829, true, new int[] { 6, 0, 1, 3, -1 }, new int[] { 5, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.001402365043759346, 0.19387529790401459, -0.0234654601663351, new int[] { 11, 3, 2, 1, -1 }, new int[] { 11, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6361160053056665E-05, -0.1317539066076279, 0.0617644004523754, new int[] { 5, 3, 2, 1, -1 }, new int[] { 6, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0057318392209708691, -0.0376738198101521, 0.14864000678062439, true, new int[] { 16, 0, 2, 4, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046025160700082779, -0.0600823499262333, 0.14757469296455381, true, new int[] { 2, 0, 4, 2, -1 }, new int[] { 3, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049826940521597862, 0.0502174682915211, -0.1770825982093811, new int[] { 0, 0, 18, 2, -1 }, new int[] { 9, 0, 9, 1, 2 }, new int[] { 0, 1, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.073296032845974, -0.49743050336837769, 0.0167066808789968, new int[] { 0, 4, 18, 4, -1 }, new int[] { 0, 4, 9, 2, 2 }, new int[] { 9, 6, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142388697713614, 0.521755576133728, -0.0113009298220277, new int[] { 10, 7, 4, 2, -1 }, new int[] { 12, 7, 2, 1, 2 }, new int[] { 10, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0181554593145847, -0.0388248786330223, 0.2092700004577637, true, new int[] { 4, 3, 3, 4, -1 }, new int[] { 5, 4, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.577970917627681E-05, 0.0649056732654572, -0.0738614425063133, new int[] { 15, 7, 2, 2, -1 }, new int[] { 16, 7, 1, 1, 2 }, new int[] { 15, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.935916927526705E-05, -0.0757590234279633, 0.1107048019766808, new int[] { 1, 7, 2, 2, -1 }, new int[] { 1, 7, 1, 1, 2 }, new int[] { 2, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.590489930298645E-05, -0.0566908791661263, 0.0705650299787521, new int[] { 10, 7, 4, 2, -1 }, new int[] { 12, 7, 2, 1, 2 }, new int[] { 10, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025659629609435797, -0.0226817093789577, 0.32642030715942377, new int[] { 6, 8, 2, 2, -1 }, new int[] { 6, 8, 1, 1, 2 }, new int[] { 7, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0431340709328651, 0.0913139432668686, -0.0776849165558815, new int[] { 8, 3, 2, 8, -1 }, new int[] { 8, 7, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1150510013103485, -0.0538835301995277, 0.17382779717445371, new int[] { 1, 4, 16, 9, -1 }, new int[] { 1, 7, 16, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0376834310591221, 0.0119111798703671, -0.1632004976272583, new int[] { 15, 6, 3, 6, -1 }, new int[] { 15, 8, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0287051200866699, 0.0230644904077053, -0.34346461296081537, new int[] { 0, 6, 3, 6, -1 }, new int[] { 0, 8, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0741745382547379, -0.0364534594118595, 0.22265499830245969, new int[] { 6, 0, 6, 11, -1 }, new int[] { 6, 0, 3, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0387266613543034, -0.0861116796731949, 0.0941641926765442, new int[] { 6, 0, 4, 10, -1 }, new int[] { 8, 0, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0041428101249039173, -0.1222383007407188, 0.0341765694320202, true, new int[] { 13, 0, 4, 4, -1 }, new int[] { 14, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0246735997498035, 0.0565831884741783, -0.1488883048295975, true, new int[] { 9, 5, 6, 2, -1 }, new int[] { 9, 5, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00998087041079998, -0.0197595097124577, 0.30300268530845642, new int[] { 8, 10, 6, 2, -1 }, new int[] { 11, 10, 3, 1, 2 }, new int[] { 8, 11, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.6217122366651893E-05, 0.0897242724895477, -0.0896338075399399, new int[] { 1, 7, 2, 2, -1 }, new int[] { 1, 7, 1, 1, 2 }, new int[] { 2, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0019440250471234322, 0.0459239892661572, -0.160874605178833, new int[] { 10, 0, 3, 3, -1 }, new int[] { 10, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00992183480411768, -0.33827519416809082, 0.023345960304141, new int[] { 4, 0, 10, 3, -1 }, new int[] { 4, 1, 10, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7032099751522765E-05, -0.0716137290000916, 0.14374250173568731, new int[] { 15, 7, 1, 2, -1 }, new int[] { 15, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0115753803402185, 0.0729895383119583, -0.1120665967464447, new int[] { 5, 7, 8, 2, -1 }, new int[] { 5, 8, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.38227710127830511, 0.00043869050568901, -0.96936649084091187, new int[] { 11, 5, 6, 9, -1 }, new int[] { 13, 8, 2, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.025604510679841, -0.0532096885144711, 0.1605699956417084, new int[] { 1, 5, 6, 9, -1 }, new int[] { 3, 8, 2, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0652327984571457, -0.0050901030190289021, 0.10526590049266819, true, new int[] { 11, 6, 3, 6, -1 }, new int[] { 9, 8, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0765335634350777, -0.27622249722480768, 0.02983707934618, true, new int[] { 7, 6, 6, 3, -1 }, new int[] { 9, 8, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-3.0668321414850652E-05, 0.0497616194188595, -0.0646989569067955, new int[] { 10, 10, 1, 3, -1 }, new int[] { 10, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0071437079459428787, 0.427419513463974, -0.0177215505391359, new int[] { 7, 10, 1, 3, -1 }, new int[] { 7, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0706991031765938, -0.3164018988609314, 0.0242118407040834, new int[] { 0, 11, 18, 4, -1 }, new int[] { 9, 11, 9, 2, 2 }, new int[] { 0, 13, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0839718133211136, 0.0076198792085051537, -0.80655181407928467, new int[] { 4, 11, 6, 4, -1 }, new int[] { 7, 11, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.49757468700408941, 0.00623872596770525, -0.83056390285491943, new int[] { 0, 2, 18, 12, -1 }, new int[] { 0, 5, 18, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054929931648075581, 0.0266029108315706, -0.22599579393863681, new int[] { 0, 10, 1, 4, -1 }, new int[] { 0, 12, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0275369994342327, 0.1843355000019074, -0.0070537109859287739, new int[] { 12, 6, 3, 3, -1 }, new int[] { 13, 7, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045211901888251305, -0.0542923994362354, 0.1254532933235169, new int[] { 3, 6, 3, 3, -1 }, new int[] { 4, 7, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0386416800320148, 0.0084282690659165382, -0.2196173965930939, true, new int[] { 13, 0, 4, 4, -1 }, new int[] { 14, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0216541700065136, -0.28082931041717529, 0.024411179125309, true, new int[] { 5, 0, 4, 4, -1 }, new int[] { 4, 1, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.029021130874753, -0.31314170360565191, 0.022386759519577, new int[] { 5, 6, 8, 4, -1 }, new int[] { 9, 6, 4, 2, 2 }, new int[] { 5, 8, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044424049556255341, 0.64938491582870483, -0.0114663699641824, new int[] { 3, 11, 2, 2, -1 }, new int[] { 3, 11, 1, 1, 2 }, new int[] { 4, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0140129495412111, -0.0560599118471146, 0.1226307973265648, new int[] { 1, 10, 16, 2, -1 }, new int[] { 1, 11, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00757738808169961, -0.0738088190555573, 0.0975568890571594, new int[] { 1, 13, 15, 2, -1 }, new int[] { 1, 14, 15, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026077621150761843, -0.0911063700914383, 0.0298527106642723, true, new int[] { 16, 12, 1, 2, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.0737720802426338, 0.0916053429245949, new int[] { 0, 8, 4, 2, -1 }, new int[] { 0, 8, 2, 1, 2 }, new int[] { 2, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0690594092011452, 0.13202320039272311, new int[] { 13, 5, 1, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0574019812047482, 0.14494429528713229, -0.0600692182779312, new int[] { 4, 4, 10, 4, -1 }, new int[] { 4, 6, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0093912649899721146, 0.50085651874542236, -0.00417069299146533, new int[] { 13, 5, 1, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6128649551537819E-05, -0.0762275531888008, 0.1260772049427033, new int[] { 4, 5, 1, 2, -1 }, new int[] { 4, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0503179281949997, 0.0103605901822448, -0.31897589564323431, true, new int[] { 13, 2, 3, 7, -1 }, new int[] { 14, 3, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051848609000444412, -0.0647242292761803, 0.1234103962779045, true, new int[] { 5, 2, 7, 3, -1 }, new int[] { 4, 3, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00339106610044837, -0.1028840020298958, 0.0440409816801548, new int[] { 13, 5, 2, 7, -1 }, new int[] { 13, 5, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0060285101644694805, 0.0370522104203701, -0.212730199098587, new int[] { 3, 5, 2, 7, -1 }, new int[] { 4, 5, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0247735399752855, 0.30380809307098389, -0.01416543032974, new int[] { 9, 2, 6, 2, -1 }, new int[] { 9, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.016291119158268, -0.0679637491703033, 0.10207100212574011, new int[] { 3, 2, 6, 2, -1 }, new int[] { 6, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0864686071872711, 0.0040547042153775692, -0.47402960062026978, new int[] { 13, 3, 5, 6, -1 }, new int[] { 13, 6, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036333149764686823, -0.0353813916444778, 0.201679602265358, new int[] { 5, 10, 4, 2, -1 }, new int[] { 5, 10, 2, 1, 2 }, new int[] { 7, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0018694689497351646, 0.0223653502762318, -0.0570879615843296, new int[] { 11, 11, 4, 2, -1 }, new int[] { 12, 11, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0037068868987262249, -0.16035629808902741, 0.0456907190382481, new int[] { 3, 11, 4, 2, -1 }, new int[] { 4, 11, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-3.0651168344775215E-05, 0.0354789905250072, -0.0344920493662357, true, new int[] { 16, 12, 1, 2, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040897028520703316, -0.26812940835952759, 0.0277175307273865, true, new int[] { 2, 12, 2, 1, -1 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0090142004191875458, 0.12767490744590759, -0.0258717201650143, true, new int[] { 16, 3, 2, 3, -1 }, new int[] { 15, 4, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0101045602932572, 0.041761290282011, -0.163332000374794, new int[] { 0, 3, 5, 6, -1 }, new int[] { 0, 6, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0232086200267076, -0.0154512897133827, 0.26844790577888489, true, new int[] { 16, 3, 2, 3, -1 }, new int[] { 15, 4, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1134508028626442, -0.0744702816009521, 0.1102133989334106, new int[] { 1, 3, 16, 9, -1 }, new int[] { 1, 6, 16, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011667109793052077, -0.0686589777469635, 0.0979631170630455, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.1848782934248447E-05, -0.0807370617985725, 0.0817197933793068, new int[] { 1, 11, 2, 2, -1 }, new int[] { 1, 11, 1, 1, 2 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7750380468205549E-05, -0.0818600133061409, 0.0863137766718864, new int[] { 15, 13, 2, 2, -1 }, new int[] { 16, 13, 1, 1, 2 }, new int[] { 15, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7259990019956604E-05, -0.0809563770890236, 0.0821038633584976, new int[] { 1, 13, 2, 2, -1 }, new int[] { 1, 13, 1, 1, 2 }, new int[] { 2, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.935916927526705E-05, 0.1045090034604073, -0.0726457983255386, new int[] { 15, 13, 2, 2, -1 }, new int[] { 16, 13, 1, 1, 2 }, new int[] { 15, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.564980968600139E-05, 0.10629419982433321, -0.0679890736937523, new int[] { 1, 13, 2, 2, -1 }, new int[] { 1, 13, 1, 1, 2 }, new int[] { 2, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0163933802396059, -0.17156420648097989, 0.0276966094970703, true, new int[] { 11, 6, 2, 4, -1 }, new int[] { 10, 7, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0233597904443741, 0.388507604598999, -0.0166453197598457, true, new int[] { 2, 3, 3, 2, -1 }, new int[] { 3, 4, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022364470642060041, -0.0172002408653498, 0.21048629283905029, new int[] { 14, 3, 2, 2, -1 }, new int[] { 15, 3, 1, 1, 2 }, new int[] { 14, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0127381896600127, -0.25325098633766169, 0.0284554697573185, new int[] { 6, 2, 6, 4, -1 }, new int[] { 6, 2, 3, 2, 2 }, new int[] { 9, 4, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0130351698026061, -0.0366394892334938, 0.0509026385843754, new int[] { 8, 1, 6, 3, -1 }, new int[] { 10, 2, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.833299913618248E-05, -0.0837918072938919, 0.0838518589735031, new int[] { 7, 3, 1, 2, -1 }, new int[] { 7, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0123362001031637, -0.0514171607792377, 0.0532306805253029, new int[] { 12, 1, 6, 4, -1 }, new int[] { 12, 1, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0327928103506565, 0.23273399472236631, -0.0373882502317429, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 12, 3, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020052760373800993, 0.027813620865345, -0.29500991106033331, new int[] { 8, 7, 2, 1, -1 }, new int[] { 8, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0139068197458982, -0.0543732605874538, 0.125259205698967, new int[] { 0, 1, 6, 4, -1 }, new int[] { 3, 1, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2173788994550705, 0.0416372790932655, -0.1780032962560654, new int[] { 1, 1, 16, 7, -1 }, new int[] { 5, 1, 8, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.67987501621246338, -0.0189819093793631, 0.3512358963489533, new int[] { 3, 3, 12, 9, -1 }, new int[] { 7, 6, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0497565008699894, -0.8002396821975708, 0.0097657497972249985, new int[] { 6, 8, 7, 2, -1 }, new int[] { 6, 9, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.004579687025398016, 0.0210781805217266, -0.28444689512252808, new int[] { 4, 0, 3, 3, -1 }, new int[] { 4, 1, 3, 1, 3 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 15
            stage = new HaarCascadeStage(-1.2427099943161011, 14, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.1051426008343697, -0.1030462011694908, 0.52641832828521729, true, new int[] { 7, 1, 8, 2, -1 }, new int[] { 9, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0218748692423105, -0.11491969972848889, 0.0879510119557381, new int[] { 6, 5, 8, 5, -1 }, new int[] { 8, 5, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25913900136947632, -1.8469070710125379E-05, -789.60552978515625, new int[] { 6, 0, 4, 11, -1 }, new int[] { 8, 0, 2, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0082329362630844116, 0.12152519822120671, -0.21997219324111941, new int[] { 12, 8, 6, 5, -1 }, new int[] { 12, 8, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00745374895632267, 0.1169904991984367, -0.19874709844589231, new int[] { 0, 1, 9, 2, -1 }, new int[] { 3, 1, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0507839918136597, 0.0343447588384151, -0.19979280233383179, new int[] { 12, 8, 6, 5, -1 }, new int[] { 12, 8, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053065801039338112, 0.1021941006183624, -0.23248760402202609, new int[] { 0, 8, 6, 5, -1 }, new int[] { 3, 8, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0198521409183741, -0.57735747098922729, 0.0107486303895712, true, new int[] { 10, 2, 2, 2, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0251020099967718, 0.0335165187716484, -0.51891112327575684, true, new int[] { 9, 1, 1, 4, -1 }, new int[] { 9, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039596240967512131, -0.1546567976474762, 0.10011819750070571, new int[] { 1, 10, 16, 2, -1 }, new int[] { 1, 11, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0059100659564137459, -0.33589190244674683, 0.0603443384170532, new int[] { 3, 2, 12, 2, -1 }, new int[] { 3, 2, 6, 1, 2 }, new int[] { 9, 3, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0090328548103570938, -0.0104679698124528, -0.35610088706016541, true, new int[] { 15, 0, 3, 2, -1 }, new int[] { 16, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0085141025483608246, 0.0334267392754555, -0.41499960422515869, true, new int[] { 3, 0, 2, 3, -1 }, new int[] { 2, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145813003182411, -0.1194749996066093, 0.1058669984340668, new int[] { 3, 0, 12, 1, -1 }, new int[] { 7, 0, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.115242101252079, -0.0234193205833435, 0.39515259861946112, true, new int[] { 9, 2, 9, 2, -1 }, new int[] { 12, 5, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021557710133492947, 0.113696001470089, -0.1149196028709412, new int[] { 16, 0, 2, 6, -1 }, new int[] { 16, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13152989745140081, -0.40761440992355352, 0.0280955005437136, new int[] { 0, 6, 18, 6, -1 }, new int[] { 0, 6, 9, 3, 2 }, new int[] { 9, 9, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0877189636230469, 0.0119158001616597, -0.62395787239074707, new int[] { 0, 1, 18, 6, -1 }, new int[] { 9, 1, 9, 3, 2 }, new int[] { 0, 4, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051810648292303085, -0.1093714982271195, 0.1119602024555206, new int[] { 6, 0, 6, 1, -1 }, new int[] { 9, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035339239984750748, 0.1208496019244194, -0.005425203125923872, true, new int[] { 9, 5, 1, 2, -1 }, new int[] { 9, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011804329697042704, -0.12307350337505341, 0.12815740704536441, true, new int[] { 9, 5, 2, 1, -1 }, new int[] { 9, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0056288531050086021, 0.0316065102815628, -0.281035989522934, new int[] { 7, 1, 5, 2, -1 }, new int[] { 7, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00099457567557692528, -0.0659783333539963, 0.14891250431537631, new int[] { 5, 8, 1, 3, -1 }, new int[] { 5, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003733726916834712, 0.0598995685577393, -0.1800362020730972, new int[] { 7, 7, 5, 2, -1 }, new int[] { 7, 8, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00010250649938825518, -0.0862240791320801, 0.13904710114002231, true, new int[] { 8, 6, 3, 3, -1 }, new int[] { 7, 7, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041721882298588753, -0.0246597994118929, 0.0794360563158989, new int[] { 11, 9, 3, 2, -1 }, new int[] { 11, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0485266894102097, 0.0381521992385387, -0.33759069442749018, new int[] { 0, 8, 18, 4, -1 }, new int[] { 0, 8, 9, 2, 2 }, new int[] { 9, 10, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00841431599110365, 0.0051525980234146118, -0.1651131063699722, new int[] { 16, 8, 2, 3, -1 }, new int[] { 16, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003570288885384798, -0.23562599718570709, 0.0417603217065334, new int[] { 0, 8, 2, 3, -1 }, new int[] { 0, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0222564004361629, -0.0281212199479342, 0.13493560254573819, new int[] { 11, 8, 4, 6, -1 }, new int[] { 11, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038191271014511585, -0.11853609979152679, 0.0843502730131149, new int[] { 0, 11, 18, 2, -1 }, new int[] { 0, 12, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14533999562263489, -0.0286314208060503, 0.35683318972587591, new int[] { 2, 5, 14, 8, -1 }, new int[] { 2, 7, 14, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00089769659098237753, 0.0549010299146175, -0.17856329679489141, new int[] { 8, 8, 2, 2, -1 }, new int[] { 8, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0416826009750366, -0.0183632392436266, 0.161685898900032, new int[] { 11, 8, 4, 6, -1 }, new int[] { 11, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0501397587358952, -0.0449284687638283, 0.21465340256690979, new int[] { 6, 0, 6, 3, -1 }, new int[] { 9, 0, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003092906903475523, 0.030171500518918, -0.3513563871383667, new int[] { 7, 1, 4, 2, -1 }, new int[] { 9, 1, 2, 1, 2 }, new int[] { 7, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0181560907512903, -0.0552617982029915, 0.1947118937969208, new int[] { 3, 8, 4, 6, -1 }, new int[] { 3, 10, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0202469304203987, 0.0373657196760178, -0.300785094499588, new int[] { 6, 6, 6, 4, -1 }, new int[] { 9, 6, 3, 2, 2 }, new int[] { 6, 8, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0117160901427269, -0.0614580996334553, 0.1639769971370697, new int[] { 1, 7, 6, 3, -1 }, new int[] { 3, 8, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081182513386011124, -0.0887261107563972, 0.0327240005135536, true, new int[] { 10, 7, 2, 3, -1 }, new int[] { 9, 8, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14681640267372131, -0.49301609396934509, 0.0201582796871662, new int[] { 0, 8, 18, 6, -1 }, new int[] { 0, 8, 9, 3, 2 }, new int[] { 9, 11, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052891620434820652, -0.25142368674278259, 0.009538767859339714, true, new int[] { 9, 1, 3, 1, -1 }, new int[] { 10, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0148622198030353, 0.25943711400032038, -0.0313785411417484, new int[] { 5, 0, 8, 5, -1 }, new int[] { 7, 0, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0177154596894979, -0.51138341426849365, 0.0075401309877634048, true, new int[] { 9, 2, 3, 2, -1 }, new int[] { 10, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00095196522306650877, 0.0692363083362579, -0.12581700086593631, true, new int[] { 7, 2, 2, 1, -1 }, new int[] { 7, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.066216342151165, -0.0098208645358681679, 0.36082351207733149, true, new int[] { 12, 1, 6, 3, -1 }, new int[] { 11, 2, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082799885421991348, -0.0748182237148285, 0.15120029449462891, true, new int[] { 6, 1, 3, 6, -1 }, new int[] { 7, 2, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0126259000971913, 0.0625171065330505, -0.1584693044424057, new int[] { 1, 3, 16, 4, -1 }, new int[] { 9, 3, 8, 2, 2 }, new int[] { 1, 5, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0506105907261372, 0.43044748902320862, -0.0195215903222561, true, new int[] { 9, 4, 4, 4, -1 }, new int[] { 8, 5, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.64415240287780762, 0.0196064803749323, -0.371227890253067, new int[] { 3, 0, 15, 14, -1 }, new int[] { 8, 0, 5, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0629194527864456, -0.1244589984416962, 0.0681276023387909, new int[] { 3, 1, 12, 10, -1 }, new int[] { 6, 1, 6, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0158867593854666, 0.0037582379300147295, -0.2513279914855957, true, new int[] { 15, 11, 3, 1, -1 }, new int[] { 16, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002367671113461256, -0.18140539526939389, 0.0453032106161118, true, new int[] { 3, 11, 1, 3, -1 }, new int[] { 2, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0252422392368317, 0.0168007891625166, -0.31515631079673773, new int[] { 15, 0, 1, 14, -1 }, new int[] { 15, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0137373497709632, -0.0329083986580372, 0.23093259334564209, new int[] { 8, 3, 2, 6, -1 }, new int[] { 8, 6, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021248359698802233, 0.0645555630326271, -0.14124630391597751, new int[] { 7, 6, 4, 2, -1 }, new int[] { 7, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0060910829342901707, -0.46051791310310358, 0.0166283007711172, new int[] { 8, 0, 2, 4, -1 }, new int[] { 8, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040456880815327168, 0.0083615174517035484, -0.26965349912643433, new int[] { 12, 3, 1, 3, -1 }, new int[] { 12, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.034469161182642, 0.21582040190696719, -0.0349247604608536, new int[] { 4, 0, 9, 9, -1 }, new int[] { 7, 0, 3, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.9153727458324283E-05, -0.051043938845396, 0.0346905216574669, true, new int[] { 9, 1, 3, 1, -1 }, new int[] { 10, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066213719546794891, -0.41585859656333918, 0.0193911194801331, true, new int[] { 9, 1, 1, 3, -1 }, new int[] { 8, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1363825052976608, -0.0445473901927471, 0.17608410120010379, new int[] { 6, 7, 12, 8, -1 }, new int[] { 6, 7, 6, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025193500332534313, -0.0905184969305992, 0.0875409692525864, new int[] { 7, 1, 2, 3, -1 }, new int[] { 8, 1, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0783995389938354, 0.264887809753418, -0.03243463113904, new int[] { 2, 4, 14, 6, -1 }, new int[] { 2, 6, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007100231945514679, -0.11403769999742509, 0.104027196764946, new int[] { 4, 4, 3, 6, -1 }, new int[] { 4, 6, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0626892074942589, -0.0568519681692123, 0.014763230457902, new int[] { 12, 1, 5, 8, -1 }, new int[] { 12, 5, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0698204934597015, 0.0167288593947887, -0.50399231910705566, new int[] { 1, 1, 5, 8, -1 }, new int[] { 1, 5, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0102383298799396, -0.0286362692713737, 0.18522030115127561, true, new int[] { 15, 0, 3, 3, -1 }, new int[] { 14, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0149942804127932, 0.22429679334163671, -0.0332668386399746, true, new int[] { 3, 0, 3, 3, -1 }, new int[] { 4, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052933390252292156, 0.0299122091382742, -0.217377707362175, new int[] { 6, 0, 10, 2, -1 }, new int[] { 11, 0, 5, 1, 2 }, new int[] { 6, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00800849124789238, 0.0341741293668747, -0.26237648725509638, new int[] { 1, 0, 16, 2, -1 }, new int[] { 1, 0, 8, 1, 2 }, new int[] { 9, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1146114021539688, -0.0244884397834539, 0.0970916673541069, new int[] { 6, 3, 12, 6, -1 }, new int[] { 9, 3, 6, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0521271787583828, -0.64139938354492188, 0.0115570602938533, new int[] { 6, 6, 6, 3, -1 }, new int[] { 8, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0748131424188614, -0.0502658300101757, 0.0502240210771561, new int[] { 6, 1, 12, 10, -1 }, new int[] { 6, 1, 6, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0191232096403837, -0.31091290712356567, 0.0227278098464012, new int[] { 2, 13, 6, 2, -1 }, new int[] { 4, 13, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.054096881300211, -0.0090643512085080147, 0.25074291229248052, true, new int[] { 12, 0, 6, 3, -1 }, new int[] { 11, 1, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0256583709269762, 0.2121652960777283, -0.0351778715848923, true, new int[] { 6, 0, 3, 6, -1 }, new int[] { 7, 1, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15096050500869751, 0.0186689905822277, -0.2159824073314667, new int[] { 8, 4, 10, 4, -1 }, new int[] { 8, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.11122240126132971, 0.0342452004551888, -0.21573379635810849, new int[] { 0, 4, 10, 4, -1 }, new int[] { 5, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.0547110479092225E-05, -0.0372137017548084, 0.0372152701020241, new int[] { 16, 9, 2, 4, -1 }, new int[] { 16, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0058619431219995022, -0.0773961320519447, 0.0930630415678024, new int[] { 1, 13, 16, 2, -1 }, new int[] { 1, 14, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0341941900551319, 0.34479939937591553, -0.0335593782365322, new int[] { 2, 13, 14, 2, -1 }, new int[] { 2, 14, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0072817560285329819, -0.29600289463996893, 0.0260884091258049, new int[] { 0, 9, 2, 4, -1 }, new int[] { 0, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0109525797888637, -0.058721199631691, 0.1384337991476059, new int[] { 2, 7, 15, 3, -1 }, new int[] { 2, 8, 15, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0810781270265579, -0.072938360273838, 0.0964554026722908, new int[] { 3, 1, 12, 8, -1 }, new int[] { 3, 3, 12, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1066536009311676, -0.0128484796732664, 0.1897089034318924, true, new int[] { 11, 4, 3, 6, -1 }, new int[] { 9, 6, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0685272365808487, -0.32469791173934942, 0.0234368797391653, true, new int[] { 7, 4, 6, 3, -1 }, new int[] { 9, 6, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0367356203496456, -0.0583354011178017, 0.0843546465039253, new int[] { 8, 0, 8, 13, -1 }, new int[] { 10, 0, 4, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0846856981515884, -0.0645033568143845, 0.16065360605716711, new int[] { 1, 0, 12, 7, -1 }, new int[] { 5, 0, 4, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036365711130201817, -0.0495950989425182, 0.17173859477043149, new int[] { 10, 9, 6, 2, -1 }, new int[] { 13, 9, 3, 1, 2 }, new int[] { 10, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0048055797815322876, -0.27324178814888, 0.0275324694812298, true, new int[] { 4, 0, 1, 3, -1 }, new int[] { 3, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0096100764349102974, -0.23277230560779569, 0.0202909894287586, true, new int[] { 14, 0, 3, 2, -1 }, new int[] { 15, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0781866833567619, 0.0119251701980829, -0.56188392639160156, true, new int[] { 6, 1, 10, 4, -1 }, new int[] { 5, 2, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0749451220035553, 0.0022771470248699188, -0.67497521638870239, true, new int[] { 15, 2, 3, 8, -1 }, new int[] { 16, 3, 1, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0366185903549194, 0.19563549757003779, -0.044303759932518, true, new int[] { 3, 2, 8, 3, -1 }, new int[] { 2, 3, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025921240448951721, 0.0411940589547157, -0.1164683029055595, true, new int[] { 14, 0, 3, 2, -1 }, new int[] { 15, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00673763919621706, 0.0310751292854548, -0.25548139214515692, true, new int[] { 4, 0, 2, 3, -1 }, new int[] { 3, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028166980482637882, -0.0413872785866261, 0.20167019963264471, new int[] { 15, 5, 2, 4, -1 }, new int[] { 16, 5, 1, 2, 2 }, new int[] { 15, 7, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0658822432160378, 0.0130075104534626, -0.55459141731262207, new int[] { 3, 0, 2, 14, -1 }, new int[] { 3, 7, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015577779849991202, -0.023746419698, 0.0413672998547554, new int[] { 16, 6, 2, 3, -1 }, new int[] { 16, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0054769590497016907, -0.26814338564872742, 0.0244701895862818, new int[] { 0, 6, 2, 3, -1 }, new int[] { 0, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0055535528808832169, 0.20323030650615689, -0.0357219502329826, new int[] { 15, 5, 2, 4, -1 }, new int[] { 16, 5, 1, 2, 2 }, new int[] { 15, 7, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.06698889285326, -0.51838552951812744, 0.0108443703502417, new int[] { 1, 0, 8, 6, -1 }, new int[] { 1, 3, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0414705388247967, 0.0027333609759807587, -0.35633009672164923, new int[] { 16, 0, 2, 6, -1 }, new int[] { 16, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0034693330526351929, 0.0982717424631119, -0.0729679390788078, new int[] { 0, 0, 2, 6, -1 }, new int[] { 0, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00821965653449297, 0.1082827970385552, -0.047256238758564, true, new int[] { 14, 0, 4, 3, -1 }, new int[] { 13, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00998765416443348, -0.0470379404723644, 0.1751355975866318, true, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.28357180953025818, 0.1180493980646133, -0.0566624216735363, new int[] { 3, 0, 12, 15, -1 }, new int[] { 3, 0, 6, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0311159901320934, 0.38079530000686651, -0.0197968706488609, new int[] { 6, 1, 4, 7, -1 }, new int[] { 8, 1, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0109928799793124, 0.0220177192240953, -0.0803828462958336, true, new int[] { 9, 0, 3, 4, -1 }, new int[] { 10, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165618509054184, -0.439990907907486, 0.0151666197925806, true, new int[] { 9, 0, 4, 3, -1 }, new int[] { 8, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001848872983828187, -0.0196843091398478, 0.16026680171489721, new int[] { 16, 3, 2, 2, -1 }, new int[] { 17, 3, 1, 1, 2 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.87090796418488E-05, 0.0893735587596893, -0.0703077465295792, new int[] { 1, 2, 2, 2, -1 }, new int[] { 1, 2, 1, 1, 2 }, new int[] { 2, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.34405407961458E-05, 0.1077063977718353, -0.0792713835835457, new int[] { 15, 2, 2, 2, -1 }, new int[] { 16, 2, 1, 1, 2 }, new int[] { 15, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1137150876456872E-05, -0.0742689892649651, 0.0928685069084167, new int[] { 1, 2, 2, 2, -1 }, new int[] { 1, 2, 1, 1, 2 }, new int[] { 2, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109409997239709, -0.60954278707504272, 0.0071117929182946682, true, new int[] { 10, 3, 3, 1, -1 }, new int[] { 11, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1670096963644028, 0.0173986200243235, -0.34830319881439209, true, new int[] { 5, 0, 9, 4, -1 }, new int[] { 5, 0, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.053627010434866, -0.25175419449806208, 0.0030668680556118488, true, new int[] { 10, 2, 3, 7, -1 }, new int[] { 11, 3, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168547891080379, -0.23226660490036011, 0.0295417997986078, true, new int[] { 8, 2, 7, 3, -1 }, new int[] { 7, 3, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00066016108030453324, 0.0844743698835373, -0.0292119607329369, new int[] { 16, 3, 2, 2, -1 }, new int[] { 17, 3, 1, 1, 2 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.8979410823667422E-05, -0.0716504007577896, 0.089446447789669, new int[] { 6, 6, 2, 2, -1 }, new int[] { 6, 6, 1, 1, 2 }, new int[] { 7, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0290991999208927, 0.15133389830589289, -0.0443021915853024, new int[] { 7, 4, 4, 4, -1 }, new int[] { 7, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0603702887892723, 0.0239160899072886, -0.28696390986442571, new int[] { 0, 1, 10, 6, -1 }, new int[] { 0, 3, 10, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.2198538469383493E-05, -0.0552247799932957, 0.0630851984024048, new int[] { 16, 3, 2, 2, -1 }, new int[] { 17, 3, 1, 1, 2 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3573388868244365E-05, 0.0917791575193405, -0.0733837336301804, new int[] { 0, 3, 2, 2, -1 }, new int[] { 0, 3, 1, 1, 2 }, new int[] { 1, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0921942219138145, 0.0845907479524612, -0.0435498803853989, new int[] { 6, 7, 12, 8, -1 }, new int[] { 6, 7, 6, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068016350269317627, -0.0395293086767197, 0.17724280059337619, true, new int[] { 7, 6, 3, 3, -1 }, new int[] { 6, 7, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0136591903865337, -0.031453400850296, 0.0921841263771057, true, new int[] { 13, 8, 2, 5, -1 }, new int[] { 13, 8, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0202402602881193, 0.12939970195293429, -0.0722166895866394, true, new int[] { 7, 7, 4, 4, -1 }, new int[] { 7, 7, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.33109420537948608, -0.56844151020050049, 0.0048965080641210079, new int[] { 6, 7, 12, 8, -1 }, new int[] { 6, 7, 6, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.3559010922908783, -0.60889261960983276, 0.0121664199978113, new int[] { 0, 2, 12, 13, -1 }, new int[] { 6, 2, 6, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.32671329379081732, 0.0114083802327514, -0.54270422458648682, new int[] { 0, 8, 18, 6, -1 }, new int[] { 0, 11, 18, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0637968480587006, -0.80737471580505371, 0.0073937238194048405, new int[] { 2, 2, 4, 13, -1 }, new int[] { 3, 2, 2, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00416563218459487, 0.0186478793621063, -0.0633438527584076, true, new int[] { 10, 3, 3, 1, -1 }, new int[] { 11, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.62817978858947754, -0.0229623205959797, 0.28442010283470148, new int[] { 3, 2, 12, 9, -1 }, new int[] { 7, 5, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.704376962967217E-05, -0.0583966001868248, 0.027118930593133, true, new int[] { 10, 3, 3, 1, -1 }, new int[] { 11, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0082484260201454163, -0.36744558811187739, 0.01796386949718, true, new int[] { 8, 3, 1, 3, -1 }, new int[] { 7, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.21319560706615451, 0.0048015988431870937, -0.25128981471061712, new int[] { 9, 2, 8, 6, -1 }, new int[] { 9, 2, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0926481783390045, 0.40808829665184021, -0.0169616807252169, new int[] { 1, 2, 8, 6, -1 }, new int[] { 5, 2, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.7387576564215124E-05, -0.1143013015389442, 0.0627095922827721, true, new int[] { 12, 0, 2, 1, -1 }, new int[] { 12, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.005226403009146452, -0.3810344934463501, 0.0188566204160452, true, new int[] { 6, 0, 1, 2, -1 }, new int[] { 6, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0045156818814575672, -0.32349079847335821, 0.0157586503773928, new int[] { 10, 0, 4, 2, -1 }, new int[] { 10, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031322699505835772, 0.0371164008975029, -0.1631309986114502, new int[] { 4, 0, 8, 2, -1 }, new int[] { 4, 0, 4, 1, 2 }, new int[] { 8, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0309491790831089, -0.224877804517746, 0.0159355606883764, new int[] { 7, 12, 8, 3, -1 }, new int[] { 9, 12, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0119997104629874, 0.1060421019792557, -0.0560035184025764, new int[] { 1, 13, 16, 1, -1 }, new int[] { 5, 13, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0336425602436066, 0.009433218277990818, -0.24610279500484469, new int[] { 7, 13, 10, 1, -1 }, new int[] { 7, 13, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0119730802252889, -0.0456926003098488, 0.15212790668010709, new int[] { 1, 13, 10, 1, -1 }, new int[] { 6, 13, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14105269312858579, -0.4025206863880158, 0.0161248706281185, new int[] { 0, 13, 18, 2, -1 }, new int[] { 0, 13, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058696339838206768, 0.1223559975624085, -0.0487510599195957, true, new int[] { 4, 6, 3, 2, -1 }, new int[] { 5, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002155571011826396, -0.0184163097292185, 0.1451521962881088, new int[] { 11, 9, 2, 2, -1 }, new int[] { 12, 9, 1, 1, 2 }, new int[] { 11, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00245343497954309, -0.0905656665563583, 0.0633557364344597, new int[] { 1, 12, 13, 2, -1 }, new int[] { 1, 13, 13, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052382410503923893, -0.0410471595823765, 0.0727308094501495, new int[] { 11, 9, 3, 6, -1 }, new int[] { 11, 11, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0143192103132606, -0.17929619550704959, 0.036573551595211, true, new int[] { 8, 7, 4, 2, -1 }, new int[] { 9, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.010585630312562, -0.38849338889122009, 0.0079265926033258438, true, new int[] { 11, 5, 1, 3, -1 }, new int[] { 10, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0089276917278766632, -0.0575792603194714, 0.1015077978372574, new int[] { 1, 9, 8, 4, -1 }, new int[] { 1, 9, 4, 2, 2 }, new int[] { 5, 11, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0579179786145687, 0.0137350102886558, -0.19172470271587369, new int[] { 10, 5, 8, 10, -1 }, new int[] { 14, 5, 4, 5, 2 }, new int[] { 10, 10, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0092071853578090668, -0.20012189447879791, 0.0331920385360718, true, new int[] { 2, 10, 3, 2, -1 }, new int[] { 3, 11, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0835009291768074, 0.29251980781555181, -0.0229036696255207, new int[] { 1, 1, 16, 9, -1 }, new int[] { 1, 4, 16, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.004570710938423872, -0.19109779596328741, 0.0408679395914078, true, new int[] { 7, 4, 4, 2, -1 }, new int[] { 8, 5, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0281076692044735, -0.13955590128898621, 0.0228978395462036, true, new int[] { 12, 0, 6, 3, -1 }, new int[] { 14, 2, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0228165406733751, -0.25770029425621033, 0.0229892395436764, new int[] { 1, 12, 6, 3, -1 }, new int[] { 3, 12, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052285268902778625, -0.0617472901940346, 0.0377134010195732, true, new int[] { 11, 7, 3, 2, -1 }, new int[] { 12, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0060513508506119251, -0.0416271314024925, 0.15567490458488459, new int[] { 4, 8, 4, 4, -1 }, new int[] { 4, 8, 2, 2, 2 }, new int[] { 6, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0407820083200932, 0.25596979260444641, -0.0251890700310469, new int[] { 6, 0, 9, 11, -1 }, new int[] { 9, 0, 3, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062671699561178684, -0.0976725667715073, 0.0727524906396866, true, new int[] { 8, 0, 6, 1, -1 }, new int[] { 10, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011280509643256664, 0.0736560374498367, -0.1138757988810539, new int[] { 8, 9, 2, 2, -1 }, new int[] { 8, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068747308105230331, -0.0667891502380371, 0.13151079416275019, new int[] { 0, 9, 17, 2, -1 }, new int[] { 0, 10, 17, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0337627902626991, -0.1893121004104614, 0.0347666181623936, new int[] { 2, 0, 14, 6, -1 }, new int[] { 2, 3, 14, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1757418987108395E-05, -0.0780986174941063, 0.0798301994800568, new int[] { 0, 13, 2, 2, -1 }, new int[] { 0, 13, 1, 1, 2 }, new int[] { 1, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1017585024237633, 0.0175233595073223, -0.2194790989160538, new int[] { 5, 4, 10, 10, -1 }, new int[] { 10, 4, 5, 5, 2 }, new int[] { 5, 9, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1176455989480019, 0.1473899036645889, -0.042805839329958, new int[] { 3, 1, 12, 9, -1 }, new int[] { 7, 4, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.19031679630279541, -0.3762378990650177, 0.0038982050027698278, true, new int[] { 12, 4, 5, 6, -1 }, new int[] { 12, 4, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2182461023330689, 0.0078864647075533867, -0.64516901969909668, true, new int[] { 6, 4, 6, 5, -1 }, new int[] { 6, 4, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.172058783704415E-05, -0.0688135400414467, 0.078313410282135, new int[] { 8, 1, 2, 2, -1 }, new int[] { 9, 1, 1, 1, 2 }, new int[] { 8, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.6815136708319187E-05, -0.0691982433199883, 0.0981492102146149, new int[] { 8, 1, 2, 2, -1 }, new int[] { 8, 1, 1, 1, 2 }, new int[] { 9, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015573709970340133, 0.0455104112625122, -0.1185887008905411, new int[] { 8, 7, 2, 2, -1 }, new int[] { 8, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0153560703620315, -0.0377323292195797, 0.161965399980545, new int[] { 0, 8, 18, 3, -1 }, new int[] { 0, 9, 18, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00064422818832099438, -0.0492143407464027, 0.0385965816676617, true, new int[] { 9, 6, 1, 3, -1 }, new int[] { 8, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0030240670312196016, 0.0198773108422756, -0.2735247015953064, new int[] { 6, 0, 2, 3, -1 }, new int[] { 6, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.24049060046672821, -0.32233241200447083, 0.0099804811179637909, new int[] { 12, 5, 6, 10, -1 }, new int[] { 12, 10, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0068453960120677948, -0.26824951171875, 0.0200939793139696, true, new int[] { 9, 6, 3, 1, -1 }, new int[] { 10, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0982210710644722, 0.367314487695694, -0.0167514402419329, new int[] { 3, 3, 12, 4, -1 }, new int[] { 3, 5, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0333984605967999, -0.75862818956375122, 0.0099286399781703949, new int[] { 5, 5, 7, 3, -1 }, new int[] { 5, 6, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0322372205555439, 0.22383579611778259, -0.0126148099079728, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0332839600741863, 0.29738378524780268, -0.0196489002555609, true, new int[] { 4, 1, 3, 4, -1 }, new int[] { 5, 2, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3496932853013277E-05, 0.0579334609210491, -0.0438858605921268, new int[] { 16, 3, 2, 2, -1 }, new int[] { 17, 3, 1, 1, 2 }, new int[] { 16, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.60122129577212E-05, -0.0718164891004562, 0.0869365110993385, new int[] { 0, 3, 2, 2, -1 }, new int[] { 0, 3, 1, 1, 2 }, new int[] { 1, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0270447190850973, 0.0075920550152659416, -0.54519557952880859, true, new int[] { 10, 0, 4, 2, -1 }, new int[] { 11, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0098314275965094566, 0.0235845800489187, -0.24372850358486181, true, new int[] { 8, 0, 2, 4, -1 }, new int[] { 7, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.014273289591074, 0.1202424988150597, -0.0208050198853016, true, new int[] { 14, 3, 4, 3, -1 }, new int[] { 13, 4, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003404742106795311, 0.0242772400379181, -0.24346110224723819, new int[] { 0, 5, 1, 4, -1 }, new int[] { 0, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021703050006181, 0.0476825311779976, -0.0285765398293734, true, new int[] { 15, 5, 2, 3, -1 }, new int[] { 14, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0646167024970055, -0.0725622028112412, 0.0955711901187897, new int[] { 0, 4, 18, 6, -1 }, new int[] { 0, 6, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0361361317336559, -0.22917810082435611, 0.0021050409413874149, true, new int[] { 15, 5, 2, 3, -1 }, new int[] { 14, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0191675499081612, 0.30063459277153021, -0.0226390194147825, true, new int[] { 3, 5, 3, 2, -1 }, new int[] { 4, 6, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0103014996275306, 0.0199798997491598, -0.1185344010591507, true, new int[] { 11, 6, 1, 4, -1 }, new int[] { 10, 7, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0250420607626438, 0.013732859864831, -0.44012320041656489, new int[] { 2, 8, 4, 7, -1 }, new int[] { 3, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1180287972092629, -0.0238245893269777, 0.0961270332336426, true, new int[] { 9, 0, 9, 4, -1 }, new int[] { 9, 0, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0082905329763889313, -0.0817760676145554, 0.0683934092521667, new int[] { 1, 9, 6, 6, -1 }, new int[] { 3, 11, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0107107702642679, 0.0433344282209873, -0.0750979110598564, new int[] { 12, 5, 6, 10, -1 }, new int[] { 12, 10, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.26918289065361017, -0.0395036600530148, 0.1450473070144653, new int[] { 0, 0, 15, 14, -1 }, new int[] { 5, 0, 5, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0227638091892004, 0.0996726229786873, -0.0775553807616234, new int[] { 7, 3, 4, 9, -1 }, new int[] { 7, 3, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.121151901781559, -0.39497470855712891, 0.0166401192545891, true, new int[] { 9, 0, 4, 9, -1 }, new int[] { 9, 0, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.1451293479185551E-05, -0.0532115213572979, 0.0365702211856842, new int[] { 9, 5, 3, 1, -1 }, new int[] { 10, 5, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038077360950410366, -0.0913413763046265, 0.0747274905443192, new int[] { 5, 4, 6, 3, -1 }, new int[] { 7, 4, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0622831098735332, 0.44904568791389471, -0.0142916804179549, new int[] { 5, 0, 8, 8, -1 }, new int[] { 7, 0, 4, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165455099195242, 0.21537640690803531, -0.0266895107924938, true, new int[] { 3, 4, 3, 3, -1 }, new int[] { 4, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0095320530235767364, -0.150287002325058, 0.0081632016226649284, true, new int[] { 9, 2, 3, 2, -1 }, new int[] { 10, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1539638661779463E-05, 0.0777021870017052, -0.074435219168663, new int[] { 3, 14, 2, 1, -1 }, new int[] { 4, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081616528332233429, 0.0125406999140978, -0.0472638383507729, true, new int[] { 9, 2, 3, 2, -1 }, new int[] { 10, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0160646103322506, -0.63055968284606934, 0.0085211051627993584, true, new int[] { 9, 2, 2, 3, -1 }, new int[] { 8, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0944218188524246, 0.138080894947052, -0.039954699575901, new int[] { 1, 5, 16, 6, -1 }, new int[] { 1, 7, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.070128433406353, -0.2750720083713532, 0.0264193192124367, new int[] { 0, 3, 4, 9, -1 }, new int[] { 0, 6, 4, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142810000106692, 0.0840907394886017, -0.0420290790498257, new int[] { 10, 8, 6, 4, -1 }, new int[] { 13, 8, 3, 2, 2 }, new int[] { 10, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0205234792083502, 0.15208010375499731, -0.0386744514107704, new int[] { 2, 8, 6, 4, -1 }, new int[] { 2, 8, 3, 2, 2 }, new int[] { 5, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.31574970483779907, 0.0088831735774874687, -0.6855131983757019, new int[] { 1, 4, 16, 6, -1 }, new int[] { 5, 4, 8, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059291431680321693, 0.0069111599586904049, -0.60731059312820435, new int[] { 6, 2, 2, 1, -1 }, new int[] { 7, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.080303865019232E-05, -0.0669746771454811, 0.0759973376989365, new int[] { 8, 1, 2, 2, -1 }, new int[] { 9, 1, 1, 1, 2 }, new int[] { 8, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00089074257994070649, -0.0574223808944225, 0.0896140709519386, new int[] { 2, 8, 6, 4, -1 }, new int[] { 2, 8, 3, 2, 2 }, new int[] { 5, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0755855664610863, 0.0054939449764788151, -0.50682210922241211, true, new int[] { 15, 3, 2, 10, -1 }, new int[] { 15, 3, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0170325208455324, -0.0700998529791832, 0.0843230485916138, true, new int[] { 3, 3, 10, 2, -1 }, new int[] { 3, 3, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0122383302077651, 0.0335065089166164, -0.15453749895095831, new int[] { 0, 12, 18, 2, -1 }, new int[] { 9, 12, 9, 1, 2 }, new int[] { 0, 13, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0126505699008703, -0.0344986617565155, 0.17358370125293729, new int[] { 5, 9, 6, 4, -1 }, new int[] { 5, 9, 3, 2, 2 }, new int[] { 8, 11, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039281910285353661, 0.033152848482132, -0.1206599026918411, true, new int[] { 16, 0, 2, 2, -1 }, new int[] { 16, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.18485839664936071, -0.44308841228485107, 0.0122470501810312, new int[] { 0, 7, 7, 8, -1 }, new int[] { 0, 11, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0065704691223800182, -0.28371539711952209, 0.011953360401094, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.8720408560475335E-05, 0.0606255605816841, -0.0905942320823669, true, new int[] { 2, 0, 2, 2, -1 }, new int[] { 2, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021587649825960398, 0.0718974173069, -0.071638710796833, new int[] { 14, 0, 3, 15, -1 }, new int[] { 15, 0, 1, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0426199585199356, -0.63012677431106567, 0.0090704262256622314, new int[] { 1, 0, 3, 15, -1 }, new int[] { 2, 0, 1, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021494319662451744, 0.070125512778759, -0.0302376300096512, new int[] { 17, 0, 1, 4, -1 }, new int[] { 17, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0090273208916187286, -0.2084393054246903, 0.0256627295166254, new int[] { 3, 14, 8, 1, -1 }, new int[] { 5, 14, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0193650294095278, -0.2186844944953919, 0.0394974797964096, new int[] { 17, 3, 1, 8, -1 }, new int[] { 17, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14133329689502719, 0.17587089538574219, -0.0300297401845455, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 11, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070533920079469681, 0.12578339874744421, -0.0422852896153927, new int[] { 8, 3, 2, 4, -1 }, new int[] { 8, 5, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(9.1119036369491369E-05, -0.080194853246212, 0.0698323473334312, new int[] { 1, 0, 10, 8, -1 }, new int[] { 1, 0, 5, 4, 2 }, new int[] { 6, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0569412186741829, 0.0166890900582075, -0.52839207649230957, new int[] { 16, 0, 2, 12, -1 }, new int[] { 16, 6, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0546842515468597, -0.20393149554729459, 0.0286209303885698, new int[] { 0, 0, 2, 12, -1 }, new int[] { 0, 6, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.8811619965126738E-05, 0.041804101318121, -0.0470252297818661, new int[] { 17, 6, 1, 2, -1 }, new int[] { 17, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017949440516531467, -0.0756849274039268, 0.0691110491752625, true, new int[] { 9, 1, 2, 2, -1 }, new int[] { 9, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029679369181394577, -0.0375063605606556, 0.16561579704284671, new int[] { 7, 2, 4, 1, -1 }, new int[] { 7, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0288094598799944, -0.1236065030097961, 0.0496754795312881, new int[] { 3, 2, 10, 8, -1 }, new int[] { 3, 4, 10, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040495251305401325, -0.0319622196257114, 0.19525900483131409, new int[] { 1, 7, 16, 2, -1 }, new int[] { 1, 8, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0620033591985703, -0.38278189301490778, 0.0150613198056817, new int[] { 3, 0, 2, 12, -1 }, new int[] { 3, 4, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1115748647134751E-05, 0.0677575394511223, -0.0526314005255699, new int[] { 15, 3, 2, 2, -1 }, new int[] { 16, 3, 1, 1, 2 }, new int[] { 15, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.5218940512277186E-05, 0.0864468365907669, -0.0672251731157303, new int[] { 1, 3, 2, 2, -1 }, new int[] { 1, 3, 1, 1, 2 }, new int[] { 2, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055194161832332611, -0.0172452796250582, 0.16542769968509671, new int[] { 15, 3, 2, 4, -1 }, new int[] { 16, 3, 1, 2, 2 }, new int[] { 15, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103026004508138, -0.23677019774913791, 0.0223297607153654, new int[] { 0, 1, 18, 2, -1 }, new int[] { 0, 1, 9, 1, 2 }, new int[] { 9, 2, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041106292046606541, -0.0202375706285238, 0.0889737829566002, new int[] { 14, 5, 3, 3, -1 }, new int[] { 15, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022337420377880335, -0.0461580082774162, 0.11012540012598041, new int[] { 1, 5, 3, 3, -1 }, new int[] { 2, 5, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0754150971770287, -0.436719685792923, 0.0070562111213803291, new int[] { 13, 8, 4, 7, -1 }, new int[] { 13, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035641689319163561, -0.20360149443149569, 0.0260564293712378, true, new int[] { 1, 12, 2, 1, -1 }, new int[] { 1, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075477738864719868, 0.0682261064648628, -0.0227576401084661, new int[] { 16, 4, 2, 10, -1 }, new int[] { 17, 4, 1, 5, 2 }, new int[] { 16, 9, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031273330096155405, -0.0515966191887856, 0.1104556024074554, new int[] { 0, 4, 2, 10, -1 }, new int[] { 0, 4, 1, 5, 2 }, new int[] { 1, 9, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.007246991153806448, -0.28128591179847717, 0.0032531570177525282, new int[] { 16, 10, 2, 1, -1 }, new int[] { 16, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.2346920710988343E-05, 0.0701061934232712, -0.0941527709364891, new int[] { 0, 10, 2, 1, -1 }, new int[] { 1, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0246129799634218, -0.8730425238609314, 0.0013450640253722668, true, new int[] { 16, 6, 2, 1, -1 }, new int[] { 16, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035978900268673897, -0.17041720449924469, 0.0319982208311558, true, new int[] { 2, 6, 1, 2, -1 }, new int[] { 2, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0729575231671333, 0.0050021768547594547, -0.4682140052318573, new int[] { 13, 8, 4, 7, -1 }, new int[] { 13, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0829254165291786, -0.6825491189956665, 0.0068542738445103168, new int[] { 1, 8, 4, 7, -1 }, new int[] { 3, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1458497941493988, 0.0044581899419426918, -0.91366928815841675, new int[] { 0, 9, 18, 4, -1 }, new int[] { 9, 9, 9, 2, 2 }, new int[] { 0, 11, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0121017899364233, 0.0244141705334187, -0.181175097823143, true, new int[] { 8, 6, 3, 2, -1 }, new int[] { 9, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0606673695147038, 0.22934840619564059, -0.0143234599381685, new int[] { 8, 7, 8, 4, -1 }, new int[] { 12, 7, 4, 2, 2 }, new int[] { 8, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0207455400377512, -0.0269107203930616, 0.19334229826927191, new int[] { 1, 12, 9, 3, -1 }, new int[] { 1, 13, 9, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00077412481186911464, -0.029913529753685, 0.0458732806146145, new int[] { 13, 13, 1, 2, -1 }, new int[] { 13, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0135493697598577, 0.0344336815178394, -0.181169793009758, new int[] { 0, 13, 18, 2, -1 }, new int[] { 0, 13, 9, 1, 2 }, new int[] { 9, 14, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1226418018341065, 0.0085802376270294189, -0.35567748546600342, new int[] { 7, 11, 8, 4, -1 }, new int[] { 7, 13, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0671608373522758, 0.0152594400569797, -0.3348085880279541, new int[] { 0, 7, 18, 4, -1 }, new int[] { 0, 7, 9, 2, 2 }, new int[] { 9, 9, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0246475301682949, 0.19604270160198209, -0.0251305196434259, new int[] { 5, 2, 9, 6, -1 }, new int[] { 5, 4, 9, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.016193950548768, 0.0255086906254292, -0.21010090410709381, new int[] { 6, 5, 6, 4, -1 }, new int[] { 6, 5, 3, 2, 2 }, new int[] { 9, 7, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44934380054473883, -0.0108507098630071, 0.263612687587738, new int[] { 6, 0, 9, 9, -1 }, new int[] { 9, 3, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0100060002878308, 0.0162830203771591, -0.33978369832038879, new int[] { 7, 0, 4, 4, -1 }, new int[] { 7, 0, 2, 2, 2 }, new int[] { 9, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.000332953903125599, 0.0482161790132523, -0.0331645794212818, new int[] { 9, 10, 2, 2, -1 }, new int[] { 10, 10, 1, 1, 2 }, new int[] { 9, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0285563599318266, -0.14011450111866, 0.0359319001436234, true, new int[] { 6, 0, 3, 6, -1 }, new int[] { 4, 2, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068772169761359692, -0.0123321795836091, 0.15525570511817929, true, new int[] { 17, 0, 1, 3, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0026129318866878748, -0.0435581207275391, 0.12221980094909669, true, new int[] { 1, 0, 3, 1, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.32784798741340643, 0.0013112389715388417, -0.81634020805358887, new int[] { 10, 0, 3, 15, -1 }, new int[] { 11, 5, 1, 5, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15350890159606931, 0.0153489299118519, -0.33603930473327642, new int[] { 5, 0, 3, 15, -1 }, new int[] { 6, 5, 1, 5, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00090102507965639234, -0.0325689390301704, 0.0637555792927742, new int[] { 16, 0, 1, 4, -1 }, new int[] { 16, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.4206269346177578E-05, 0.0817376524209976, -0.0669129565358162, new int[] { 1, 0, 1, 2, -1 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003356515895575285, -0.1260069012641907, 0.0223339106887579, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0652299970388412, -0.0320342108607292, 0.1782056987285614, new int[] { 0, 0, 17, 10, -1 }, new int[] { 0, 5, 17, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020175189711153507, 0.0244843903928995, -0.0572246313095093, new int[] { 12, 0, 3, 10, -1 }, new int[] { 12, 5, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007074608001857996, 0.0098791662603616714, -0.54220247268676758, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.59178986097686E-05, -0.0516582205891609, 0.0567629300057888, new int[] { 15, 2, 2, 2, -1 }, new int[] { 16, 2, 1, 1, 2 }, new int[] { 15, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.30828830599784851, -0.0095853386446833611, 0.5343317985534668, new int[] { 3, 3, 9, 6, -1 }, new int[] { 6, 5, 3, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0102557903155684, 0.0248383395373821, -0.16516630351543429, new int[] { 6, 3, 11, 2, -1 }, new int[] { 6, 4, 11, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3460840717889369E-05, 0.0798209980130196, -0.0650218427181244, new int[] { 1, 2, 2, 2, -1 }, new int[] { 1, 2, 1, 1, 2 }, new int[] { 2, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0023789680562913418, 0.0478302501142025, -0.0529914908111095, true, new int[] { 14, 1, 4, 2, -1 }, new int[] { 14, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0056755929253995419, 0.1244622021913528, -0.0447519905865192, true, new int[] { 4, 1, 2, 4, -1 }, new int[] { 4, 1, 1, 4, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 16
            stage = new HaarCascadeStage(-1.1628010272979736, 15, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.1075673997402191, 0.34051141142845148, -0.1520918011665344, true, new int[] { 8, 4, 4, 6, -1 }, new int[] { 6, 6, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.043516431003809, -0.0135334003716707, 0.2857075035572052, true, new int[] { 14, 0, 4, 4, -1 }, new int[] { 13, 1, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15090970695018771, 0.00050420017214491963, -560.7666015625, new int[] { 0, 9, 8, 4, -1 }, new int[] { 0, 9, 4, 2, 2 }, new int[] { 4, 11, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081543149426579475, -0.0573937706649303, 0.1638182997703552, true, new int[] { 15, 8, 3, 3, -1 }, new int[] { 16, 9, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10340789705514911, 0.22989919781684881, -0.1285800039768219, new int[] { 2, 7, 14, 4, -1 }, new int[] { 2, 9, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075287488289177418, 0.071470782160759, -0.0257890298962593, true, new int[] { 9, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036443499848246574, -0.22227230668067929, 0.1241116970777512, true, new int[] { 9, 0, 1, 4, -1 }, new int[] { 9, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.53749978542327881, 0.0139470295980573, 0.52125108242034912, new int[] { 2, 0, 14, 15, -1 }, new int[] { 2, 0, 7, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.27013081312179571, -0.0199047792702913, -630.8125, new int[] { 1, 7, 14, 4, -1 }, new int[] { 1, 9, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103687699884176, 0.10527289658784871, -0.1294572055339813, new int[] { 9, 8, 8, 7, -1 }, new int[] { 11, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0156045500189066, 0.215954601764679, -0.0988422036170959, true, new int[] { 4, 0, 4, 4, -1 }, new int[] { 5, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.2028758972883225, -0.27739518880844122, 0.0034634380135685205, new int[] { 8, 6, 9, 8, -1 }, new int[] { 11, 6, 3, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0271604191511869, 0.1002269983291626, -0.2054217010736466, new int[] { 1, 6, 9, 8, -1 }, new int[] { 4, 6, 3, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052366848103702068, 0.12705439329147339, -0.1254777014255524, new int[] { 7, 6, 4, 2, -1 }, new int[] { 7, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046215238980948925, 0.0448268912732601, -0.27245700359344482, true, new int[] { 8, 0, 2, 3, -1 }, new int[] { 7, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0067956638522446156, -0.13386589288711551, 0.0271778404712677, true, new int[] { 10, 6, 3, 2, -1 }, new int[] { 11, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.2197666019201279, -0.25276950001716608, 0.0464650392532349, new int[] { 0, 1, 18, 14, -1 }, new int[] { 0, 1, 9, 7, 2 }, new int[] { 9, 8, 9, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046517988666892052, 0.0109347002580762, -0.355980396270752, new int[] { 11, 0, 2, 2, -1 }, new int[] { 11, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015317969955503941, -0.24999429285526281, 0.0443512909114361, new int[] { 5, 0, 2, 2, -1 }, new int[] { 5, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046969428658485413, 0.0218366198241711, -0.28716519474983221, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0481894090771675, 0.0288693699985743, -0.36160790920257568, new int[] { 4, 4, 10, 6, -1 }, new int[] { 4, 4, 5, 3, 2 }, new int[] { 9, 7, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0076267770491540432, 0.13116089999675751, -0.0371875613927841, new int[] { 8, 9, 6, 2, -1 }, new int[] { 11, 9, 3, 1, 2 }, new int[] { 8, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.5027391024632379E-05, 0.0719915106892586, -0.1249687001109123, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3772819228470325E-05, 0.0795105397701263, -0.0796041265130043, new int[] { 16, 1, 1, 2, -1 }, new int[] { 16, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092382878065109253, -0.0459494404494762, 0.205514594912529, true, new int[] { 2, 3, 3, 2, -1 }, new int[] { 3, 4, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0336009599268436, 0.0239669401198626, -0.22747719287872309, new int[] { 16, 3, 2, 10, -1 }, new int[] { 16, 8, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0418576300144196, -0.2567035853862763, 0.0433881990611553, new int[] { 0, 3, 2, 10, -1 }, new int[] { 0, 8, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0043434980325400829, -0.0360659398138523, 0.13354070484638211, new int[] { 8, 9, 6, 2, -1 }, new int[] { 11, 9, 3, 1, 2 }, new int[] { 8, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0087262392044067383, -0.0280333999544382, 0.29659709334373469, new int[] { 1, 7, 10, 2, -1 }, new int[] { 1, 7, 5, 1, 2 }, new int[] { 6, 8, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0725063979625702, 0.0339310988783836, -0.26456800103187561, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069837369956076145, 0.0230753999203444, -0.3671954870223999, true, new int[] { 4, 0, 1, 4, -1 }, new int[] { 3, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0939587205648422, 0.00051443470874801278, -0.69157868623733521, true, new int[] { 10, 4, 4, 4, -1 }, new int[] { 11, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0546111688017845, 0.35633870959281921, -0.0255911909043789, true, new int[] { 8, 4, 4, 4, -1 }, new int[] { 7, 5, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0083599044010043144, -0.1183891966938973, 0.054096020758152, true, new int[] { 16, 1, 2, 10, -1 }, new int[] { 16, 1, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0085311960428953171, 0.25801640748977661, -0.0432965084910393, new int[] { 5, 0, 8, 9, -1 }, new int[] { 7, 0, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0530957616865635, 0.0134461699053645, -0.20017629861831671, true, new int[] { 16, 1, 2, 10, -1 }, new int[] { 16, 1, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071099922060966492, -0.1717357933521271, 0.0664152875542641, true, new int[] { 2, 1, 10, 2, -1 }, new int[] { 2, 1, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0121456598863006, -0.34982419013977051, 0.0152532299980521, true, new int[] { 13, 0, 3, 2, -1 }, new int[] { 14, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0491840504109859, -0.1462731063365936, 0.0766353383660316, new int[] { 3, 0, 12, 6, -1 }, new int[] { 6, 0, 6, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0642079263925552, -0.0426980294287205, 0.0898953378200531, new int[] { 6, 0, 12, 4, -1 }, new int[] { 9, 0, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0505671091377735, -0.0342714004218578, 0.32117810845375061, new int[] { 6, 3, 6, 6, -1 }, new int[] { 6, 6, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.38187500834465032, 0.00597370695322752, -0.41509181261062622, new int[] { 6, 3, 12, 7, -1 }, new int[] { 6, 3, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2414198964834213, 0.0428920909762383, -0.25744560360908508, new int[] { 0, 3, 12, 7, -1 }, new int[] { 6, 3, 6, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00873350165784359, 0.0215238109230995, -0.25816148519515991, true, new int[] { 13, 0, 3, 2, -1 }, new int[] { 14, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055905920453369617, 0.0368825495243073, -0.26805239915847778, true, new int[] { 5, 0, 2, 3, -1 }, new int[] { 4, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145109295845032, -0.1092017963528633, 0.0991731509566307, new int[] { 0, 9, 18, 4, -1 }, new int[] { 0, 11, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0274284295737743, -0.25048801302909851, 0.0452128499746323, true, new int[] { 9, 6, 1, 8, -1 }, new int[] { 9, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1233676970005035, 0.2255768030881882, -0.0428952686488628, new int[] { 2, 5, 14, 6, -1 }, new int[] { 2, 7, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0616077184677124, -0.27772828936576838, 0.0325213186442852, new int[] { 2, 0, 5, 8, -1 }, new int[] { 2, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0762168914079666, 0.36572670936584473, -0.0255184806883335, new int[] { 4, 3, 10, 4, -1 }, new int[] { 4, 5, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0043231542222201824, -0.0599518194794655, 0.12853649258613589, new int[] { 8, 0, 2, 3, -1 }, new int[] { 9, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.2015187470242381E-05, 0.0668459609150887, -0.0653621777892113, true, new int[] { 15, 6, 1, 3, -1 }, new int[] { 14, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038772630505263805, -0.0746818333864212, 0.149043396115303, new int[] { 3, 6, 3, 4, -1 }, new int[] { 3, 7, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0308424606919289, 0.0467762798070908, -0.0792699083685875, new int[] { 9, 4, 8, 10, -1 }, new int[] { 13, 4, 4, 5, 2 }, new int[] { 9, 9, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029754610732197762, -0.0631382465362549, 0.12994049489498141, new int[] { 3, 8, 3, 3, -1 }, new int[] { 4, 9, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043571940623223782, 0.1760174036026001, -0.0209502801299095, new int[] { 13, 9, 1, 2, -1 }, new int[] { 13, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.564980968600139E-05, -0.0934598371386528, 0.1056388020515442, new int[] { 4, 9, 1, 2, -1 }, new int[] { 4, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0190466307103634, 0.0378969013690948, -0.20427240431308749, new int[] { 8, 4, 2, 10, -1 }, new int[] { 8, 9, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.059084378182888, -0.26028269529342651, 0.0318774096667767, new int[] { 7, 7, 4, 4, -1 }, new int[] { 7, 9, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0399503409862518, -0.350638210773468, 0.0092909233644604683, true, new int[] { 14, 0, 2, 10, -1 }, new int[] { 14, 0, 1, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0508347414433956, 0.0219123102724552, -0.38032969832420349, true, new int[] { 4, 0, 10, 2, -1 }, new int[] { 4, 0, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0136031899601221, 0.2038068026304245, -0.0212994609028101, true, new int[] { 16, 4, 2, 3, -1 }, new int[] { 15, 5, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0674393326044083, -0.4756908118724823, 0.016315059736371, new int[] { 4, 12, 9, 3, -1 }, new int[] { 7, 12, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0177440494298935, -0.0262153502553701, 0.17312249541282651, true, new int[] { 16, 4, 2, 3, -1 }, new int[] { 15, 5, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0408229492604733, 0.0269718896597624, -0.25315660238265991, new int[] { 2, 1, 14, 4, -1 }, new int[] { 2, 3, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035472789313644171, -0.193899005651474, 0.015181340277195, new int[] { 9, 1, 4, 2, -1 }, new int[] { 9, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.01345095038414, -0.0560166388750076, 0.1336188018321991, new int[] { 1, 3, 12, 4, -1 }, new int[] { 1, 3, 6, 2, 2 }, new int[] { 7, 5, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0702156871557236, 0.0121993301436305, -0.29756540060043341, new int[] { 9, 3, 8, 2, -1 }, new int[] { 9, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0158290397375822, -0.0871118977665901, 0.0889551267027855, new int[] { 1, 3, 8, 2, -1 }, new int[] { 5, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0203911308199167, 0.17829939723014829, -0.0371981598436832, true, new int[] { 15, 8, 3, 4, -1 }, new int[] { 16, 9, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036189330276101828, -0.0762976333498955, 0.0969681292772293, true, new int[] { 3, 8, 4, 3, -1 }, new int[] { 2, 9, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0010060019558295608, -0.0498901791870594, 0.0658943429589272, new int[] { 15, 12, 2, 3, -1 }, new int[] { 15, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029275720007717609, 0.0298173800110817, -0.24240310490131381, new int[] { 0, 7, 1, 4, -1 }, new int[] { 0, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0122589897364378, 0.19031849503517151, -0.0075331269763410091, new int[] { 15, 12, 2, 3, -1 }, new int[] { 15, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.0887768194079399, 0.080645427107811, new int[] { 1, 12, 2, 3, -1 }, new int[] { 1, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0128609901294112, 0.0695679932832718, -0.0297688208520412, true, new int[] { 10, 0, 3, 8, -1 }, new int[] { 8, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.049192551523447, 0.151136502623558, -0.0546999201178551, new int[] { 3, 0, 12, 8, -1 }, new int[] { 9, 0, 6, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0194404404610395, -0.17859379947185519, 0.017632320523262, new int[] { 5, 14, 12, 1, -1 }, new int[] { 8, 14, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025363420136272907, 0.0300990603864193, -0.21704940497875211, new int[] { 8, 0, 2, 4, -1 }, new int[] { 8, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0209271106868982, 0.15293440222740171, -0.0265916306525469, new int[] { 8, 0, 4, 3, -1 }, new int[] { 8, 0, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021768060978502035, -0.0801318064332008, 0.0870366171002388, new int[] { 7, 0, 4, 2, -1 }, new int[] { 8, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032644919119775295, -0.0506618581712246, 0.050410520285368, new int[] { 8, 13, 8, 2, -1 }, new int[] { 8, 14, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0531350895762444, 0.0313573814928532, -0.24327489733695981, new int[] { 0, 11, 18, 4, -1 }, new int[] { 0, 11, 9, 2, 2 }, new int[] { 9, 13, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065658721141517162, -0.0314484387636185, 0.13142390549182889, new int[] { 9, 9, 8, 2, -1 }, new int[] { 13, 9, 4, 1, 2 }, new int[] { 9, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016994590405374765, 0.0787288174033165, -0.0930547267198563, new int[] { 8, 9, 2, 2, -1 }, new int[] { 8, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0231965091079474, 0.20170910656452179, -0.0152339404448867, new int[] { 9, 9, 8, 2, -1 }, new int[] { 13, 9, 4, 1, 2 }, new int[] { 9, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0071990801952779293, -0.0436348989605904, 0.2130060940980911, new int[] { 1, 9, 8, 2, -1 }, new int[] { 1, 9, 4, 1, 2 }, new int[] { 5, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069829211570322514, 0.0317675210535526, -0.2128593027591705, true, new int[] { 11, 7, 1, 3, -1 }, new int[] { 10, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044900798238813877, -0.17512929439544681, 0.0440214611589909, true, new int[] { 7, 7, 3, 1, -1 }, new int[] { 8, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.12099999934434889, -0.36906799674034119, 0.0044225710444152355, true, new int[] { 10, 0, 3, 8, -1 }, new int[] { 8, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0380082689225674, 0.52773797512054443, -0.0147407604381442, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0111320000141859, 0.0634055435657501, -0.1106311976909638, new int[] { 5, 7, 8, 2, -1 }, new int[] { 5, 8, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1212562024593353, 0.1124370023608208, -0.0671258494257927, new int[] { 4, 1, 9, 9, -1 }, new int[] { 7, 4, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0588735602796078, 0.1949198991060257, -0.00079787842696532607, true, new int[] { 10, 3, 3, 7, -1 }, new int[] { 11, 4, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0123289301991463, -0.18806469440460211, 0.0393505804240704, true, new int[] { 8, 3, 7, 3, -1 }, new int[] { 7, 4, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044250390492379665, 0.11267340183258059, -0.0681002363562584, new int[] { 7, 5, 4, 2, -1 }, new int[] { 7, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00509668281301856, -0.17945580184459689, 0.0475732088088989, new int[] { 7, 0, 2, 6, -1 }, new int[] { 7, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0403452403843403, -0.57044768333435059, 0.0055092480033636093, new int[] { 8, 0, 3, 6, -1 }, new int[] { 9, 2, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.112549401819706, -0.0269452705979347, 0.25808990001678472, true, new int[] { 9, 3, 6, 3, -1 }, new int[] { 11, 5, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0699782967567444, -0.0011665009660646319, 0.86768251657485962, new int[] { 5, 14, 12, 1, -1 }, new int[] { 8, 14, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0165449008345604, 0.0243071895092726, -0.25596928596496582, new int[] { 1, 14, 12, 1, -1 }, new int[] { 4, 14, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0822774171829224, -0.0268739499151707, 0.24098409712314611, new int[] { 1, 7, 16, 6, -1 }, new int[] { 1, 9, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046195117756724358, -0.16582019627094269, 0.0400424189865589, new int[] { 0, 10, 3, 4, -1 }, new int[] { 0, 11, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0024694160092622042, 0.0927710607647896, -0.0273753199726343, new int[] { 14, 7, 4, 1, -1 }, new int[] { 15, 7, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0001085738986148499, -0.1348482966423035, 0.0436066016554832, true, new int[] { 8, 1, 1, 4, -1 }, new int[] { 8, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0164907705038786, -0.1666806042194367, 0.0177498105913401, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 15, 2, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0047164629213511944, 0.17804649472236631, -0.0365630798041821, new int[] { 2, 6, 2, 4, -1 }, new int[] { 2, 6, 1, 2, 2 }, new int[] { 3, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0906244590878487, 0.0174008794128895, -0.48980259895324713, new int[] { 15, 5, 3, 10, -1 }, new int[] { 15, 10, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0027714879252016544, -0.0659386664628983, 0.0964076220989227, new int[] { 3, 9, 6, 4, -1 }, new int[] { 3, 9, 3, 2, 2 }, new int[] { 6, 11, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0434898696839809, 0.0139165297150612, -0.27095559239387512, true, new int[] { 13, 3, 3, 7, -1 }, new int[] { 14, 4, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063884491100907326, -0.0581430904567242, 0.1046271026134491, true, new int[] { 6, 2, 7, 4, -1 }, new int[] { 5, 3, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142638003453612, 0.14017640054225919, -0.0269160307943821, true, new int[] { 15, 1, 3, 3, -1 }, new int[] { 14, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.005662744864821434, -0.18962329626083371, 0.0316337496042252, new int[] { 0, 4, 8, 3, -1 }, new int[] { 0, 5, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075204060412943363, -0.0435900315642357, 0.10007920116186141, true, new int[] { 14, 4, 3, 5, -1 }, new int[] { 15, 5, 1, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0110979797318578, 0.30840259790420532, -0.0212082397192717, true, new int[] { 5, 4, 5, 2, -1 }, new int[] { 5, 4, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0618321411311626, 0.18315550684928891, -0.0077433600090444088, true, new int[] { 10, 7, 1, 6, -1 }, new int[] { 8, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034768159966915846, 0.0506381392478943, -0.13400410115718839, true, new int[] { 8, 7, 6, 1, -1 }, new int[] { 10, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0977838635444641, 0.00205444497987628, -0.68779617547988892, new int[] { 10, 4, 6, 10, -1 }, new int[] { 13, 4, 3, 5, 2 }, new int[] { 10, 9, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0918209478259087, -0.25586891174316412, 0.0251086503267288, new int[] { 2, 4, 6, 10, -1 }, new int[] { 2, 4, 3, 5, 2 }, new int[] { 5, 9, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0140088303014636, -0.36381798982620239, 0.0155368996784091, new int[] { 4, 5, 10, 2, -1 }, new int[] { 9, 5, 5, 1, 2 }, new int[] { 4, 6, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0470989495515823, 0.4120045006275177, -0.0147856995463371, true, new int[] { 6, 2, 3, 6, -1 }, new int[] { 7, 3, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0240776594728231, -0.26497179269790649, 0.0043284958228468895, new int[] { 16, 0, 2, 4, -1 }, new int[] { 16, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030720019713044167, 0.1134819984436035, -0.0527238808572292, new int[] { 0, 0, 2, 4, -1 }, new int[] { 0, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0232353191822767, -0.161824107170105, 0.0139071401208639, new int[] { 8, 0, 10, 4, -1 }, new int[] { 8, 2, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.02175322920084, 0.0320463292300701, -0.1815026998519898, new int[] { 0, 0, 18, 4, -1 }, new int[] { 0, 0, 9, 2, 2 }, new int[] { 9, 2, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.028419379144907, 0.0735991299152374, -0.0121852997690439, new int[] { 6, 0, 12, 2, -1 }, new int[] { 9, 0, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0990353375673294, -0.80039161443710327, 0.007554355077445507, new int[] { 0, 0, 12, 2, -1 }, new int[] { 3, 0, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036745260003954172, -0.0425384715199471, 0.13135530054569239, true, new int[] { 17, 3, 1, 3, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.24902099370956421, 0.570973813533783, -0.0100652799010277, new int[] { 0, 4, 12, 6, -1 }, new int[] { 3, 4, 6, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025670630857348442, 0.1004543974995613, -0.0438447706401348, true, new int[] { 17, 3, 1, 3, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062725669704377651, 0.0282882191240788, -0.19911240041255951, true, new int[] { 8, 7, 3, 1, -1 }, new int[] { 9, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0121860196813941, -0.00892985612154007, 0.172361895442009, true, new int[] { 17, 3, 1, 3, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00840808730572462, 0.22059670090675351, -0.0254241600632668, true, new int[] { 1, 3, 3, 1, -1 }, new int[] { 2, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0076226810924708843, 0.022617619484663, -0.35040241479873657, true, new int[] { 15, 2, 3, 1, -1 }, new int[] { 16, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035278380382806063, -0.2129029035568237, 0.0337668098509312, true, new int[] { 3, 2, 1, 3, -1 }, new int[] { 2, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0487591288983822, 0.26399469375610352, -0.0227282308042049, new int[] { 2, 3, 14, 6, -1 }, new int[] { 2, 5, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.042163010686636, 0.0164839699864388, -0.37255099415779108, new int[] { 4, 2, 3, 8, -1 }, new int[] { 4, 6, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0412516593933105, -0.0056340959854424, 0.1074742004275322, true, new int[] { 14, 2, 4, 3, -1 }, new int[] { 13, 3, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.033506590873003, 0.32449829578399658, -0.0198305491358042, true, new int[] { 4, 2, 3, 4, -1 }, new int[] { 5, 3, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040785958990454674, 0.0712641105055809, -0.0864052474498749, new int[] { 12, 2, 4, 13, -1 }, new int[] { 13, 2, 2, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.039688128978014, -0.35533818602561951, 0.0168110895901918, new int[] { 2, 2, 4, 13, -1 }, new int[] { 3, 2, 2, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.26250740885734558, 0.0033027199096977711, -0.30452561378479, true, new int[] { 9, 4, 8, 3, -1 }, new int[] { 9, 4, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10336879640817639, -0.44277548789978027, 0.0152687802910805, true, new int[] { 9, 4, 3, 8, -1 }, new int[] { 9, 4, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0025352418888360262, 0.0226268991827965, -0.19356669485569, new int[] { 17, 10, 1, 2, -1 }, new int[] { 17, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013277910184115171, -0.084263376891613, 0.0657716765999794, new int[] { 7, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0692616030573845, 0.1914274990558624, -0.0148142697289586, new int[] { 6, 0, 9, 13, -1 }, new int[] { 9, 0, 3, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0319452695548534, -0.30996501445770258, 0.0180993191897869, new int[] { 0, 1, 1, 10, -1 }, new int[] { 0, 6, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007150053046643734, -0.0755150690674782, 0.0713425576686859, new int[] { 0, 11, 18, 2, -1 }, new int[] { 0, 12, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024518880527466536, -0.0526761785149574, 0.11914870142936709, new int[] { 4, 7, 3, 6, -1 }, new int[] { 5, 9, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0254793707281351, -0.0215268898755312, 0.1125423014163971, true, new int[] { 12, 7, 2, 5, -1 }, new int[] { 12, 7, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.3662307588383555E-05, -0.123724102973938, 0.0447584912180901, new int[] { 6, 5, 2, 1, -1 }, new int[] { 7, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052631269209086895, 0.016644699499011, -0.27927619218826288, new int[] { 8, 2, 6, 2, -1 }, new int[] { 11, 2, 3, 1, 2 }, new int[] { 8, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.9906251408392563E-05, -0.0590216182172298, 0.0907072424888611, new int[] { 4, 6, 3, 1, -1 }, new int[] { 5, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.40492799878120422, 0.0098951030522584915, -0.53900748491287231, new int[] { 0, 1, 18, 14, -1 }, new int[] { 9, 1, 9, 7, 2 }, new int[] { 0, 8, 9, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045421868562698364, -0.0830420330166817, 0.057933621108532, new int[] { 0, 9, 6, 6, -1 }, new int[] { 0, 9, 3, 3, 2 }, new int[] { 3, 12, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0286024697124958, 0.098798967897892, -0.0411834083497524, new int[] { 8, 9, 10, 6, -1 }, new int[] { 13, 9, 5, 3, 2 }, new int[] { 8, 12, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00609810883179307, -0.0496008917689323, 0.1082315966486931, new int[] { 1, 10, 15, 3, -1 }, new int[] { 1, 11, 15, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024081019219011068, 0.031793300062418, -0.0897006466984749, true, new int[] { 16, 11, 1, 2, -1 }, new int[] { 16, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1049328967928886, -0.1838400065898895, 0.0292720291763544, true, new int[] { 4, 4, 9, 2, -1 }, new int[] { 7, 7, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00072810851270332932, 0.0346079505980015, -0.18057569861412051, new int[] { 7, 7, 4, 2, -1 }, new int[] { 7, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002398305106908083, -0.0366495698690414, 0.1469368040561676, new int[] { 2, 7, 4, 2, -1 }, new int[] { 2, 7, 2, 1, 2 }, new int[] { 4, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044842050410807133, 0.0254560094326735, -0.17060090601444239, new int[] { 8, 0, 5, 2, -1 }, new int[] { 8, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0559289082884789, 0.0069079152308404446, -0.74263191223144531, new int[] { 6, 2, 4, 11, -1 }, new int[] { 7, 2, 2, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0113146202638745, -0.65691602230072021, 0.0030682450160384178, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052855871617794037, 0.0122091500088573, -0.41138368844985962, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035499120131134987, 0.15674009919166571, -0.0136733297258615, new int[] { 11, 9, 2, 2, -1 }, new int[] { 12, 9, 1, 1, 2 }, new int[] { 11, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.016200989484787, -0.45118838548660278, 0.0105137201026082, new int[] { 7, 8, 4, 4, -1 }, new int[] { 8, 8, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0063212178647518158, 0.2467146962881088, -0.0221792291849852, new int[] { 5, 0, 8, 1, -1 }, new int[] { 7, 0, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0678062811493874, 0.0141928596422076, -0.45575699210166931, new int[] { 4, 0, 6, 8, -1 }, new int[] { 4, 0, 3, 4, 2 }, new int[] { 7, 4, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44995149970054632, -0.0205099303275347, 0.238416999578476, new int[] { 5, 1, 9, 9, -1 }, new int[] { 8, 4, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.16068010032176969, -0.79124170541763306, 0.0054184817709028721, new int[] { 0, 4, 4, 10, -1 }, new int[] { 0, 9, 4, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0094610815867781639, -0.2421163022518158, 0.00911825243383646, true, new int[] { 16, 11, 1, 2, -1 }, new int[] { 16, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0147587396204472, -0.0416104607284069, 0.13534289598464971, new int[] { 1, 7, 16, 4, -1 }, new int[] { 1, 8, 16, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065756370313465595, 0.0093746017664670944, -0.0832142680883408, true, new int[] { 16, 11, 1, 2, -1 }, new int[] { 16, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0057711522094905376, 0.0266925692558289, -0.19803330302238459, true, new int[] { 2, 11, 2, 1, -1 }, new int[] { 2, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0509134791791439, 0.32146498560905462, -0.0169861502945423, new int[] { 2, 13, 14, 2, -1 }, new int[] { 2, 14, 14, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.3694868003949523E-05, -0.0845351293683052, 0.0685012266039848, new int[] { 0, 13, 4, 2, -1 }, new int[] { 0, 13, 2, 1, 2 }, new int[] { 2, 14, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021522149909287691, 0.0548588298261166, -0.0481257401406765, true, new int[] { 15, 1, 3, 2, -1 }, new int[] { 15, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040621249936521053, 0.31572619080543518, -0.0174344405531883, new int[] { 5, 9, 2, 2, -1 }, new int[] { 5, 9, 1, 1, 2 }, new int[] { 6, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0351190604269505, -0.4585689902305603, 0.0149546898901463, new int[] { 6, 9, 6, 2, -1 }, new int[] { 6, 10, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0127988802269101, -0.15211139619350431, 0.0345015898346901, new int[] { 0, 6, 18, 2, -1 }, new int[] { 0, 6, 9, 1, 2 }, new int[] { 9, 7, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053432481363415718, -0.20269839465618131, 0.0139673100784421, new int[] { 15, 13, 2, 2, -1 }, new int[] { 15, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030109770596027374, 0.2396494001150131, -0.0214331708848476, new int[] { 7, 9, 2, 2, -1 }, new int[] { 7, 9, 1, 1, 2 }, new int[] { 8, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0795640870928764, 0.016967510804534, -0.31260809302330023, new int[] { 0, 8, 18, 4, -1 }, new int[] { 9, 8, 9, 2, 2 }, new int[] { 0, 10, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168946702033281, 0.145903006196022, -0.034819670021534, new int[] { 8, 4, 2, 4, -1 }, new int[] { 8, 6, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.65786761045455933, -0.0130230896174908, 0.41044768691062927, new int[] { 3, 4, 12, 9, -1 }, new int[] { 7, 7, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1127222031354904, -0.3777270913124085, 0.0159226898103952, true, new int[] { 9, 1, 4, 7, -1 }, new int[] { 9, 1, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0177928805351257, 0.0118195097893476, -0.24668039381504059, true, new int[] { 11, 0, 4, 2, -1 }, new int[] { 12, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0033843109849840403, 0.0420966595411301, -0.13628929853439331, true, new int[] { 7, 0, 2, 4, -1 }, new int[] { 6, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0129303801804781, 0.0156342405825853, -0.31559720635414118, true, new int[] { 12, 0, 2, 2, -1 }, new int[] { 12, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.019866120070219, -0.019867179915309, 0.272928386926651, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0202569793909788, -0.75079268217086792, 0.0036987708881497383, true, new int[] { 12, 0, 3, 2, -1 }, new int[] { 13, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00681325001642108, -0.18717190623283389, 0.0291250105947256, true, new int[] { 6, 0, 2, 3, -1 }, new int[] { 5, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0134505499154329, 0.24198499321937561, -0.0111368801444769, new int[] { 13, 1, 4, 4, -1 }, new int[] { 15, 1, 2, 2, 2 }, new int[] { 13, 3, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.38661697646603E-05, 0.0751902163028717, -0.0758378133177757, new int[] { 2, 0, 2, 1, -1 }, new int[] { 3, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.048590901424177E-05, -0.0479880385100842, 0.0507909804582596, new int[] { 14, 0, 2, 2, -1 }, new int[] { 15, 0, 1, 1, 2 }, new int[] { 14, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.4496016420889646E-05, 0.08631631731987, -0.0676591396331787, new int[] { 2, 0, 2, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }, new int[] { 3, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.8561800213065E-05, 0.0952962711453438, -0.0720320492982864, new int[] { 14, 0, 2, 2, -1 }, new int[] { 15, 0, 1, 1, 2 }, new int[] { 14, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0147060392191634E-05, -0.0706219524145126, 0.0916848704218864, new int[] { 2, 0, 2, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }, new int[] { 3, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0007700761198066175, -0.0312023907899857, 0.0549915507435799, new int[] { 16, 0, 1, 4, -1 }, new int[] { 16, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00967198796570301, -0.0433308891952038, 0.1151764988899231, new int[] { 0, 3, 6, 2, -1 }, new int[] { 3, 3, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007568074855953455, -0.0232947506010532, 0.206037700176239, new int[] { 16, 1, 2, 6, -1 }, new int[] { 17, 1, 1, 3, 2 }, new int[] { 16, 4, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00050460308557376266, 0.051032479852438, -0.11277139931917191, new int[] { 1, 13, 2, 2, -1 }, new int[] { 2, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003729179035872221, 0.0791396573185921, -0.0201081596314907, new int[] { 16, 1, 2, 6, -1 }, new int[] { 17, 1, 1, 3, 2 }, new int[] { 16, 4, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0155905103310943, 0.017876299098134, -0.3296821117401123, new int[] { 5, 0, 3, 4, -1 }, new int[] { 5, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.054314311593771, -0.56021261215209961, 0.0010424769716337323, true, new int[] { 16, 0, 2, 3, -1 }, new int[] { 15, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0078423749655485153, -0.0343349911272526, 0.177660197019577, true, new int[] { 2, 0, 3, 2, -1 }, new int[] { 3, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069496310316026211, 0.0119108697399497, -0.28336969017982477, new int[] { 11, 2, 3, 3, -1 }, new int[] { 11, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0042853900231420994, -0.23308420181274411, 0.0223415307700634, new int[] { 4, 2, 3, 3, -1 }, new int[] { 4, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.866586095886305E-05, -0.0438981205224991, 0.0437583401799202, new int[] { 10, 3, 1, 2, -1 }, new int[] { 10, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.6118220527423546E-05, 0.0808287113904953, -0.0694800913333893, new int[] { 7, 3, 2, 2, -1 }, new int[] { 7, 3, 1, 1, 2 }, new int[] { 8, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0484328605234623, -0.79129552841186523, 0.0065139750950038433, new int[] { 6, 8, 6, 2, -1 }, new int[] { 6, 9, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0152241997420788, -0.0400892198085785, 0.13455769419670111, new int[] { 0, 10, 9, 3, -1 }, new int[] { 3, 10, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0128723401576281, 0.0560490600764751, -0.0245438907295465, new int[] { 6, 12, 10, 1, -1 }, new int[] { 6, 12, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0282472502440214, -0.0394716411828995, 0.15137889981269839, new int[] { 2, 12, 8, 3, -1 }, new int[] { 6, 12, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074682589620351791, 0.0130424499511719, -0.20481270551681521, new int[] { 13, 12, 4, 2, -1 }, new int[] { 14, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0469749011099339, 0.80171698331832886, -0.0071750162169337273, new int[] { 4, 11, 3, 4, -1 }, new int[] { 4, 12, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.013225469738245, -0.013960069976747, 0.1729875057935715, true, new int[] { 13, 10, 2, 2, -1 }, new int[] { 13, 10, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021193178836256266, 0.046903520822525, -0.1572621017694473, true, new int[] { 5, 10, 2, 2, -1 }, new int[] { 5, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.21487170457839971, 0.0037922300398349762, -0.38143840432167048, true, new int[] { 11, 0, 6, 9, -1 }, new int[] { 13, 2, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.15091340243816381, -0.013922699727118, 0.4097478985786438, true, new int[] { 6, 2, 8, 3, -1 }, new int[] { 8, 4, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.23029349744319921, -0.58206570148468018, 0.0011216839775443077, true, new int[] { 11, 0, 6, 9, -1 }, new int[] { 13, 2, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14030410349369049, 0.0169044900685549, -0.36825358867645258, true, new int[] { 7, 0, 9, 6, -1 }, new int[] { 5, 2, 9, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.0036112447269261E-05, -0.0551543496549129, 0.0726215615868568, new int[] { 9, 3, 2, 2, -1 }, new int[] { 10, 3, 1, 1, 2 }, new int[] { 9, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.49608469009399409, 0.0073583098128438, -0.70183300971984863, new int[] { 3, 2, 10, 13, -1 }, new int[] { 8, 2, 5, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0023255969863384962, -0.14822499454021451, 0.0326147899031639, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0138854403048754, 0.16097649931907651, -0.0331473685801029, new int[] { 5, 0, 7, 8, -1 }, new int[] { 5, 2, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0076077110134065151, -0.50956517457962036, 0.0050284918397665024, new int[] { 9, 6, 1, 3, -1 }, new int[] { 9, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029671129304915667, 0.0319776199758053, -0.19695889949798581, new int[] { 8, 6, 1, 3, -1 }, new int[] { 8, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055358321405947208, -0.0565205812454224, 0.1075361967086792, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0710219964385033, 0.0791943371295929, -0.0813843309879303, new int[] { 0, 7, 17, 4, -1 }, new int[] { 0, 9, 17, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0458000712096691, -0.0307503994554281, 0.15652079880237579, new int[] { 12, 6, 6, 9, -1 }, new int[] { 12, 9, 6, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0067807468585669994, 0.0189444404095411, -0.30112281441688538, new int[] { 1, 0, 3, 3, -1 }, new int[] { 2, 0, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.001945507014170289, 0.12722960114479071, -0.0254848394542933, new int[] { 11, 8, 2, 2, -1 }, new int[] { 12, 8, 1, 1, 2 }, new int[] { 11, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1861845999956131, 0.0090244021266698837, -0.54486262798309326, new int[] { 0, 8, 18, 4, -1 }, new int[] { 0, 10, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.9605968999676406E-05, 0.0626633614301682, -0.0534323900938034, new int[] { 9, 0, 1, 3, -1 }, new int[] { 9, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.023714829236269, -0.60180211067199707, 0.0079368790611624718, new int[] { 0, 4, 18, 2, -1 }, new int[] { 0, 4, 9, 1, 2 }, new int[] { 9, 5, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0313583016395569, -0.17721989750862119, 0.0092706838622689247, new int[] { 5, 2, 12, 4, -1 }, new int[] { 11, 2, 6, 2, 2 }, new int[] { 5, 4, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0349689982831478, 0.3794535100460053, -0.0169909205287695, new int[] { 1, 2, 12, 4, -1 }, new int[] { 1, 2, 6, 2, 2 }, new int[] { 7, 4, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0624166503548622, -0.41591739654541021, 0.0048467209562659264, true, new int[] { 15, 4, 1, 8, -1 }, new int[] { 13, 6, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0422837510704994, 0.0098220221698284149, -0.47655558586120611, true, new int[] { 3, 4, 8, 1, -1 }, new int[] { 5, 6, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051127527840435505, -0.0367820709943771, 0.16474020481109619, new int[] { 9, 8, 8, 2, -1 }, new int[] { 13, 8, 4, 1, 2 }, new int[] { 9, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0112114502117038, 0.18803590536117551, -0.0276528596878052, new int[] { 4, 8, 6, 2, -1 }, new int[] { 4, 8, 3, 1, 2 }, new int[] { 7, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052367132157087326, 0.0286790002137423, -0.17751020193099981, new int[] { 3, 3, 12, 2, -1 }, new int[] { 9, 3, 6, 1, 2 }, new int[] { 3, 4, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3686140745412558E-05, 0.0753717795014381, -0.0666650682687759, new int[] { 4, 1, 1, 4, -1 }, new int[] { 4, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0128402002155781, 0.0218078903853893, -0.12720319628715521, new int[] { 9, 5, 3, 3, -1 }, new int[] { 10, 6, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0427928082644939, 0.0075381440110504627, -0.71861362457275391, new int[] { 0, 1, 3, 14, -1 }, new int[] { 1, 1, 1, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052706589922308922, 0.0988220199942589, -0.044858880341053, true, new int[] { 16, 6, 2, 3, -1 }, new int[] { 15, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00082180468598380685, -0.10595670342445369, 0.044027641415596, new int[] { 4, 1, 3, 2, -1 }, new int[] { 4, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0192952807992697, -0.41217219829559332, 0.002904857974499464, new int[] { 16, 1, 2, 6, -1 }, new int[] { 17, 1, 1, 3, 2 }, new int[] { 16, 4, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020072490442544222, 0.1149147972464562, -0.0455907806754112, new int[] { 0, 1, 2, 6, -1 }, new int[] { 0, 1, 1, 3, 2 }, new int[] { 1, 4, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.055046308785677, 0.1894032955169678, -0.011900239624083, new int[] { 6, 0, 9, 7, -1 }, new int[] { 9, 0, 3, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1124947965145111, 0.24269099533557889, -0.0220534801483154, new int[] { 3, 0, 9, 7, -1 }, new int[] { 6, 0, 3, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0095265945419669151, -0.0385538190603256, 0.0301385801285505, true, new int[] { 9, 0, 1, 6, -1 }, new int[] { 9, 0, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0088573405519127846, -0.0646601468324661, 0.0850300714373589, true, new int[] { 9, 0, 6, 1, -1 }, new int[] { 9, 0, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053099901415407658, -0.0779245272278786, 0.0518223904073238, new int[] { 11, 3, 5, 4, -1 }, new int[] { 11, 5, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1524796932935715, 0.0170198101550341, -0.28019899129867548, true, new int[] { 9, 0, 9, 6, -1 }, new int[] { 7, 2, 9, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0514544583857059, -0.2223165035247803, 0.008854166604578495, true, new int[] { 9, 6, 4, 3, -1 }, new int[] { 9, 6, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0254663806408644, -0.054948728531599, 0.0890722572803497, new int[] { 7, 3, 4, 7, -1 }, new int[] { 9, 3, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25437718629837042, 0.0020636660046875477, -0.87088710069656372, true, new int[] { 12, 1, 4, 6, -1 }, new int[] { 10, 3, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.22862739861011511, 0.20034660398960111, -0.025318780913949, new int[] { 4, 1, 4, 14, -1 }, new int[] { 4, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0118133397772908, 0.13387170433998111, -0.0365035310387611, new int[] { 1, 6, 16, 3, -1 }, new int[] { 1, 7, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0201183203607798, -0.20123849809169769, 0.0280736796557903, new int[] { 5, 6, 6, 3, -1 }, new int[] { 7, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0217740796506405, -0.0065130768343806267, 0.280221700668335, new int[] { 9, 8, 8, 2, -1 }, new int[] { 13, 8, 4, 1, 2 }, new int[] { 9, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048404871486127377, -0.029814250767231, 0.15977649390697479, new int[] { 1, 8, 8, 2, -1 }, new int[] { 1, 8, 4, 1, 2 }, new int[] { 5, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00011922290286747739, 0.0340446382761002, -0.160576805472374, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0072792158462107182, -0.48334380984306341, 0.0099527724087238312, new int[] { 0, 9, 1, 4, -1 }, new int[] { 0, 10, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.590489930298645E-05, -0.0381436906754971, 0.0470281802117825, new int[] { 10, 8, 2, 2, -1 }, new int[] { 11, 8, 1, 1, 2 }, new int[] { 10, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0909861028194427, 0.26971128582954412, -0.0179479792714119, new int[] { 6, 4, 5, 8, -1 }, new int[] { 6, 6, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.20878769457340241, 0.23006640374660489, -0.0216091796755791, new int[] { 1, 8, 16, 6, -1 }, new int[] { 1, 10, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070507721975445747, -0.2504821121692658, 0.0200520195066929, true, new int[] { 6, 5, 3, 1, -1 }, new int[] { 7, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0089825186878442764, -0.0180237293243408, 0.29516848921775818, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 8, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0597062110900879, -0.0128449099138379, 0.3559386134147644, true, new int[] { 8, 3, 6, 1, -1 }, new int[] { 10, 5, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103647699579597, -0.20093110203742981, 0.0278272200375795, new int[] { 0, 4, 18, 2, -1 }, new int[] { 9, 4, 9, 1, 2 }, new int[] { 0, 5, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0194542594254017, -0.53035300970077515, 0.00907062366604805, new int[] { 0, 9, 2, 5, -1 }, new int[] { 1, 9, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021027070470154285, 0.0885996073484421, -0.0361577197909355, true, new int[] { 17, 5, 1, 3, -1 }, new int[] { 16, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075333649292588234, -0.0244578700512648, 0.1936513036489487, true, new int[] { 1, 5, 3, 1, -1 }, new int[] { 2, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051182601600885391, 0.0174081493169069, -0.22554570436477661, new int[] { 17, 6, 1, 4, -1 }, new int[] { 17, 7, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021947720088064671, 0.0296904593706131, -0.19585029780864721, new int[] { 0, 6, 1, 4, -1 }, new int[] { 0, 7, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0412029810249805, -0.0132970996201038, 0.1000028029084206, new int[] { 14, 3, 4, 6, -1 }, new int[] { 14, 5, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0161616802215576, 0.0401702187955379, -0.132104903459549, new int[] { 0, 3, 4, 6, -1 }, new int[] { 0, 5, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12740600109100339, 0.0092737795785069466, -0.2394157946109772, new int[] { 7, 7, 6, 6, -1 }, new int[] { 9, 9, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026743640191853046, 0.23251029849052429, -0.0232730191200972, new int[] { 7, 10, 2, 2, -1 }, new int[] { 7, 10, 1, 1, 2 }, new int[] { 8, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1170528009533882, -0.21834470331668851, 0.0135161597281694, new int[] { 2, 9, 16, 3, -1 }, new int[] { 6, 9, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0066700777970254421, -0.0436670817434788, 0.10799729824066159, true, new int[] { 3, 4, 3, 2, -1 }, new int[] { 4, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0400560796260834, -0.0068564810790121555, 0.29377210140228271, true, new int[] { 15, 5, 3, 3, -1 }, new int[] { 14, 6, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0045556342229247093, 0.1104653999209404, -0.0465722493827343, true, new int[] { 3, 5, 3, 3, -1 }, new int[] { 4, 6, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.031573511660099, 0.0098816202953457832, -0.41573968529701227, new int[] { 10, 2, 3, 10, -1 }, new int[] { 11, 2, 1, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0248094201087952, -0.33196479082107538, 0.0140330903232098, new int[] { 3, 2, 4, 5, -1 }, new int[] { 4, 2, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0002840495144482702, -0.0977882891893387, 0.0236715003848076, true, new int[] { 13, 4, 2, 2, -1 }, new int[] { 13, 4, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.004079878795892, 0.0679533332586288, -0.0907793864607811, true, new int[] { 5, 4, 2, 2, -1 }, new int[] { 5, 4, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0226807501167059, -0.80813902616500854, 0.0031646140851080418, new int[] { 9, 2, 1, 6, -1 }, new int[] { 9, 4, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016572299646213651, 0.1429641991853714, -0.0321753397583961, new int[] { 6, 13, 3, 1, -1 }, new int[] { 7, 13, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0209627896547318, -0.75405949354171753, 0.003187268041074276, new int[] { 9, 8, 2, 6, -1 }, new int[] { 10, 8, 1, 3, 2 }, new int[] { 9, 11, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010227429447695613, 0.0832900702953339, -0.0552086904644966, new int[] { 7, 12, 4, 2, -1 }, new int[] { 8, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030178760644048452, -0.0410230606794357, 0.0196295809000731, true, new int[] { 9, 0, 3, 7, -1 }, new int[] { 10, 1, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1914006024599075, 0.0175436791032553, -0.255665510892868, true, new int[] { 6, 3, 6, 6, -1 }, new int[] { 6, 3, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0189527608454227, 0.32863160967826838, -0.0048918230459094048, new int[] { 15, 10, 3, 2, -1 }, new int[] { 15, 11, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00452493317425251, -0.1561917066574097, 0.0295387599617243, new int[] { 0, 8, 3, 3, -1 }, new int[] { 0, 9, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0049335299991071224, -0.153610497713089, 0.01271259970963, true, new int[] { 9, 2, 1, 3, -1 }, new int[] { 8, 3, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0189859308302403, -0.0395853891968727, 0.12031179666519171, new int[] { 1, 6, 16, 3, -1 }, new int[] { 1, 7, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015369809698313475, 0.0511838011443615, -0.0198078006505966, true, new int[] { 9, 9, 1, 2, -1 }, new int[] { 9, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0313022881746292, 0.00790486391633749, -0.5422518253326416, new int[] { 7, 9, 3, 3, -1 }, new int[] { 8, 10, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.000790994381532073, 0.0733341798186302, -0.024761039763689, new int[] { 17, 0, 1, 3, -1 }, new int[] { 17, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.5027391024632379E-05, -0.0677618235349655, 0.0672639682888985, new int[] { 0, 0, 1, 3, -1 }, new int[] { 0, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1923059800174087E-05, -0.0342731587588787, 0.0385947003960609, new int[] { 17, 0, 1, 2, -1 }, new int[] { 17, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.709586912416853E-05, 0.0838238298892975, -0.0660852268338203, new int[] { 0, 0, 1, 2, -1 }, new int[] { 0, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1215929016470909, -0.70010262727737427, 0.0018631670391187072, new int[] { 13, 2, 4, 6, -1 }, new int[] { 13, 5, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0174945406615734, 0.0259598605334759, -0.1810075044631958, new int[] { 1, 2, 4, 6, -1 }, new int[] { 1, 5, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0633600726723671, 0.13021109998226171, -0.0088773788884282112, new int[] { 4, 0, 12, 11, -1 }, new int[] { 8, 0, 4, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.39351868629455572, -0.63525807857513428, 0.008234822191298008, new int[] { 0, 1, 12, 14, -1 }, new int[] { 6, 1, 6, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0147491302341223, 0.0573673695325851, -0.07745411247015, new int[] { 10, 5, 8, 9, -1 }, new int[] { 12, 5, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074586831033229828, -0.0738315135240555, 0.0729713514447212, new int[] { 0, 12, 18, 2, -1 }, new int[] { 9, 12, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.0465059505077079E-05, -0.0687413066625595, 0.0833826810121536, new int[] { 8, 14, 2, 1, -1 }, new int[] { 8, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.3182349549606442E-05, -0.0648377612233162, 0.0794876664876938, new int[] { 8, 13, 2, 2, -1 }, new int[] { 8, 13, 1, 1, 2 }, new int[] { 9, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0179907493293285, -0.34188538789749151, 0.0082358242943882942, new int[] { 9, 11, 4, 4, -1 }, new int[] { 10, 11, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017810800345614553, 0.0831420794129372, -0.0662932470440865, new int[] { 5, 11, 4, 4, -1 }, new int[] { 6, 11, 2, 4, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 17
            stage = new HaarCascadeStage(-1.1554880142211914, 16, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.52821958065032959, -0.1120738014578819, 0.46492001414299011, new int[] { 3, 2, 12, 9, -1 }, new int[] { 7, 5, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0083934608846902847, 0.1242000982165337, -0.0984233617782593, new int[] { 11, 8, 6, 3, -1 }, new int[] { 11, 8, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0125337103381753, 0.1294067054986954, -0.21826070547103879, new int[] { 2, 7, 8, 4, -1 }, new int[] { 4, 7, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026514590717852116, 0.10746669769287109, -0.0652235969901085, new int[] { 14, 2, 4, 2, -1 }, new int[] { 14, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0012469879584386945, 0.094827726483345, -0.19725410640239721, new int[] { 8, 6, 2, 2, -1 }, new int[] { 8, 7, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0105062201619148, -0.1786229014396668, 0.0707185864448547, true, new int[] { 9, 5, 4, 2, -1 }, new int[] { 10, 6, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0014628679491579533, 0.0773052126169205, -0.1588167995214462, new int[] { 0, 2, 4, 2, -1 }, new int[] { 2, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011747170239687, 0.0412793383002281, -0.1657488942146301, true, new int[] { 10, 6, 3, 2, -1 }, new int[] { 11, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021636099554598331, -0.0817365422844887, 0.1844726949930191, true, new int[] { 8, 6, 2, 3, -1 }, new int[] { 7, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0156048499047756, 0.1840981990098953, 0.0091587323695421219, true, new int[] { 11, 8, 2, 2, -1 }, new int[] { 11, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.006790901068598032, 0.1927130073308945, -0.0610056594014168, true, new int[] { 7, 8, 2, 2, -1 }, new int[] { 7, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046382728032767773, 0.0721243992447853, -0.15475249290466311, new int[] { 8, 8, 2, 4, -1 }, new int[] { 8, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.105950802564621, 0.169883206486702, -0.0774008184671402, new int[] { 5, 3, 8, 6, -1 }, new int[] { 5, 6, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0222781002521515, 0.0300818495452404, -0.3189120888710022, true, new int[] { 13, 0, 2, 4, -1 }, new int[] { 13, 0, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0383511297404766, -0.0293571297079325, 0.37845009565353388, new int[] { 4, 5, 10, 4, -1 }, new int[] { 4, 7, 10, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0127405496314168, 0.0121086901053786, -0.2898040115833283, new int[] { 10, 14, 6, 1, -1 }, new int[] { 12, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0119678396731615, -0.27529829740524292, 0.0334202796220779, new int[] { 5, 0, 3, 6, -1 }, new int[] { 5, 3, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062382412143051624, 0.0232270695269108, -0.2876886129379273, new int[] { 7, 0, 6, 2, -1 }, new int[] { 7, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012571290135383606, -0.1228341981768608, 0.0775459334254265, new int[] { 0, 10, 18, 2, -1 }, new int[] { 0, 11, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.097746416926384, 0.0120771396905184, -0.32092699408531189, new int[] { 14, 3, 4, 12, -1 }, new int[] { 14, 9, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0059180860407650471, -0.22756209969520569, 0.0447532683610916, new int[] { 3, 0, 12, 3, -1 }, new int[] { 3, 1, 12, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0064139030873775482, 0.040146928280592, -0.0504605211317539, true, new int[] { 9, 7, 1, 2, -1 }, new int[] { 9, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0092285759747028351, 0.0234754905104637, -0.377289205789566, new int[] { 2, 14, 6, 1, -1 }, new int[] { 4, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026009760331362486, 0.05803607031703, -0.0397480018436909, true, new int[] { 9, 0, 2, 1, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0045100939460098743, -0.150070995092392, 0.0647656172513962, new int[] { 3, 0, 12, 1, -1 }, new int[] { 6, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.30929979681968689, -0.36162209510803223, 0.0052778669632971287, new int[] { 8, 0, 10, 15, -1 }, new int[] { 8, 0, 5, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.166436105966568, 0.0580257400870323, -0.1667063981294632, new int[] { 0, 0, 10, 15, -1 }, new int[] { 5, 0, 5, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.029249170795083, -0.1041812002658844, 0.047381941229105, new int[] { 15, 0, 1, 14, -1 }, new int[] { 15, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0578976906836033, -0.0827134624123573, 0.1230174973607063, true, new int[] { 9, 1, 9, 2, -1 }, new int[] { 12, 4, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0439998507499695, 0.0031090460252016783, -0.38884210586547852, new int[] { 15, 0, 1, 14, -1 }, new int[] { 15, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1334455013275147, -0.27564039826393127, 0.03073425963521, new int[] { 3, 5, 12, 10, -1 }, new int[] { 3, 5, 6, 5, 2 }, new int[] { 9, 10, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0084765329957008362, 0.0265623796731234, -0.28648358583450317, new int[] { 1, 0, 16, 2, -1 }, new int[] { 9, 0, 8, 1, 2 }, new int[] { 1, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0062942858785390854, 0.0198616907000542, -0.36465620994567871, new int[] { 0, 6, 2, 3, -1 }, new int[] { 0, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0118541996926069, -0.0481690689921379, 0.1577796936035156, true, new int[] { 14, 1, 4, 3, -1 }, new int[] { 13, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10978949815034871, -0.21610009670257571, 0.0352399796247482, new int[] { 0, 0, 18, 10, -1 }, new int[] { 0, 0, 9, 5, 2 }, new int[] { 9, 5, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012859810376539826, -0.0768053531646729, 0.0990003198385239, new int[] { 10, 2, 2, 2, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.10880099982023241, -0.0982203707098961, 0.1162839010357857, new int[] { 4, 0, 10, 8, -1 }, new int[] { 9, 0, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0142060602083802, 0.0048896879889070988, -0.38383349776268011, new int[] { 8, 3, 4, 1, -1 }, new int[] { 8, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0132633903995156, 0.0221766997128725, -0.38806360960006708, new int[] { 6, 3, 4, 1, -1 }, new int[] { 8, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079566845670342445, -0.0713148191571236, 0.0741146504878998, new int[] { 3, 12, 15, 2, -1 }, new int[] { 3, 13, 15, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0769576579332352, -0.0361662209033966, 0.25757679343223572, new int[] { 0, 7, 18, 8, -1 }, new int[] { 0, 9, 18, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0100203501060605, -0.0785313323140144, 0.0633838027715683, new int[] { 11, 4, 3, 6, -1 }, new int[] { 11, 6, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00920175202190876, 0.0293919891119003, -0.25732880830764771, true, new int[] { 3, 3, 2, 3, -1 }, new int[] { 2, 4, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0307231806218624, -0.0187381394207478, 0.22832340002059939, true, new int[] { 15, 3, 3, 3, -1 }, new int[] { 14, 4, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0110199600458145, -0.0532967299222946, 0.174945205450058, true, new int[] { 3, 3, 3, 3, -1 }, new int[] { 4, 4, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0274540707468987, 0.1702467948198319, -0.0082028387114405632, true, new int[] { 15, 2, 3, 3, -1 }, new int[] { 14, 3, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0136898197233677, 0.2001978009939194, -0.0419919602572918, true, new int[] { 3, 2, 3, 3, -1 }, new int[] { 4, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081678535789251328, -0.26262301206588751, 0.0103546399623156, true, new int[] { 9, 0, 3, 2, -1 }, new int[] { 10, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0100999800488353, -0.044948261231184, 0.18523730337619779, true, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.20024929940700531, -0.0368244796991348, 0.2407283037900925, new int[] { 3, 5, 12, 10, -1 }, new int[] { 3, 5, 6, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037789789494127035, -0.13910900056362149, 0.0761268436908722, true, new int[] { 9, 1, 1, 4, -1 }, new int[] { 9, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0111010000109673, 0.23991499841213229, -0.0364109985530376, new int[] { 6, 0, 6, 5, -1 }, new int[] { 8, 0, 2, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0620720200240612, 0.0276025105267763, -0.297624409198761, new int[] { 0, 1, 18, 6, -1 }, new int[] { 0, 1, 9, 3, 2 }, new int[] { 9, 4, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00049415021203458309, 0.0430329516530037, -0.1610901951789856, new int[] { 10, 7, 1, 2, -1 }, new int[] { 10, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035258450079709291, -0.17413130402565, 0.0575136989355087, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 7, 3, 1, 2 }, new int[] { 9, 8, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0056127668358385563, -0.0242344699800015, 0.0987889915704727, new int[] { 12, 6, 1, 6, -1 }, new int[] { 12, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037660789676010609, -0.0366232991218567, 0.2009083032608032, new int[] { 5, 9, 4, 2, -1 }, new int[] { 5, 9, 2, 1, 2 }, new int[] { 7, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0154594099149108, 0.00766490213572979, -0.20163559913635251, true, new int[] { 9, 0, 3, 2, -1 }, new int[] { 10, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103579899296165, -0.42395240068435669, 0.0170050095766783, true, new int[] { 9, 0, 2, 3, -1 }, new int[] { 8, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0131801199167967, -0.28122058510780329, 0.0253022592514753, true, new int[] { 14, 0, 4, 2, -1 }, new int[] { 15, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.3639352023601532, 0.0106940995901823, -0.65183037519454956, true, new int[] { 8, 0, 9, 6, -1 }, new int[] { 8, 0, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0457970909774303, -0.0010829409584403038, -0.60917937755584717, new int[] { 6, 9, 8, 2, -1 }, new int[] { 10, 9, 4, 1, 2 }, new int[] { 6, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0168178994208574, 0.24067279696464541, -0.0288416408002377, new int[] { 4, 9, 8, 2, -1 }, new int[] { 4, 9, 4, 1, 2 }, new int[] { 8, 10, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0699327737092972, -0.24569059908390051, 0.00014374910097103566, new int[] { 15, 0, 1, 14, -1 }, new int[] { 15, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0370729491114616, 0.0120472796261311, -0.61824947595596313, new int[] { 2, 0, 1, 14, -1 }, new int[] { 2, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0022509139962494373, -0.13868570327758789, 0.0234417803585529, new int[] { 17, 9, 1, 3, -1 }, new int[] { 17, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0411305986344814, -0.49580198526382452, 0.0126163000240922, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 8, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.0702746585011482, 0.0652459263801575, new int[] { 17, 10, 1, 4, -1 }, new int[] { 17, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0032828738912940025, -0.21801410615444181, 0.0284525100141764, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0589578114449978, -0.11310169845819471, 0.0356478206813335, new int[] { 14, 2, 4, 8, -1 }, new int[] { 14, 6, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(3.2863670639926568E-05, -0.0697758123278618, 0.0949401631951332, new int[] { 1, 9, 16, 2, -1 }, new int[] { 1, 10, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0730367004871368, 0.1069146022200584, -0.0896811932325363, new int[] { 5, 7, 8, 6, -1 }, new int[] { 5, 10, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1058195978403091, 0.1823062002658844, -0.0388196706771851, new int[] { 0, 2, 16, 8, -1 }, new int[] { 0, 2, 8, 4, 2 }, new int[] { 8, 6, 8, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00046694820048287511, -0.1007533967494965, 0.0651198998093605, new int[] { 14, 10, 4, 4, -1 }, new int[] { 16, 10, 2, 2, 2 }, new int[] { 14, 12, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025920490734279156, -0.2544820904731751, 0.0231018606573343, new int[] { 0, 9, 1, 3, -1 }, new int[] { 0, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104395002126694, 0.0040941308252513409, -0.58273357152938843, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.0606367290019989, -0.10014739632606511, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0022808990906924009, 0.18519909679889679, -0.0254341196268797, new int[] { 16, 0, 1, 4, -1 }, new int[] { 16, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020937379449605942, -0.1919911056756973, 0.0333683788776398, new int[] { 0, 10, 1, 4, -1 }, new int[] { 0, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.21821799874305731, 0.30659881234169012, -0.0218403805047274, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 11, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.011518020182848, -0.1070621013641357, 0.0582328587770462, true, new int[] { 3, 0, 8, 2, -1 }, new int[] { 3, 0, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0315043888986111, 0.11767739802598951, -0.0459064915776253, new int[] { 8, 9, 10, 6, -1 }, new int[] { 13, 9, 5, 3, 2 }, new int[] { 8, 12, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0294614192098379, -0.2296009957790375, 0.0288945809006691, new int[] { 1, 12, 12, 1, -1 }, new int[] { 5, 12, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0079243192449212074, 0.14196249842643741, -0.0125654498115182, true, new int[] { 13, 0, 2, 8, -1 }, new int[] { 11, 2, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041360300965607166, -0.0285923406481743, 0.20373730361461639, new int[] { 1, 8, 4, 2, -1 }, new int[] { 1, 8, 2, 1, 2 }, new int[] { 3, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104305995628238, -0.0423329882323742, 0.0525090992450714, new int[] { 12, 4, 2, 6, -1 }, new int[] { 12, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.2438413947820664, 0.3361566960811615, -0.0189900696277618, new int[] { 4, 0, 10, 8, -1 }, new int[] { 9, 0, 5, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00756867416203022, 0.0064027151092886925, -0.30588310956954962, true, new int[] { 9, 0, 2, 1, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012688450515270233, -0.0901417508721352, 0.0729410126805305, true, new int[] { 9, 0, 1, 2, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0308157093822956, 0.0029594700317829847, -0.2435165941715241, new int[] { 13, 11, 3, 4, -1 }, new int[] { 13, 12, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021978209260851145, -0.0633767321705818, 0.1006520017981529, new int[] { 2, 11, 3, 4, -1 }, new int[] { 2, 12, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.006128211971372366, -0.0383862592279911, 0.0665621683001518, new int[] { 8, 13, 7, 2, -1 }, new int[] { 8, 14, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028037100564688444, 0.035719309002161, -0.15420930087566381, true, new int[] { 3, 1, 1, 3, -1 }, new int[] { 2, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0039568650536239147, 0.0709167122840881, -0.0399580597877502, true, new int[] { 16, 0, 2, 3, -1 }, new int[] { 15, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0139292301610112, -0.0233923103660345, 0.28147700428962708, true, new int[] { 2, 0, 3, 2, -1 }, new int[] { 3, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0101550603285432, -0.140423595905304, 0.01851562038064, new int[] { 13, 0, 4, 3, -1 }, new int[] { 14, 0, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0146013703197241, 0.0123592196032405, -0.54975450038909912, new int[] { 1, 0, 4, 3, -1 }, new int[] { 2, 0, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.3091858717380092E-05, -0.0439675599336624, 0.0347095616161823, new int[] { 16, 0, 1, 4, -1 }, new int[] { 16, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00210163788869977, 0.2275288999080658, -0.0287020802497864, new int[] { 1, 0, 1, 4, -1 }, new int[] { 1, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0064648198895156384, 0.0181927904486656, -0.22275130450725561, true, new int[] { 14, 0, 4, 1, -1 }, new int[] { 15, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026089660823345184, -0.1483312994241715, 0.0421623699367046, true, new int[] { 4, 0, 1, 4, -1 }, new int[] { 3, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0491728708148003, 0.182160496711731, -0.0349443815648556, new int[] { 0, 4, 18, 6, -1 }, new int[] { 0, 6, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017964000580832362, 0.0488241016864777, -0.1821431964635849, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00738500477746129, -0.0418660007417202, 0.1861997991800308, new int[] { 1, 8, 16, 2, -1 }, new int[] { 1, 9, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0205026101320982, -0.0581343583762646, 0.13789500296115881, true, new int[] { 3, 4, 8, 2, -1 }, new int[] { 3, 4, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1163681969046593, -0.0551596693694592, 0.067019522190094, new int[] { 6, 0, 9, 11, -1 }, new int[] { 9, 0, 3, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058732312172651291, 0.23400300741195679, -0.0273893792182207, new int[] { 7, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.28881600499153143, 0.019362909719348, -0.161901205778122, new int[] { 4, 0, 12, 11, -1 }, new int[] { 7, 0, 6, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1196641996502876, 0.24559150636196139, -0.0259939599782228, new int[] { 2, 0, 12, 11, -1 }, new int[] { 5, 0, 6, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038372459821403027, -0.138967901468277, 0.0567790493369102, new int[] { 11, 2, 6, 4, -1 }, new int[] { 11, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011065569706261158, -0.16209499537944791, 0.036041721701622, new int[] { 5, 1, 2, 2, -1 }, new int[] { 5, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0863595679402351, -0.0102093601599336, 0.25007158517837519, true, new int[] { 10, 3, 2, 6, -1 }, new int[] { 8, 5, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0359533615410328, -0.75698071718215942, 0.0081533808261156082, new int[] { 4, 2, 8, 4, -1 }, new int[] { 4, 3, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0827576965093613, -0.0119722299277782, 0.13151499629020691, true, new int[] { 11, 1, 2, 8, -1 }, new int[] { 9, 3, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14555160701274869, 0.0256695207208395, -0.23377719521522519, new int[] { 0, 1, 18, 14, -1 }, new int[] { 0, 1, 9, 7, 2 }, new int[] { 9, 8, 9, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0666986927390099, 0.0182299092411995, -0.123862698674202, new int[] { 13, 5, 5, 10, -1 }, new int[] { 13, 10, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0987812727689743, -0.0197382606565952, 0.32106879353523249, true, new int[] { 9, 3, 6, 2, -1 }, new int[] { 11, 5, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.28243279457092291, -0.54694139957427979, 0.002388776047155261, true, new int[] { 9, 0, 7, 8, -1 }, new int[] { 7, 2, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2101342976093292, 0.0181991197168827, -0.36248031258583069, true, new int[] { 9, 0, 8, 7, -1 }, new int[] { 11, 2, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00065322709269821644, 0.0552163012325764, -0.030892439186573, true, new int[] { 13, 2, 4, 3, -1 }, new int[] { 12, 3, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0345937386155128, 0.33557349443435669, -0.0155041199177504, true, new int[] { 5, 2, 3, 4, -1 }, new int[] { 6, 3, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052095651626586914, -0.25957450270652771, 0.0123718800023198, new int[] { 10, 0, 3, 3, -1 }, new int[] { 10, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.067268118262291, -0.0627519264817238, 0.0915589928627014, new int[] { 2, 3, 13, 6, -1 }, new int[] { 2, 5, 13, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040582818910479546, 0.0410736314952374, -0.156754806637764, new int[] { 8, 5, 2, 3, -1 }, new int[] { 8, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0444693900644779, -0.19344259798526761, 0.0311934594064951, new int[] { 0, 3, 4, 6, -1 }, new int[] { 0, 6, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0028536471072584391, -0.0742046609520912, 0.0826525837182999, new int[] { 1, 9, 16, 2, -1 }, new int[] { 1, 10, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1215196028351784, -0.0172205492854118, 0.37725690007209778, true, new int[] { 5, 5, 6, 4, -1 }, new int[] { 5, 5, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0527439787983894, 0.0073638479225337505, -0.39580649137496948, new int[] { 13, 0, 2, 8, -1 }, new int[] { 13, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0133668296039104, 0.0302810091525316, -0.1715900003910065, new int[] { 3, 0, 2, 8, -1 }, new int[] { 3, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007848663255572319, -0.0223950203508139, 0.1505244970321655, true, new int[] { 16, 1, 2, 3, -1 }, new int[] { 15, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0038255099207162857, 0.13788110017776489, -0.0390050299465656, true, new int[] { 1, 3, 3, 1, -1 }, new int[] { 2, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14737069606781009, 0.0984983816742897, -0.0175660997629166, new int[] { 13, 4, 5, 10, -1 }, new int[] { 13, 9, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0714110434055328, 0.0232200995087624, -0.26759588718414312, new int[] { 0, 4, 5, 10, -1 }, new int[] { 0, 9, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0166891291737556, -0.0217618402093649, 0.14617420732975009, true, new int[] { 16, 5, 2, 3, -1 }, new int[] { 15, 6, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0032251640222966671, 0.1193147972226143, -0.0540297999978065, true, new int[] { 2, 5, 3, 2, -1 }, new int[] { 3, 6, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079702045768499374, -0.0543896183371544, 0.0729502886533737, new int[] { 10, 10, 8, 4, -1 }, new int[] { 14, 10, 4, 2, 2 }, new int[] { 10, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0116266896948218, 0.0324149206280708, -0.170573502779007, new int[] { 1, 5, 4, 9, -1 }, new int[] { 3, 5, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0332335010170937, -0.15321509540081019, 0.0276584308594465, new int[] { 10, 1, 8, 9, -1 }, new int[] { 10, 1, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0162025205790997, -0.0798396766185761, 0.0804151371121407, new int[] { 0, 1, 8, 9, -1 }, new int[] { 4, 1, 4, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0169930998235941, 0.1070884987711906, -0.0270955804735422, new int[] { 10, 10, 8, 4, -1 }, new int[] { 14, 10, 4, 2, 2 }, new int[] { 10, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052699539810419083, -0.0776714086532593, 0.0904784426093102, new int[] { 0, 9, 8, 4, -1 }, new int[] { 0, 9, 4, 2, 2 }, new int[] { 4, 11, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0112306997179985, -0.36888670921325678, 0.0147642102092505, new int[] { 3, 0, 14, 2, -1 }, new int[] { 10, 0, 7, 1, 2 }, new int[] { 3, 1, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0216833408921957, 0.0211919397115707, -0.2431215047836304, new int[] { 0, 13, 18, 2, -1 }, new int[] { 0, 13, 9, 1, 2 }, new int[] { 9, 14, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0027136749122291803, 0.12931990623474121, -0.0180541593581438, new int[] { 11, 13, 1, 2, -1 }, new int[] { 11, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038232649676501751, -0.0677571818232536, 0.0790435373783112, new int[] { 3, 13, 8, 2, -1 }, new int[] { 3, 14, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0129264900460839, 0.0228535197675228, -0.25793269276618958, new int[] { 4, 13, 10, 2, -1 }, new int[] { 9, 13, 5, 1, 2 }, new int[] { 4, 14, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026950810570269823, 0.21666090190410611, -0.0270976908504963, new int[] { 6, 13, 1, 2, -1 }, new int[] { 6, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.21591490507125849, 0.0046611670404672623, -0.8688737154006958, true, new int[] { 14, 0, 4, 9, -1 }, new int[] { 14, 0, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1681632995605469, 0.0141299199312925, -0.35010749101638788, true, new int[] { 4, 0, 9, 4, -1 }, new int[] { 4, 0, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0491994395852089, -0.77299457788467407, 0.00609645014628768, new int[] { 6, 5, 6, 3, -1 }, new int[] { 8, 6, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0261047407984734, 0.0061850231140851974, -0.66869372129440308, true, new int[] { 2, 9, 3, 2, -1 }, new int[] { 3, 10, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0145413503050804, 0.0050752838142216206, -0.74292498826980591, new int[] { 6, 0, 7, 2, -1 }, new int[] { 6, 1, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011107119498774409, -0.0341122597455978, 0.15071749687194819, new int[] { 4, 7, 1, 2, -1 }, new int[] { 4, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0107706598937511, -0.0934311375021935, 0.0101868798956275, true, new int[] { 9, 4, 4, 3, -1 }, new int[] { 10, 5, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0942776203155518, -0.0600805804133415, 0.0837868973612785, new int[] { 0, 0, 15, 6, -1 }, new int[] { 5, 0, 5, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1235508024692535, -0.0419926010072231, 0.0931324735283852, new int[] { 7, 0, 9, 14, -1 }, new int[] { 10, 0, 3, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.8364567756652832, 0.0113448603078723, -0.54795432090759277, new int[] { 0, 0, 18, 15, -1 }, new int[] { 6, 0, 6, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0352501794695854, -0.0108188204467297, 0.0904011875391006, true, new int[] { 9, 4, 4, 3, -1 }, new int[] { 10, 5, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1221748435636982E-05, 0.0795160531997681, -0.0667194202542305, new int[] { 7, 4, 2, 2, -1 }, new int[] { 7, 4, 1, 1, 2 }, new int[] { 8, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.716275674989447E-05, -0.0442888401448727, 0.0536684095859528, new int[] { 9, 4, 2, 2, -1 }, new int[] { 10, 4, 1, 1, 2 }, new int[] { 9, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046395221725106239, -0.0847273468971252, 0.0621006116271019, new int[] { 0, 11, 8, 4, -1 }, new int[] { 0, 11, 4, 2, 2 }, new int[] { 4, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0013368109939619899, -0.0803513526916504, 0.0279868002980947, new int[] { 16, 12, 2, 3, -1 }, new int[] { 16, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0337816514074802, 0.32461521029472351, -0.0163126401603222, new int[] { 0, 10, 8, 4, -1 }, new int[] { 0, 10, 4, 2, 2 }, new int[] { 4, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017830280121415854, -0.16490410268306729, 0.0217570792883635, new int[] { 12, 0, 2, 2, -1 }, new int[] { 12, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020984211005270481, 0.0295347701758146, -0.17951250076293951, new int[] { 4, 0, 2, 2, -1 }, new int[] { 4, 1, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3364270570455119E-05, 0.0443317405879498, -0.0367653109133244, new int[] { 9, 0, 2, 2, -1 }, new int[] { 10, 0, 1, 1, 2 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1226925998926163, 0.0124071799218655, -0.40553379058837891, new int[] { 0, 11, 11, 4, -1 }, new int[] { 0, 13, 11, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0949875265359879, -0.00035644270246848464, -0.99994051456451416, true, new int[] { 9, 4, 4, 3, -1 }, new int[] { 10, 5, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0637726783752441, 0.74163448810577393, -0.0068990588188171387, true, new int[] { 9, 4, 3, 4, -1 }, new int[] { 8, 5, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0555911287665367, -0.003510219044983387, 0.2164891064167023, true, new int[] { 9, 3, 4, 3, -1 }, new int[] { 10, 4, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0157034005969763, -0.23365770280361181, 0.0235169809311628, new int[] { 6, 4, 6, 4, -1 }, new int[] { 6, 4, 3, 2, 2 }, new int[] { 9, 6, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1162799000740051, -1, 0.0005000365199521184, true, new int[] { 9, 3, 3, 9, -1 }, new int[] { 10, 4, 1, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0639397129416466, 0.0085324635729193687, -0.56500917673110962, true, new int[] { 9, 3, 9, 3, -1 }, new int[] { 8, 4, 9, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0018591650296002626, -0.0215167496353388, 0.0431870110332966, new int[] { 16, 12, 2, 3, -1 }, new int[] { 16, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0023360128980129957, 0.0451245903968811, -0.1088766977190971, new int[] { 3, 0, 10, 2, -1 }, new int[] { 3, 0, 5, 1, 2 }, new int[] { 8, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0587388910353184, -0.56496912240982056, 0.0052059069275856018, new int[] { 13, 1, 3, 13, -1 }, new int[] { 14, 1, 1, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037132750730961561, -0.0134631600230932, 0.37635311484336847, new int[] { 1, 0, 1, 2, -1 }, new int[] { 1, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020255730487406254, 0.0314449593424797, -0.123226098716259, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3382161897607148E-05, 0.0770330131053925, -0.066739097237587, new int[] { 3, 0, 1, 2, -1 }, new int[] { 3, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12969060242176059, 0.0036417250521481037, -0.411312997341156, new int[] { 14, 6, 4, 8, -1 }, new int[] { 14, 10, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1191373988986015, -0.60263478755950928, 0.007990347221493721, new int[] { 0, 6, 4, 8, -1 }, new int[] { 0, 10, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.012801880016923, -0.5977100133895874, 0.0010519300121814013, true, new int[] { 16, 0, 2, 1, -1 }, new int[] { 16, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1910737007856369, -0.81298089027404785, 0.0057100728154182434, new int[] { 0, 0, 6, 13, -1 }, new int[] { 2, 0, 2, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0228933207690716, 0.0194525197148323, -0.1632170975208283, new int[] { 5, 14, 12, 1, -1 }, new int[] { 9, 14, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.170331597328186, -0.0198107101023197, 0.2434374988079071, true, new int[] { 7, 2, 9, 2, -1 }, new int[] { 10, 5, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.38168561458587652, 0.0074787861667573452, -0.83872407674789429, new int[] { 6, 1, 6, 12, -1 }, new int[] { 6, 5, 6, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0062416237778961658, -0.14228279888629911, 0.0332785397768021, true, new int[] { 9, 5, 2, 4, -1 }, new int[] { 9, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0845880135893822, 0.0167654994875193, -0.0928579717874527, true, new int[] { 11, 5, 4, 4, -1 }, new int[] { 11, 5, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0225149597972631, 0.0879255905747414, -0.0715503692626953, true, new int[] { 4, 0, 12, 1, -1 }, new int[] { 4, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1966812014579773, 0.0833218693733215, -0.0203528292477131, new int[] { 10, 2, 5, 10, -1 }, new int[] { 10, 7, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.21616910398006439, 0.29649278521537781, -0.0161115303635597, new int[] { 3, 2, 5, 10, -1 }, new int[] { 3, 7, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0088920090347528458, 0.13778349757194519, -0.0358431711792946, new int[] { 2, 2, 14, 6, -1 }, new int[] { 2, 4, 14, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0120847998186946, -0.43843948841094971, 0.0123654901981354, new int[] { 4, 4, 5, 3, -1 }, new int[] { 4, 5, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25806298851966858, -0.0052921390160918236, 0.3777414858341217, new int[] { 2, 1, 15, 3, -1 }, new int[] { 7, 2, 5, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0148832304403186, 0.0090738674625754356, -0.55208402872085571, new int[] { 5, 2, 2, 3, -1 }, new int[] { 6, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.66914242506027222, -0.0149384997785091, 0.17851120233535769, new int[] { 4, 2, 12, 9, -1 }, new int[] { 8, 5, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0049930079840123653, -0.231485903263092, 0.0234815701842308, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.20315469801425931, 0.0021833679638803005, -0.49344301223754877, new int[] { 8, 0, 6, 11, -1 }, new int[] { 10, 0, 2, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00567800784483552, 0.19343170523643491, -0.0277863405644894, true, new int[] { 1, 1, 3, 1, -1 }, new int[] { 2, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.005930453073233366, -0.0200895592570305, 0.109096996486187, new int[] { 16, 0, 1, 6, -1 }, new int[] { 16, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.0694196820259094, -0.0834254324436188, new int[] { 0, 4, 1, 2, -1 }, new int[] { 0, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0062176208011806011, 0.0786899477243423, -0.0139514803886414, true, new int[] { 15, 6, 3, 2, -1 }, new int[] { 15, 6, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055320560932159424, -0.0663150474429131, 0.0798476189374924, true, new int[] { 3, 6, 2, 3, -1 }, new int[] { 3, 6, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0369591601192951, -0.29380309581756592, 0.0157649908214808, new int[] { 0, 0, 18, 4, -1 }, new int[] { 9, 0, 9, 2, 2 }, new int[] { 0, 2, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0163652505725622, -0.0322352685034275, 0.1461254954338074, true, new int[] { 4, 5, 3, 4, -1 }, new int[] { 5, 6, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0785978734493256, -0.19322140514850619, 0.00977293960750103, new int[] { 13, 1, 4, 12, -1 }, new int[] { 15, 1, 2, 6, 2 }, new int[] { 13, 7, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.037147980183363, -0.0805545896291733, 0.0657810792326927, new int[] { 3, 1, 2, 14, -1 }, new int[] { 3, 8, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0117284599691629, 0.0272431094199419, -0.14649729430675509, new int[] { 7, 6, 5, 3, -1 }, new int[] { 7, 7, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0103350402787328, 0.06276735663414, -0.0815778523683548, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 8, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0225539691746235, -0.0534550100564957, 0.0260324496775866, new int[] { 7, 8, 9, 3, -1 }, new int[] { 10, 9, 3, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0209841597825289, -0.0704301372170448, 0.0790670588612556, new int[] { 2, 8, 9, 3, -1 }, new int[] { 5, 9, 3, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070778899826109409, 0.0680953115224838, -0.0216820295900106, new int[] { 10, 4, 4, 7, -1 }, new int[] { 11, 4, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0019395829876884818, 0.0617897398769856, -0.1004408970475197, new int[] { 4, 4, 4, 7, -1 }, new int[] { 5, 4, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015511269448325038, -0.0237703006714582, 0.1048393994569778, new int[] { 10, 14, 3, 1, -1 }, new int[] { 11, 14, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.747781248530373E-05, 0.0735548809170723, -0.0689330399036407, new int[] { 5, 14, 3, 1, -1 }, new int[] { 6, 14, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00048028680612333119, 0.0447285212576389, -0.0435139797627926, new int[] { 5, 14, 12, 1, -1 }, new int[] { 9, 14, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1720701009035111, -0.59279191493988037, 0.0088808601722121239, new int[] { 0, 1, 18, 8, -1 }, new int[] { 0, 1, 9, 4, 2 }, new int[] { 9, 5, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.158473402261734, 0.0030388650484383106, -0.27436259388923651, true, new int[] { 9, 1, 6, 4, -1 }, new int[] { 9, 1, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.14971689879894259, -0.760021984577179, 0.0064801289699971676, true, new int[] { 9, 1, 4, 6, -1 }, new int[] { 9, 1, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020640289876610041, 0.15531200170516971, -0.0304844807833433, new int[] { 11, 8, 2, 2, -1 }, new int[] { 12, 8, 1, 1, 2 }, new int[] { 11, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0771084874868393, 0.43029859662055969, -0.0116477198898792, new int[] { 5, 0, 8, 11, -1 }, new int[] { 7, 0, 4, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0343285612761974, -0.2315476983785629, 0.0161607693880796, new int[] { 7, 7, 6, 3, -1 }, new int[] { 9, 8, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.043574009090662, -0.028146019205451, 0.16973729431629181, new int[] { 4, 0, 6, 9, -1 }, new int[] { 6, 0, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.4282230343669653E-05, -0.0652616396546364, 0.0352620482444763, true, new int[] { 10, 0, 3, 2, -1 }, new int[] { 11, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021579340100288391, 0.0431658513844013, -0.11010999977588649, true, new int[] { 8, 0, 2, 3, -1 }, new int[] { 7, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020436691120266914, 0.0295867193490267, -0.0619979798793793, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.18425910174846649, 0.0053550167940557, -0.92895787954330444, new int[] { 6, 4, 6, 11, -1 }, new int[] { 8, 4, 2, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0191197507083416, 0.0053580361418426037, -0.65347892045974731, new int[] { 10, 8, 3, 3, -1 }, new int[] { 11, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0641443729400635, -0.0103305000811815, 0.46719509363174438, true, new int[] { 8, 1, 9, 2, -1 }, new int[] { 8, 1, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043394681997597218, -0.1537874042987824, 0.0111428704112768, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.223211795091629, -0.94697248935699463, 0.0048918798565864563, new int[] { 0, 9, 18, 6, -1 }, new int[] { 0, 9, 9, 3, 2 }, new int[] { 9, 12, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.6038159527815878E-05, 0.0709768906235695, -0.0623531192541122, new int[] { 8, 8, 2, 2, -1 }, new int[] { 9, 8, 1, 1, 2 }, new int[] { 8, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013452749699354172, -0.0286097601056099, 0.15549249947071081, new int[] { 4, 7, 2, 2, -1 }, new int[] { 4, 7, 1, 1, 2 }, new int[] { 5, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013946880353614688, -0.0402705408632755, 0.0586122795939446, new int[] { 10, 8, 3, 3, -1 }, new int[] { 11, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0156203303486109, 0.0073195630684494972, -0.63210958242416382, new int[] { 5, 8, 3, 3, -1 }, new int[] { 6, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.5555468861712143E-05, 0.045023500919342, -0.0287142004817724, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0111428601667285, 0.0157248601317406, -0.28536120057106018, new int[] { 0, 6, 1, 6, -1 }, new int[] { 0, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0131013197824359, -0.0355839505791664, 0.1051271036267281, new int[] { 11, 8, 5, 6, -1 }, new int[] { 11, 10, 5, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0087957009673118591, 0.0244174394756556, -0.18935090303421021, new int[] { 0, 8, 1, 6, -1 }, new int[] { 0, 10, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0499279797077179, 0.0787372216582298, -0.0277854092419147, new int[] { 11, 8, 6, 6, -1 }, new int[] { 11, 10, 6, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0398713387548923, -0.0298023894429207, 0.19444610178470609, new int[] { 1, 8, 6, 6, -1 }, new int[] { 1, 10, 6, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0157816000282764, -0.76653957366943359, 0.00095044961199164391, true, new int[] { 10, 0, 3, 1, -1 }, new int[] { 11, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00611749617382884, -0.26769641041755682, 0.0171274207532406, true, new int[] { 8, 0, 1, 3, -1 }, new int[] { 7, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.44994831085205078, -0.0190667398273945, 0.23485369980335241, new int[] { 5, 3, 9, 9, -1 }, new int[] { 8, 6, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0433428809046745, -0.7188379168510437, 0.0062806149944663048, true, new int[] { 7, 0, 8, 2, -1 }, new int[] { 7, 0, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0301288608461618, -0.65766400098800659, 0.00497265113517642, new int[] { 11, 9, 3, 6, -1 }, new int[] { 12, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0227169692516327, -0.1927156001329422, 0.0224213097244501, new int[] { 1, 14, 12, 1, -1 }, new int[] { 5, 14, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010098509956151247, 0.0785590186715126, -0.035671528428793, new int[] { 8, 14, 4, 1, -1 }, new int[] { 9, 14, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010692490031942725, 0.12817870080471039, -0.0513950809836388, new int[] { 6, 14, 4, 1, -1 }, new int[] { 7, 14, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0057365992106497288, -0.45711138844490051, 0.0040395711548626423, new int[] { 14, 14, 2, 1, -1 }, new int[] { 14, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.0038979679811746E-05, 0.0696846470236778, -0.0743912681937218, new int[] { 2, 14, 2, 1, -1 }, new int[] { 3, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0336750186979771, 0.0032588799949735403, -0.80500680208206177, new int[] { 11, 9, 3, 6, -1 }, new int[] { 12, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0159147903323174, 0.0107761099934578, -0.4024600088596344, new int[] { 4, 9, 3, 6, -1 }, new int[] { 5, 9, 1, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00042607940849848092, -0.0471980608999729, 0.0233493093401194, new int[] { 5, 12, 12, 2, -1 }, new int[] { 5, 13, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22485710680484769, -0.0398878902196884, 0.1068518981337547, new int[] { 0, 0, 15, 15, -1 }, new int[] { 5, 0, 5, 15, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0029953860212117434, 0.091609388589859, -0.0748484134674072, new int[] { 8, 1, 2, 4, -1 }, new int[] { 8, 1, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0041523170657455921, 0.11539760231971739, -0.0425119213759899, new int[] { 0, 3, 2, 8, -1 }, new int[] { 0, 3, 1, 4, 2 }, new int[] { 1, 7, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0498369298875332, -0.00392977986484766, 0.518172025680542, true, new int[] { 15, 1, 3, 3, -1 }, new int[] { 14, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0200233291834593, 0.19128979742527011, -0.0231510493904352, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0062091718427836895, 0.1013979017734528, -0.0324465110898018, new int[] { 14, 0, 4, 6, -1 }, new int[] { 16, 0, 2, 3, 2 }, new int[] { 14, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052683670073747635, -0.18189090490341189, 0.0307422205805779, new int[] { 3, 3, 12, 2, -1 }, new int[] { 3, 3, 6, 1, 2 }, new int[] { 9, 4, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.002545441035181284, 0.0155313396826386, -0.0760350972414017, true, new int[] { 16, 10, 2, 1, -1 }, new int[] { 16, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0033172550611197948, -0.13509359955787659, 0.0359591096639633, true, new int[] { 2, 10, 1, 2, -1 }, new int[] { 2, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0261108204722404, 0.0872836336493492, -0.0217705499380827, new int[] { 10, 7, 6, 4, -1 }, new int[] { 13, 7, 3, 2, 2 }, new int[] { 10, 9, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24312630295753479, 0.0361428782343864, -0.146251305937767, new int[] { 0, 4, 18, 5, -1 }, new int[] { 6, 4, 6, 5, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1904131025075913, 0.0073239780031144619, -0.27729520201683039, true, new int[] { 11, 1, 5, 6, -1 }, new int[] { 9, 3, 5, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0163597594946623, -0.1068542972207069, 0.0491146706044674, true, new int[] { 8, 0, 6, 6, -1 }, new int[] { 10, 2, 2, 6, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.06885776668787, -0.42388990521430969, 0.0085399514064192772, new int[] { 4, 4, 14, 4, -1 }, new int[] { 11, 4, 7, 2, 2 }, new int[] { 4, 6, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0203291904181242, -0.0396039597690105, 0.16347900032997131, new int[] { 0, 4, 14, 4, -1 }, new int[] { 0, 4, 7, 2, 2 }, new int[] { 7, 6, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0129730198532343, -0.0195611193776131, 0.1110479012131691, new int[] { 10, 7, 6, 4, -1 }, new int[] { 13, 7, 3, 2, 2 }, new int[] { 10, 9, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00629903981462121, -0.0387555509805679, 0.16495589911937711, new int[] { 2, 7, 6, 4, -1 }, new int[] { 2, 7, 3, 2, 2 }, new int[] { 5, 9, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00046493619447574019, -0.0703989788889885, 0.0591666884720325, new int[] { 10, 2, 2, 2, -1 }, new int[] { 10, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0114370100200176, -0.25582531094551092, 0.0225616004317999, new int[] { 6, 14, 6, 1, -1 }, new int[] { 9, 14, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0605634413659573, -0.15025900304317469, 0.0358815304934978, new int[] { 0, 9, 18, 6, -1 }, new int[] { 9, 9, 9, 3, 2 }, new int[] { 0, 12, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0571704693138599, -0.55165249109268188, 0.0088588111102581024, new int[] { 1, 0, 1, 14, -1 }, new int[] { 1, 7, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0037495139986276627, -0.1063347011804581, 0.0165663603693247, true, new int[] { 14, 0, 3, 1, -1 }, new int[] { 15, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036156480200588703, -0.0469515882432461, 0.0984329879283905, true, new int[] { 4, 7, 3, 3, -1 }, new int[] { 3, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039375461637973785, 0.0158571396023035, -0.12761549651622769, true, new int[] { 14, 0, 3, 1, -1 }, new int[] { 15, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00069156679091975093, -0.0969325676560402, 0.0460354201495647, new int[] { 6, 2, 2, 2, -1 }, new int[] { 7, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0171396601945162, 0.18325529992580411, -0.0297442600131035, new int[] { 5, 0, 8, 12, -1 }, new int[] { 7, 0, 4, 12, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021130971144884825, -0.14694969356060031, 0.0371412001550198, true, new int[] { 9, 8, 2, 1, -1 }, new int[] { 9, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3239036535378546E-05, 0.0560943596065044, -0.0452513098716736, new int[] { 9, 4, 2, 2, -1 }, new int[] { 10, 4, 1, 1, 2 }, new int[] { 9, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.252463961369358E-05, -0.0660794675350189, 0.084846131503582, new int[] { 7, 4, 2, 2, -1 }, new int[] { 7, 4, 1, 1, 2 }, new int[] { 8, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022989229764789343, -0.062885507941246, 0.0724585726857185, new int[] { 7, 8, 4, 4, -1 }, new int[] { 8, 8, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00852392427623272, 0.0245985891669989, -0.20404249429702759, true, new int[] { 9, 6, 3, 1, -1 }, new int[] { 10, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0152474995702505, -0.046305138617754, 0.0926802083849907, new int[] { 0, 10, 18, 2, -1 }, new int[] { 0, 11, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0411155596375465, -0.16479089856147769, 0.0320520587265491, true, new int[] { 8, 5, 2, 8, -1 }, new int[] { 8, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0570124983787537, 0.17691320180892939, -0.0289100594818592, new int[] { 3, 3, 12, 4, -1 }, new int[] { 3, 5, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0361419506371021, 0.33573868870735168, -0.0146681498736143, true, new int[] { 4, 0, 8, 1, -1 }, new int[] { 6, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0473424009978771, -0.36468461155891418, 0.009702109731733799, new int[] { 9, 0, 4, 10, -1 }, new int[] { 11, 0, 2, 5, 2 }, new int[] { 9, 5, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00015224410162772983, -0.0855662599205971, 0.0563358217477798, new int[] { 4, 2, 10, 2, -1 }, new int[] { 4, 3, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010744449682533741, 0.0676028802990913, -0.0449445992708206, new int[] { 11, 0, 3, 4, -1 }, new int[] { 12, 0, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024688818957656622, 0.0393917709589005, -0.1143665015697479, new int[] { 6, 0, 5, 2, -1 }, new int[] { 6, 1, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0223950799554586, -0.41499689221382141, 0.0033534979447722435, new int[] { 11, 0, 3, 4, -1 }, new int[] { 12, 0, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0141458800062537, 0.007806004025042057, -0.56246250867843628, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 0, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.6172739277826622E-05, 0.0422396287322044, -0.0399822406470776, new int[] { 10, 0, 3, 3, -1 }, new int[] { 10, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046720141544938087, -0.3006666898727417, 0.0159843992441893, new int[] { 5, 0, 3, 3, -1 }, new int[] { 5, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.928966129431501E-05, -0.041034109890461, 0.0526925884187222, new int[] { 11, 8, 2, 2, -1 }, new int[] { 12, 8, 1, 1, 2 }, new int[] { 11, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0019730681087821722, 0.15118849277496341, -0.032511081546545, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3879110813140869E-05, 0.0414035692811012, -0.042990118265152, new int[] { 11, 8, 2, 2, -1 }, new int[] { 12, 8, 1, 1, 2 }, new int[] { 11, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1802700909320265E-05, -0.0583424791693687, 0.0939400717616081, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022840979509055614, 0.0185070801526308, -0.045831311494112, true, new int[] { 11, 7, 1, 4, -1 }, new int[] { 10, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13125069439411161, -0.1768728047609329, 0.0260149408131838, new int[] { 3, 3, 5, 12, -1 }, new int[] { 3, 6, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011948959436267614, 0.0419367291033268, -0.0555466488003731, true, new int[] { 11, 10, 4, 1, -1 }, new int[] { 11, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0722346305847168, 0.010688959620893, -0.40127620100975042, true, new int[] { 6, 1, 3, 6, -1 }, new int[] { 4, 3, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0563969314098358, -0.88491988182067871, 0.00036692508729174733, new int[] { 2, 1, 16, 1, -1 }, new int[] { 6, 1, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0541536509990692, -0.2249650955200195, 0.0179232098162174, new int[] { 3, 6, 12, 4, -1 }, new int[] { 3, 6, 6, 2, 2 }, new int[] { 9, 8, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.025167329236865, 0.1300235986709595, -0.0366861596703529, new int[] { 1, 7, 16, 3, -1 }, new int[] { 1, 8, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0137102100998163, -0.0405139811336994, 0.11201860010623931, new int[] { 2, 6, 6, 6, -1 }, new int[] { 4, 8, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0278908200562, -0.731376588344574, 0.0037337029352784157, new int[] { 14, 4, 4, 3, -1 }, new int[] { 14, 5, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0075335809960961342, -0.23119840025901789, 0.0176145397126675, new int[] { 0, 4, 4, 3, -1 }, new int[] { 0, 5, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032403611112385988, -0.0087237963452935219, 0.20382650196552279, new int[] { 14, 1, 1, 3, -1 }, new int[] { 14, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0844089612364769, 0.0051954388618469238, -0.82454532384872437, new int[] { 1, 9, 9, 3, -1 }, new int[] { 4, 9, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00082196877337992191, -0.0817157030105591, 0.0218308698385954, true, new int[] { 14, 2, 2, 3, -1 }, new int[] { 14, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029956221114844084, -0.0280322693288326, 0.15127840638160711, new int[] { 3, 2, 2, 4, -1 }, new int[] { 3, 2, 1, 2, 2 }, new int[] { 4, 4, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0703764632344246, -0.13520090281963351, 0.003968109842389822, new int[] { 10, 5, 4, 10, -1 }, new int[] { 12, 5, 2, 5, 2 }, new int[] { 10, 10, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0321913808584213, 0.0131358997896314, -0.33470198512077332, new int[] { 4, 5, 4, 10, -1 }, new int[] { 4, 5, 2, 5, 2 }, new int[] { 6, 10, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054974909871816635, -0.0265497900545597, 0.1170909032225609, true, new int[] { 11, 2, 2, 2, -1 }, new int[] { 11, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.016429390758276, -0.0533613413572311, 0.0821190625429153, new int[] { 5, 4, 3, 6, -1 }, new int[] { 5, 6, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0074506900273263454, 0.0808582007884979, -0.0223928596824408, true, new int[] { 11, 2, 2, 2, -1 }, new int[] { 11, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049851150251924992, -0.0205729696899652, 0.25987860560417181, true, new int[] { 7, 2, 2, 2, -1 }, new int[] { 7, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029100670944899321, -0.0231053698807955, 0.0452293008565903, true, new int[] { 9, 5, 2, 1, -1 }, new int[] { 9, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.135223001241684, 0.11169719696044921, -0.0436136610805988, new int[] { 2, 0, 9, 14, -1 }, new int[] { 5, 0, 3, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0098680844530463219, -0.18346819281578061, 0.0038948319852352142, true, new int[] { 14, 0, 3, 1, -1 }, new int[] { 15, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0040301959961652756, 0.0233750492334366, -0.20566239953041079, true, new int[] { 4, 0, 1, 3, -1 }, new int[] { 3, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0396324507892132, 0.007700175978243351, -0.166393905878067, new int[] { 9, 0, 4, 10, -1 }, new int[] { 11, 0, 2, 5, 2 }, new int[] { 9, 5, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.012742499820888, 0.14852410554885859, -0.0306067708879709, true, new int[] { 5, 0, 2, 5, -1 }, new int[] { 5, 0, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037017830181866884, 0.0209220908582211, -0.11472299695014949, true, new int[] { 14, 0, 2, 1, -1 }, new int[] { 14, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022704519797116518, 0.0270258691161871, -0.16540579497814181, true, new int[] { 4, 0, 1, 2, -1 }, new int[] { 4, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14953289926052091, -0.0020300289615988731, 0.598150908946991, true, new int[] { 12, 1, 4, 4, -1 }, new int[] { 12, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0051417769864201546, 0.38440889120101929, -0.0112848002463579, new int[] { 0, 1, 1, 4, -1 }, new int[] { 0, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00636163679882884, -0.31090161204338068, 0.0143518401309848, new int[] { 8, 7, 2, 4, -1 }, new int[] { 9, 7, 1, 2, 2 }, new int[] { 8, 9, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0598138608038425, -0.70378690958023071, 0.00579686788842082, new int[] { 1, 5, 16, 2, -1 }, new int[] { 5, 5, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.35357210040092468, 0.0112126599997282, -0.33229699730873108, new int[] { 5, 0, 8, 12, -1 }, new int[] { 5, 4, 8, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.6899908185005188, -0.0105861099436879, 0.3837656974792481, new int[] { 2, 2, 12, 9, -1 }, new int[] { 6, 5, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048297038301825523, 0.0210381299257278, -0.0573535598814487, true, new int[] { 13, 0, 3, 3, -1 }, new int[] { 14, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0178284905850887, -0.0106050595641136, 0.39563548564910889, true, new int[] { 5, 0, 3, 3, -1 }, new int[] { 4, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0642841011285782, -0.0638428777456284, 0.0267954096198082, new int[] { 2, 2, 16, 7, -1 }, new int[] { 6, 2, 8, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25491470098495478, 0.0193274095654488, -0.24302749335765839, new int[] { 0, 2, 16, 7, -1 }, new int[] { 4, 2, 8, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051334970630705357, 0.011508010327816, -0.23830899596214289, new int[] { 16, 13, 2, 2, -1 }, new int[] { 16, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0039797872304916382, -0.20426890254020691, 0.0203900802880526, new int[] { 0, 13, 2, 2, -1 }, new int[] { 1, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0027258729096502066, -0.0465084612369537, 0.0794106870889664, new int[] { 16, 7, 2, 8, -1 }, new int[] { 17, 7, 1, 4, 2 }, new int[] { 16, 11, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0149838598445058, 0.39586919546127319, -0.0113431699573994, new int[] { 0, 7, 2, 8, -1 }, new int[] { 0, 7, 1, 4, 2 }, new int[] { 1, 11, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029130540788173676, 0.0363716296851635, -0.0906147211790085, new int[] { 11, 2, 7, 3, -1 }, new int[] { 11, 3, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00090548500884324312, 0.0620919205248356, -0.0684250965714455, new int[] { 0, 8, 2, 3, -1 }, new int[] { 1, 8, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1165482997894287, 0.1156952977180481, -0.0132687203586102, true, new int[] { 10, 5, 6, 4, -1 }, new int[] { 12, 7, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0107813002541661, 0.017420019954443, -0.28036078810691828, true, new int[] { 9, 7, 2, 1, -1 }, new int[] { 9, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.534478485584259, -0.47411590814590449, 0.0086649907752871513, new int[] { 0, 3, 18, 8, -1 }, new int[] { 0, 7, 18, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.66155395063106E-05, -0.0586382709443569, 0.0750202611088753, new int[] { 7, 6, 2, 2, -1 }, new int[] { 7, 6, 1, 1, 2 }, new int[] { 8, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.2536040786653757E-05, -0.0498466081917286, 0.0593500696122646, new int[] { 11, 6, 2, 2, -1 }, new int[] { 12, 6, 1, 1, 2 }, new int[] { 11, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0730543434619904, -0.0140366898849607, 0.35884460806846619, true, new int[] { 8, 5, 4, 6, -1 }, new int[] { 6, 7, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165301598608494, -0.34632420539855963, 0.0067927599884569645, true, new int[] { 16, 3, 2, 2, -1 }, new int[] { 16, 3, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3628758653067052E-05, 0.0716383680701256, -0.0592160597443581, new int[] { 8, 14, 2, 1, -1 }, new int[] { 9, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0194537602365017, -0.51694720983505249, 0.0062814089469611645, new int[] { 11, 2, 7, 3, -1 }, new int[] { 11, 3, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.21202109754085541, 0.0076583931222558022, -0.50985842943191528, new int[] { 0, 2, 9, 12, -1 }, new int[] { 0, 5, 9, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0196576490998268, -0.0431430488824844, 0.0518909394741058, true, new int[] { 16, 0, 2, 11, -1 }, new int[] { 16, 0, 1, 11, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 18
            stage = new HaarCascadeStage(-1.1236120462417603, 17, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(0.0868941992521286, -0.18969950079917911, 0.22035740315914151, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0096704717725515366, 0.1185135021805763, -0.0863395631313324, true, new int[] { 12, 8, 4, 2, -1 }, new int[] { 12, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0814679488539696, 0.1499083936214447, -0.12963719666004181, true, new int[] { 6, 4, 5, 6, -1 }, new int[] { 4, 6, 5, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017537999665364623, 0.17750880122184751, -0.1069336980581284, new int[] { 16, 2, 2, 3, -1 }, new int[] { 16, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.438779721269384E-05, 0.0960103869438171, -0.16225080192089081, new int[] { 0, 2, 2, 3, -1 }, new int[] { 1, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.008001105859875679, -0.0185400806367397, 0.24660170078277591, new int[] { 13, 7, 3, 3, -1 }, new int[] { 14, 8, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0170908197760582, 0.0325614809989929, -0.26181620359420782, new int[] { 2, 4, 4, 9, -1 }, new int[] { 4, 4, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0099246148020029068, -0.0193589702248573, 0.12542679905891421, true, new int[] { 10, 0, 4, 1, -1 }, new int[] { 10, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0122903902083635, 0.0343302115797997, -0.32864710688591, true, new int[] { 8, 2, 2, 2, -1 }, new int[] { 8, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021256268955767155, -0.0717979818582535, 0.0692160725593567, new int[] { 9, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24960160255432129, -0.1123834997415543, 0.1429843008518219, new int[] { 2, 0, 14, 15, -1 }, new int[] { 9, 0, 7, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00495578907430172, 0.1379792988300324, -0.0583309903740883, true, new int[] { 13, 4, 4, 3, -1 }, new int[] { 12, 5, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0697411075234413, 0.0297146998345852, -0.34425801038742071, new int[] { 3, 6, 12, 8, -1 }, new int[] { 3, 6, 6, 4, 2 }, new int[] { 9, 10, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.008152778260409832, -0.0469510108232498, 0.0782470628619194, new int[] { 12, 5, 3, 6, -1 }, new int[] { 13, 7, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0103493202477694, -0.0694328024983406, 0.15855890512466431, true, new int[] { 5, 4, 3, 4, -1 }, new int[] { 6, 5, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063299350440502167, -0.0399102084338665, 0.15249839425086981, true, new int[] { 12, 7, 3, 3, -1 }, new int[] { 13, 8, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.030955720692873, 0.0419439598917961, -0.23227399587631231, new int[] { 0, 1, 4, 14, -1 }, new int[] { 0, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0125044696033001, -0.0183122493326664, 0.0996528565883636, true, new int[] { 12, 8, 3, 4, -1 }, new int[] { 13, 9, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074256081134080887, -0.0621832795441151, 0.16638119518756869, true, new int[] { 6, 8, 4, 3, -1 }, new int[] { 5, 9, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0200669895857573, 0.0226579904556274, -0.34708911180496221, new int[] { 3, 0, 12, 4, -1 }, new int[] { 9, 0, 6, 2, 2 }, new int[] { 3, 2, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.582889974117279, 0.28628429770469671, -0.0296743903309107, new int[] { 3, 2, 12, 12, -1 }, new int[] { 3, 8, 12, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142788495868444, 0.17780199646949771, -0.029107179492712, new int[] { 12, 7, 2, 4, -1 }, new int[] { 12, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059483898803591728, -0.0514614395797253, 0.2133691012859345, new int[] { 5, 6, 1, 6, -1 }, new int[] { 5, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.037677738815546, -0.369326114654541, 0.0577233098447323, new int[] { 17, 1, 1, 14, -1 }, new int[] { 17, 8, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.065808892250061, 0.0406517907977104, -0.21074700355529791, new int[] { 0, 0, 18, 10, -1 }, new int[] { 0, 0, 9, 5, 2 }, new int[] { 9, 5, 9, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.23132109642028811, 0.41835370659828192, -0.0121959000825882, new int[] { 5, 0, 12, 11, -1 }, new int[] { 9, 0, 4, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00092640687944367528, -0.14468879997730261, 0.058539729565382, true, new int[] { 7, 0, 2, 4, -1 }, new int[] { 7, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0010040670167654753, -0.0440565086901188, 0.0339186899363995, new int[] { 8, 13, 10, 2, -1 }, new int[] { 13, 13, 5, 1, 2 }, new int[] { 8, 14, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0161782503128052, -0.253731906414032, 0.0289683602750301, true, new int[] { 4, 1, 3, 4, -1 }, new int[] { 3, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00060239218873903155, 0.0413237288594246, -0.0400842092931271, new int[] { 13, 8, 4, 1, -1 }, new int[] { 14, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0060449438169598579, 0.14372240006923681, -0.0471708290278912, true, new int[] { 3, 6, 3, 2, -1 }, new int[] { 4, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022208129521459341, 0.0451353900134563, -0.16863340139389041, true, new int[] { 13, 0, 3, 1, -1 }, new int[] { 14, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0254353806376457, 0.27486249804496771, -0.0250210706144571, true, new int[] { 3, 2, 3, 3, -1 }, new int[] { 4, 3, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0097569217905402184, -0.35105359554290771, 0.0067487931810319424, true, new int[] { 13, 0, 3, 1, -1 }, new int[] { 14, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058798119425773621, -0.236527606844902, 0.0292028002440929, true, new int[] { 2, 11, 2, 2, -1 }, new int[] { 2, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017566860187798739, -0.0990074127912521, 0.0523698702454567, new int[] { 2, 11, 16, 2, -1 }, new int[] { 2, 12, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0742733180522919, -0.2623257040977478, 0.0324768982827663, true, new int[] { 8, 6, 6, 3, -1 }, new int[] { 10, 8, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0311077497899532, -0.0317390114068985, 0.19744929671287539, new int[] { 2, 12, 16, 3, -1 }, new int[] { 2, 13, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0637038722634315, 0.0268714595586061, -0.27673950791358948, new int[] { 0, 10, 18, 4, -1 }, new int[] { 0, 10, 9, 2, 2 }, new int[] { 9, 12, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0475392416119576, -0.40510261058807367, 0.0122220404446125, new int[] { 6, 12, 9, 2, -1 }, new int[] { 9, 12, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035632580984383821, -0.1999291926622391, 0.0335399098694324, true, new int[] { 5, 0, 1, 3, -1 }, new int[] { 4, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.026527589187026, -0.1629005968570709, 0.0278331693261862, new int[] { 8, 0, 9, 6, -1 }, new int[] { 8, 3, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.28048178553581238, 0.0288105905056, -0.2271182984113693, new int[] { 0, 3, 12, 9, -1 }, new int[] { 6, 3, 6, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.45591941475868231, -0.0227571800351143, 0.310296893119812, new int[] { 3, 4, 12, 6, -1 }, new int[] { 7, 6, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0867485329508781, 0.0726863965392113, -0.10276269912719729, true, new int[] { 8, 0, 8, 3, -1 }, new int[] { 10, 2, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016994470497593284, -0.0318094082176685, 0.0871460884809494, true, new int[] { 11, 1, 1, 2, -1 }, new int[] { 11, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011253879638388753, 0.0680664330720901, -0.12390069663524631, true, new int[] { 7, 1, 2, 1, -1 }, new int[] { 7, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0508721508085728, -0.00875171646475792, 0.31184211373329163, true, new int[] { 13, 1, 4, 3, -1 }, new int[] { 12, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.19961729645729071, -0.030910549685359, 0.21652880311012271, new int[] { 1, 0, 12, 11, -1 }, new int[] { 5, 0, 4, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0638386905193329, -0.60265827178955078, 0.001323336036875844, true, new int[] { 13, 1, 4, 3, -1 }, new int[] { 12, 2, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063007958233356476, -0.052063349634409, 0.12607939541339869, true, new int[] { 5, 1, 3, 4, -1 }, new int[] { 6, 2, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0056697470135986805, 0.0090780286118388176, -0.19445329904556269, true, new int[] { 9, 0, 2, 1, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014293550048023462, -0.0857814326882362, 0.0712894573807716, new int[] { 0, 13, 10, 2, -1 }, new int[] { 0, 13, 5, 1, 2 }, new int[] { 5, 14, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0138120101764798, 0.0080618355423212051, -0.38797891139984131, new int[] { 12, 12, 4, 3, -1 }, new int[] { 13, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.0624911710619926, 0.1092092990875244, new int[] { 3, 6, 1, 2, -1 }, new int[] { 3, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0039398381486535072, 0.0509323291480541, -0.1498032063245773, new int[] { 7, 7, 4, 2, -1 }, new int[] { 7, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.12358880043029789, 0.31476518511772161, -0.0257598794996738, new int[] { 0, 10, 18, 4, -1 }, new int[] { 0, 12, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109574301168323, -0.26074820756912231, 0.0158497299998999, new int[] { 12, 12, 4, 3, -1 }, new int[] { 13, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0076301600784063339, 0.26100659370422358, -0.0243298895657063, new int[] { 5, 10, 4, 2, -1 }, new int[] { 5, 10, 2, 1, 2 }, new int[] { 7, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0678390711545944, 0.196913093328476, -0.0083496840670704842, new int[] { 13, 1, 2, 8, -1 }, new int[] { 13, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0186073090881109, 0.0256039593368769, -0.25413629412651062, new int[] { 3, 1, 2, 8, -1 }, new int[] { 3, 5, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.8711939345812425E-05, 0.0356258116662502, -0.041084211319685, new int[] { 12, 12, 4, 3, -1 }, new int[] { 13, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.391460869461298E-05, -0.1306141018867493, 0.0493933893740177, true, new int[] { 9, 0, 1, 2, -1 }, new int[] { 9, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0177341904491186, -0.0342735201120377, 0.1212686002254486, new int[] { 8, 0, 4, 4, -1 }, new int[] { 8, 0, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0068113701418042183, 0.0226712208241224, -0.26590269804000849, new int[] { 2, 12, 4, 3, -1 }, new int[] { 3, 12, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0454825609922409, -0.0061395200900733471, 0.47231659293174738, true, new int[] { 9, 5, 1, 6, -1 }, new int[] { 7, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070767141878604889, -0.31650930643081671, 0.0200363900512457, new int[] { 8, 0, 2, 4, -1 }, new int[] { 8, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.000732222106307745, -0.0228806100785732, 0.0647242665290833, new int[] { 10, 9, 2, 2, -1 }, new int[] { 11, 9, 1, 1, 2 }, new int[] { 10, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0032817400060594082, 0.25166231393814092, -0.0231686402112246, new int[] { 6, 9, 2, 2, -1 }, new int[] { 6, 9, 1, 1, 2 }, new int[] { 7, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0461158901453018, -0.35920459032058721, 0.0159878805279732, new int[] { 3, 6, 12, 4, -1 }, new int[] { 9, 6, 6, 2, 2 }, new int[] { 3, 8, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0105268899351358, 0.0096597811207175255, -0.583083987236023, new int[] { 6, 1, 2, 2, -1 }, new int[] { 7, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0218886006623507, 0.0028070888947695494, -0.29022130370140081, new int[] { 17, 7, 1, 6, -1 }, new int[] { 17, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0037969578988850117, 0.26823088526725769, -0.0220357701182365, new int[] { 6, 8, 2, 2, -1 }, new int[] { 6, 8, 1, 1, 2 }, new int[] { 7, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.02915059030056, 0.0370618589222431, -0.0972430408000946, new int[] { 7, 6, 6, 3, -1 }, new int[] { 9, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0796693712472916, -0.0613007396459579, 0.1079474985599518, new int[] { 0, 4, 17, 6, -1 }, new int[] { 0, 6, 17, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0276291705667973, 0.2252894937992096, -0.0325724296271801, new int[] { 1, 6, 16, 3, -1 }, new int[] { 1, 7, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0120179802179337, 0.10100489854812621, -0.0664613619446754, new int[] { 0, 0, 12, 1, -1 }, new int[] { 3, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0119251403957605, -0.18590609729290011, 0.0324855595827103, true, new int[] { 11, 5, 3, 4, -1 }, new int[] { 12, 6, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25123509764671331, -0.0248921401798725, 0.28030058741569519, true, new int[] { 7, 1, 8, 4, -1 }, new int[] { 7, 1, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0069036600179970264, -0.0628988519310951, 0.0317778214812279, new int[] { 2, 0, 16, 1, -1 }, new int[] { 6, 0, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0535753183066845, -0.0124396402388811, 0.46091410517692571, true, new int[] { 9, 5, 6, 1, -1 }, new int[] { 11, 7, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0094652660191059113, 0.0841030478477478, -0.1130022034049034, new int[] { 11, 6, 6, 8, -1 }, new int[] { 13, 6, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1846922039985657, 0.021576140075922, -0.2691057026386261, true, new int[] { 9, 0, 8, 7, -1 }, new int[] { 11, 2, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1181607022881508, -0.47206339240074158, 0.0090096276253461838, new int[] { 11, 6, 6, 8, -1 }, new int[] { 13, 6, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(3.6900841223541647E-05, -0.058833159506321, 0.0994533821940422, true, new int[] { 7, 5, 4, 3, -1 }, new int[] { 6, 6, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1633061021566391, -0.6099013090133667, 0.0013118899660184979, new int[] { 11, 6, 6, 8, -1 }, new int[] { 13, 6, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0965555906295776, -0.52723962068557739, 0.0116685898974538, new int[] { 1, 6, 6, 8, -1 }, new int[] { 3, 6, 2, 8, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0401624515652657, -0.0327838994562626, 0.18107770383358, new int[] { 6, 2, 6, 6, -1 }, new int[] { 6, 5, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0296869408339262, 0.10548429936170579, -0.0615133084356785, new int[] { 6, 4, 6, 4, -1 }, new int[] { 6, 6, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.54369466449134E-05, -0.0359807685017586, 0.0499344505369663, new int[] { 17, 8, 1, 2, -1 }, new int[] { 17, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020552529022097588, 0.0275182090699673, -0.24573980271816251, new int[] { 0, 8, 1, 2, -1 }, new int[] { 0, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.0258090496063232, 0.0299507193267345, new int[] { 16, 8, 2, 4, -1 }, new int[] { 16, 9, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0050713717937469482, -0.20639109611511231, 0.0320026017725468, new int[] { 0, 8, 2, 4, -1 }, new int[] { 0, 9, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0048216218128800392, -0.0975668132305145, 0.0551092401146889, new int[] { 1, 4, 17, 2, -1 }, new int[] { 1, 5, 17, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0652106925845146, 0.0063420450314879417, -0.78828340768814087, new int[] { 0, 0, 18, 4, -1 }, new int[] { 0, 0, 9, 2, 2 }, new int[] { 9, 2, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0158219691365957, -0.0214756801724434, 0.1222712993621826, true, new int[] { 12, 9, 4, 2, -1 }, new int[] { 13, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0300759393721819, 0.37011429667472839, -0.0154766896739602, true, new int[] { 6, 9, 2, 4, -1 }, new int[] { 5, 10, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00055496598361060023, 0.0414319299161434, -0.1214471980929375, new int[] { 9, 8, 1, 2, -1 }, new int[] { 9, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0247548408806324, -0.35262298583984381, 0.0153448497876525, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0087477359920740128, 0.19155359268188479, -0.0225379504263401, true, new int[] { 17, 3, 1, 3, -1 }, new int[] { 16, 4, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0003550080000422895, -0.0846040025353432, 0.0653416514396667, true, new int[] { 5, 9, 2, 3, -1 }, new int[] { 4, 10, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0578844510018826, 0.2597366869449616, -0.0210837107151747, new int[] { 1, 3, 16, 6, -1 }, new int[] { 1, 5, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0037522350903600454, 0.0316149704158306, -0.187950000166893, true, new int[] { 2, 12, 1, 2, -1 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0002026628062594682, -0.0488242693245411, 0.0477622412145138, new int[] { 17, 0, 1, 4, -1 }, new int[] { 17, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0179599896073341, -0.18358300626277921, 0.0270573794841766, true, new int[] { 6, 5, 4, 2, -1 }, new int[] { 6, 5, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0512004382908344, 0.27234628796577448, -0.0199546292424202, new int[] { 0, 13, 18, 2, -1 }, new int[] { 0, 14, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0073698651976883411, -0.1229937970638275, 0.0452794395387173, true, new int[] { 8, 4, 6, 3, -1 }, new int[] { 7, 5, 6, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00081579107791185379, 0.0460813082754612, -0.0212064106017351, new int[] { 9, 4, 2, 1, -1 }, new int[] { 9, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.7019751188345253E-05, -0.1122386977076531, 0.0467198304831982, new int[] { 7, 4, 2, 1, -1 }, new int[] { 8, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0337534099817276, -0.0296947807073593, 0.0309586394578218, new int[] { 8, 0, 6, 10, -1 }, new int[] { 10, 0, 2, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0288798399269581, -0.0476091802120209, 0.1637064069509506, new int[] { 4, 0, 6, 10, -1 }, new int[] { 6, 0, 2, 10, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13803930580616, -0.7450910210609436, 0.0023958049714565277, new int[] { 8, 3, 6, 6, -1 }, new int[] { 10, 5, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0903065428137779, 0.0284100994467735, -0.2060600072145462, new int[] { 4, 3, 6, 6, -1 }, new int[] { 6, 5, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13130649924278259, 0.0058837989345192909, -0.25894629955291748, true, new int[] { 9, 5, 4, 6, -1 }, new int[] { 9, 5, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.13623699545860291, 0.0184906795620918, -0.29096630215644842, true, new int[] { 9, 5, 6, 4, -1 }, new int[] { 9, 5, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001148396055214107, -0.0253341905772686, 0.0819629207253456, new int[] { 14, 3, 4, 1, -1 }, new int[] { 15, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.0390116889029741E-05, -0.0650801733136177, 0.0823377668857574, new int[] { 0, 3, 4, 1, -1 }, new int[] { 1, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0058111059479415417, -0.20126000046730039, 0.0141831701621413, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.012150090187788, 0.21021689474582669, -0.0297118108719587, true, new int[] { 2, 1, 3, 2, -1 }, new int[] { 3, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00432203896343708, 0.0221526604145765, -0.19705900549888611, true, new int[] { 15, 0, 3, 1, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00466731796041131, 0.0223421193659306, -0.263421893119812, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0013583960244432092, 0.0737654492259026, -0.0178339798003435, new int[] { 17, 0, 1, 4, -1 }, new int[] { 17, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0050764158368110657, -0.17490370571613309, 0.0299977697432041, true, new int[] { 3, 0, 2, 3, -1 }, new int[] { 2, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0019497750326991081, -0.0271147508174181, 0.1616608947515488, new int[] { 14, 5, 3, 1, -1 }, new int[] { 15, 5, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025937429163604975, 0.18078009784221649, -0.0271914806216955, new int[] { 1, 5, 3, 1, -1 }, new int[] { 2, 5, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0217158906161785, 0.0960418581962585, -0.0522431582212448, new int[] { 1, 13, 16, 2, -1 }, new int[] { 5, 13, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.564980968600139E-05, 0.0830500423908234, -0.0617705583572388, new int[] { 2, 3, 2, 2, -1 }, new int[] { 2, 3, 1, 1, 2 }, new int[] { 3, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00098641996737569571, -0.0246842093765736, 0.0971914604306221, new int[] { 14, 3, 2, 2, -1 }, new int[] { 15, 3, 1, 1, 2 }, new int[] { 14, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3739310563541949E-05, -0.0695554167032242, 0.077152892947197, new int[] { 2, 3, 2, 2, -1 }, new int[] { 2, 3, 1, 1, 2 }, new int[] { 3, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109101701527834, -0.25444790720939642, 0.0161350406706333, new int[] { 4, 1, 11, 3, -1 }, new int[] { 4, 2, 11, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.6066219258354977E-05, -0.0764008387923241, 0.0709967613220215, new int[] { 7, 3, 1, 2, -1 }, new int[] { 7, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0277181603014469, 0.007712789811193943, -0.30201679468154907, new int[] { 7, 14, 9, 1, -1 }, new int[] { 10, 14, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0043827071785926819, -0.0343367606401443, 0.13955120742321009, new int[] { 5, 10, 6, 2, -1 }, new int[] { 5, 10, 3, 1, 2 }, new int[] { 8, 11, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0375617109239101, -0.456894189119339, 0.0118549996986985, new int[] { 0, 10, 18, 2, -1 }, new int[] { 9, 10, 9, 1, 2 }, new int[] { 0, 11, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0137532595545053, -0.0834474489092827, 0.0594723001122475, new int[] { 0, 11, 15, 4, -1 }, new int[] { 0, 13, 15, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0275797992944717, 0.21291820704936981, -0.0230544097721577, new int[] { 2, 12, 16, 3, -1 }, new int[] { 2, 13, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.040822759270668, -0.50263231992721558, 0.0106398798525333, new int[] { 0, 0, 16, 1, -1 }, new int[] { 4, 0, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.14743439853191381, 0.0077440468594431877, -0.18454490602016449, new int[] { 6, 5, 12, 5, -1 }, new int[] { 9, 5, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.19371560215950009, 0.46490699052810669, -0.0140745798125863, new int[] { 0, 5, 12, 5, -1 }, new int[] { 3, 5, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0414674803614616, -0.1333149969577789, 0.0317224115133286, new int[] { 8, 11, 9, 3, -1 }, new int[] { 11, 12, 3, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021617549937218428, 0.0348884016275406, -0.1198396012187004, new int[] { 7, 0, 4, 2, -1 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0028305849991738796, -0.21483750641345981, 0.0255391206592321, new int[] { 7, 0, 4, 2, -1 }, new int[] { 7, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0108386399224401, 0.33803048729896551, -0.0135911796241999, new int[] { 5, 1, 4, 3, -1 }, new int[] { 7, 1, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021821239497512579, -0.0311352293938398, 0.0836798921227455, new int[] { 9, 0, 4, 1, -1 }, new int[] { 10, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.8489680415950716E-05, -0.15453560650348661, 0.0330539792776108, true, new int[] { 3, 0, 7, 2, -1 }, new int[] { 3, 0, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0052545121870934963, -0.0294149704277515, 0.1650622040033341, new int[] { 3, 5, 12, 4, -1 }, new int[] { 3, 7, 12, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075199748389422894, 0.0233634002506733, -0.21771569550037381, true, new int[] { 8, 7, 3, 1, -1 }, new int[] { 9, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0451239906251431, -0.32536029815673828, 0.0132816601544619, new int[] { 5, 11, 8, 4, -1 }, new int[] { 7, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020451450254768133, 0.0958046466112137, -0.0509931109845638, new int[] { 6, 14, 6, 1, -1 }, new int[] { 8, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029070109594613314, -0.0276902206242085, 0.19595550000667569, new int[] { 7, 13, 4, 1, -1 }, new int[] { 8, 13, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0255583897233009, -0.27625438570976257, 0.0211479291319847, new int[] { 2, 12, 8, 2, -1 }, new int[] { 4, 12, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0026447090785950422, -0.0326275005936623, 0.0412402711808681, true, new int[] { 15, 10, 3, 2, -1 }, new int[] { 16, 11, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.8334530725260265E-05, -0.0848775878548622, 0.0558658987283707, true, new int[] { 3, 10, 2, 3, -1 }, new int[] { 2, 11, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00066109612816944718, -0.0328278504312038, 0.0740109831094742, new int[] { 16, 12, 2, 2, -1 }, new int[] { 17, 12, 1, 1, 2 }, new int[] { 16, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2091878950595856, 0.0100189801305532, -0.47411561012268072, new int[] { 0, 10, 18, 4, -1 }, new int[] { 0, 12, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.0340400523273274E-05, 0.0483234487473965, -0.0327794998884201, new int[] { 16, 12, 2, 2, -1 }, new int[] { 17, 12, 1, 1, 2 }, new int[] { 16, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.6149746999144554E-05, -0.0749692469835281, 0.0619521290063858, new int[] { 0, 0, 1, 4, -1 }, new int[] { 0, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00051479000831022859, -0.0949240326881409, 0.0353007800877094, true, new int[] { 16, 1, 2, 4, -1 }, new int[] { 16, 1, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073261340148746967, 0.0385022200644016, -0.148406594991684, true, new int[] { 2, 1, 4, 2, -1 }, new int[] { 2, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0244394596666098, -0.0134110199287534, 0.18843680620193479, true, new int[] { 14, 0, 4, 3, -1 }, new int[] { 13, 1, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0041021420620381832, -0.0499801896512508, 0.10747750103473661, true, new int[] { 4, 0, 3, 4, -1 }, new int[] { 5, 1, 1, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0022003119811415672, 0.15202569961547849, -0.0104131698608398, new int[] { 16, 2, 2, 2, -1 }, new int[] { 17, 2, 1, 1, 2 }, new int[] { 16, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3748419051989913E-05, 0.083184726536274, -0.0730274766683578, new int[] { 0, 2, 2, 2, -1 }, new int[] { 0, 2, 1, 1, 2 }, new int[] { 1, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0169174205511808, 0.0226879809051752, -0.17060829699039459, new int[] { 12, 5, 6, 1, -1 }, new int[] { 12, 5, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0023382799699902534, -0.0599084608256817, 0.0865803733468056, new int[] { 0, 5, 6, 1, -1 }, new int[] { 3, 5, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0035319819580763578, 0.033012930303812, -0.159266397356987, new int[] { 5, 3, 8, 2, -1 }, new int[] { 9, 3, 4, 1, 2 }, new int[] { 5, 4, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082293795421719551, -0.0760265216231346, 0.0753199979662895, new int[] { 7, 0, 2, 8, -1 }, new int[] { 8, 0, 1, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0413003005087376, -0.610956072807312, 0.0021895230747759342, true, new int[] { 9, 2, 2, 3, -1 }, new int[] { 9, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043179420754313469, 0.14404989778995511, -0.0388708002865314, true, new int[] { 9, 2, 3, 2, -1 }, new int[] { 9, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.7153229388641194E-05, -0.0498175993561745, 0.0487685203552246, new int[] { 16, 12, 2, 2, -1 }, new int[] { 17, 12, 1, 1, 2 }, new int[] { 16, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.9003963037393987E-05, -0.0683221071958542, 0.0680771768093109, new int[] { 0, 12, 2, 2, -1 }, new int[] { 0, 12, 1, 1, 2 }, new int[] { 1, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.0340400523273274E-05, 0.0513286590576172, -0.0355508588254452, new int[] { 16, 12, 2, 2, -1 }, new int[] { 17, 12, 1, 1, 2 }, new int[] { 16, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1807070121867582E-05, 0.0842122733592987, -0.0549248084425926, new int[] { 0, 12, 2, 2, -1 }, new int[] { 0, 12, 1, 1, 2 }, new int[] { 1, 13, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0472138598561287, 0.0023352450225502253, -0.34417921304702759, new int[] { 6, 13, 8, 2, -1 }, new int[] { 8, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040626591071486473, -0.1841911971569061, 0.0257207695394754, new int[] { 5, 0, 8, 2, -1 }, new int[] { 5, 0, 4, 1, 2 }, new int[] { 9, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0227853395044804, -0.1396211981773377, 0.0121513595804572, true, new int[] { 13, 8, 1, 4, -1 }, new int[] { 13, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0758542269468308, 0.11256880313158039, -0.0392036698758602, new int[] { 0, 5, 16, 6, -1 }, new int[] { 0, 7, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075154039077460766, -0.0197846591472626, 0.0587355606257916, new int[] { 12, 5, 1, 6, -1 }, new int[] { 12, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031700478866696358, -0.0542454309761524, 0.0902648568153381, new int[] { 5, 5, 1, 6, -1 }, new int[] { 5, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032852489966899157, -0.0545393712818623, 0.0909095332026482, new int[] { 12, 8, 6, 4, -1 }, new int[] { 15, 8, 3, 2, 2 }, new int[] { 12, 10, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0938187167048454, -0.481680691242218, 0.0097587006166577339, new int[] { 0, 5, 18, 4, -1 }, new int[] { 0, 5, 9, 2, 2 }, new int[] { 9, 7, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3132712966762483E-05, 0.041089840233326, -0.0365439392626286, new int[] { 10, 3, 2, 2, -1 }, new int[] { 11, 3, 1, 1, 2 }, new int[] { 10, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0198575109243393, -0.1172147020697594, 0.0405645594000816, new int[] { 2, 10, 6, 3, -1 }, new int[] { 4, 11, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0067911748774349689, 0.0064080609008669853, -0.32277610898017878, new int[] { 17, 5, 1, 3, -1 }, new int[] { 17, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.089469276368618, -0.357415109872818, 0.0124983703717589, true, new int[] { 8, 3, 3, 8, -1 }, new int[] { 8, 3, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0074639841914176941, -0.0199772007763386, 0.18343879282474521, true, new int[] { 12, 7, 4, 1, -1 }, new int[] { 13, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.3588905930519104, 0.0110323298722506, -0.55673301219940186, new int[] { 1, 3, 9, 12, -1 }, new int[] { 4, 7, 3, 4, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0288398806005716, 0.19993069767951971, -0.008988572284579277, true, new int[] { 12, 7, 4, 1, -1 }, new int[] { 13, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013966220431029797, -0.0439058393239975, 0.11055959761142729, true, new int[] { 6, 7, 1, 4, -1 }, new int[] { 5, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.008622707799077034, -0.43030598759651179, 0.0049329511821269989, new int[] { 17, 5, 1, 3, -1 }, new int[] { 17, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081372596323490143, 0.0061173681169748306, -0.70870321989059448, new int[] { 0, 5, 1, 3, -1 }, new int[] { 0, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.2080889872740954E-05, 0.0546860583126545, -0.0489871315658093, new int[] { 13, 1, 1, 3, -1 }, new int[] { 13, 2, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.2907347455620766E-05, 0.0777546167373657, -0.0597959607839584, new int[] { 6, 3, 2, 2, -1 }, new int[] { 6, 3, 1, 1, 2 }, new int[] { 7, 4, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0226010698825121, -0.1179111003875732, 0.0073637152090668678, new int[] { 6, 13, 8, 2, -1 }, new int[] { 8, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046634320169687271, 0.075231060385704, -0.057572990655899, true, new int[] { 5, 5, 4, 3, -1 }, new int[] { 6, 6, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0047270618379116058, 0.0710658580064774, -0.0859678834676743, new int[] { 6, 9, 6, 2, -1 }, new int[] { 6, 10, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.72711610794067383, 0.010272890329361, -0.46845859289169312, new int[] { 0, 0, 18, 11, -1 }, new int[] { 6, 0, 6, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030634279828518629, 0.1082748025655747, -0.0231780707836151, new int[] { 16, 3, 2, 4, -1 }, new int[] { 17, 3, 1, 2, 2 }, new int[] { 16, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0512203201651573, 0.0100829303264618, -0.46223670244216919, new int[] { 5, 3, 6, 6, -1 }, new int[] { 5, 3, 3, 3, 2 }, new int[] { 8, 6, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0233622491359711, 0.22211229801177981, -0.0204992592334747, new int[] { 5, 2, 8, 6, -1 }, new int[] { 7, 2, 4, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0226982291787863, -0.114096499979496, 0.0413477197289467, new int[] { 3, 2, 12, 4, -1 }, new int[] { 6, 2, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0032806419767439365, -0.0227168798446655, 0.1028605028986931, new int[] { 16, 3, 2, 4, -1 }, new int[] { 17, 3, 1, 2, 2 }, new int[] { 16, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0035968020092695951, 0.0211614202708006, -0.2068026065826416, new int[] { 1, 0, 3, 2, -1 }, new int[] { 2, 0, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0120496097952127, -0.26006719470024109, 0.0020481001120060682, new int[] { 16, 3, 2, 4, -1 }, new int[] { 17, 3, 1, 2, 2 }, new int[] { 16, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026617539115250111, 0.1557877063751221, -0.032414011657238, new int[] { 0, 3, 2, 4, -1 }, new int[] { 0, 3, 1, 2, 2 }, new int[] { 1, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0147399995476007, -0.16306230425834661, 0.0071668480522930622, true, new int[] { 14, 3, 4, 1, -1 }, new int[] { 15, 4, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0702147036790848, 0.36760380864143372, -0.0122618498280644, new int[] { 5, 5, 6, 6, -1 }, new int[] { 5, 5, 3, 3, 2 }, new int[] { 8, 8, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1149382963776588, -0.41006609797477722, 0.0111378999426961, new int[] { 8, 3, 2, 10, -1 }, new int[] { 8, 8, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0165353007614613, -0.49331170320510859, 0.0089259371161460876, true, new int[] { 4, 3, 1, 4, -1 }, new int[] { 3, 4, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0684577375650406, -0.629443883895874, 0.0013810090022161603, true, new int[] { 11, 8, 6, 1, -1 }, new int[] { 11, 8, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017950909677892923, 0.0439951792359352, -0.0981230884790421, true, new int[] { 7, 8, 1, 6, -1 }, new int[] { 7, 8, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00924097653478384, -0.0319279804825783, 0.0786244422197342, new int[] { 6, 13, 12, 1, -1 }, new int[] { 6, 13, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0150848804041743, -0.0652311071753502, 0.0835528671741486, new int[] { 0, 13, 16, 2, -1 }, new int[] { 8, 13, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.014755530282855, 0.0596954599022865, -0.0246289800852537, new int[] { 10, 9, 4, 4, -1 }, new int[] { 10, 11, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0138705503195524, 0.0068354210816323757, -0.6697801947593689, new int[] { 4, 1, 7, 3, -1 }, new int[] { 4, 2, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.4027196862734854E-05, -0.0388491488993168, 0.0505469888448715, new int[] { 11, 2, 2, 2, -1 }, new int[] { 12, 2, 1, 1, 2 }, new int[] { 11, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3879110813140869E-05, 0.0776163190603256, -0.0570690892636776, new int[] { 5, 2, 2, 2, -1 }, new int[] { 5, 2, 1, 1, 2 }, new int[] { 6, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00371186388656497, 0.0576838590204716, -0.0364302918314934, new int[] { 6, 13, 8, 2, -1 }, new int[] { 8, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0293781608343124, 0.0116572398692369, -0.3750464916229248, new int[] { 4, 13, 8, 2, -1 }, new int[] { 6, 13, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.75752860307693481, -0.0124912802129984, 0.30145660042762762, new int[] { 4, 3, 12, 9, -1 }, new int[] { 8, 6, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0284970905631781, -0.073959968984127, 0.06259386241436, true, new int[] { 7, 0, 6, 4, -1 }, new int[] { 9, 2, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0307283699512482, 0.0085481833666563034, -0.25127428770065308, true, new int[] { 13, 8, 1, 4, -1 }, new int[] { 13, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0336146205663681, -0.0114417197182775, 0.49361410737037659, true, new int[] { 5, 8, 4, 1, -1 }, new int[] { 5, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0226515103131533, 0.206863597035408, -0.0094910562038421631, new int[] { 4, 1, 12, 5, -1 }, new int[] { 7, 1, 6, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.5092899856390432E-05, 0.0643607303500175, -0.0726891383528709, new int[] { 5, 0, 4, 1, -1 }, new int[] { 6, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0065959710627794266, -0.1754118949174881, 0.0161602105945349, true, new int[] { 9, 0, 1, 4, -1 }, new int[] { 8, 1, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.0941398260183632E-05, 0.0750486701726913, -0.0528231002390385, new int[] { 4, 2, 2, 2, -1 }, new int[] { 4, 2, 1, 1, 2 }, new int[] { 5, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.590489930298645E-05, -0.04973965883255, 0.0585739016532898, new int[] { 12, 2, 2, 2, -1 }, new int[] { 13, 2, 1, 1, 2 }, new int[] { 12, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0394570280332118E-05, -0.0618803091347218, 0.0666748136281967, new int[] { 4, 2, 2, 2, -1 }, new int[] { 4, 2, 1, 1, 2 }, new int[] { 5, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0125536797568202, 0.0249107405543327, -0.12772439420223239, new int[] { 7, 0, 5, 4, -1 }, new int[] { 7, 2, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.058084350079298, -0.0178222507238388, 0.22898909449577329, true, new int[] { 9, 3, 1, 6, -1 }, new int[] { 9, 3, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0080750687047839165, -0.0227536000311375, 0.14363150298595431, new int[] { 15, 6, 2, 4, -1 }, new int[] { 15, 7, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0121633401140571, 0.0267546195536852, -0.182559996843338, new int[] { 0, 6, 18, 2, -1 }, new int[] { 0, 6, 9, 1, 2 }, new int[] { 9, 7, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015941649908199906, 0.0994387790560722, -0.0237834397703409, new int[] { 13, 6, 2, 2, -1 }, new int[] { 14, 6, 1, 1, 2 }, new int[] { 13, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1208584979176521, -0.59585529565811157, 0.0068441159091889858, new int[] { 0, 0, 5, 8, -1 }, new int[] { 0, 4, 5, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00874815322458744, -0.022079860791564, 0.26656690239906311, true, new int[] { 12, 2, 2, 2, -1 }, new int[] { 12, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0161353591829538, 0.0678508132696152, -0.077386163175106, true, new int[] { 8, 0, 10, 2, -1 }, new int[] { 8, 0, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.22907149791717529, -0.0353788398206234, 0.0487073697149754, new int[] { 5, 0, 11, 12, -1 }, new int[] { 5, 4, 11, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.50671470165252686, 0.005834176205098629, -0.66830587387084961, new int[] { 2, 0, 11, 12, -1 }, new int[] { 2, 4, 11, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0358187593519688, -0.26823309063911438, 0.0017747150268405676, new int[] { 11, 1, 2, 14, -1 }, new int[] { 12, 1, 1, 7, 2 }, new int[] { 11, 8, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0265013501048088, -0.30137398838996893, 0.0139737101271749, new int[] { 5, 1, 2, 14, -1 }, new int[] { 5, 1, 1, 7, 2 }, new int[] { 6, 8, 1, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0247978400439024, 0.0024552580434828997, -0.595221221446991, true, new int[] { 11, 8, 2, 1, -1 }, new int[] { 11, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016543349483981729, -0.0251259692013264, 0.19396910071372989, new int[] { 3, 6, 2, 2, -1 }, new int[] { 3, 6, 1, 1, 2 }, new int[] { 4, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0070274528115987778, 0.0204041302204132, -0.0531757883727551, true, new int[] { 11, 8, 2, 1, -1 }, new int[] { 11, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0742075890302658, 0.0124620702117682, -0.33352059125900269, new int[] { 0, 8, 18, 4, -1 }, new int[] { 0, 8, 9, 2, 2 }, new int[] { 9, 10, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0023010969161987305, -0.14958749711513519, 0.020109549164772, true, new int[] { 14, 0, 2, 1, -1 }, new int[] { 14, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013790120137855411, 0.0333775207400322, -0.12395989894866941, true, new int[] { 4, 0, 1, 2, -1 }, new int[] { 4, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.82677090167999268, 0.0046560140326619148, -0.7640576958656311, new int[] { 3, 0, 15, 14, -1 }, new int[] { 8, 0, 5, 14, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.29461461305618292, -0.0152309397235513, 0.31044191122055048, new int[] { 4, 0, 9, 13, -1 }, new int[] { 7, 0, 3, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0746835619211197, 0.0088676074519753456, -0.522868275642395, new int[] { 7, 5, 4, 9, -1 }, new int[] { 7, 5, 2, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0880003422498703, -0.0119359400123358, 0.40419429540634161, true, new int[] { 9, 1, 4, 4, -1 }, new int[] { 9, 1, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0053336159326136112, 0.0136402798816562, -0.244797095656395, new int[] { 7, 2, 6, 2, -1 }, new int[] { 10, 2, 3, 1, 2 }, new int[] { 7, 3, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0543241314589977, -0.33548229932785029, 0.0117584997788072, true, new int[] { 7, 4, 6, 2, -1 }, new int[] { 9, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0325612500309944, 0.0013724969467148185, -0.33259418606758118, true, new int[] { 11, 8, 3, 2, -1 }, new int[] { 12, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048455069772899151, -0.0363678596913815, 0.1394127011299133, true, new int[] { 7, 8, 2, 3, -1 }, new int[] { 6, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0094578228890895844, -0.15179359912872309, 0.0071280989795923233, new int[] { 12, 14, 4, 1, -1 }, new int[] { 12, 14, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065718130208551884, 0.0160512197762728, -0.2522624135017395, new int[] { 2, 14, 4, 1, -1 }, new int[] { 4, 14, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0234677102416754, 0.006124687846750021, -0.23419499397277829, new int[] { 12, 11, 4, 4, -1 }, new int[] { 14, 11, 2, 2, 2 }, new int[] { 12, 13, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017358670011162758, -0.0396148599684238, 0.1216652020812035, new int[] { 6, 12, 1, 3, -1 }, new int[] { 6, 13, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00080753577640280128, -0.0265275705605745, 0.0391027294099331, new int[] { 11, 13, 1, 2, -1 }, new int[] { 11, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0055824369192123413, -0.10073930025100709, 0.0372616909444332, new int[] { 3, 10, 4, 4, -1 }, new int[] { 3, 11, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026079979725182056, 0.0740168169140816, -0.010955180041492, new int[] { 11, 13, 1, 2, -1 }, new int[] { 11, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.9571033236570656E-05, -0.0852629169821739, 0.064489983022213, new int[] { 6, 13, 1, 2, -1 }, new int[] { 6, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0819417685270309, 0.0020980359986424446, -0.61844950914382935, new int[] { 7, 7, 10, 4, -1 }, new int[] { 12, 7, 5, 2, 2 }, new int[] { 7, 9, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.019427090883255, -0.022283710539341, 0.19918359816074371, new int[] { 1, 7, 10, 4, -1 }, new int[] { 1, 7, 5, 2, 2 }, new int[] { 6, 9, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1507761031389237, -0.64394700527191162, 0.0070817708037793636, new int[] { 0, 4, 18, 4, -1 }, new int[] { 6, 4, 6, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015093310503289104, -0.10650269687175749, 0.0375769101083279, new int[] { 2, 11, 4, 4, -1 }, new int[] { 2, 11, 2, 2, 2 }, new int[] { 4, 13, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0362875610589981, 0.00062272557988762856, -1.0000269412994385, new int[] { 9, 14, 6, 1, -1 }, new int[] { 11, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017432459862902761, 0.0829876065254211, -0.0519000887870789, new int[] { 3, 14, 6, 1, -1 }, new int[] { 5, 14, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.1345883295871317E-05, 0.0411302000284195, -0.0397632196545601, new int[] { 11, 12, 3, 1, -1 }, new int[] { 12, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6694999178289436E-05, -0.057489451020956, 0.0767864733934402, new int[] { 4, 12, 3, 1, -1 }, new int[] { 5, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0332492999732494, 0.0608417689800262, new int[] { 13, 5, 1, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.021666070446372, -0.42399600148200989, 0.0095887510105967522, new int[] { 5, 3, 6, 4, -1 }, new int[] { 5, 4, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.65124082565307617, -0.0139236301183701, 0.20358699560165411, new int[] { 4, 3, 12, 9, -1 }, new int[] { 8, 6, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081125432625412941, 0.0472846701741219, -0.0877940282225609, new int[] { 4, 6, 10, 2, -1 }, new int[] { 4, 7, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0087661407887935638, 0.0003612214932218194, -0.46132668852806091, new int[] { 13, 5, 1, 2, -1 }, new int[] { 13, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.6974760809680447E-05, -0.0540806017816067, 0.0876793190836906, new int[] { 4, 5, 1, 2, -1 }, new int[] { 4, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.2681202911771834E-05, -0.0361079499125481, 0.0403531081974506, new int[] { 11, 4, 4, 2, -1 }, new int[] { 11, 5, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0036902779247611761, 0.0328456684947014, -0.1765446066856384, new int[] { 3, 4, 4, 2, -1 }, new int[] { 3, 5, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.002488462021574378, -0.1116909012198448, 0.0380927696824074, new int[] { 5, 4, 8, 2, -1 }, new int[] { 9, 4, 4, 1, 2 }, new int[] { 5, 5, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0061029191128909588, -0.0218723006546497, 0.21474809944629669, true, new int[] { 6, 2, 2, 2, -1 }, new int[] { 6, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00842163898050785, 0.0250333193689585, -0.10524729639291761, new int[] { 14, 3, 2, 11, -1 }, new int[] { 14, 3, 1, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0112776597961783, -0.1206863969564438, 0.0366918705403805, new int[] { 2, 3, 2, 11, -1 }, new int[] { 3, 3, 1, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025908139068633318, 0.0489619709551334, -0.0271127801388502, new int[] { 14, 6, 4, 3, -1 }, new int[] { 15, 6, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0039354357868433, -0.0488033294677734, 0.0915941670536995, new int[] { 0, 6, 4, 5, -1 }, new int[] { 1, 6, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0057140849530696869, 0.0652810335159302, -0.0544281415641308, new int[] { 11, 0, 6, 3, -1 }, new int[] { 13, 0, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075044799596071243, 0.0404559001326561, -0.1001691967248917, true, new int[] { 7, 6, 2, 2, -1 }, new int[] { 7, 6, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00360394106246531, -0.0484412014484406, 0.0443660393357277, new int[] { 13, 3, 1, 6, -1 }, new int[] { 13, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0142484996467829, -0.18958659470081329, 0.0223791096359491, new int[] { 4, 4, 4, 4, -1 }, new int[] { 5, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1074685975909233, -0.0145733403041959, 0.18533800542354581, new int[] { 8, 1, 3, 9, -1 }, new int[] { 9, 4, 1, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00654483400285244, 0.0309639498591423, -0.15456229448318479, true, new int[] { 9, 4, 3, 1, -1 }, new int[] { 10, 5, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.405587911605835, -0.0106067704036832, 0.0930665135383606, new int[] { 6, 2, 9, 9, -1 }, new int[] { 9, 5, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.4504162073135376, -0.0119176097214222, 0.37239480018615723, new int[] { 3, 2, 9, 9, -1 }, new int[] { 6, 5, 3, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(1.0484869480133057, 0.024846600368619, -0.20550209283828741, new int[] { 0, 0, 18, 12, -1 }, new int[] { 6, 4, 6, 4, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0317365005612373, 0.18238979578018191, -0.0208370704203844, new int[] { 1, 3, 14, 4, -1 }, new int[] { 1, 3, 7, 2, 2 }, new int[] { 8, 5, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1016217023134232, 0.015214960090816, -0.28738000988960272, new int[] { 0, 0, 18, 8, -1 }, new int[] { 9, 0, 9, 4, 2 }, new int[] { 0, 4, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016911029815673828, -0.0272036101669073, 0.15361389517784119, new int[] { 5, 10, 2, 2, -1 }, new int[] { 5, 10, 1, 1, 2 }, new int[] { 6, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0550902001559734, 0.40182009339332581, -0.0026924409903585911, true, new int[] { 9, 8, 2, 3, -1 }, new int[] { 8, 9, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066355741582810879, -0.1039951965212822, 0.0399309694766998, true, new int[] { 9, 8, 3, 2, -1 }, new int[] { 10, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.28234618902206421, -0.6573529839515686, 0.0022085180971771479, true, new int[] { 9, 0, 6, 9, -1 }, new int[] { 9, 0, 3, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.356060802936554, 0.0088273994624614716, -0.41840550303459167, true, new int[] { 9, 0, 9, 6, -1 }, new int[] { 9, 0, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028794088866561651, -0.0477025806903839, 0.0486192405223846, new int[] { 9, 4, 4, 2, -1 }, new int[] { 9, 4, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0345713905990124, -0.16541080176830289, 0.0324508398771286, new int[] { 0, 2, 18, 1, -1 }, new int[] { 9, 2, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0700211822986603, 0.00713475001975894, -0.51421910524368286, new int[] { 9, 10, 6, 3, -1 }, new int[] { 11, 11, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0253863092511892, -0.1287622004747391, 0.0291819702833891, new int[] { 0, 3, 8, 4, -1 }, new int[] { 0, 5, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007792747113853693, 0.0385298691689968, -0.0494838394224644, new int[] { 14, 3, 3, 8, -1 }, new int[] { 14, 5, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0142815597355366, 0.0056447219103574753, -0.70385247468948364, true, new int[] { 5, 0, 2, 1, -1 }, new int[] { 5, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.0420181788504124, 0.0442302897572517, new int[] { 13, 1, 2, 2, -1 }, new int[] { 14, 1, 1, 1, 2 }, new int[] { 13, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0035789560060948133, 0.46143290400505071, -0.0097652971744537354, new int[] { 3, 1, 2, 2, -1 }, new int[] { 3, 1, 1, 1, 2 }, new int[] { 4, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.9024448748677969E-05, 0.0501331388950348, -0.0589645393192768, new int[] { 13, 0, 4, 1, -1 }, new int[] { 14, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030192299745976925, -0.19493819773197171, 0.0247106906026602, new int[] { 1, 0, 4, 1, -1 }, new int[] { 2, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025278010871261358, 0.0835050269961357, -0.0252687390893698, true, new int[] { 17, 0, 1, 2, -1 }, new int[] { 17, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017980269622057676, -0.048482459038496, 0.0943117365241051, true, new int[] { 1, 0, 2, 1, -1 }, new int[] { 1, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0226906202733517, -0.2997882068157196, 0.0022890099789947271, new int[] { 16, 0, 1, 8, -1 }, new int[] { 16, 2, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014375130413100123, -0.0624394081532955, 0.0752900913357735, new int[] { 1, 0, 1, 8, -1 }, new int[] { 1, 2, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0082696974277496338, -0.0303539503365755, 0.0880893915891647, new int[] { 8, 0, 4, 2, -1 }, new int[] { 8, 0, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15055939555168149, 0.1941386014223099, -0.0227722208946943, new int[] { 2, 0, 12, 15, -1 }, new int[] { 5, 0, 6, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017811149591580033, -0.060310248285532, 0.0200738906860352, new int[] { 11, 2, 6, 4, -1 }, new int[] { 11, 2, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00474506476894021, -0.0518799908459187, 0.0740923434495926, new int[] { 4, 2, 8, 6, -1 }, new int[] { 4, 2, 4, 3, 2 }, new int[] { 8, 5, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0039645358920097351, -0.12223859876394271, 0.0184847600758076, true, new int[] { 10, 1, 1, 4, -1 }, new int[] { 9, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.2112957984209061, 0.006967875175178051, -0.63405531644821167, true, new int[] { 9, 0, 7, 6, -1 }, new int[] { 7, 2, 7, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0679322928190231, 0.0112383002415299, -0.29897838830947882, new int[] { 10, 6, 8, 2, -1 }, new int[] { 10, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.35460498929023743, 0.0108207296580076, -0.40180310606956482, new int[] { 0, 0, 17, 9, -1 }, new int[] { 0, 3, 17, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0678805708885193, -0.0090837832540273666, 0.28558140993118292, new int[] { 7, 0, 5, 6, -1 }, new int[] { 7, 3, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0231790095567703, 0.0120336599647999, -0.34283030033111572, new int[] { 5, 1, 8, 4, -1 }, new int[] { 5, 1, 4, 2, 2 }, new int[] { 9, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0250181294977665, 0.16851060092449191, -0.0148548297584057, new int[] { 9, 0, 3, 9, -1 }, new int[] { 9, 3, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0108465002849698, -0.0498660691082478, 0.0913302898406982, true, new int[] { 9, 2, 4, 2, -1 }, new int[] { 9, 2, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0674327909946442, -0.0671769231557846, 0.052287008613348, new int[] { 4, 2, 11, 8, -1 }, new int[] { 4, 4, 11, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1040098965167999, 0.21269090473651889, -0.0196353103965521, new int[] { 1, 4, 16, 6, -1 }, new int[] { 1, 6, 16, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0195524599403143, -0.0859493836760521, 0.0108785601332784, new int[] { 10, 6, 8, 2, -1 }, new int[] { 10, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010041260393336415, -0.0881467536091805, 0.0533496886491776, new int[] { 0, 6, 8, 2, -1 }, new int[] { 4, 6, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0031779510900378227, -0.0257080793380737, 0.12620189785957339, new int[] { 13, 8, 4, 2, -1 }, new int[] { 15, 8, 2, 1, 2 }, new int[] { 13, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0051974221132695675, -0.14909990131855011, 0.0257342308759689, new int[] { 0, 7, 3, 3, -1 }, new int[] { 0, 8, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.008438553661108017, 0.17627319693565369, -0.0173361804336309, true, new int[] { 17, 5, 1, 3, -1 }, new int[] { 16, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0063723679631948471, -0.0288299303501844, 0.1601462066173554, true, new int[] { 1, 5, 3, 1, -1 }, new int[] { 2, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00064913480309769511, 0.0250607505440712, -0.0684819966554642, new int[] { 17, 6, 1, 2, -1 }, new int[] { 17, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3739310563541949E-05, 0.0597767196595669, -0.0690794587135315, new int[] { 0, 6, 1, 2, -1 }, new int[] { 0, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.021902309730649, 0.0158000495284796, -0.2590233981609345, new int[] { 17, 1, 1, 12, -1 }, new int[] { 17, 7, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0232256501913071, -0.15240189433097839, 0.0343589708209038, new int[] { 0, 1, 1, 12, -1 }, new int[] { 0, 7, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0173969995230436, -0.0445144101977348, 0.086146168410778, new int[] { 0, 6, 18, 4, -1 }, new int[] { 0, 7, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043821102008223534, -0.0655946731567383, 0.0700312927365303, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0522718392312527, -0.84593230485916138, 0.00407365383580327, new int[] { 6, 7, 6, 2, -1 }, new int[] { 6, 8, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.6945039280690253E-05, 0.0711033865809441, -0.0569700710475445, new int[] { 0, 9, 3, 1, -1 }, new int[] { 1, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073246699757874012, 0.0101481601595879, -0.164958193898201, new int[] { 16, 9, 2, 2, -1 }, new int[] { 16, 10, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0796489417552948, 0.0049309800378978252, -0.73935991525650024, new int[] { 3, 10, 6, 3, -1 }, new int[] { 5, 11, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0256457198411226, -0.0099361119791865349, 0.195734903216362, true, new int[] { 15, 1, 3, 3, -1 }, new int[] { 14, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0215177107602358, -0.37398171424865723, 0.010564680211246, new int[] { 2, 5, 14, 2, -1 }, new int[] { 2, 5, 7, 1, 2 }, new int[] { 9, 6, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00310848793014884, -0.0232892800122499, 0.0444528982043266, true, new int[] { 15, 1, 3, 3, -1 }, new int[] { 14, 2, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0203057900071144, 0.184503898024559, -0.0220416504889727, true, new int[] { 3, 1, 3, 3, -1 }, new int[] { 4, 2, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00023073209740687162, -0.042533028870821, 0.0405342392623425, new int[] { 14, 4, 3, 2, -1 }, new int[] { 14, 5, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0051654567942023277, 0.0195509009063244, -0.2752223014831543, new int[] { 1, 4, 3, 2, -1 }, new int[] { 1, 5, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0133738899603486, -0.10676769912242889, 0.0157130900770426, new int[] { 15, 3, 3, 11, -1 }, new int[] { 16, 3, 1, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0305575095117092, -0.49036020040512091, 0.0084824627265334129, new int[] { 0, 3, 3, 11, -1 }, new int[] { 1, 3, 1, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00449386378750205, 0.2458741962909699, -0.0073765181005001068, new int[] { 14, 5, 2, 2, -1 }, new int[] { 15, 5, 1, 1, 2 }, new int[] { 14, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015328789595514536, -0.0219983607530594, 0.1710575073957443, new int[] { 2, 5, 2, 2, -1 }, new int[] { 2, 5, 1, 1, 2 }, new int[] { 3, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0284645706415176, -0.0044271750375628471, 0.37864500284194952, new int[] { 15, 5, 3, 4, -1 }, new int[] { 15, 6, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036278439220041037, -0.1194301024079323, 0.0363873392343521, new int[] { 0, 5, 3, 4, -1 }, new int[] { 0, 6, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0065880590118467808, 0.0047421031631529331, -0.23040629923343661, new int[] { 17, 8, 1, 3, -1 }, new int[] { 17, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017257609870284796, -0.15124620497226721, 0.0245305094867945, new int[] { 0, 8, 1, 3, -1 }, new int[] { 0, 9, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0040079229511320591, 0.1179575026035309, -0.0284553095698357, new int[] { 16, 6, 2, 4, -1 }, new int[] { 17, 6, 1, 2, 2 }, new int[] { 16, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0060597620904445648, -0.0159428808838129, 0.263492614030838, new int[] { 0, 6, 2, 4, -1 }, new int[] { 0, 6, 1, 2, 2 }, new int[] { 1, 8, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.10206180065870291, 0.0228738095611334, -0.1756930947303772, new int[] { 0, 6, 18, 6, -1 }, new int[] { 9, 6, 9, 3, 2 }, new int[] { 0, 9, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00536059495061636, -0.284327894449234, 0.0135392798110843, new int[] { 5, 1, 6, 2, -1 }, new int[] { 5, 1, 3, 1, 2 }, new int[] { 8, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013634009519591928, 0.0150163397192955, -0.2169246971607208, new int[] { 10, 1, 2, 2, -1 }, new int[] { 11, 1, 1, 1, 2 }, new int[] { 10, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.18671513418667E-05, 0.0715956836938858, -0.0591941215097904, new int[] { 6, 1, 2, 2, -1 }, new int[] { 6, 1, 1, 1, 2 }, new int[] { 7, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055599510669708252, -0.0504433810710907, 0.0246312096714973, new int[] { 10, 1, 6, 3, -1 }, new int[] { 10, 1, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0081721879541873932, 0.14858539402484891, -0.0320550985634327, new int[] { 2, 1, 6, 3, -1 }, new int[] { 5, 1, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0511872991919518, -0.25399059057235718, 0.0068093240261077881, new int[] { 12, 0, 6, 3, -1 }, new int[] { 14, 0, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0402427017688751, 0.0073603428900241852, -0.53896129131317139, new int[] { 0, 0, 6, 3, -1 }, new int[] { 2, 0, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0086354929953813553, 0.2015924006700516, -0.0168281905353069, new int[] { 13, 8, 4, 2, -1 }, new int[] { 15, 8, 2, 1, 2 }, new int[] { 13, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.2959326896816492E-05, -0.0544128902256489, 0.073297880589962, new int[] { 1, 8, 4, 2, -1 }, new int[] { 1, 8, 2, 1, 2 }, new int[] { 3, 9, 2, 1, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion

            #region Stage 19
            stage = new HaarCascadeStage(-1.0771520137786865, 18, -1); nodes = new List<HaarFeatureNode[]>();
            nodes.Add(new[] { new HaarFeatureNode(-0.0465844385325909, 0.39758908748626709, -0.10487789660692221, true, new int[] { 9, 6, 1, 6, -1 }, new int[] { 7, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0135460803285241, 0.1016070991754532, -0.0605821199715137, new int[] { 10, 0, 6, 6, -1 }, new int[] { 12, 2, 2, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0212406199425459, -0.21520909667015081, 0.0991928800940514, new int[] { 5, 0, 4, 8, -1 }, new int[] { 7, 0, 2, 8, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0068675312213599682, 0.34559080004692078, -0.0272973105311394, new int[] { 12, 7, 1, 3, -1 }, new int[] { 12, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001887471997179091, -0.0626463666558266, 0.22028639912605291, new int[] { 4, 7, 1, 3, -1 }, new int[] { 4, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00966489315032959, 0.12642039358615881, -0.0029440899379551411, true, new int[] { 15, 5, 1, 3, -1 }, new int[] { 14, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0057599171996116638, -0.0645451918244362, 0.2116688936948776, true, new int[] { 3, 5, 3, 1, -1 }, new int[] { 4, 6, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0426046885550022, 0.0816654786467552, -0.2211515009403229, new int[] { 0, 9, 18, 4, -1 }, new int[] { 9, 9, 9, 2, 2 }, new int[] { 0, 11, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011809020070359111, 0.0537825897336006, -0.2183254957199097, true, new int[] { 2, 0, 1, 2, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0258668307214975, -0.0034579040948301554, -0.2280915975570679, new int[] { 8, 0, 7, 4, -1 }, new int[] { 8, 2, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0130240898579359, -0.23362639546394351, 0.0455196797847748, new int[] { 3, 0, 12, 4, -1 }, new int[] { 3, 0, 6, 2, 2 }, new int[] { 9, 2, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.6178720872849226E-05, 0.0630585104227066, -0.0357771515846252, new int[] { 9, 1, 2, 2, -1 }, new int[] { 10, 1, 1, 1, 2 }, new int[] { 9, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003864985890686512, 0.0413089096546173, -0.21261259913444519, new int[] { 7, 0, 4, 4, -1 }, new int[] { 7, 0, 2, 2, 2 }, new int[] { 9, 2, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043429462239146233, 0.1096725985407829, -0.067377433180809, new int[] { 12, 8, 2, 2, -1 }, new int[] { 12, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.003246336942538619, -0.0599126406013966, 0.24788300693035131, new int[] { 4, 8, 2, 2, -1 }, new int[] { 4, 9, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0446722097694874, -0.137876495718956, 0.0075812488794326782, new int[] { 11, 5, 4, 10, -1 }, new int[] { 11, 5, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0596978403627872, -0.3720127940177918, 0.0243327803909779, new int[] { 3, 5, 4, 10, -1 }, new int[] { 5, 5, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.009966626763343811, 0.0740873217582703, -0.12867400050163269, new int[] { 1, 10, 16, 3, -1 }, new int[] { 5, 10, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011090090265497565, -0.0450637899339199, 0.1985294967889786, new int[] { 5, 8, 2, 2, -1 }, new int[] { 5, 8, 1, 1, 2 }, new int[] { 6, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1913764029741287, 0.0166084691882133, -0.406623899936676, new int[] { 0, 5, 18, 8, -1 }, new int[] { 9, 5, 9, 4, 2 }, new int[] { 0, 9, 9, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0291308406740427, 0.0361067317426205, -0.2113531976938248, new int[] { 0, 3, 3, 9, -1 }, new int[] { 0, 6, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.003912351094186306, -0.1371506005525589, 0.0311542004346848, new int[] { 9, 3, 8, 2, -1 }, new int[] { 9, 4, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0102061899378896, 0.0290562491863966, -0.250322699546814, new int[] { 0, 3, 18, 2, -1 }, new int[] { 0, 3, 9, 1, 2 }, new int[] { 9, 4, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0544211715459824, -0.36787760257720947, 0.0049542388878762722, new int[] { 6, 11, 8, 4, -1 }, new int[] { 8, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0105043696239591, -0.039119478315115, 0.1786668002605438, true, new int[] { 2, 4, 3, 2, -1 }, new int[] { 3, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0389032289385796, -0.1115652024745941, 0.0494851097464561, new int[] { 14, 4, 4, 6, -1 }, new int[] { 14, 7, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0030581050086766481, 0.1185448989272118, -0.0652535036206245, true, new int[] { 9, 0, 9, 3, -1 }, new int[] { 8, 1, 9, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0120711103081703, 0.0169083792716265, -0.0460892505943775, new int[] { 6, 11, 8, 4, -1 }, new int[] { 8, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0361215807497501, -0.28585109114646912, 0.0273920707404613, new int[] { 4, 11, 8, 4, -1 }, new int[] { 6, 11, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-7.0450740167871118E-05, 0.0811922177672386, -0.0853394791483879, new int[] { 15, 0, 2, 1, -1 }, new int[] { 15, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0614753998816013, -0.30502641201019293, 0.021672610193491, true, new int[] { 4, 4, 6, 2, -1 }, new int[] { 6, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12384360283613211, -0.0086616817861795425, 0.0958835631608963, true, new int[] { 9, 3, 6, 4, -1 }, new int[] { 11, 5, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13729789853096011, 0.32487779855728149, -0.0273847002536058, true, new int[] { 9, 3, 4, 6, -1 }, new int[] { 7, 5, 4, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.37660130858421331, 0.0695123001933098, -0.0875100269913673, new int[] { 3, 1, 15, 14, -1 }, new int[] { 3, 8, 15, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1042848974466324, -0.1743391007184982, 0.0465723089873791, new int[] { 0, 1, 4, 14, -1 }, new int[] { 0, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0153772495687008, 0.0072437077760696411, -0.3706468939781189, true, new int[] { 12, 0, 2, 2, -1 }, new int[] { 12, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0103409802541137, 0.0195991508662701, -0.35058119893074041, true, new int[] { 6, 0, 2, 2, -1 }, new int[] { 6, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.6178720872849226E-05, -0.0371437408030033, 0.0463190414011478, new int[] { 15, 0, 2, 1, -1 }, new int[] { 15, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.1104918384226039E-05, 0.0750196501612663, -0.0955687314271927, new int[] { 1, 0, 2, 1, -1 }, new int[] { 2, 0, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012594480067491531, -0.0361403413116932, 0.1402405053377152, new int[] { 11, 7, 2, 2, -1 }, new int[] { 12, 7, 1, 1, 2 }, new int[] { 11, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044775051064789295, 0.11984299868345261, -0.0559747815132141, new int[] { 0, 0, 4, 6, -1 }, new int[] { 0, 0, 2, 3, 2 }, new int[] { 2, 3, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025892409030348063, 0.20983800292015081, -0.0216069091111422, new int[] { 11, 7, 2, 2, -1 }, new int[] { 12, 7, 1, 1, 2 }, new int[] { 11, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.8334530725260265E-05, -0.0646458193659782, 0.1100763976573944, new int[] { 5, 7, 2, 2, -1 }, new int[] { 5, 7, 1, 1, 2 }, new int[] { 6, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0493306517601013, -0.0343082509934902, 0.1055921986699104, true, new int[] { 14, 6, 4, 5, -1 }, new int[] { 14, 6, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00031046869116835296, 0.0380286201834679, -0.20678110420703891, new int[] { 8, 7, 2, 2, -1 }, new int[] { 8, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0112909199669957, -0.0430234186351299, 0.16977259516716, new int[] { 1, 8, 16, 3, -1 }, new int[] { 1, 9, 16, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0019364829640835524, -0.1082670986652374, 0.0643948465585709, new int[] { 1, 10, 16, 2, -1 }, new int[] { 1, 11, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1330419927835465, -0.0107648801058531, 0.30249550938606262, true, new int[] { 14, 6, 4, 5, -1 }, new int[] { 14, 6, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1217804998159409, -0.40108850598335272, 0.019901329651475, true, new int[] { 4, 6, 5, 4, -1 }, new int[] { 4, 6, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.8507350584259257E-05, 0.0578306503593922, -0.0554163902997971, new int[] { 15, 1, 1, 2, -1 }, new int[] { 15, 2, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0081427283585071564, -0.13038429617881781, 0.0504461117088795, true, new int[] { 2, 1, 10, 2, -1 }, new int[] { 2, 1, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.25049310922622681, 0.0049552097916603088, -0.84521442651748657, true, new int[] { 10, 0, 6, 9, -1 }, new int[] { 12, 2, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0029000479262322187, -0.04863416031003, 0.13975860178470609, new int[] { 4, 9, 3, 3, -1 }, new int[] { 4, 10, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0095292963087558746, -0.48227089643478388, 0.0089182211086153984, new int[] { 10, 0, 3, 4, -1 }, new int[] { 10, 1, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.001260858029127121, -0.14396399259567261, 0.044625461101532, true, new int[] { 1, 10, 2, 1, -1 }, new int[] { 1, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.000698642514180392, -0.0534688793122768, 0.0444802902638912, true, new int[] { 15, 11, 3, 1, -1 }, new int[] { 16, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0955888582393527E-05, -0.0910912230610847, 0.0615591295063496, true, new int[] { 3, 11, 1, 3, -1 }, new int[] { 2, 12, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0422890111804008, -0.1452918946743012, 0.0229476597160101, true, new int[] { 10, 0, 6, 9, -1 }, new int[] { 12, 2, 2, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0839773416519165, 0.0371137298643589, -0.1620655953884125, true, new int[] { 8, 0, 9, 6, -1 }, new int[] { 6, 2, 9, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0091143082827329636, -0.0084407972171902657, 0.1036289036273956, new int[] { 7, 10, 6, 2, -1 }, new int[] { 10, 10, 3, 1, 2 }, new int[] { 7, 11, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.6319790271809325E-05, -0.0675051584839821, 0.0853116363286972, new int[] { 7, 7, 2, 2, -1 }, new int[] { 7, 7, 1, 1, 2 }, new int[] { 8, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.52136087417602539, -0.0144045604392886, 0.44966968894004822, new int[] { 3, 3, 12, 6, -1 }, new int[] { 7, 5, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0158583596348763, 0.0245071090757847, -0.28061380982398992, new int[] { 6, 3, 1, 6, -1 }, new int[] { 6, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00090295937843620777, -0.0197774693369865, 0.0582239516079426, new int[] { 16, 0, 1, 3, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016763530438765883, -0.15801259875297549, 0.0340122990310192, new int[] { 4, 0, 3, 2, -1 }, new int[] { 4, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.4684870368218981E-05, 0.0519807413220406, -0.0352598205208778, new int[] { 16, 0, 1, 3, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.0777395367622375, 0.0757706016302109, new int[] { 1, 0, 1, 3, -1 }, new int[] { 1, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0059450380504131317, -0.1076762974262238, 0.0473425313830376, new int[] { 10, 3, 8, 2, -1 }, new int[] { 10, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0338867083191872, 0.2539583146572113, -0.0263967607170343, new int[] { 0, 3, 8, 2, -1 }, new int[] { 4, 3, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055312961339950562, -0.0277216397225857, 0.2323354035615921, new int[] { 7, 0, 4, 1, -1 }, new int[] { 7, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0060472032055258751, -0.17387150228023529, 0.03456149995327, new int[] { 0, 0, 18, 2, -1 }, new int[] { 0, 0, 9, 1, 2 }, new int[] { 9, 1, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0319555215537548, -0.0191999804228544, 0.0308420602232218, new int[] { 12, 3, 2, 12, -1 }, new int[] { 12, 9, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.090737096965313, 0.0077871060930192471, -0.75864750146865845, new int[] { 4, 3, 2, 12, -1 }, new int[] { 4, 9, 2, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0124458596110344, 0.14370959997177121, -0.0104776499792933, new int[] { 7, 10, 6, 2, -1 }, new int[] { 10, 10, 3, 1, 2 }, new int[] { 7, 11, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.01130152028054, -0.1322194039821625, 0.0409673303365707, true, new int[] { 6, 4, 7, 2, -1 }, new int[] { 6, 4, 7, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.010558320209384, -0.3396332859992981, 0.0126309199258685, new int[] { 13, 13, 4, 1, -1 }, new int[] { 13, 13, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0046060150489211082, -0.0353191308677197, 0.1581331938505173, new int[] { 4, 9, 6, 2, -1 }, new int[] { 4, 9, 3, 1, 2 }, new int[] { 7, 10, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0306612607091665, -0.587932825088501, 0.0096826143562793732, new int[] { 7, 8, 4, 2, -1 }, new int[] { 7, 9, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0062674311921000481, -0.1976262032985687, 0.0269288308918476, new int[] { 1, 13, 4, 1, -1 }, new int[] { 3, 13, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012989880051463842, -0.0291242301464081, 0.0762825235724449, new int[] { 12, 10, 1, 3, -1 }, new int[] { 12, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0048161852173507214, 0.0180221293121576, -0.29259270429611212, new int[] { 6, 0, 6, 2, -1 }, new int[] { 6, 0, 3, 1, 2 }, new int[] { 9, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0044622411951422691, 0.0485544018447399, -0.0468474701046944, true, new int[] { 9, 0, 2, 2, -1 }, new int[] { 9, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.9135680455947295E-05, 0.0812152177095413, -0.0633795633912086, new int[] { 6, 2, 2, 2, -1 }, new int[] { 6, 2, 1, 1, 2 }, new int[] { 7, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0050573139451444149, 0.0140971401706338, -0.20685930550098419, new int[] { 8, 0, 3, 4, -1 }, new int[] { 8, 1, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0013823669869452715, -0.0426558181643486, 0.1154166981577873, new int[] { 6, 7, 1, 3, -1 }, new int[] { 6, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0401844494044781, -0.2984366118907929, 0.01746373064816, new int[] { 4, 8, 10, 4, -1 }, new int[] { 9, 8, 5, 2, 2 }, new int[] { 4, 10, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0070384680293500423, -0.0521952509880066, 0.0946906581521034, new int[] { 0, 9, 18, 2, -1 }, new int[] { 0, 10, 18, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.6935990869533271E-05, 0.0507361218333244, -0.122299499809742, new int[] { 9, 9, 1, 2, -1 }, new int[] { 9, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.9834190324181691E-05, -0.0615346282720566, 0.0821938663721085, new int[] { 8, 9, 2, 2, -1 }, new int[] { 8, 9, 1, 1, 2 }, new int[] { 9, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0239803306758404, 0.0899486094713211, -0.0531572587788105, new int[] { 3, 5, 12, 6, -1 }, new int[] { 7, 7, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0198573190718889, -0.0290171504020691, 0.19026429951190951, new int[] { 6, 4, 6, 4, -1 }, new int[] { 6, 6, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1887260973453522, -0.189160093665123, 0.00914729293435812, true, new int[] { 9, 0, 7, 8, -1 }, new int[] { 7, 2, 7, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0053056180477142334, 0.0595022700726986, -0.1106636002659798, new int[] { 6, 6, 6, 2, -1 }, new int[] { 6, 7, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0179616697132587, 0.0069341547787189484, -0.29351618885993958, true, new int[] { 12, 0, 3, 3, -1 }, new int[] { 13, 1, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044897631742060184, 0.0345449112355709, -0.14389620721340179, true, new int[] { 6, 0, 3, 3, -1 }, new int[] { 5, 1, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1378097981214523, 0.66656690835952759, -0.0076799020171165466, new int[] { 5, 1, 8, 6, -1 }, new int[] { 5, 4, 8, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0250661708414555, 0.0270246397703886, -0.18130689859390259, true, new int[] { 9, 0, 7, 3, -1 }, new int[] { 8, 1, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0066011329181492329, -0.0471079796552658, 0.0535648204386234, new int[] { 14, 5, 4, 4, -1 }, new int[] { 14, 7, 4, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0446340888738632, -0.0582992509007454, 0.0854041278362274, new int[] { 0, 11, 18, 4, -1 }, new int[] { 0, 13, 18, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.020959110930562, 0.17154890298843381, -0.0302498191595078, new int[] { 1, 13, 16, 2, -1 }, new int[] { 1, 14, 16, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0486911907792091, 0.0214052200317383, -0.231359601020813, new int[] { 2, 0, 6, 10, -1 }, new int[] { 2, 0, 3, 5, 2 }, new int[] { 5, 5, 3, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0334771387279034, -0.0175353996455669, 0.2070588022470474, true, new int[] { 14, 3, 4, 3, -1 }, new int[] { 13, 4, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0157824493944645, 0.20446999371051791, -0.0294545702636242, true, new int[] { 4, 3, 3, 3, -1 }, new int[] { 5, 4, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0216255001723766, -0.0121418898925185, 0.25204500555992132, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0051940139383077621, -0.1221897974610329, 0.0451432801783085, true, new int[] { 7, 7, 4, 1, -1 }, new int[] { 8, 8, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0313102789223194, 0.28687921166419977, -0.00829025823622942, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0155427400022745, 0.0274001006036997, -0.20353409647941589, new int[] { 4, 0, 2, 10, -1 }, new int[] { 4, 5, 2, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0052836928516626358, 0.054194588214159, -0.0240161493420601, true, new int[] { 16, 2, 2, 3, -1 }, new int[] { 15, 3, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0064056441187858582, 0.13316440582275391, -0.04658317938447, true, new int[] { 2, 2, 3, 2, -1 }, new int[] { 3, 3, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0047195679508149624, -0.1046644002199173, 0.0291981901973486, true, new int[] { 15, 1, 3, 2, -1 }, new int[] { 16, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0122418403625488, -0.35400021076202393, 0.0156168602406979, true, new int[] { 3, 1, 2, 3, -1 }, new int[] { 2, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0034770739730447531, 0.0471543706953526, -0.0372542105615139, true, new int[] { 10, 0, 2, 8, -1 }, new int[] { 8, 2, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1831195950508118, -0.0496848896145821, 0.1203569024801254, new int[] { 0, 0, 15, 13, -1 }, new int[] { 5, 0, 5, 13, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1365886926651001, -0.22701020538806921, 0.0083362739533185959, new int[] { 5, 6, 12, 9, -1 }, new int[] { 8, 6, 6, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0449327491223812, 0.0796067118644714, -0.0694770887494087, new int[] { 1, 6, 12, 9, -1 }, new int[] { 4, 6, 6, 9, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010785179911181331, 0.1114739030599594, -0.0302823390811682, new int[] { 16, 5, 2, 2, -1 }, new int[] { 17, 5, 1, 1, 2 }, new int[] { 16, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00086406682385131717, -0.1434711962938309, 0.0378380417823792, new int[] { 7, 2, 2, 2, -1 }, new int[] { 7, 3, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0014584630262106657, -0.0272518005222082, 0.15474230051040649, new int[] { 16, 5, 2, 2, -1 }, new int[] { 17, 5, 1, 1, 2 }, new int[] { 16, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.18864479660987851, 0.17952759563922879, -0.0304256193339825, new int[] { 4, 0, 10, 13, -1 }, new int[] { 9, 0, 5, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-9.0535402705427259E-05, 0.0379448309540749, -0.0349269211292267, new int[] { 16, 5, 2, 2, -1 }, new int[] { 17, 5, 1, 1, 2 }, new int[] { 16, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00098015682306140661, 0.1471706032752991, -0.0350825004279613, new int[] { 0, 5, 2, 2, -1 }, new int[] { 0, 5, 1, 1, 2 }, new int[] { 1, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0126139298081398, -0.23039579391479489, 0.0261014793068171, new int[] { 0, 5, 18, 2, -1 }, new int[] { 9, 5, 9, 1, 2 }, new int[] { 0, 6, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1353210437810048E-05, -0.0731913670897484, 0.0707238763570786, new int[] { 0, 13, 2, 2, -1 }, new int[] { 0, 13, 1, 1, 2 }, new int[] { 1, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011017440119758248, 0.10001300275325779, -0.019991559907794, new int[] { 16, 10, 2, 2, -1 }, new int[] { 17, 10, 1, 1, 2 }, new int[] { 16, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.3879110813140869E-05, -0.0730697214603424, 0.0769988894462585, new int[] { 0, 10, 2, 2, -1 }, new int[] { 0, 10, 1, 1, 2 }, new int[] { 1, 11, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0085628936067223549, 0.0538700483739376, -0.0811710432171822, new int[] { 7, 7, 5, 2, -1 }, new int[] { 7, 8, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.24042169749736789, -0.0140129402279854, 0.503661572933197, true, new int[] { 8, 3, 9, 3, -1 }, new int[] { 11, 6, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054416628554463387, 0.025490989908576, -0.12167359888553619, true, new int[] { 15, 1, 3, 2, -1 }, new int[] { 16, 2, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0123843001201749, 0.0125095099210739, -0.38121658563613892, true, new int[] { 3, 1, 2, 3, -1 }, new int[] { 2, 2, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0969182103872299, -0.0125396698713303, 0.1020260006189346, true, new int[] { 11, 2, 1, 10, -1 }, new int[] { 11, 2, 1, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.12472909688949591, 0.0086807161569595337, -0.60219871997833252, true, new int[] { 7, 2, 10, 1, -1 }, new int[] { 7, 2, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.1862320106010884E-05, -0.0602015890181065, 0.0648947283625603, new int[] { 13, 0, 2, 2, -1 }, new int[] { 14, 0, 1, 1, 2 }, new int[] { 13, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.2220391808077693E-05, 0.0786095485091209, -0.0601177997887135, new int[] { 3, 0, 2, 2, -1 }, new int[] { 3, 0, 1, 1, 2 }, new int[] { 4, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-2.3879110813140869E-05, 0.0795721486210823, -0.0547612011432648, new int[] { 13, 0, 2, 2, -1 }, new int[] { 14, 0, 1, 1, 2 }, new int[] { 13, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.4684870368218981E-05, -0.0759956613183022, 0.0895266085863113, new int[] { 3, 0, 2, 2, -1 }, new int[] { 3, 0, 1, 1, 2 }, new int[] { 4, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0666326731443405, 0.0116960098966956, -0.38171160221099848, new int[] { 6, 8, 6, 3, -1 }, new int[] { 8, 9, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020522400736808777, -0.0348950810730457, 0.134132906794548, new int[] { 5, 10, 1, 3, -1 }, new int[] { 5, 11, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00393071910366416, -0.066283293068409, 0.029610849916935, new int[] { 17, 7, 1, 6, -1 }, new int[] { 17, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.012441479600966, 0.0159051697701216, -0.32050350308418268, new int[] { 0, 7, 1, 6, -1 }, new int[] { 0, 9, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0388024896383286, -0.0152452699840069, 0.063629612326622, new int[] { 11, 4, 3, 9, -1 }, new int[] { 12, 7, 1, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.3351631979458034E-05, 0.0617886707186699, -0.0717490166425705, new int[] { 0, 5, 1, 2, -1 }, new int[] { 0, 6, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.024020129814744, 0.2426270991563797, -0.0087506501004099846, new int[] { 7, 10, 8, 2, -1 }, new int[] { 11, 10, 4, 1, 2 }, new int[] { 7, 11, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0067699998617172241, -0.0331209786236286, 0.14404219388961789, new int[] { 3, 10, 8, 2, -1 }, new int[] { 3, 10, 4, 1, 2 }, new int[] { 7, 11, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.16888360679149631, 0.35152590274810791, -0.0071931672282516956, true, new int[] { 10, 5, 3, 6, -1 }, new int[] { 8, 7, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.067578062415123, -0.22686310112476349, 0.0256022103130817, true, new int[] { 8, 5, 6, 3, -1 }, new int[] { 10, 7, 2, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0113558797165751, -0.62450700998306274, 0.0025642369873821735, new int[] { 12, 11, 1, 2, -1 }, new int[] { 12, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0778802484273911, 0.007915940135717392, -0.56059461832046509, true, new int[] { 8, 0, 9, 4, -1 }, new int[] { 7, 1, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0029031829908490181, 0.0941536873579025, -0.0496119000017643, new int[] { 8, 5, 2, 2, -1 }, new int[] { 8, 6, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0024730090517550707, 0.1085821017622948, -0.0538938194513321, new int[] { 4, 1, 10, 6, -1 }, new int[] { 4, 3, 10, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038511860184371471, 0.0234237797558308, -0.13090899586677551, new int[] { 6, 0, 7, 3, -1 }, new int[] { 6, 1, 7, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00223901798017323, -0.21743249893188479, 0.0244357194751501, new int[] { 7, 0, 3, 2, -1 }, new int[] { 7, 1, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073695750907063484, -0.0247745793312788, 0.1158865988254547, true, new int[] { 15, 2, 3, 2, -1 }, new int[] { 15, 2, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0046323970891535282, 0.12989379465579989, -0.0381496995687485, true, new int[] { 3, 2, 2, 3, -1 }, new int[] { 3, 2, 1, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0199226494878531, 0.0158690698444843, -0.18562960624694819, new int[] { 14, 0, 2, 14, -1 }, new int[] { 14, 0, 1, 14, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0167268496006727, 0.16922770440578461, -0.032117698341608, new int[] { 3, 3, 12, 6, -1 }, new int[] { 7, 5, 4, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0014559989795088768, 0.0727108269929886, -0.0531024895608425, true, new int[] { 12, 3, 1, 2, -1 }, new int[] { 12, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1436896026134491, -0.10999070107936861, 0.0632115080952644, true, new int[] { 8, 0, 9, 6, -1 }, new int[] { 8, 0, 9, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0059681031852960587, 0.0853514671325684, -0.0319969989359379, true, new int[] { 16, 0, 2, 3, -1 }, new int[] { 15, 1, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0008606793126091361, -0.0677398666739464, 0.0783357918262482, true, new int[] { 2, 0, 3, 2, -1 }, new int[] { 3, 1, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022462129127234221, 0.0421381592750549, -0.15379780530929571, true, new int[] { 16, 12, 2, 1, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.023184010758996, 0.23559680581092829, -0.0220876298844814, new int[] { 3, 12, 12, 2, -1 }, new int[] { 3, 13, 12, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(8.35188475321047E-05, -0.0491336695849895, 0.0353255607187748, new int[] { 12, 11, 1, 2, -1 }, new int[] { 12, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0022380428854376078, 0.179789200425148, -0.0249581690877676, new int[] { 4, 9, 2, 2, -1 }, new int[] { 4, 9, 1, 1, 2 }, new int[] { 5, 10, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036487199831753969, -0.0488890595734119, 0.0157207604497671, true, new int[] { 16, 12, 2, 1, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0024686430115252733, 0.0342142805457115, -0.1369293928146362, true, new int[] { 2, 12, 1, 2, -1 }, new int[] { 2, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0179013404995203, 0.20170210301876071, -0.0058616171590983868, true, new int[] { 14, 10, 4, 1, -1 }, new int[] { 15, 11, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00054372870363295078, -0.0817660167813301, 0.0578251294791698, true, new int[] { 4, 10, 1, 4, -1 }, new int[] { 3, 11, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0007220237166620791, 0.024502370506525, -0.0610220991075039, true, new int[] { 16, 9, 2, 1, -1 }, new int[] { 16, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0036474859807640314, -0.14141079783439639, 0.0364049896597862, true, new int[] { 2, 9, 1, 2, -1 }, new int[] { 2, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0007320601143874228, -0.0436596609652042, 0.0481952391564846, new int[] { 11, 5, 2, 1, -1 }, new int[] { 11, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0310860797762871, 0.0367696695029736, -0.1427676975727081, true, new int[] { 9, 0, 7, 4, -1 }, new int[] { 8, 1, 7, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0069447411224246025, 0.350436806678772, -0.0070687229745090008, new int[] { 11, 4, 2, 2, -1 }, new int[] { 11, 4, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.020435830578208E-05, -0.1218914985656738, 0.0413166508078575, new int[] { 5, 4, 2, 2, -1 }, new int[] { 6, 4, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0366099290549755, 0.0199259296059608, -0.0984719917178154, new int[] { 7, 4, 6, 3, -1 }, new int[] { 9, 5, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0109604299068451, 0.1281152069568634, -0.0383881889283657, new int[] { 5, 4, 4, 7, -1 }, new int[] { 6, 4, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0023295450955629349, 0.0707607492804527, -0.0289194602519274, new int[] { 17, 0, 1, 4, -1 }, new int[] { 17, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0618558302521706, -0.0475871004164219, 0.0985863581299782, true, new int[] { 4, 3, 8, 4, -1 }, new int[] { 4, 3, 8, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0234752092510462, 0.086964562535286, -0.0122541096061468, true, new int[] { 9, 7, 4, 2, -1 }, new int[] { 9, 7, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00093669712077826262, 0.0812510773539543, -0.0542218498885632, true, new int[] { 7, 8, 3, 2, -1 }, new int[] { 7, 8, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1315189003944397, -0.1539728045463562, 0.0100725498050451, new int[] { 8, 3, 6, 9, -1 }, new int[] { 10, 6, 2, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0088957380503416061, 0.0319623500108719, -0.13615429401397711, new int[] { 7, 4, 4, 6, -1 }, new int[] { 7, 4, 2, 3, 2 }, new int[] { 9, 7, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-8.2765902334358543E-05, 0.0532807409763336, -0.0550383105874062, new int[] { 16, 1, 2, 1, -1 }, new int[] { 16, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0020361710339784622, 0.0354836508631706, -0.1206891983747482, true, new int[] { 3, 0, 1, 3, -1 }, new int[] { 2, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028764940798282623, -0.0278693605214357, 0.104407399892807, new int[] { 17, 0, 1, 4, -1 }, new int[] { 17, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00099125062115490437, 0.0979837700724602, -0.0593339614570141, new int[] { 0, 0, 1, 4, -1 }, new int[] { 0, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0300707891583443, 0.0164330396801233, -0.0933536067605019, new int[] { 13, 3, 4, 10, -1 }, new int[] { 13, 3, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.2220391808077693E-05, 0.0752206817269325, -0.057729821652174, new int[] { 3, 0, 2, 2, -1 }, new int[] { 3, 0, 1, 1, 2 }, new int[] { 4, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1495593935251236, -0.57173091173172, 0.0074865440838038921, new int[] { 0, 0, 18, 6, -1 }, new int[] { 0, 3, 18, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0101018501445651, 0.18661679327487951, -0.0265819206833839, new int[] { 4, 3, 2, 6, -1 }, new int[] { 4, 5, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0235938206315041, -0.36165231466293341, 0.0086832279339432716, true, new int[] { 14, 3, 1, 6, -1 }, new int[] { 12, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.056298989802599, 0.38091570138931269, -0.0125403897836804, true, new int[] { 9, 0, 8, 4, -1 }, new int[] { 11, 2, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.8374498874181882E-05, -0.0372395589947701, 0.0435059703886509, new int[] { 12, 4, 2, 2, -1 }, new int[] { 12, 5, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.319483847822994E-05, -0.0574802309274673, 0.0801668912172318, new int[] { 6, 7, 2, 2, -1 }, new int[] { 6, 7, 1, 1, 2 }, new int[] { 7, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0226483792066574, -0.0914651080965996, 0.0060311011038720608, new int[] { 14, 0, 2, 15, -1 }, new int[] { 14, 0, 1, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0075446818955242634, 0.0277416408061981, -0.171812504529953, new int[] { 2, 0, 2, 15, -1 }, new int[] { 3, 0, 1, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1057740971446037, 0.53441429138183594, -0.0051590129733085632, new int[] { 11, 1, 6, 6, -1 }, new int[] { 11, 1, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0044444771483540535, 0.0343015491962433, -0.1451483964920044, true, new int[] { 8, 7, 3, 1, -1 }, new int[] { 9, 8, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0056781400926411152, -0.0430911704897881, 0.14633339643478391, new int[] { 13, 9, 3, 3, -1 }, new int[] { 14, 10, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.10109309852123261, -0.17478010058403021, 0.0280684307217598, new int[] { 0, 5, 16, 6, -1 }, new int[] { 4, 5, 8, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0473572388291359, 0.1670453995466232, -0.0158186703920364, new int[] { 4, 0, 12, 15, -1 }, new int[] { 7, 0, 6, 15, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.5767403244972229, -0.62243127822875977, 0.0079542007297277451, new int[] { 0, 5, 16, 10, -1 }, new int[] { 8, 5, 8, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.001805974985472858, -0.0164429899305105, 0.0462612397968769, true, new int[] { 9, 9, 1, 3, -1 }, new int[] { 8, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0146800400689244, 0.0081173582002520561, -0.55666857957839966, true, new int[] { 9, 9, 3, 1, -1 }, new int[] { 10, 10, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1689784973859787, -0.31401479244232178, 0.0125729897990823, new int[] { 13, 0, 5, 10, -1 }, new int[] { 13, 5, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0193899292498827, 0.1551029980182648, -0.0279963091015816, new int[] { 2, 6, 14, 3, -1 }, new int[] { 2, 7, 14, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.026446670293808, -0.3146206140518189, 0.017393559217453, new int[] { 6, 6, 6, 3, -1 }, new int[] { 8, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00457324692979455, -0.13583140075206759, 0.0376659594476223, new int[] { 0, 6, 3, 4, -1 }, new int[] { 0, 7, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0078531084582209587, -0.0036102959420531988, 0.1896488964557648, new int[] { 12, 11, 1, 2, -1 }, new int[] { 12, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7107769710710272E-05, -0.0843098610639572, 0.0545401610434055, new int[] { 5, 11, 1, 2, -1 }, new int[] { 5, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0203770492225885, 0.11659640073776251, -0.0136959999799728, new int[] { 13, 9, 3, 3, -1 }, new int[] { 14, 10, 1, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.17351469397544861, -0.0126557499170303, 0.35746860504150391, new int[] { 0, 3, 12, 11, -1 }, new int[] { 3, 3, 6, 11, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0542285591363907, 0.0092725036665797234, -0.17699269950389859, new int[] { 7, 12, 9, 3, -1 }, new int[] { 10, 12, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054582608863711357, -0.0437470003962517, 0.1033746972680092, new int[] { 3, 8, 2, 6, -1 }, new int[] { 3, 10, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0637689232826233, 0.0219606403261423, -0.2052810937166214, new int[] { 12, 3, 6, 12, -1 }, new int[] { 12, 9, 6, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0112160202115774, -0.0601588003337383, 0.0776893869042397, new int[] { 2, 13, 12, 2, -1 }, new int[] { 8, 13, 6, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0393657200038433, -0.0201384108513594, 0.12760840356349951, new int[] { 8, 12, 8, 3, -1 }, new int[] { 8, 12, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0161337107419968, 0.1127976030111313, -0.0601407214999199, new int[] { 2, 12, 8, 3, -1 }, new int[] { 6, 12, 4, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016923110233619809, 0.0280561596155167, -0.0492299310863018, true, new int[] { 9, 0, 2, 2, -1 }, new int[] { 9, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.9907790526049212E-05, 0.0722095370292664, -0.057712871581316, new int[] { 5, 4, 2, 2, -1 }, new int[] { 5, 4, 1, 1, 2 }, new int[] { 6, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00838569924235344, 0.0042978320270776749, -0.48725709319114691, new int[] { 11, 0, 2, 4, -1 }, new int[] { 11, 1, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0068764640018343925, -0.35551750659942633, 0.0109930103644729, true, new int[] { 8, 0, 2, 2, -1 }, new int[] { 8, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0074763470329344273, 0.161957398056984, -0.0268841590732336, new int[] { 5, 0, 8, 4, -1 }, new int[] { 7, 0, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0038878160994499922, -0.1101962998509407, 0.0409429408609867, new int[] { 3, 1, 12, 1, -1 }, new int[] { 6, 1, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0632312968373299, 0.49099159240722662, -0.0051781800575554371, new int[] { 10, 3, 6, 4, -1 }, new int[] { 10, 3, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0566077493131161, 0.37937331199646, -0.0108209000900388, new int[] { 2, 3, 6, 4, -1 }, new int[] { 5, 3, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.26267269253730768, -0.44802859425544739, 0.0105561902746558, new int[] { 1, 2, 16, 13, -1 }, new int[] { 5, 2, 8, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.4856478527653962E-05, 0.0653926804661751, -0.0620450004935265, new int[] { 1, 0, 2, 2, -1 }, new int[] { 1, 0, 1, 1, 2 }, new int[] { 2, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(4.7022080252645537E-05, -0.0353392213582993, 0.0484495908021927, new int[] { 15, 0, 2, 2, -1 }, new int[] { 16, 0, 1, 1, 2 }, new int[] { 15, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(7.6384996646083891E-05, -0.0554682798683643, 0.0811991393566132, new int[] { 1, 0, 2, 2, -1 }, new int[] { 1, 0, 1, 1, 2 }, new int[] { 2, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.13491000235080719, -0.56497681140899658, 0.005841605830937624, new int[] { 10, 8, 8, 7, -1 }, new int[] { 12, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0173286907374859, 0.0686116516590118, -0.0624860487878323, new int[] { 0, 8, 8, 7, -1 }, new int[] { 2, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1159003973007202, 0.35991528630256647, -0.0070457011461257935, new int[] { 11, 8, 6, 3, -1 }, new int[] { 13, 9, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0025972709991037846, -0.0610489808022976, 0.0729080066084862, new int[] { 1, 8, 6, 3, -1 }, new int[] { 3, 9, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.58517122268676758, 0.170673206448555, -0.0274902693927288, new int[] { 1, 0, 16, 12, -1 }, new int[] { 1, 6, 16, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0164765398949385, 0.13038930296897891, -0.0331927388906479, true, new int[] { 9, 0, 5, 4, -1 }, new int[] { 9, 0, 5, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0474574081599712, 0.0938887968659401, -0.0477792508900166, new int[] { 7, 0, 4, 10, -1 }, new int[] { 7, 0, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0071776830591261387, -0.1972271949052811, 0.0238158907741308, true, new int[] { 7, 4, 3, 2, -1 }, new int[] { 8, 5, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00089368229964748025, -0.0385106988251209, 0.12537740170955661, new int[] { 12, 7, 2, 2, -1 }, new int[] { 13, 7, 1, 1, 2 }, new int[] { 12, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.15897080302238459, 0.42691999673843378, -0.0113530196249485, new int[] { 0, 6, 9, 3, -1 }, new int[] { 3, 7, 3, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0015724339755252004, 0.130340501666069, -0.029230359941721, new int[] { 12, 7, 2, 2, -1 }, new int[] { 13, 7, 1, 1, 2 }, new int[] { 12, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(6.2912302382756025E-05, -0.053911529481411, 0.0894209668040276, new int[] { 4, 7, 2, 2, -1 }, new int[] { 4, 7, 1, 1, 2 }, new int[] { 5, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.007953789085149765, 0.0292203202843666, -0.16147419810295111, new int[] { 5, 7, 8, 2, -1 }, new int[] { 9, 7, 4, 1, 2 }, new int[] { 5, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.02785431034863, 0.0081461891531944275, -0.501079797744751, new int[] { 6, 4, 3, 9, -1 }, new int[] { 7, 4, 1, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0307268109172583, -0.3919588029384613, 0.006921504158526659, new int[] { 12, 4, 3, 7, -1 }, new int[] { 13, 4, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0356646999716759, -0.75857192277908325, 0.0053641172125935555, new int[] { 3, 4, 3, 7, -1 }, new int[] { 4, 4, 1, 7, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0360276810824871, -0.0191031396389008, 0.24392929673194891, new int[] { 4, 12, 10, 3, -1 }, new int[] { 4, 13, 10, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00065820151939988136, -0.0886877924203873, 0.0565083399415016, new int[] { 4, 13, 8, 2, -1 }, new int[] { 4, 14, 8, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1285891979932785, -0.1347049027681351, 0.0150261903181672, new int[] { 13, 3, 5, 12, -1 }, new int[] { 13, 6, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0254423692822456, -0.19021469354629519, 0.0212604906409979, new int[] { 0, 0, 6, 8, -1 }, new int[] { 0, 2, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0512643307447433, -0.0036050491034984589, 0.37001758813858032, true, new int[] { 14, 5, 3, 3, -1 }, new int[] { 13, 6, 3, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0326501503586769, -0.0135911498218775, 0.32766878604888922, true, new int[] { 4, 5, 3, 3, -1 }, new int[] { 5, 6, 1, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0055878241546452045, -0.0084945466369390488, 0.0897279679775238, true, new int[] { 17, 0, 1, 3, -1 }, new int[] { 16, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0458750911056995, 0.41267889738082891, -0.0098934909328818321, true, new int[] { 9, 4, 3, 4, -1 }, new int[] { 8, 5, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0054674488492310047, -0.0308022703975439, 0.0607560500502586, new int[] { 7, 3, 6, 2, -1 }, new int[] { 7, 3, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.106912799179554, -0.0305466204881668, 0.14703939855098719, true, new int[] { 9, 0, 9, 2, -1 }, new int[] { 12, 3, 3, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0582343190908432, 0.0017207229975610972, -0.60017997026443481, new int[] { 7, 3, 6, 2, -1 }, new int[] { 7, 3, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.054181519895792, 0.0111133400350809, -0.42601078748703, new int[] { 5, 3, 6, 2, -1 }, new int[] { 8, 3, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.19892099499702451, 0.0015127729857340455, -0.66665178537368774, new int[] { 13, 3, 5, 12, -1 }, new int[] { 13, 6, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0836698114871979, -0.1597495973110199, 0.0258307307958603, new int[] { 0, 3, 5, 12, -1 }, new int[] { 0, 6, 5, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0383935607969761, -0.41582900285720831, 0.0097704501822590828, new int[] { 4, 14, 10, 1, -1 }, new int[] { 4, 14, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0576191917061806, 0.0093507859855890274, -0.4187014102935791, new int[] { 2, 12, 9, 3, -1 }, new int[] { 5, 12, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0440335609018803, -0.0463782697916031, 0.0919744595885277, new int[] { 2, 9, 14, 4, -1 }, new int[] { 2, 11, 14, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.26608958840370178, 0.0100852102041245, -0.38973841071128851, new int[] { 0, 2, 18, 8, -1 }, new int[] { 0, 4, 18, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0536184795200825, -0.50888961553573608, 0.0040682330727577209, new int[] { 7, 6, 6, 3, -1 }, new int[] { 9, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.6047519794665277E-05, 0.0691266432404518, -0.0591945089399815, new int[] { 7, 0, 2, 2, -1 }, new int[] { 7, 0, 1, 1, 2 }, new int[] { 8, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.5685410188743845E-05, -0.0400558486580849, 0.0543046407401562, new int[] { 9, 0, 2, 2, -1 }, new int[] { 10, 0, 1, 1, 2 }, new int[] { 9, 1, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-5.3049330745125189E-05, 0.0731744170188904, -0.0598583295941353, new int[] { 7, 0, 1, 3, -1 }, new int[] { 7, 1, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0124693904072046, -0.315225213766098, 0.0117351301014423, new int[] { 4, 0, 11, 3, -1 }, new int[] { 4, 1, 11, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0927336066961288, 0.32328981161117548, -0.0127641502767801, new int[] { 0, 9, 10, 6, -1 }, new int[] { 0, 9, 5, 3, 2 }, new int[] { 5, 12, 5, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.59549742937088013, 0.0083142714574933052, -0.56721991300582886, new int[] { 0, 4, 18, 9, -1 }, new int[] { 6, 4, 6, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.53787451982498169, -0.0141389099881053, 0.32671380043029791, new int[] { 2, 4, 12, 9, -1 }, new int[] { 6, 7, 4, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.19027920067310331, -0.6616215705871582, 0.0074805710464715958, new int[] { 6, 7, 6, 6, -1 }, new int[] { 6, 10, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.067436046898365, 0.0053405929356813431, -0.57537007331848145, new int[] { 0, 9, 3, 6, -1 }, new int[] { 0, 12, 3, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.001784904976375401, 0.0343016088008881, -0.1244985982775688, new int[] { 8, 8, 3, 2, -1 }, new int[] { 8, 9, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.017916489392519, 0.2131116986274719, -0.0218786392360926, new int[] { 4, 4, 10, 2, -1 }, new int[] { 4, 5, 10, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0034813389647752047, 0.0268206801265478, -0.10166029632091519, new int[] { 8, 2, 5, 3, -1 }, new int[] { 8, 3, 5, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0016392209799960256, -0.0226296707987785, 0.16795350611209869, new int[] { 8, 13, 2, 2, -1 }, new int[] { 8, 13, 1, 1, 2 }, new int[] { 9, 14, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.8717228966997936E-05, -0.0969148203730583, 0.054079819470644, true, new int[] { 14, 0, 2, 2, -1 }, new int[] { 14, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0011430910089984536, -0.0913046523928642, 0.0478410087525845, new int[] { 4, 4, 3, 2, -1 }, new int[] { 4, 5, 3, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.12747140228748319, 0.1231575012207031, -0.039322629570961, new int[] { 1, 5, 16, 4, -1 }, new int[] { 1, 7, 16, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0409889809088781E-05, -0.046518798917532, 0.0935849994421005, new int[] { 4, 6, 1, 2, -1 }, new int[] { 4, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.007715822197496891, -0.6546670794487, 0.0039967028424143791, new int[] { 16, 11, 2, 2, -1 }, new int[] { 17, 11, 1, 1, 2 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(2.7107769710710272E-05, -0.0640250220894814, 0.0632654428482056, new int[] { 0, 11, 2, 2, -1 }, new int[] { 0, 11, 1, 1, 2 }, new int[] { 1, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0015383179998025298, 0.0226351507008076, -0.1935117989778519, new int[] { 16, 12, 2, 1, -1 }, new int[] { 16, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.4936917624436319E-05, 0.0578822083771229, -0.0738588199019432, new int[] { 0, 12, 2, 1, -1 }, new int[] { 1, 12, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.136530801653862, -0.0149675700813532, 0.26669740676879877, new int[] { 0, 7, 18, 6, -1 }, new int[] { 0, 9, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.18993720412254331, 0.012506790459156, -0.35344779491424561, new int[] { 4, 2, 7, 12, -1 }, new int[] { 4, 5, 7, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0314559191465378, 0.0183809790760279, -0.0603883489966393, new int[] { 7, 6, 6, 3, -1 }, new int[] { 9, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0269035492092371, -0.22182409465312961, 0.0186347793787718, new int[] { 5, 6, 6, 3, -1 }, new int[] { 7, 7, 2, 1, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.25814530253410339, -0.80185538530349731, 0.00038440190837718546, true, new int[] { 12, 3, 6, 6, -1 }, new int[] { 12, 3, 6, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.1513974070549011, 0.0267061796039343, -0.15360879898071289, true, new int[] { 6, 3, 6, 6, -1 }, new int[] { 6, 3, 3, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0440951585769653, 0.0494831092655659, -0.0132203595712781, new int[] { 4, 2, 12, 9, -1 }, new int[] { 8, 2, 4, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017376670148223639, -0.0296104997396469, 0.12741160392761231, new int[] { 1, 7, 3, 1, -1 }, new int[] { 2, 7, 1, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0067472518421709538, 0.0369098298251629, -0.1863466948270798, true, new int[] { 14, 0, 2, 2, -1 }, new int[] { 14, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.27132511138916021, 0.43453308939933782, -0.0090847145766019821, new int[] { 2, 2, 12, 9, -1 }, new int[] { 6, 2, 4, 9, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0047428919933736324, 0.0166457295417786, -0.0998101606965065, true, new int[] { 9, 1, 1, 4, -1 }, new int[] { 8, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0098173134028911591, -0.0557747483253479, 0.0711958929896355, new int[] { 2, 0, 12, 1, -1 }, new int[] { 5, 0, 6, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011679739691317082, -0.0676950290799141, 0.0412361510097981, true, new int[] { 11, 0, 4, 1, -1 }, new int[] { 11, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0031285739969462156, 0.0793463066220284, -0.0644870027899742, true, new int[] { 9, 0, 4, 1, -1 }, new int[] { 9, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0021147250663489103, -0.1048358008265495, 0.0149682499468327, new int[] { 9, 3, 2, 1, -1 }, new int[] { 9, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017796000465750694, 0.28925609588623052, -0.0134435798972845, new int[] { 7, 3, 2, 1, -1 }, new int[] { 8, 3, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.21853840351104739, -0.56218802928924561, 0.0024572419933974743, true, new int[] { 9, 2, 8, 4, -1 }, new int[] { 9, 2, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0542420297861099, -0.21208059787750241, 0.019283739849925, new int[] { 0, 3, 18, 1, -1 }, new int[] { 9, 3, 9, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00225058407522738, 0.0087050450965762138, -0.0469894893467426, new int[] { 13, 13, 2, 2, -1 }, new int[] { 13, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0273687392473221, 0.0053823711350560188, -0.73394852876663208, true, new int[] { 7, 0, 1, 4, -1 }, new int[] { 7, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0171208307147026, 0.17836299538612371, -0.00798860564827919, new int[] { 16, 7, 2, 4, -1 }, new int[] { 16, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0058321221731603146, 0.0193902608007193, -0.2057818025350571, new int[] { 0, 7, 2, 4, -1 }, new int[] { 0, 8, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00069258757866919041, 0.0525361597537994, -0.0348935909569263, new int[] { 9, 13, 3, 2, -1 }, new int[] { 10, 13, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021873079240322113, -0.0308929309248924, 0.11824580281972891, new int[] { 6, 13, 3, 2, -1 }, new int[] { 7, 13, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0026870400179177523, -0.0478884391486645, 0.0109662897884846, new int[] { 13, 13, 2, 2, -1 }, new int[] { 13, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0017761799972504377, 0.0283233094960451, -0.13571000099182129, new int[] { 3, 13, 2, 2, -1 }, new int[] { 4, 13, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0268767699599266, 0.0109366700053215, -0.13214470446109769, new int[] { 10, 11, 6, 4, -1 }, new int[] { 12, 11, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0397437512874603, -0.27749490737915039, 0.014792799949646, new int[] { 2, 11, 6, 4, -1 }, new int[] { 4, 11, 2, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0519120208919048, -0.0306210797280073, 0.1386394947767258, new int[] { 0, 11, 18, 3, -1 }, new int[] { 6, 11, 6, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-4.965993866790086E-05, 0.0652230083942413, -0.0611205287277699, new int[] { 6, 13, 4, 1, -1 }, new int[] { 7, 13, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0208992697298527, 0.0100139798596501, -0.3789927065372467, new int[] { 7, 9, 4, 3, -1 }, new int[] { 7, 10, 4, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0346408486366272, -0.0236316304653883, 0.1669196039438248, new int[] { 5, 5, 4, 9, -1 }, new int[] { 5, 8, 4, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028383019380271435, 0.0228540804237127, -0.0597838684916496, true, new int[] { 13, 5, 2, 8, -1 }, new int[] { 11, 7, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0021739569492638111, -0.0186796691268682, 0.1997753977775574, new int[] { 6, 8, 2, 2, -1 }, new int[] { 6, 8, 1, 1, 2 }, new int[] { 7, 9, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0150487199425697, -0.31850379705429083, 0.0032470070291310549, true, new int[] { 9, 1, 1, 4, -1 }, new int[] { 8, 2, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0070679760538041592, -0.349465012550354, 0.0113516096025705, true, new int[] { 9, 1, 4, 1, -1 }, new int[] { 10, 2, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.20126479864120481, -0.0153439603745937, 0.270695686340332, new int[] { 6, 2, 6, 9, -1 }, new int[] { 6, 5, 6, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0454341918230057, -0.1544011980295181, 0.0267359893769026, true, new int[] { 9, 6, 1, 6, -1 }, new int[] { 7, 8, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(5.0224931328557432E-05, -0.0454120188951492, 0.0583584196865559, new int[] { 10, 6, 2, 2, -1 }, new int[] { 11, 6, 1, 1, 2 }, new int[] { 10, 7, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0028120330534875393, -0.0352263003587723, 0.1206099987030029, new int[] { 6, 7, 3, 2, -1 }, new int[] { 7, 7, 1, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.10989969968795781, -0.0082655288279056549, 0.27113300561904907, true, new int[] { 12, 4, 2, 6, -1 }, new int[] { 10, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0350026711821556, 0.0418249294161797, -0.14443680644035339, true, new int[] { 6, 4, 6, 2, -1 }, new int[] { 8, 6, 2, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0569862984120846, -0.0448646917939186, 0.0947646573185921, new int[] { 0, 5, 18, 6, -1 }, new int[] { 0, 7, 18, 2, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.001924803014844656, 0.0438571982085705, -0.11556690186262129, new int[] { 7, 6, 4, 2, -1 }, new int[] { 7, 7, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0364132300019264, -0.0259249694645405, 0.0877993777394295, new int[] { 12, 6, 6, 9, -1 }, new int[] { 14, 9, 2, 3, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049817138351500034, -0.0624991990625858, 0.062983013689518, new int[] { 4, 6, 10, 3, -1 }, new int[] { 4, 7, 10, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0157324392348528, 0.1091820001602173, -0.0354424603283405, new int[] { 13, 3, 4, 8, -1 }, new int[] { 13, 5, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0323861613869667, -0.61410897970199585, 0.0061990139074623585, new int[] { 0, 4, 11, 2, -1 }, new int[] { 0, 5, 11, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.019163029268384, -0.0030063120648264885, 0.48029029369354248, new int[] { 16, 5, 2, 2, -1 }, new int[] { 16, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.6093212808482349E-05, 0.0573367811739445, -0.0716157332062721, new int[] { 0, 5, 2, 2, -1 }, new int[] { 1, 5, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0011779610067605972, 0.0471811406314373, -0.0946075767278671, new int[] { 16, 0, 2, 13, -1 }, new int[] { 16, 0, 1, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0148553596809506, -0.13877260684967041, 0.0338439010083675, new int[] { 0, 0, 2, 13, -1 }, new int[] { 1, 0, 1, 13, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.023859990760684, 0.19980570673942569, -0.0122430603951216, new int[] { 6, 1, 9, 3, -1 }, new int[] { 9, 1, 3, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.078580729663372, -0.49618101119995123, 0.0095836250111460686, true, new int[] { 9, 2, 4, 8, -1 }, new int[] { 9, 2, 4, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0289697498083115, 0.20147210359573359, -0.0211850497871637, new int[] { 3, 1, 12, 9, -1 }, new int[] { 3, 4, 12, 3, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.045099250972271, 0.0072327218949794769, -0.57577252388000488, new int[] { 0, 10, 8, 3, -1 }, new int[] { 0, 11, 8, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0393024682998657, 0.0255729109048843, -0.14938560128211981, new int[] { 0, 11, 18, 4, -1 }, new int[] { 9, 11, 9, 2, 2 }, new int[] { 0, 13, 9, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0384178198873997, 0.0043327999301254749, -0.84697937965393066, true, new int[] { 5, 5, 2, 4, -1 }, new int[] { 4, 6, 2, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0157523807138205, 0.0215584896504879, -0.0945848673582077, new int[] { 8, 0, 3, 6, -1 }, new int[] { 9, 2, 1, 2, 9 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00065488961990922689, -0.1137140020728111, 0.0342830009758472, true, new int[] { 6, 1, 4, 2, -1 }, new int[] { 6, 1, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0060493252240121365, -0.0153995295986533, 0.10828500241041179, new int[] { 13, 12, 2, 3, -1 }, new int[] { 13, 13, 2, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0380066595971584, 0.00871944334357977, -0.45662951469421392, new int[] { 6, 0, 4, 8, -1 }, new int[] { 6, 0, 2, 4, 2 }, new int[] { 8, 4, 2, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0022284449078142643, -0.0540577992796898, 0.0205975491553545, new int[] { 10, 2, 6, 2, -1 }, new int[] { 10, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0116986101493239, 0.18344320356845861, -0.0235534105449915, new int[] { 2, 2, 6, 2, -1 }, new int[] { 5, 2, 3, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0235775094479322, -0.337787002325058, 0.0042076371610164642, new int[] { 13, 0, 4, 3, -1 }, new int[] { 14, 0, 2, 3, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0017685960046947002, -0.1034085005521774, 0.0397500097751617, new int[] { 2, 3, 5, 2, -1 }, new int[] { 2, 4, 5, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0626740828156471, 0.25634589791297913, -0.0026633420493453741, true, new int[] { 13, 0, 3, 11, -1 }, new int[] { 14, 1, 1, 11, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0049983179196715355, -0.0596107505261898, 0.0683519020676613, true, new int[] { 5, 0, 11, 3, -1 }, new int[] { 4, 1, 11, 1, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.013796079903841, -0.12925280630588529, 0.013147160410881, new int[] { 11, 0, 2, 12, -1 }, new int[] { 12, 0, 1, 6, 2 }, new int[] { 11, 6, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0073155229911208153, 0.0236708596348763, -0.173146203160286, new int[] { 5, 0, 2, 12, -1 }, new int[] { 5, 0, 1, 6, 2 }, new int[] { 6, 6, 1, 6, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0160576999187469, 0.0210999101400375, -0.0365347005426884, true, new int[] { 11, 5, 2, 4, -1 }, new int[] { 11, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.1364033967256546, 0.3252066969871521, -0.0125922495499253, new int[] { 1, 3, 3, 12, -1 }, new int[] { 1, 7, 3, 4, 3 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0043760128319263458, -0.0689269527792931, 0.0126556698232889, true, new int[] { 11, 5, 2, 4, -1 }, new int[] { 11, 5, 1, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0251937098801136, 0.63607120513916016, -0.0069624311290681362, true, new int[] { 7, 5, 4, 2, -1 }, new int[] { 7, 5, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0992545634508133, -0.16383069753646851, 0.0402428992092609, new int[] { 13, 0, 5, 10, -1 }, new int[] { 13, 5, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0041403169743716717, 0.0453241616487503, -0.0904397219419479, new int[] { 0, 1, 6, 8, -1 }, new int[] { 0, 5, 6, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0092972591519355774, 0.0730063766241074, -0.021570960059762, new int[] { 14, 7, 4, 2, -1 }, new int[] { 14, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0065849390812218189, -0.14133420586586, 0.0347219407558441, new int[] { 0, 7, 4, 2, -1 }, new int[] { 0, 8, 4, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0825936570763588, 0.0022461370099335909, -0.33250170946121221, new int[] { 14, 3, 3, 8, -1 }, new int[] { 14, 5, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0447855107486248, -0.0163932293653488, 0.31968909502029419, new int[] { 1, 3, 3, 8, -1 }, new int[] { 1, 5, 3, 4, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0149416103959084, -0.013618029654026, 0.091183677315712, new int[] { 11, 3, 4, 10, -1 }, new int[] { 12, 3, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.00048578871064819396, 0.0450273416936398, -0.0991435274481773, new int[] { 3, 3, 4, 10, -1 }, new int[] { 4, 3, 2, 10, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0010591340251266956, 0.0437940806150436, -0.0463229306042194, new int[] { 11, 2, 4, 7, -1 }, new int[] { 12, 2, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0124091897159815, -0.1189147979021072, 0.0417256988584995, new int[] { 3, 2, 4, 7, -1 }, new int[] { 4, 2, 2, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0020622629672288895, 0.1331578940153122, -0.0239935107529163, new int[] { 13, 7, 2, 2, -1 }, new int[] { 14, 7, 1, 1, 2 }, new int[] { 13, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.00088945101015269756, -0.0329415686428547, 0.13120089471340179, new int[] { 3, 7, 2, 2, -1 }, new int[] { 3, 7, 1, 1, 2 }, new int[] { 4, 8, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.0016302269650623202, -0.0539117492735386, 0.0144488299265504, true, new int[] { 14, 0, 2, 2, -1 }, new int[] { 14, 0, 1, 2, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0079654958099126816, 0.0144072799012065, -0.26187309622764587, true, new int[] { 4, 0, 2, 2, -1 }, new int[] { 4, 0, 2, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-6.1501268646679819E-05, 0.0330021195113659, -0.0297673903405666, new int[] { 12, 4, 2, 2, -1 }, new int[] { 13, 4, 1, 1, 2 }, new int[] { 12, 5, 1, 1, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(-0.20129390060901639, -0.49312359094619751, 0.0073236711323261261, new int[] { 5, 1, 8, 14, -1 }, new int[] { 5, 1, 4, 7, 2 }, new int[] { 9, 8, 4, 7, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0012285460252314806, 0.0346601791679859, -0.0940746665000916, new int[] { 13, 0, 5, 10, -1 }, new int[] { 13, 5, 5, 5, 2 }) });
            nodes.Add(new[] { new HaarFeatureNode(0.0104913795366883, -0.0389849282801151, 0.12683519721031189, new int[] { 1, 4, 16, 4, -1 }, new int[] { 1, 6, 16, 2, 2 }) });
            stage.Trees = nodes.ToArray(); stages.Add(stage);
            #endregion


            Stages = stages.ToArray();
        }
    }
}